@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2003 Stuart Parmenter and others,
+ * Copyright (C) 1997-2000 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,10 +27,14 @@
  http://cvs.handhelds.org/cgi-bin/viewcvs.cgi/gpe/base/gpe-contacts
  Tags must be UPPERCASE.
  */
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
 
 #if defined(HAVE_SQLITE)
 
+#include "address-book-gpe.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -39,7 +43,6 @@
 #include <time.h>
 
 #include "address-book.h"
-#include "address-book-gpe.h"
 #include "information.h"
 
 #define ASSURE_GPE_DIR\
@@ -193,21 +196,21 @@ libbalsa_address_book_gpe_open_db(LibBalsaAddressBookGpe * ab)
 {
 #ifdef HAVE_SQLITE3
     gchar *dir, *name;
-    int r;
 
     dir = g_build_filename(g_get_home_dir(), ".gpe", NULL);
     mkdir(dir, S_IRUSR|S_IWUSR|S_IXUSR);
     name = g_build_filename(dir, "contacts", NULL);
     g_free(dir);
-    r = sqlite3_open(name, &ab->db);
-    g_free(name);
 
-    if (r != SQLITE_OK) {
-        printf("Cannot open: %s\n", sqlite3_errmsg(ab->db));
+    if (sqlite3_open(name, &ab->db) != SQLITE_OK) {
+        printf("Cannot open \"%s\": %s\n", name, sqlite3_errmsg(ab->db));
+        g_free(name);
         sqlite3_close(ab->db);
         ab->db = NULL;
         return 0;
     }
+    g_free(name);
+
     sqlite3_exec(ab->db, schema_str,  NULL, NULL, NULL);
     sqlite3_exec(ab->db, schema2_str, NULL, NULL, NULL);
 #else                           /* HAVE_SQLITE3 */
@@ -397,7 +400,11 @@ libbalsa_address_book_gpe_load(LibBalsaAddressBook * ab,
     if(r != SQLITE_OK) {
         printf("r=%d err=%s\n", r, err);
         libbalsa_address_book_set_status(ab, g_strdup(err));
+#ifdef HAVE_SQLITE3
+        sqlite3_free(err);
+#else                           /* HAVE_SQLITE3 */
         free(err);
+#endif                          /* HAVE_SQLITE3 */
         ret = LBABERR_CANNOT_READ;
     } else {
         libbalsa_address_book_set_status(ab, NULL);
@@ -444,13 +451,16 @@ libbalsa_address_book_gpe_add_address(LibBalsaAddressBook *ab,
 #ifdef HAVE_SQLITE3
     r = sqlite3_exec(gpe_ab->db, "insert into contacts_urn values (NULL)",
                      NULL, NULL, &err);
+    if (r != SQLITE_OK) {
+        libbalsa_address_book_set_status(ab, g_strdup(err));
+        sqlite3_free(err);
 #else                           /* HAVE_SQLITE3 */
     r = sqlite_exec(gpe_ab->db, "insert into contacts_urn values (NULL)",
                     NULL, NULL, &err);
-#endif                          /* HAVE_SQLITE3 */
     if (r != SQLITE_OK) {
         libbalsa_address_book_set_status(ab, g_strdup(err));
         free(err);
+#endif                          /* HAVE_SQLITE3 */
         return LBABERR_CANNOT_WRITE;
     }
     /* FIXME: duplicate detection! */
@@ -560,7 +570,11 @@ libbalsa_address_book_gpe_remove_address(LibBalsaAddressBook *ab,
     err = db_delete_by_uid(gpe_ab->db, uid);
     if(err) {
         libbalsa_address_book_set_status(ab, g_strdup(err));
+#ifdef HAVE_SQLITE3
+        sqlite3_free(err);
+#else                           /* HAVE_SQLITE3 */
         free(err);
+#endif                          /* HAVE_SQLITE3 */
         return LBABERR_CANNOT_WRITE;
     } else return LBABERR_OK;
 }
@@ -616,7 +630,7 @@ libbalsa_address_book_gpe_modify_address(LibBalsaAddressBook *ab,
     if ((r = sqlite3_exec(gpe_ab->db, "begin transaction",
                           NULL, NULL, &err)) != SQLITE_OK) {
         libbalsa_address_book_set_status(ab, g_strdup(err));
-        free(err);              /* failed, so soon!? */
+        sqlite3_free(err);              /* failed, so soon!? */
         return LBABERR_CANNOT_WRITE;
     }
 
@@ -678,10 +692,11 @@ libbalsa_address_book_gpe_modify_address(LibBalsaAddressBook *ab,
 
  rollback:
     libbalsa_address_book_set_status(ab, g_strdup(err));
-    free(err);
 #ifdef HAVE_SQLITE3
+    sqlite3_free(err);
     sqlite3_exec(gpe_ab->db, "rollback transaction", NULL, NULL, NULL);
 #else                           /* HAVE_SQLITE3 */
+    free(err);
     sqlite_exec(gpe_ab->db, "rollback transaction", NULL, NULL, NULL);
 #endif                          /* HAVE_SQLITE3 */
 
@@ -729,7 +744,7 @@ gpe_read_completion(void *arg, int argc, char **argv, char **names)
     if(!a->full_name)
         a->full_name = create_name(a->first_name, a->last_name);
     for(l=a->address_list; l; l = l->next) {
-        ia = internet_address_new_name(a->full_name, l->data);
+        ia = internet_address_mailbox_new(a->full_name, l->data);
         gc->res = g_list_prepend(gc->res, ia);
     }
     if(gc->new_prefix && !*gc->new_prefix)
@@ -778,6 +793,10 @@ libbalsa_address_book_gpe_alias_complete(LibBalsaAddressBook * ab,
         r = sqlite3_exec(gpe_ab->db,
                          "select distinct urn from contacts_urn",
                          gpe_read_completion, &gcc, &err);
+    if(err) {
+        printf("r=%d err=%s\n", r, err);
+        sqlite3_free(err);
+    }
 #else                           /* HAVE_SQLITE3 */
     if(prefix)
         r = sqlite_exec_printf(gpe_ab->db, query,
@@ -786,11 +805,11 @@ libbalsa_address_book_gpe_alias_complete(LibBalsaAddressBook * ab,
     else
         r = sqlite_exec(gpe_ab->db, "select distinct urn from contacts_urn",
                         gpe_read_completion, &gcc, &err);
-#endif                          /* HAVE_SQLITE3 */
     if(err) {
         printf("r=%d err=%s\n", r, err);
         free(err);
     }
+#endif                          /* HAVE_SQLITE3 */
     return gcc.res;
 }
 #endif				/* WITH_SQLITE */
