@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2002 Stuart Parmenter and others,
+ * Copyright (C) 1997-2009 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,7 +20,9 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -78,9 +80,10 @@ libbalsa_message_body_free(LibBalsaMessageBody * body)
     g_free(body->filename);
     if (body->file_uri)
         g_object_unref(body->file_uri);
-    if (body->temp_filename)
+    if (body->temp_filename) {
 	unlink(body->temp_filename);
-    g_free(body->temp_filename);
+        g_free(body->temp_filename);
+    }
 
     g_free(body->charset);
 
@@ -114,11 +117,7 @@ libbalsa_message_body_extract_embedded_headers(GMimeMessage* msg)
     subj = g_mime_message_get_subject(msg);
     if (subj) {
 	ehdr->subject =
-#if HAVE_GMIME_2_2_5
 	    g_mime_utils_header_decode_text(subj);
-#else
-	    g_mime_utils_header_decode_text((const unsigned char *) subj);
-#endif
 	libbalsa_utf8_sanitize(&ehdr->subject, TRUE, NULL);
     } else 
 	ehdr->subject = g_strdup(_("(No subject)"));
@@ -147,7 +146,7 @@ libbalsa_message_body_set_filename(LibBalsaMessageBody * body)
 static void
 libbalsa_message_body_set_types(LibBalsaMessageBody * body)
 {
-    const GMimeContentType *type;
+    GMimeContentType *type;
 
     type = g_mime_object_get_content_type(body->mime_part);
     if      (g_mime_content_type_is_type(type, "audio", "*"))
@@ -191,7 +190,6 @@ libbalsa_message_body_set_message_part(LibBalsaMessageBody * body,
             *next_part = libbalsa_message_body_new(body->message);
         libbalsa_message_body_set_mime_body(*next_part,
                                             embedded_message->mime_part);
-        g_object_unref(embedded_message);
     }
 
     return *next_part ? &(*next_part)->next : next_part;
@@ -201,13 +199,16 @@ static LibBalsaMessageBody **
 libbalsa_message_body_set_multipart(LibBalsaMessageBody * body,
 				    LibBalsaMessageBody ** next_part)
 {
-    GList *child;
-
-    for (child = GMIME_MULTIPART(body->mime_part)->subparts; child;
-	 child = child->next) {
+    GMimeMultipart *multipart = GMIME_MULTIPART(body->mime_part);
+    GMimeObject *part;
+    int count, i;
+    
+    count = g_mime_multipart_get_count (multipart);
+    for (i = 0; i < count; i++) {
+	part = g_mime_multipart_get_part (multipart, i);
 	if (!*next_part)
 	    *next_part = libbalsa_message_body_new(body->message);
-	libbalsa_message_body_set_mime_body(*next_part, child->data);
+	libbalsa_message_body_set_mime_body(*next_part, part);
 	next_part = &(*next_part)->next;
     }
 
@@ -261,24 +262,31 @@ gchar *
 libbalsa_message_body_get_parameter(LibBalsaMessageBody * body,
 				    const gchar * param)
 {
+    GMimeContentType *type;
     gchar *res = NULL;
 
     g_return_val_if_fail(body != NULL, NULL);
 
     if (body->mime_part) {
-	const GMimeContentType *type =
-	    g_mime_object_get_content_type(body->mime_part);
+	type = g_mime_object_get_content_type(body->mime_part);
 	res = g_strdup(g_mime_content_type_get_parameter(type, param));
     } else if (body->content_type) {
-	GMimeContentType *type =
-	    g_mime_content_type_new_from_string(body->content_type);
+	type = g_mime_content_type_new_from_string(body->content_type);
 	res = g_strdup(g_mime_content_type_get_parameter(type, param));
-	g_mime_content_type_destroy(type);
+	g_object_unref(type);
     }
 
     return res;
 }
 
+static const gchar *
+libbalsa_message_body_get_content_id(LibBalsaMessageBody * body)
+{
+    if (body->mime_part)
+        return g_mime_object_get_content_id(body->mime_part);
+    return body->content_id;
+}
+
 /* libbalsa_message_body_save_temporary:
    check if body has already its copy in temporary file and if not,
    allocates a temporary file name and saves the body there.
@@ -293,59 +301,50 @@ libbalsa_message_body_save_temporary(LibBalsaMessageBody * body, GError **err)
     }
 
     if (body->temp_filename == NULL) {
-	gint count = 100; /* Magic number, same as in g_mkstemp. */
+        const gchar *filename;
+        gint fd = -1;
+        GMimeStream *tmp_stream;
 
-	/* We want a temporary file with a name that ends with the same
-	 * set of suffices as body->filename (for the benefit of helpers
-	 * that depend on such things); however, g_file_open_tmp() works
-	 * only with templates that end with "XXXXXX", so we fake it. */
-	do {
-	    gint fd;
-	    gchar *tmp_file_name;
-	    gchar *dotpos = NULL;
+        filename = body->filename;
+        if (!filename)
+            filename = libbalsa_message_body_get_content_id(body);
 
-	    fd = g_file_open_tmp("balsa-body-XXXXXX", &tmp_file_name,
-				 err);
-	    if (fd < 0)
-		return FALSE;
-	    if (err && *err  &&  /* We should have returned by now! */
-                (*err)->message) {
-		printf("libbalsa_message_body_save_temporary:\n %s\n",
-		       (*err)->message);
-		g_clear_error(err);
-	    }
-	    close(fd);
-	    unlink(tmp_file_name);
+        if (!filename)
+	    fd = g_file_open_tmp("balsa-body-XXXXXX",
+                                 &body->temp_filename, err);
+        else {
+            const gchar *tempdir =
+                libbalsa_message_get_tempdir(body->message);
 
-	    if (body->filename) {
-		gchar *seppos = strrchr(body->filename, G_DIR_SEPARATOR);
-		dotpos = strchr(seppos ? seppos : body->filename, '.');
-	    }
-	    g_free(body->temp_filename);
-	    if (dotpos) {
-		body->temp_filename =
-		    g_strdup_printf("%s%s", tmp_file_name, dotpos);
-		g_free(tmp_file_name);
-	    } else
-		body->temp_filename = tmp_file_name;
-	    fd = open(body->temp_filename, O_WRONLY | O_EXCL | O_CREAT,
-                      LIBBALSA_MESSAGE_BODY_SAFE);
-	    if (fd >= 0) {
-                GMimeStream *tmp_stream;
-
-                if ((tmp_stream = g_mime_stream_fs_new(fd)) != NULL)
-                    return libbalsa_message_body_save_stream(body, tmp_stream, FALSE, err);
-                else {
-                    g_set_error(err, LIBBALSA_ERROR_QUARK, 1,
-                                _("Failed to create output stream"));
-                    close(fd);
-                    return FALSE;
-                }
+            if (!tempdir) {
+                g_set_error(err, LIBBALSA_MAILBOX_ERROR,
+                            LIBBALSA_MAILBOX_TEMPDIR_ERROR,
+                            "Failed to create temporary directory");
+            } else {
+                body->temp_filename =
+                    g_build_filename(tempdir, filename, NULL);
+                fd = open(body->temp_filename,
+                          O_WRONLY | O_EXCL | O_CREAT,
+                          S_IRUSR);
             }
-	} while (errno == EEXIST && --count > 0);
+        }
 
-	/* Either we hit a real error, or we used up 100 attempts. */
-	return FALSE;
+        if (fd < 0) {
+            if (err && !*err)
+                g_set_error(err, LIBBALSA_ERROR_QUARK, 1,
+                            "Failed to create temporary file");
+            return FALSE;
+        }
+
+        if ((tmp_stream = g_mime_stream_fs_new(fd)) != NULL)
+            return libbalsa_message_body_save_stream(body, tmp_stream,
+                                                     FALSE, err);
+        else {
+            g_set_error(err, LIBBALSA_ERROR_QUARK, 1,
+                        _("Failed to create output stream"));
+            close(fd);
+            return FALSE;
+        }
     } else {
 	/* the temporary name has been already allocated on previous
 	   save_temporary action. We just check if the file is still there.
@@ -357,7 +356,7 @@ libbalsa_message_body_save_temporary(LibBalsaMessageBody * body, GError **err)
 	    return TRUE;
 	else
 	    return libbalsa_message_body_save(body, body->temp_filename,
-                                              LIBBALSA_MESSAGE_BODY_SAFE,
+                                              S_IRUSR,
                                               FALSE, err);
     }
 }
@@ -409,14 +408,13 @@ libbalsa_message_body_save_vfs(LibBalsaMessageBody * body,
     return libbalsa_message_body_save_stream(body, out_stream, filter_crlf, err);
 }
 
-
 static GMimeStream *
 libbalsa_message_body_stream_add_filter(GMimeStream * stream,
                                         GMimeFilter * filter)
 {
     if (!GMIME_IS_STREAM_FILTER(stream)) {
         GMimeStream *filtered_stream =
-            g_mime_stream_filter_new_with_stream(stream);
+            g_mime_stream_filter_new(stream);
         g_object_unref(stream);
         stream = filtered_stream;
     }
@@ -427,85 +425,52 @@ libbalsa_message_body_stream_add_filter(GMimeStream * stream,
     return stream;
 }
 
-GMimeStream *
-libbalsa_message_body_get_stream(LibBalsaMessageBody * body, GError **err)
+static GMimeStream *
+libbalsa_message_body_get_part_stream(LibBalsaMessageBody * body,
+                                      GError ** err)
 {
     GMimeStream *stream;
+    GMimeDataWrapper *wrapper;
+    GMimeContentEncoding encoding;
     GMimeFilter *filter;
     gchar *mime_type = NULL;
     const gchar *charset;
 
-    g_return_val_if_fail(body != NULL, NULL);
-    g_return_val_if_fail(body->message != NULL, NULL);
-
-    if (body->message->mailbox) {
-        GMimeDataWrapper *wrapper;
-        GMimePartEncodingType encoding;
-        if(!libbalsa_mailbox_get_message_part(body->message, body, err))
-            return NULL;
-	
-	/* We handle "real" parts and embedded rfc822 messages
-	   differently. There is probably a way to unify if we use
-	   GMimeObject common denominator.  */
-	if(GMIME_IS_MESSAGE_PART(body->mime_part)) {
-	    ssize_t bytes_written;
-	    GMimeMessage *msg = g_mime_message_part_get_message
-		(GMIME_MESSAGE_PART(body->mime_part));
-	    stream = g_mime_stream_mem_new();
-	    libbalsa_mailbox_lock_store(body->message->mailbox);
-	    bytes_written =
-		g_mime_object_write_to_stream(GMIME_OBJECT(msg), stream);
-	    libbalsa_mailbox_unlock_store(body->message->mailbox);
-            printf("Written %ld bytes of embedded message\n",
-                   (long) bytes_written);
-	    if(bytes_written < 0) {
-		g_object_unref(stream);
-		g_set_error(err, LIBBALSA_MAILBOX_ERROR,
-			    LIBBALSA_MAILBOX_ACCESS_ERROR,
-			    _("Could not read embedded message"));
-		return NULL;
-	    }
-	    g_mime_stream_reset(stream);
-	    return stream;
-	}
-
-        wrapper =
-            g_mime_part_get_content_object(GMIME_PART(body->mime_part));
-	if(!wrapper) /* part is incomplete. */
-	    return NULL;
-        stream = g_mime_data_wrapper_get_stream(wrapper);
-        encoding = g_mime_data_wrapper_get_encoding(wrapper);
-        g_object_unref(wrapper);
-
-        switch (encoding) {
-        case GMIME_PART_ENCODING_BASE64:
-            filter =
-                g_mime_filter_basic_new_type
-                (GMIME_FILTER_BASIC_BASE64_DEC);
-            stream =
-                libbalsa_message_body_stream_add_filter(stream, filter);
-            break;
-        case GMIME_PART_ENCODING_QUOTEDPRINTABLE:
-            filter =
-                g_mime_filter_basic_new_type(GMIME_FILTER_BASIC_QP_DEC);
-            stream =
-                libbalsa_message_body_stream_add_filter(stream, filter);
-            break;
-        case GMIME_PART_ENCODING_UUENCODE:
-            filter =
-                g_mime_filter_basic_new_type(GMIME_FILTER_BASIC_UU_DEC);
-            stream =
-                libbalsa_message_body_stream_add_filter(stream, filter);
-            break;
-        default:
-            break;
-        }
-    } else {
-        g_set_error(err, LIBBALSA_MAILBOX_ERROR, LIBBALSA_MAILBOX_ACCESS_ERROR,
+    wrapper = g_mime_part_get_content_object(GMIME_PART(body->mime_part));
+    if (!wrapper) {
+        /* part is incomplete. */
+        g_set_error(err, LIBBALSA_MAILBOX_ERROR,
+                    LIBBALSA_MAILBOX_ACCESS_ERROR,
                     "Internal error in get_stream");
         return NULL;
     }
 
+    stream = g_object_ref(g_mime_data_wrapper_get_stream(wrapper));
+    encoding = g_mime_data_wrapper_get_encoding(wrapper);
+
+    switch (encoding) {
+    case GMIME_CONTENT_ENCODING_BASE64:
+        filter =
+            g_mime_filter_basic_new(GMIME_CONTENT_ENCODING_BASE64,
+                                    FALSE);
+        stream = libbalsa_message_body_stream_add_filter(stream, filter);
+        break;
+    case GMIME_CONTENT_ENCODING_QUOTEDPRINTABLE:
+        filter =
+            g_mime_filter_basic_new(GMIME_CONTENT_ENCODING_QUOTEDPRINTABLE,
+                                    FALSE);
+        stream = libbalsa_message_body_stream_add_filter(stream, filter);
+        break;
+    case GMIME_CONTENT_ENCODING_UUENCODE:
+        filter =
+            g_mime_filter_basic_new(GMIME_CONTENT_ENCODING_UUENCODE,
+                                    FALSE);
+        stream = libbalsa_message_body_stream_add_filter(stream, filter);
+        break;
+    default:
+        break;
+    }
+
     /* convert text bodies but HTML - gtkhtml does conversion on its own. */
     if (libbalsa_message_body_type(body) == LIBBALSA_MESSAGE_BODY_TYPE_TEXT
         && strcmp(mime_type = libbalsa_message_body_get_mime_type(body),
@@ -517,7 +482,7 @@ libbalsa_message_body_get_stream(LibBalsaMessageBody * body, GError **err)
         GMimeFilter *filter_windows;
 
         stream_null = g_mime_stream_null_new();
-        stream_filter = g_mime_stream_filter_new_with_stream(stream_null);
+        stream_filter = g_mime_stream_filter_new(stream_null);
         g_object_unref(stream_null);
 
         filter_windows = g_mime_filter_windows_new(charset);
@@ -546,6 +511,69 @@ libbalsa_message_body_get_stream(LibBalsaMessageBody * body, GError **err)
     return stream;
 }
 
+static GMimeStream *
+libbalsa_message_body_get_message_part_stream(LibBalsaMessageBody * body,
+                                              GError ** err)
+{
+    GMimeStream *stream;
+    ssize_t bytes_written;
+    GMimeMessage *msg = g_mime_message_part_get_message
+        (GMIME_MESSAGE_PART(body->mime_part));
+
+    stream = g_mime_stream_mem_new();
+    libbalsa_mailbox_lock_store(body->message->mailbox);
+    bytes_written =
+        g_mime_object_write_to_stream(GMIME_OBJECT(msg), stream);
+    libbalsa_mailbox_unlock_store(body->message->mailbox);
+    printf("Written %ld bytes of embedded message\n",
+           (long) bytes_written);
+
+    if (bytes_written < 0) {
+        g_object_unref(stream);
+        g_set_error(err, LIBBALSA_MAILBOX_ERROR,
+                    LIBBALSA_MAILBOX_ACCESS_ERROR,
+                    _("Could not read embedded message"));
+        return NULL;
+    }
+
+    g_mime_stream_reset(stream);
+    return stream;
+}
+
+GMimeStream *
+libbalsa_message_body_get_stream(LibBalsaMessageBody * body, GError **err)
+{
+    g_return_val_if_fail(body != NULL, NULL);
+    g_return_val_if_fail(body->message != NULL, NULL);
+
+    if (!body->message->mailbox) {
+        g_set_error(err, LIBBALSA_MAILBOX_ERROR,
+                    LIBBALSA_MAILBOX_ACCESS_ERROR,
+                    "Internal error in get_stream");
+        return NULL;
+    }
+
+    if (!libbalsa_mailbox_get_message_part(body->message, body, err)
+        || !(GMIME_IS_PART(body->mime_part)
+             || GMIME_IS_MESSAGE_PART(body->mime_part))) {
+        if (err && !*err)
+            g_set_error(err, LIBBALSA_MAILBOX_ERROR,
+                        LIBBALSA_MAILBOX_ACCESS_ERROR,
+                        "Cannot get stream for part of type %s",
+                        g_type_name(G_TYPE_FROM_INSTANCE
+                                    (body->mime_part)));
+        return NULL;
+    }
+
+    /* We handle "real" parts and embedded rfc822 messages
+       differently. There is probably a way to unify if we use
+       GMimeObject common denominator.  */
+    if (GMIME_IS_MESSAGE_PART(body->mime_part))
+        return libbalsa_message_body_get_message_part_stream(body, err);
+
+    return libbalsa_message_body_get_part_stream(body, err);
+}
+
 gssize
 libbalsa_message_body_get_content(LibBalsaMessageBody * body, gchar ** buf,
                                   GError **err)
@@ -580,8 +608,12 @@ libbalsa_message_body_get_content(LibBalsaMessageBody * body, gchar ** buf,
 	/* NULL-terminate, in case it is used as a string. */
 	g_byte_array_append(array, &zero, 1);
         *buf = (gchar *) g_byte_array_free(array, FALSE);
-    } else
+    } else {
         g_byte_array_free(array, TRUE);
+        g_set_error(err, LIBBALSA_MAILBOX_ERROR,
+                    LIBBALSA_MAILBOX_ACCESS_ERROR,
+                    "Write error in get_content");
+    }
 
     return len;
 }
@@ -589,38 +621,48 @@ libbalsa_message_body_get_content(LibBalsaMessageBody * body, gchar ** buf,
 GdkPixbuf *
 libbalsa_message_body_get_pixbuf(LibBalsaMessageBody * body, GError ** err)
 {
-    GdkPixbufLoader *loader;
     GMimeStream *stream;
-    gssize count;
-    gboolean ok = TRUE;
-    gchar buf[4096];
+    gchar *mime_type;
+    GdkPixbufLoader *loader;
     GdkPixbuf *pixbuf = NULL;
 
     stream = libbalsa_message_body_get_stream(body, err);
     if (!stream)
         return pixbuf;
+
     libbalsa_mailbox_lock_store(body->message->mailbox);
     g_mime_stream_reset(stream);
 
-    loader = gdk_pixbuf_loader_new();
-    while ((count = g_mime_stream_read(stream, buf, sizeof(buf))) > 0) {
-        if (!gdk_pixbuf_loader_write(loader, (guchar *) buf, count, err)) {
-            ok = FALSE;
-            break;
-        }
+    mime_type = libbalsa_message_body_get_mime_type(body);
+    loader = gdk_pixbuf_loader_new_with_mime_type(mime_type, err);
+
+#define ENABLE_WORKAROUND_FOR_IE_NON_IANA_MIME_TYPE TRUE
+#if ENABLE_WORKAROUND_FOR_IE_NON_IANA_MIME_TYPE
+    if (!loader && g_ascii_strcasecmp(mime_type, "image/pjpeg") == 0) {
+        g_clear_error(err);
+        loader = gdk_pixbuf_loader_new_with_mime_type("image/jpeg", err);
     }
+#endif                          /* ENABLE_WORKAROUND_FOR_IE_NON_IANA_MIME_TYPE */
+
+    g_free(mime_type);
+
+    if (loader) {
+        gssize count;
+        gchar buf[4096];
+
+        while ((count = g_mime_stream_read(stream, buf, sizeof(buf))) > 0)
+            if (!gdk_pixbuf_loader_write(loader, (guchar *) buf, count, err))
+                break;
+
+        if (gdk_pixbuf_loader_close(loader, *err ? NULL : err))
+            pixbuf = g_object_ref(gdk_pixbuf_loader_get_pixbuf(loader));
+
+        g_object_unref(loader);
+    }
+
     libbalsa_mailbox_unlock_store(body->message->mailbox);
     g_object_unref(stream);
 
-    if (!gdk_pixbuf_loader_close(loader, ok ? err : NULL))
-	ok = FALSE;
-
-    if (ok) {
-        pixbuf = gdk_pixbuf_loader_get_pixbuf(loader);
-	g_object_ref(pixbuf);
-    }
-    g_object_unref(loader);
-
     return pixbuf;
 }
 
@@ -635,6 +677,7 @@ libbalsa_message_body_save_stream(LibBalsaMessageBody * body,
     stream = libbalsa_message_body_get_stream(body, err);
     if (!body->mime_part)
         return FALSE;
+    g_clear_error(err);
 
     libbalsa_mailbox_lock_store(body->message->mailbox);
 
@@ -642,39 +685,24 @@ libbalsa_message_body_save_stream(LibBalsaMessageBody * body,
         g_mime_stream_reset(stream);
 
         if (filter_crlf) {
-            GMimeFilter *filter;
-
-            if (!GMIME_IS_STREAM_FILTER(stream)) {
-                GMimeStream *stream_filter =
-                    g_mime_stream_filter_new_with_stream(stream);
-                g_object_unref(stream);
-                stream = stream_filter;
-            }
-
-            filter = g_mime_filter_crlf_new(GMIME_FILTER_CRLF_DECODE,
-                                            GMIME_FILTER_CRLF_MODE_CRLF_ONLY);
-            g_mime_stream_filter_add(GMIME_STREAM_FILTER(stream), filter);
-            g_object_unref(filter);
+            GMimeFilter *filter = g_mime_filter_crlf_new(FALSE, FALSE);
+            stream =
+                libbalsa_message_body_stream_add_filter(stream, filter);
         }
 
         len = g_mime_stream_write_to_stream(stream, dest);
         g_object_unref(stream);
-    } else {
-        /* body->mime_part is not a GMimePart. */
-        GMimeObject *mime_part = body->mime_part;
-
-        if (GMIME_IS_MESSAGE_PART(mime_part))
-            /* The user probably wants the message without the part's
-             * mime headers. */
-            mime_part =
-                GMIME_OBJECT(GMIME_MESSAGE_PART(mime_part)->message);
-
-        len = g_mime_object_write_to_stream(mime_part, dest);
-    }
+    } else
+        /* body->mime_part is neither a GMimePart nor a GMimeMessagePart. */
+        len = g_mime_object_write_to_stream(body->mime_part, dest);
 
     libbalsa_mailbox_unlock_store(body->message->mailbox);
     g_object_unref(dest);
 
+    if (len < 0)
+        g_set_error(err, LIBBALSA_MAILBOX_ERROR, LIBBALSA_MAILBOX_ACCESS_ERROR,
+                    "Write error in save_stream");
+
     return len >= 0;
 }
 
@@ -779,22 +807,16 @@ libbalsa_message_body_get_by_id(LibBalsaMessageBody * body,
 				const gchar * id)
 {
     LibBalsaMessageBody *res;
+    const gchar *content_id;
 
     g_return_val_if_fail(id != NULL, NULL);
 
     if (!body)
 	return NULL;
 
-    if (body->mime_part) {
-	const gchar *bodyid =
-	    g_mime_object_get_content_id(body->mime_part);
-
-	if (bodyid && strcmp(id, bodyid) == 0)
-	    return body;
-    } else {
-        if(body->content_id && strcmp(id, body->content_id) == 0)
-            return body;
-    }
+    if ((content_id = libbalsa_message_body_get_content_id(body))
+        && !strcmp(id, content_id))
+        return body;
 
     if ((res = libbalsa_message_body_get_by_id(body->parts, id)) != NULL)
 	return res;
@@ -802,7 +824,6 @@ libbalsa_message_body_get_by_id(LibBalsaMessageBody * body,
     return libbalsa_message_body_get_by_id(body->next, id);
 }
 
-
 #ifdef HAVE_GPGME
 LibBalsaMsgProtectState
 libbalsa_message_body_protect_state(LibBalsaMessageBody *body)
