@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2002 Stuart Parmenter and others,
+ * Copyright (C) 1997-2009 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,10 +27,11 @@
  *
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "filter-file.h"
 
-/* define _XOPEN_SOURCE to make strptime visible */
-#define _XOPEN_SOURCE
 /* extensions  needed additonally on Solaris for strptime */
 #define __EXTENSIONS__
 #include <stdlib.h>
@@ -41,14 +42,13 @@
 #endif
 #include <string.h>
 #include <time.h>
-
-#include <libgnome/libgnome.h>
-
 #include <sys/types.h>
-#include "filter-file.h"
+
 #include "filter-private.h"
 #include "filter-funcs.h"
+#include "libbalsa-conf.h"
 #include "mailbox-filter.h"
+#include <glib/gi18n.h>
 
 /* Load the header of a filter filter (you have to separately load the
  * associated conditions) Filter is marked as invalid Position
@@ -61,12 +61,12 @@ libbalsa_filter_new_from_config(void)
     LibBalsaFilter * newf = libbalsa_filter_new();
     gchar *str, *p;
     /* First we load the fixed part of the filter */
-    newf->name          = gnome_config_get_string("Name");
-    p = str             = gnome_config_get_string("Condition");
-    newf->sound         = gnome_config_get_string("Sound");
-    newf->popup_text    = gnome_config_get_string("Popup-text");
-    newf->action        = gnome_config_get_int("Action-type");
-    newf->action_string = gnome_config_get_string("Action-string");
+    newf->name          = libbalsa_conf_get_string("Name");
+    p = str             = libbalsa_conf_get_string("Condition");
+    newf->sound         = libbalsa_conf_get_string("Sound");
+    newf->popup_text    = libbalsa_conf_get_string("Popup-text");
+    newf->action        = libbalsa_conf_get_int("Action-type");
+    newf->action_string = libbalsa_conf_get_string("Action-string");
     newf->condition     = libbalsa_condition_new_from_string(&p);
     g_free(str);
     if (newf->sound[0]=='\0') {
@@ -101,12 +101,12 @@ void
 libbalsa_filter_save_config(LibBalsaFilter * fil)
 {
     gchar *str = libbalsa_condition_to_string(fil->condition);
-    gnome_config_set_string("Name",          fil->name);
-    gnome_config_set_string("Condition",     str);
-    gnome_config_set_string("Sound",         fil->sound);
-    gnome_config_set_string("Popup-text",    fil->popup_text);
-    gnome_config_set_int("Action-type",      fil->action);
-    gnome_config_set_string("Action-string", fil->action_string);
+    libbalsa_conf_set_string("Name",          fil->name);
+    libbalsa_conf_set_string("Condition",     str);
+    libbalsa_conf_set_string("Sound",         fil->sound);
+    libbalsa_conf_set_string("Popup-text",    fil->popup_text);
+    libbalsa_conf_set_int("Action-type",      fil->action);
+    libbalsa_conf_set_string("Action-string", fil->action_string);
     g_free(str);
 }
 
@@ -125,7 +125,7 @@ libbalsa_mailbox_filters_load_config(LibBalsaMailbox* mbox)
     GSList * lst;
 
     /* We load the associated filters */
-    gnome_config_get_vector_with_default(MAILBOX_FILTERS_KEY,&nb_filters,
+    libbalsa_conf_get_vector_with_default(MAILBOX_FILTERS_KEY,&nb_filters,
 					 &filters_names,&def);
     if (!def) {
 	for(i=0;i<nb_filters;i++) {
@@ -145,7 +145,7 @@ libbalsa_mailbox_filters_load_config(LibBalsaMailbox* mbox)
     }
     g_strfreev(filters_names);
     if (!def) {
-	gnome_config_get_vector_with_default(MAILBOX_FILTERS_WHEN_KEY,
+	libbalsa_conf_get_vector_with_default(MAILBOX_FILTERS_WHEN_KEY,
                                              &nb_filters,&filters_names,&def);
 	if (def)
 	    for(lst=mbox->filters;lst;lst=g_slist_next(lst))
@@ -192,7 +192,7 @@ libbalsa_mailbox_filters_save_config(LibBalsaMailbox * mbox)
 	lst=g_slist_next(lst);
     }
     g_slist_free(names);
-    gnome_config_set_vector(MAILBOX_FILTERS_KEY,nb_filters,
+    libbalsa_conf_set_vector(MAILBOX_FILTERS_KEY,nb_filters,
                             (const gchar**)filters_names);
 
     fil=mbox->filters;
@@ -202,13 +202,14 @@ libbalsa_mailbox_filters_save_config(LibBalsaMailbox * mbox)
                             ((LibBalsaMailboxFilter*)fil->data)->when);
 	fil=g_slist_next(fil);
     }
-    gnome_config_set_vector(MAILBOX_FILTERS_WHEN_KEY,nb_filters,
+    libbalsa_conf_set_vector(MAILBOX_FILTERS_WHEN_KEY,nb_filters,
                             (const gchar**)filters_names);
     for (i=0;i<nb_filters;i++)
 	g_free(filters_names[i]);
     g_free(filters_names);
 }
 
+/* FIXME: #ifdef HAVE_GNOME ?? */
 /* Temporary code for transition from 2.0.x */
 static LibBalsaCondition *
 libbalsa_condition_new_from_config()
@@ -225,23 +226,23 @@ libbalsa_condition_new_from_config()
 
     newc = libbalsa_condition_new();
 
-    newc->type = gnome_config_get_int("Type");
-    newc->negate = gnome_config_get_bool("Condition-not");
-    fields = gnome_config_get_int("Match-fields");
+    newc->type = libbalsa_conf_get_int("Type");
+    newc->negate = libbalsa_conf_get_bool("Condition-not");
+    fields = libbalsa_conf_get_int("Match-fields");
 
     switch (newc->type) {
     case CONDITION_STRING:
 	newc->match.string.fields = fields;
 	newc->match.string.string =
-	    gnome_config_get_string("Match-string");
+	    libbalsa_conf_get_string("Match-string");
 	newc->match.string.user_header =
 	    CONDITION_CHKMATCH(newc, CONDITION_MATCH_US_HEAD) ?
-	    gnome_config_get_string("User-header") : NULL;
+	    libbalsa_conf_get_string("User-header") : NULL;
 	break;
     case CONDITION_REGEX:
 	newc->match.regex.fields = fields;
 #if 0
-	gnome_config_get_vector("Reg-exps", &nbregexs, &regexs);
+	libbalsa_conf_get_vector("Reg-exps", &nbregexs, &regexs);
 	for (i = 0; i < nbregexs && (filter_errno == FILTER_NOERR); i++) {
 	    newreg = g_new(LibBalsaConditionRegex, 1);
 	    newreg->string = regexs[i];
@@ -255,7 +256,7 @@ libbalsa_condition_new_from_config()
 #endif
 	break;
     case CONDITION_DATE:
-	str = gnome_config_get_string("Low-date");
+	str = libbalsa_conf_get_string("Low-date");
 	if (str[0] == '\0')
 	    newc->match.date.date_low = 0;
 	else {
@@ -267,7 +268,7 @@ libbalsa_condition_new_from_config()
 		newc->match.date.date_low = mktime(&date);
 	}
 	g_free(str);
-	str = gnome_config_get_string("High-date");
+	str = libbalsa_conf_get_string("High-date");
 	if (str[0] == '\0')
 	    newc->match.date.date_high = 0;
 	else {
@@ -281,7 +282,7 @@ libbalsa_condition_new_from_config()
 	g_free(str);
 	break;
     case CONDITION_FLAG:
-	newc->match.flags = gnome_config_get_int("Flags");
+	newc->match.flags = libbalsa_conf_get_int("Flags");
 	break;
     default:
 	filter_errno = FILTER_EFILESYN;
@@ -314,75 +315,84 @@ compare_conditions_order(gconstpointer a, gconstpointer b)
 
 #define CONDITION_SECTION_PREFIX "condition-"
 
-LibBalsaCondition *
-libbalsa_condition_new_2_0(gchar * prefix, gchar * filter_section_name,
-			   ConditionMatchType cmt)
+struct lbc_new_info {
+    GList *tmp_list;
+    gint err;
+};
+
+static gboolean
+lbc_new_helper(const gchar * key, const gchar * value, gpointer data)
 {
+    struct lbc_new_info *info = data;
     LibBalsaCondition *cond;
+
+    libbalsa_conf_push_group(key);
+
+    cond = libbalsa_condition_new_from_config();
+    if (cond) {
+        if (filter_errno == FILTER_EFILESYN) {
+            /* We don't stop the process for syntax error, we
+             * just discard the malformed condition we also
+             * remember (with err) that a syntax error occurs
+             */
+            info->err = FILTER_EFILESYN;
+            filter_errno = FILTER_NOERR;
+            libbalsa_condition_unref(cond);
+        } else {
+            LibBalsaTempCondition *tmp_cond;
+
+            tmp_cond = g_new(LibBalsaTempCondition, 1);
+            tmp_cond->cnd = cond;
+            tmp_cond->order = atoi(strrchr(key, ':') + 1);
+            info->tmp_list = g_list_prepend(info->tmp_list, tmp_cond);
+        }
+    }
+
+    libbalsa_conf_pop_group();
+
+    return filter_errno != FILTER_NOERR;
+}
+
+LibBalsaCondition *
+libbalsa_condition_new_2_0(const gchar * filter_section_name,
+                           ConditionMatchType cmt)
+{
+    struct lbc_new_info info;
+    gchar *section_prefix;
     LibBalsaCondition *cond_2_0 = NULL;
-    void *iterator;
-    gchar *tmp, *condprefix, *key;
-    gint pref_len =
-	strlen(CONDITION_SECTION_PREFIX) + strlen(filter_section_name) + 1;
-    gint err = FILTER_NOERR;
-    GList *tmp_list = NULL;
     GList *l;
 
-    tmp =
-	g_strconcat(CONDITION_SECTION_PREFIX, filter_section_name, ":",
-		    NULL);
-    iterator = gnome_config_init_iterator_sections(prefix);
-    filter_errno = FILTER_NOERR;
+    info.tmp_list = NULL;
+    info.err = filter_errno = FILTER_NOERR;
 
-    while ((filter_errno == FILTER_NOERR) &&
-	   (iterator = gnome_config_iterator_next(iterator, &key, NULL))) {
+    section_prefix =
+        g_strconcat(CONDITION_SECTION_PREFIX, filter_section_name, ":",
+                    NULL);
+    libbalsa_conf_foreach_group(section_prefix, lbc_new_helper, &info);
+    g_free(section_prefix);
 
-	if (strncmp(key, tmp, pref_len) == 0) {
-	    condprefix = g_strconcat(prefix, key, "/", NULL);
-	    gnome_config_push_prefix(condprefix);
-	    g_free(condprefix);
-	    cond = libbalsa_condition_new_from_config();
-	    if (cond) {
-		if (filter_errno == FILTER_EFILESYN) {
-		    /* We don't stop the process for syntax error, we
-		     * just discard the malformed condition we also
-		     * remember (with err) that a syntax error occurs
-		     */
-		    err = FILTER_EFILESYN;
-		    filter_errno = FILTER_NOERR;
-		    libbalsa_condition_free(cond);
-		} else {
-		    LibBalsaTempCondition *tmp =
-			g_new(LibBalsaTempCondition, 1);
-
-		    tmp->cnd = cond;
-		    tmp->order = atoi(strrchr(key, ':') + 1);
-		    tmp_list = g_list_prepend(tmp_list, tmp);
-		}
-	    }
-	    gnome_config_pop_prefix();
-	}
-	g_free(key);
-    }
-    g_free(tmp);
     /* We position filter_errno to the last non-critical error */
     if (filter_errno == FILTER_NOERR) {
-	LibBalsaTempCondition *tmp;
-
-	filter_errno = err;
-	/* We sort the list of temp conditions, then
-	   we create the combined condition. */
-	tmp_list = g_list_sort(tmp_list, compare_conditions_order);
-	l = tmp_list;
-	for (; tmp_list; tmp_list = g_list_next(tmp_list)) {
-	    tmp = (LibBalsaTempCondition *) (tmp_list->data);
-	    cond_2_0 = cond_2_0 ?
-		libbalsa_condition_new_bool_ptr(FALSE, cmt, tmp->cnd,
-						cond_2_0) : tmp->cnd;
-	    g_free(tmp);
-	}
-	g_list_free(l);
-    } /* else we leak the list and structures?? */
-
+        filter_errno = info.err;
+        /* We sort the list of temp conditions, then
+           we create the combined condition. */
+        info.tmp_list =
+            g_list_sort(info.tmp_list, compare_conditions_order);
+        l = info.tmp_list;
+        for (; info.tmp_list; info.tmp_list = info.tmp_list->next) {
+            LibBalsaTempCondition *tmp =
+                (LibBalsaTempCondition *) (info.tmp_list->data);
+            LibBalsaCondition *res = 
+                libbalsa_condition_new_bool_ptr(FALSE, cmt, tmp->cnd,
+                                                cond_2_0);
+            libbalsa_condition_unref(tmp->cnd);
+            libbalsa_condition_unref(cond_2_0);
+            cond_2_0 = res;
+            g_free(tmp);
+        }
+        g_list_free(l);
+    }
+    /* else we leak the list and structures?? */
     return cond_2_0;
 }
+/* #endif *//* HAVE_GNOME */
