@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2002 Stuart Parmenter and others,
+ * Copyright (C) 1997-2009 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -29,29 +29,28 @@
  * Primary author: Emmanuel ALLAUD
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "filter.h"
 
 #include <ctype.h>
 #include <string.h>
 
-#ifdef HAVE_GNOME
+#if HAVE_CANBERRA
+#include <canberra-gtk.h>
+#elif HAVE_GNOME
 #include <libgnome/gnome-sound.h>
-#endif
+#endif                          /* HAVE_CANBERRA */
 
 #include "libbalsa.h"
 #include "libbalsa_private.h"
 
-#include "filter.h"
 #include "filter-funcs.h"
 #include "filter-private.h"
 #include "misc.h"
 #include <glib/gi18n.h>
 
-/* from libmutt */
-#define REGCOMP(X,Y,Z) regcomp(X, Y, REG_WORDS|REG_EXTENDED|(Z))
-#define REGEXEC(X,Y) regexec(&X, Y, (size_t)0, (regmatch_t *)0, (int)0)
-
-
 /* filters_trash_mbox points to a mailbox that is used as Trash by filters
  *  code. */
 static LibBalsaMailbox* filters_trash_mbox = NULL;
@@ -133,7 +132,8 @@ libbalsa_condition_matches(LibBalsaCondition* cond,
             }
         }
         /* do the work */
-	if (CONDITION_CHKMATCH(cond,CONDITION_MATCH_TO)) {
+	if (CONDITION_CHKMATCH(cond,CONDITION_MATCH_TO)
+            && message->headers->to_list) {
             str =
                 internet_address_list_to_string(message->headers->to_list,
                                                 FALSE);
@@ -157,7 +157,8 @@ libbalsa_condition_matches(LibBalsaCondition* cond,
                 break;
             }
 	}
-	if (CONDITION_CHKMATCH(cond,CONDITION_MATCH_CC)) {
+	if (CONDITION_CHKMATCH(cond,CONDITION_MATCH_CC)
+            && message->headers->cc_list) {
             str =
                 internet_address_list_to_string(message->headers->cc_list,
                                                 FALSE);
@@ -252,14 +253,22 @@ libbalsa_filter_mailbox_messages(LibBalsaFilter * filt,
     gboolean result=FALSE;
     LibBalsaMailbox *mbox;
     GError *err = NULL;
+    gchar **parts, **p;
 
     if (msgnos->len == 0)
 	return FALSE;
 
-#ifdef HAVE_GNOME
+#if HAVE_CANBERRA
+    if (filt->sound) {
+        gint rc = ca_context_play(ca_gtk_context_get(), 0,
+                                  CA_PROP_MEDIA_FILENAME, filt->sound,
+                                  NULL);
+        g_message("(%s) play %s, %s", __func__, filt->sound, ca_strerror(rc));
+    }
+#elif HAVE_GNOME
     if (filt->sound)
 	gnome_sound_play(filt->sound);
-#endif /* HAVE_GNOME */
+#endif                          /* HAVE_CANBERRA */
     if (filt->popup_text)
 	libbalsa_information(LIBBALSA_INFORMATION_MESSAGE,
 			     "%s",
@@ -304,6 +313,18 @@ libbalsa_filter_mailbox_messages(LibBalsaFilter * filt,
 	else if (mbox == filters_trash_mbox)
 	    result = TRUE;
 	break;
+    case FILTER_COLOR:
+        parts = g_strsplit(filt->action_string, ";", 2);
+        for (p = parts; *p; p++) {
+            if (g_str_has_prefix(*p, "foreground:"))
+                libbalsa_mailbox_set_foreground(mailbox, msgnos,
+                                                (*p) + 11);
+            if (g_str_has_prefix(*p, "background:"))
+                libbalsa_mailbox_set_background(mailbox, msgnos,
+                                                (*p) + 11);
+        }
+        g_strfreev(parts);
+        break;
     case FILTER_PRINT:
 	/* FIXME : to be implemented */
 	break;
@@ -311,6 +332,7 @@ libbalsa_filter_mailbox_messages(LibBalsaFilter * filt,
 	/* FIXME : to be implemented */
 	break;
     case FILTER_NOTHING:
+    case FILTER_N_TYPES:
 	/* Nothing to do */
 	break;
     }
