@@ -23,7 +23,9 @@
 #ifndef   __LIBBALSA_HTML_H__
 # define  __LIBBALSA_HTML_H__
 
-#include "config.h"
+#ifndef BALSA_VERSION
+# error "Include config.h before this file."
+#endif
 
 # if defined(HAVE_GTKHTML2)
 /* gtkhtml2 uses deprecated api */
@@ -31,6 +33,7 @@
 # endif
 
 #  include <gtk/gtk.h>
+#include "libbalsa.h"
 
 /* We need this enum even if we're not using GtkHtml. */
 typedef enum {
@@ -40,13 +43,11 @@ typedef enum {
     LIBBALSA_HTML_TYPE_RICHTEXT
 } LibBalsaHTMLType;
 
-# ifdef HAVE_GTKHTML
+# ifdef HAVE_HTML_WIDGET
 
 typedef void (*LibBalsaHtmlCallback) (const gchar * uri);
 
-GtkWidget *libbalsa_html_new(const gchar * text, size_t len,
-			     const gchar * charset,
-			     gpointer message,
+GtkWidget *libbalsa_html_new(LibBalsaMessageBody * body,
                              LibBalsaHtmlCallback hover_cb,
                              LibBalsaHtmlCallback clicked_cb);
 void libbalsa_html_to_string(gchar ** text, size_t len);
@@ -58,31 +59,18 @@ void libbalsa_html_copy(GtkWidget * widget);
 guint libbalsa_html_filter(LibBalsaHTMLType html_type, gchar ** text,
 			   guint len);
 
-#if defined(HAVE_GNOME) && !defined(HAVE_GTK_PRINT)
+gboolean libbalsa_html_can_search(GtkWidget * widget);
+gboolean libbalsa_html_search_text(GtkWidget * widget, const gchar * text,
+                                   gboolean find_forward, gboolean wrap);
+void libbalsa_html_get_selection_bounds(GtkWidget * widget,
+                                        GdkRectangle * selection_bounds);
 
-#  include <libgnomeprint/gnome-print.h>
+GtkWidget *libbalsa_html_popup_menu_widget(GtkWidget * widget);
 
-gboolean libbalsa_html_can_print(void);
-typedef void (*LibBalsaHTMLPrintCallback) (GtkWidget * widget,
-					   GnomePrintContext *
-					   print_context, gdouble x,
-					   gdouble y, gdouble width,
-					   gdouble height,
-					   gpointer user_data);
-void libbalsa_html_print(GtkWidget * widget,
-			 GnomePrintContext * print_context,
-			 gdouble header_height, gdouble footer_height,
-			 LibBalsaHTMLPrintCallback header_print,
-			 LibBalsaHTMLPrintCallback footer_print,
-			 gpointer user_data);
-gint libbalsa_html_print_get_pages_num(GtkWidget * widget,
-				       GnomePrintContext * print_context,
-				       gdouble header_height,
-				       gdouble footer_height);
+gboolean libbalsa_html_can_print(GtkWidget * widget);
+void libbalsa_html_print(GtkWidget * widget);
 
-#endif /* defined(HAVE_GNOME) && !defined(HAVE_GTK_PRINT) */
-
-# endif				/* HAVE_GTKHTML */
+# endif				/* HAVE_HTML_WIDGET */
 
 LibBalsaHTMLType libbalsa_html_type(const gchar * mime_type);
 
