@@ -19,22 +19,24 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
-
-#ifdef HAVE_GNOME
-#include <libgnome/gnome-help.h>
-#endif
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "identity.h"
 
 #ifdef HAVE_GPGME
 #  include "rfc3156.h"
 #endif
 #include "libbalsa.h"
-#include "identity.h"
 #include "information.h"
 #include "libbalsa-conf.h"
 #include <glib/gi18n.h>
 #include "misc.h"
 
+#if HAVE_MACOSX_DESKTOP
+#  include "macosx-helpers.h"
+#endif
+
 #if ENABLE_ESMTP
 #include <string.h>
 #include "smtp-server.h"
@@ -96,12 +98,13 @@ static void
 libbalsa_identity_init(LibBalsaIdentity* ident)
 {
     ident->identity_name = NULL;
-    ident->ia = internet_address_new();
+    ident->ia = NULL;
     ident->replyto = NULL;
     ident->domain = NULL;
     ident->bcc = NULL;
     ident->reply_string = g_strdup(_("Re:"));
     ident->forward_string = g_strdup(_("Fwd:"));
+    ident->send_mp_alternative = FALSE;
     ident->signature_path = NULL;
     ident->sig_executable = FALSE;
     ident->sig_sending = TRUE;
@@ -113,7 +116,9 @@ libbalsa_identity_init(LibBalsaIdentity* ident)
     ident->gpg_sign = FALSE;
     ident->gpg_encrypt = FALSE;
     ident->always_trust = FALSE;
+    ident->warn_send_plain = TRUE;
     ident->crypt_protocol = LIBBALSA_PROTECT_OPENPGP;
+    ident->force_key_id = NULL;
 #endif
     ident->request_mdn = FALSE;
     /*
@@ -130,7 +135,8 @@ libbalsa_identity_finalize(GObject * object)
 {
     LibBalsaIdentity *ident = LIBBALSA_IDENTITY(object);
 
-    internet_address_unref(ident->ia);
+    if (ident->ia)
+	g_object_unref(ident->ia);
     g_free(ident->identity_name);
     g_free(ident->replyto);
     g_free(ident->domain);
@@ -144,6 +150,9 @@ libbalsa_identity_finalize(GObject * object)
 #endif                          /* ENABLE_ESMTP */
     g_free(ident->face);
     g_free(ident->x_face);
+#ifdef HAVE_GPGME
+    g_free(ident->force_key_id);
+#endif
 
     G_OBJECT_CLASS(parent_class)->finalize(object);
 }
@@ -195,7 +204,8 @@ libbalsa_identity_set_address(LibBalsaIdentity * ident,
 {
     g_return_if_fail(ident != NULL);
 
-    internet_address_unref(ident->ia);
+    if (ident->ia)
+	g_object_unref(ident->ia);
     ident->ia = ia;
 }
 
@@ -250,6 +260,14 @@ libbalsa_identity_set_forward_string(LibBalsaIdentity* ident, const gchar* forwa
 }
 
 
+void
+libbalsa_identity_set_send_mp_alternative(LibBalsaIdentity* ident, gboolean send_mp_alternative)
+{
+    g_return_if_fail(ident != NULL);
+    ident->send_mp_alternative = send_mp_alternative;
+}
+
+
 void
 libbalsa_identity_set_signature_path(LibBalsaIdentity* ident, const gchar* path)
 {
@@ -454,7 +472,7 @@ libbalsa_identity_select_dialog(GtkWindow * parent,
     sdi = g_object_get_data(G_OBJECT(parent),
                             LIBBALSA_IDENTITY_SELECT_DIALOG_KEY);
     if (sdi) {
-        gdk_window_raise(sdi->dialog->window);
+        gtk_window_present(GTK_WINDOW(sdi->dialog));
         return;
     }
 
@@ -472,6 +490,10 @@ libbalsa_identity_select_dialog(GtkWindow * parent,
                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                     GTK_STOCK_OK, GTK_RESPONSE_OK,
                                     NULL);
+#if HAVE_MACOSX_DESKTOP
+    libbalsa_macosx_menu_for_parent(dialog, parent);
+#endif
+ 
     g_signal_connect(dialog, "response",
                      G_CALLBACK(sd_response_cb), sdi);
     gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
@@ -484,7 +506,8 @@ libbalsa_identity_select_dialog(GtkWindow * parent,
     identity_list_update_real(GTK_TREE_VIEW(tree), identities, initial_id);
 
     frame = gtk_frame_new(NULL);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),
+    gtk_box_pack_start(GTK_BOX
+                       (gtk_dialog_get_content_area(GTK_DIALOG(dialog))),
                        frame, TRUE, TRUE, 0);
     gtk_container_add(GTK_CONTAINER(frame), tree);
     gtk_container_set_border_width(GTK_CONTAINER(frame), padding);
@@ -948,7 +971,8 @@ struct {
 };
 
 #define LIBBALSA_IDENTITY_CHECK "libbalsa-identity-check"
-static void md_sig_path_changed_cb(GtkWidget *sig_button, GObject *dialog);
+static void md_sig_path_changed_cb(GtkToggleButton * sig_button,
+                                   GObject * dialog);
 
 #if ENABLE_ESMTP
 static GtkWidget*
@@ -969,26 +993,29 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
     /* create the "General" tab */
     table = append_ident_notebook_page(notebook, 5, _("General"), NULL);
     row = 0;
-    ident_dialog_add_entry(table, row++, dialog, _("_Identity Name:"), 
+    ident_dialog_add_entry(table, row++, dialog, _("_Identity name:"), 
 		           "identity-name");
-    ident_dialog_add_entry(table, row++, dialog, _("_Full Name:"), 
+    ident_dialog_add_entry(table, row++, dialog, _("_Full name:"), 
                            "identity-fullname");
-    ident_dialog_add_entry(table, row++, dialog, _("_Mailing Address:"), 
+    ident_dialog_add_entry(table, row++, dialog, _("_Mailing address:"), 
                            "identity-address");
-    ident_dialog_add_entry(table, row++, dialog, _("Reply _To:"), 
+    ident_dialog_add_entry(table, row++, dialog, _("Reply _to:"), 
                            "identity-replyto");
     ident_dialog_add_entry(table, row++, dialog, _("_Domain:"), 
                            "identity-domain");
 
     /* create the "Messages" tab */
-    table = append_ident_notebook_page(notebook, 8, _("Messages"), NULL);
+    table = append_ident_notebook_page(notebook, 9, _("Messages"), NULL);
     row = 0;
     ident_dialog_add_entry(table, row++, dialog, _("_Bcc:"), 
                            "identity-bcc");
-    ident_dialog_add_entry(table, row++, dialog, _("Reply _String:"), 
+    ident_dialog_add_entry(table, row++, dialog, _("Reply _string:"), 
                            "identity-replystring");
-    ident_dialog_add_entry(table, row++, dialog, _("F_orward String:"), 
+    ident_dialog_add_entry(table, row++, dialog, _("F_orward string:"), 
                            "identity-forwardstring");
+    ident_dialog_add_checkbutton(table, row++, dialog, 
+                                 _("send messages in both plain text and _HTML format"),
+                                 "identity-sendmpalternative", TRUE);
     ident_dialog_add_checkbutton(table, row++, dialog, 
                                  _("request _Message Disposition Notification by default"),
                                  "identity-requestmdn", TRUE);
@@ -1000,7 +1027,7 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
                            path_info[LBI_PATH_TYPE_FACE].box_key,
                            path_info[LBI_PATH_TYPE_XFACE].box_key);
 #if ENABLE_ESMTP
-    ident_dialog_add_smtp_menu(table, row++, dialog, _("SMT_P Server:"),
+    ident_dialog_add_smtp_menu(table, row++, dialog, _("SMT_P server:"),
                                "identity-smtp-server", smtp_servers);
 #endif /* ENABLE_ESMTP */
 
@@ -1008,25 +1035,25 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
     table = append_ident_notebook_page(notebook, 7, _("Signature"), NULL);
     row = 0;
     ident_dialog_add_check_and_entry(table, row++, dialog,
-                                     _("Signature _Path"),
+                                     _("Signature _path"),
                                      "identity-sigpath");
     ident_dialog_add_checkbutton(table, row++, dialog,
-                                _("_Execute Signature"),
+                                _("_Execute signature"),
 				 "identity-sigexecutable", FALSE);
     ident_dialog_add_checkbutton(table, row++, dialog,
-                                 _("Incl_ude Signature"), 
+                                 _("Incl_ude signature"), 
                                  "identity-sigappend", FALSE);
     ident_dialog_add_checkbutton(table, row++, dialog, 
-                                 _("Include Signature When For_warding"),
+                                 _("Include signature when for_warding"),
                                  "identity-whenforward", FALSE);
     ident_dialog_add_checkbutton(table, row++, dialog,
-                                 _("Include Signature When Rep_lying"),
+                                 _("Include signature when rep_lying"),
                                  "identity-whenreply", FALSE);
     ident_dialog_add_checkbutton(table, row++, dialog, 
-                                 _("_Add Signature Separator"),
+                                 _("_Add signature separator"),
                                  "identity-sigseparator", FALSE);
     ident_dialog_add_checkbutton(table, row++, dialog,
-                                 _("Prepend Si_gnature"),
+                                 _("Prepend si_gnature"),
                                  "identity-sigprepend", FALSE);
 
 #ifdef HAVE_GPGME
@@ -1037,7 +1064,7 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
 #endif
     /* create the "Security" tab */
     table =
-        append_ident_notebook_page(notebook, 4, _("Security"), footnote);
+        append_ident_notebook_page(notebook, 5, _("Security"), footnote);
     row = 0;
     ident_dialog_add_checkbutton(table, row++, dialog, 
                                  _("sign messages by default"),
@@ -1051,6 +1078,12 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
     ident_dialog_add_checkbutton(table, row++, dialog,
                                  _("always trust GnuPG keys when encrypting"),
                                  "identity-trust-always", TRUE);
+    ident_dialog_add_checkbutton(table, row++, dialog,
+                                 _("remind me if messages can be encrypted"),
+                                 "identity-warn-send-plain", TRUE);
+    ident_dialog_add_entry(table, row++, dialog,
+                           _("use secret key with this id for signing\n(leave empty for automatic selection)"), 
+                           "identity-keyid");
 #ifndef HAVE_GPGME
     gtk_widget_set_sensitive(table, FALSE);
 #endif
@@ -1159,11 +1192,7 @@ file_chooser_check_cb(GtkToggleButton * button, GtkWidget * chooser)
     gtk_widget_set_sensitive(chooser,
                              gtk_toggle_button_get_active(button));
     /* Force validation of current path, if any. */
-#if GTK_CHECK_VERSION(2, 6, 0)
     g_signal_emit_by_name(chooser, "selection-changed");
-#else                           /* GTK_CHECK_VERSION(2, 6, 0) */
-    g_signal_emit_by_name(chooser, "changed");
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
 }
 
 static void
@@ -1186,11 +1215,9 @@ md_face_path_changed(const gchar * filename, gboolean active,
                                             filename, &size, &err);
 
     if (err) {
-#if GTK_CHECK_VERSION(2, 6, 0)
         libbalsa_information(LIBBALSA_INFORMATION_WARNING,
                              _("Error reading file %s: %s"), filename,
                              err->message);
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
         g_error_free(err);
         gtk_widget_hide(face_box);
         return;
@@ -1267,10 +1294,9 @@ md_sig_path_changed(gboolean active, GObject * dialog)
 }
 
 static void
-md_sig_path_changed_cb(GtkWidget *sig_button, GObject *dialog)
+md_sig_path_changed_cb(GtkToggleButton *sig_button, GObject *dialog)
 {
-    md_sig_path_changed(GTK_TOGGLE_BUTTON(sig_button)->active,
-                        dialog);
+    md_sig_path_changed(gtk_toggle_button_get_active(sig_button), dialog);
 }
 
 #define LIBBALSA_IDENTITY_INFO "libbalsa-identity-info"
@@ -1282,11 +1308,7 @@ file_chooser_cb(GtkWidget * chooser, gpointer data)
     GtkToggleButton *check;
     gboolean active;
 
-#if GTK_CHECK_VERSION(2, 6, 0)
     filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser));
-#else                           /* GTK_CHECK_VERSION(2, 6, 0) */
-    filename = gtk_editable_get_chars(GTK_EDITABLE(chooser), 0, -1);
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
     if (!filename || !*filename) {
         g_free(filename);
         return;
@@ -1313,9 +1335,7 @@ ident_dialog_add_file_chooser_button(GtkWidget * table, gint row,
 {
     GtkWidget *check;
     gchar *filename;
-#if GTK_CHECK_VERSION(2, 6, 0)
     gchar *title;
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
     GtkWidget *button;
 
     check =
@@ -1325,17 +1345,12 @@ ident_dialog_add_file_chooser_button(GtkWidget * table, gint row,
 
     filename =
         g_build_filename(g_get_home_dir(), path_info[type].basename, NULL);
-#if GTK_CHECK_VERSION(2, 6, 0)
     title = g_strdup_printf("Choose %s file", _(path_info[type].info));
     button = gtk_file_chooser_button_new(title,
                                          GTK_FILE_CHOOSER_ACTION_OPEN);
     g_free(title);
     gtk_file_chooser_set_show_hidden(GTK_FILE_CHOOSER(button), TRUE);
     gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(button), filename);
-#else                           /* GTK_CHECK_VERSION(2, 6, 0) */
-    button = gtk_entry_new();
-    gtk_entry_set_text(GTK_ENTRY(button), filename);
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
     g_free(filename);
 
     gtk_table_attach(GTK_TABLE(table), button, 1, 2, row, row + 1,
@@ -1347,13 +1362,8 @@ ident_dialog_add_file_chooser_button(GtkWidget * table, gint row,
                       GUINT_TO_POINTER(type));
     g_signal_connect(check, "toggled",
                      G_CALLBACK(file_chooser_check_cb), button);
-#if GTK_CHECK_VERSION(2, 6, 0)
     g_signal_connect(button, "selection-changed",
                      G_CALLBACK(file_chooser_cb), dialog);
-#else                           /* GTK_CHECK_VERSION(2, 6, 0) */
-    g_signal_connect(button, "changed",
-                     G_CALLBACK(file_chooser_cb), dialog);
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
 }
 
 static void
@@ -1444,16 +1454,15 @@ ident_dialog_update(GObject * dlg)
     g_free(id->identity_name); id->identity_name = text;
     set_identity_name_in_tree(GTK_TREE_VIEW(tree), id, text);
 
-    text = ident_dialog_get_text(dlg, "identity-fullname");
-    g_return_val_if_fail(text != NULL, FALSE);
-    ia = internet_address_new();
-    internet_address_set_name(ia, text);
-    g_free(text);
-    
     text = ident_dialog_get_text(dlg, "identity-address");
-    internet_address_set_addr(ia, text);
+    g_return_val_if_fail(text != NULL, FALSE);
+    ia = internet_address_mailbox_new(NULL, text);
     g_free(text);
+
+    text = ident_dialog_get_text(dlg, "identity-fullname");
+    internet_address_set_name(ia, text);
     libbalsa_identity_set_address(id, ia);
+    g_free(text);
 
     g_free(id->replyto);
     id->replyto         = ident_dialog_get_text(dlg, "identity-replyto");
@@ -1465,6 +1474,7 @@ ident_dialog_update(GObject * dlg)
     id->reply_string    = ident_dialog_get_text(dlg, "identity-replystring");
     g_free(id->forward_string);
     id->forward_string  = ident_dialog_get_text(dlg, "identity-forwardstring");
+    id->send_mp_alternative = ident_dialog_get_bool(dlg, "identity-sendmpalternative");
 #if ENABLE_ESMTP
     if(id->smtp_server) g_object_unref(id->smtp_server);
     id->smtp_server = ident_dialog_get_value(dlg, "identity-smtp-server");
@@ -1491,8 +1501,11 @@ ident_dialog_update(GObject * dlg)
     id->gpg_sign        = ident_dialog_get_bool(dlg, "identity-gpgsign");
     id->gpg_encrypt     = ident_dialog_get_bool(dlg, "identity-gpgencrypt");
     id->always_trust    = ident_dialog_get_bool(dlg, "identity-trust-always");
+    id->warn_send_plain = ident_dialog_get_bool(dlg, "identity-warn-send-plain");
     id->crypt_protocol  = GPOINTER_TO_INT(ident_dialog_get_value
                                           (dlg, "identity-crypt-protocol"));
+    g_free(id->force_key_id);
+    id->force_key_id    = g_strstrip(ident_dialog_get_text(dlg, "identity-keyid"));
 #endif
    
     return TRUE;
@@ -1542,14 +1555,15 @@ ident_dialog_get_path(GObject * dialog, const gchar * key)
     GtkWidget *chooser;
 
     chooser = g_object_get_data(dialog, key);
+#if GTK_CHECK_VERSION(2, 18, 0)
+    if (!gtk_widget_get_sensitive(chooser))
+        return NULL;
+#else                           /* GTK_CHECK_VERSION(2, 18, 0) */
     if (!GTK_WIDGET_SENSITIVE(chooser))
         return NULL;
+#endif                          /* GTK_CHECK_VERSION(2, 18, 0) */
 
-#if GTK_CHECK_VERSION(2, 6, 0)
     return gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser));
-#else                           /* GTK_CHECK_VERSION(2, 6, 0) */
-    return gtk_editable_get_chars(GTK_EDITABLE(chooser), 0, -1);
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
 }
 
 
@@ -1634,6 +1648,9 @@ delete_ident_cb(GtkTreeView * tree, GtkWidget * dialog)
                                      GTK_BUTTONS_OK_CANCEL,
                                      _("Do you really want to delete"
                                        " the selected identity?"));
+#if HAVE_MACOSX_DESKTOP
+    libbalsa_macosx_menu_for_parent(confirm, GTK_WINDOW(dialog));
+#endif
     di = g_new(IdentityDeleteInfo, 1);
     di->tree = tree;
     di->dialog = dialog;
@@ -1662,9 +1679,8 @@ help_ident_cb(void)
 {
     GError *err = NULL;
 
-#ifdef HAVE_GNOME
-    gnome_help_display("balsa", "identities", &err);
-#endif
+    gtk_show_uri(NULL, "ghelp:balsa?identities",
+                 gtk_get_current_event_time(), &err);
 
     if (err) {
         g_print(_("Error displaying help for identities: %s\n"),
@@ -1708,7 +1724,7 @@ libbalsa_identity_config_dialog(GtkWindow *parent, GList **identities,
 
     /* Show only one dialog at a time. */
     if (dialog) {
-        gdk_window_raise(dialog->window);
+        gtk_window_present(GTK_WINDOW(dialog));
         return;
     }
 
@@ -1721,6 +1737,9 @@ libbalsa_identity_config_dialog(GtkWindow *parent, GList **identities,
                                     GTK_STOCK_REMOVE, IDENTITY_RESPONSE_REMOVE,
                                     GTK_STOCK_CLOSE, IDENTITY_RESPONSE_CLOSE,
                                     NULL);
+#if HAVE_MACOSX_DESKTOP
+    libbalsa_macosx_menu_for_parent(dialog, parent);
+#endif
 
     frame = libbalsa_identity_config_frame(identities, default_id, dialog,
                                            changed_cb, parent);
@@ -1734,7 +1753,8 @@ libbalsa_identity_config_dialog(GtkWindow *parent, GList **identities,
                                     IDENTITY_RESPONSE_CLOSE);
 
     hbox = gtk_hbox_new(FALSE, padding);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), 
+    gtk_box_pack_start(GTK_BOX
+                       (gtk_dialog_get_content_area(GTK_DIALOG(dialog))),
                        hbox, TRUE, TRUE, 0);
 
     gtk_box_pack_start(GTK_BOX(hbox), frame, FALSE, FALSE, 0);
@@ -1817,10 +1837,10 @@ display_frame_update(GObject * dialog, LibBalsaIdentity* ident)
 
     ident_dialog_update(dialog);
     display_frame_set_field(dialog, "identity-name", ident->identity_name);
-    display_frame_set_field(dialog, "identity-fullname", ident->ia->name);
-    if (ident->ia->type == INTERNET_ADDRESS_NAME)
+    display_frame_set_field(dialog, "identity-fullname", ident->ia ? ident->ia->name : NULL);
+    if (ident->ia && INTERNET_ADDRESS_IS_MAILBOX (ident->ia))
         display_frame_set_field(dialog, "identity-address", 
-                                ident->ia->value.addr);
+                                INTERNET_ADDRESS_MAILBOX(ident->ia)->addr);
     else
         display_frame_set_field(dialog, "identity-address", NULL);
     
@@ -1831,6 +1851,8 @@ display_frame_update(GObject * dialog, LibBalsaIdentity* ident)
                             ident->reply_string);
     display_frame_set_field(dialog, "identity-forwardstring", 
                             ident->forward_string);
+    display_frame_set_boolean(dialog, "identity-sendmpalternative",
+                              ident->send_mp_alternative);
 #if ENABLE_ESMTP
     display_frame_set_server(dialog, "identity-smtp-server",
                              ident->smtp_server);
@@ -1871,8 +1893,11 @@ display_frame_update(GObject * dialog, LibBalsaIdentity* ident)
                               ident->gpg_encrypt);    
     display_frame_set_boolean(dialog, "identity-trust-always", 
                               ident->always_trust);    
+    display_frame_set_boolean(dialog, "identity-warn-send-plain", 
+                              ident->warn_send_plain);    
     display_frame_set_gpg_mode(dialog, "identity-crypt-protocol",
 			   &ident->crypt_protocol);
+    display_frame_set_field(dialog, "identity-keyid", ident->force_key_id);
 #endif
 }
 
@@ -1926,17 +1951,16 @@ LibBalsaIdentity*
 libbalsa_identity_new_config(const gchar* name)
 {
     LibBalsaIdentity* ident;
+    gchar *fname, *email;
     gchar* tmpstr;
     
+    fname = libbalsa_conf_get_string("FullName");
+    email = libbalsa_conf_get_string("Address");
+    
     ident = LIBBALSA_IDENTITY(libbalsa_identity_new_with_name(name));
-
-    tmpstr = libbalsa_conf_get_string("FullName");
-    internet_address_set_name(ident->ia, tmpstr);
-    g_free(tmpstr);
-
-    tmpstr = libbalsa_conf_get_string("Address");
-    internet_address_set_addr(ident->ia, tmpstr);
-    g_free(tmpstr);
+    ident->ia = internet_address_mailbox_new (fname, email);
+    g_free(fname);
+    g_free(email);
 
     ident->replyto = libbalsa_conf_get_string("ReplyTo");
     ident->domain = libbalsa_conf_get_string("Domain");
@@ -1955,6 +1979,8 @@ libbalsa_identity_new_config(const gchar* name)
         g_free(ident->forward_string);
         ident->forward_string = tmpstr;
     }
+    ident->send_mp_alternative =
+        libbalsa_conf_get_bool("SendMultipartAlternative");
     
     ident->signature_path = libbalsa_conf_get_string("SignaturePath");
     ident->sig_executable = libbalsa_conf_get_bool("SigExecutable");
@@ -1971,7 +1997,9 @@ libbalsa_identity_new_config(const gchar* name)
     ident->gpg_sign = libbalsa_conf_get_bool("GpgSign");
     ident->gpg_encrypt = libbalsa_conf_get_bool("GpgEncrypt");
     ident->always_trust = libbalsa_conf_get_bool("GpgTrustAlways");
+    ident->warn_send_plain = libbalsa_conf_get_bool("GpgWarnSendPlain=true");
     ident->crypt_protocol = libbalsa_conf_get_int("CryptProtocol=16");
+    ident->force_key_id = libbalsa_conf_get_string("ForceKeyID");
 #endif
 
     return ident;
@@ -1983,16 +2011,17 @@ libbalsa_identity_save(LibBalsaIdentity* ident, const gchar* group)
     g_return_if_fail(ident);
 
     libbalsa_conf_push_group(group);
-    libbalsa_conf_set_string("FullName", ident->ia->name);
+    libbalsa_conf_set_string("FullName", ident->ia ? ident->ia->name : NULL);
     
-    if (ident->ia->type == INTERNET_ADDRESS_NAME)
-        libbalsa_conf_set_string("Address", ident->ia->value.addr);
+    if (ident->ia && INTERNET_ADDRESS_IS_MAILBOX (ident->ia))
+        libbalsa_conf_set_string("Address", INTERNET_ADDRESS_MAILBOX(ident->ia)->addr);
 
     libbalsa_conf_set_string("ReplyTo", ident->replyto);
     libbalsa_conf_set_string("Domain", ident->domain);
     libbalsa_conf_set_string("Bcc", ident->bcc);
     libbalsa_conf_set_string("ReplyString", ident->reply_string);
     libbalsa_conf_set_string("ForwardString", ident->forward_string);
+    libbalsa_conf_set_bool("SendMultipartAlternative", ident->send_mp_alternative);
 #if ENABLE_ESMTP
     libbalsa_conf_set_string("SmtpServer",
                              libbalsa_smtp_server_get_name(ident->
@@ -2016,7 +2045,9 @@ libbalsa_identity_save(LibBalsaIdentity* ident, const gchar* group)
     libbalsa_conf_set_bool("GpgSign", ident->gpg_sign);
     libbalsa_conf_set_bool("GpgEncrypt", ident->gpg_encrypt);
     libbalsa_conf_set_bool("GpgTrustAlways", ident->always_trust);
+    libbalsa_conf_set_bool("GpgWarnSendPlain", ident->warn_send_plain);
     libbalsa_conf_set_int("CryptProtocol", ident->crypt_protocol);
+    libbalsa_conf_set_string("ForceKeyID", ident->force_key_id);
 #endif
 
     libbalsa_conf_pop_group();
