@@ -1,21 +1,49 @@
 /* -*-mode:c; c-basic-offset:4; -*- */
+/* Balsa E-Mail Client
+ * Copyright (C) 1997-2010 Stuart Parmenter and others,
+ *                         See the file AUTHORS for a list.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option) 
+ * any later version.
+ *  
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
+ * GNU General Public License for more details.
+ *  
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
+ * 02111-1307, USA.
+ */
 /*
   LibBalsaImapServer is a class for managing connections to one IMAP
   server. Idle connections are disconnected after a timeout, or when
   the user switches to offline mode.
 */
+
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "imap-server.h"
+
 #include <string.h>
 #include <stdlib.h>
 #include <pthread.h>
 
+#if defined(HAVE_GNOME_KEYRING)
+#include <gnome-keyring.h>
+#endif
+
 #include "libbalsa.h"
 #include "libbalsa-conf.h"
 #include "server.h"
 
 #include "imap-handle.h"
-#include "imap-server.h"
 #include "imap-commands.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 
 #ifdef USE_TLS
 #define REQ_SSL(s) (LIBBALSA_SERVER(s)->use_ssl)
@@ -36,7 +64,9 @@ struct LibBalsaImapServer_ {
     guint max_connections;
     gboolean offline_mode;
     
+#if defined(BALSA_USE_THREADS)
     GMutex *lock; /* protects the following members */
+#endif
     guint used_connections;
     GList *used_handles;
     GList *free_handles;
@@ -68,7 +98,20 @@ static gboolean connection_cleanup(gpointer ptr);
 /* We try to avoid too many connections per server */
 #define MAX_CONNECTIONS_PER_SERVER 20
 
+#if defined(BALSA_USE_THREADS)
 static pthread_mutex_t imap_servers_lock = PTHREAD_MUTEX_INITIALIZER;
+#define LOCK_SERVERS()   pthread_mutex_lock(&imap_servers_lock)
+#define UNLOCK_SERVERS() pthread_mutex_unlock(&imap_servers_lock)
+#define LOCK_SERVER(server)    g_mutex_lock((server)->lock)
+#define TRYLOCK_SERVER(server) g_mutex_trylock((server)->lock)
+#define UNLOCK_SERVER(server)  g_mutex_unlock((server)->lock)
+#else
+#define LOCK_SERVERS()
+#define UNLOCK_SERVERS()
+#define LOCK_SERVER(server)
+#define TRYLOCK_SERVER(server) TRUE
+#define UNLOCK_SERVER(server)
+#endif
 static GHashTable *imap_servers = NULL;
 
 struct handle_info {
@@ -119,12 +162,12 @@ static void libbalsa_imap_server_set_username(LibBalsaServer * server,
     if(server->host && name) { /* we have been initialized... */
         LibBalsaImapServer *imap_server = LIBBALSA_IMAP_SERVER(server);
         
-        pthread_mutex_lock(&imap_servers_lock);
+        LOCK_SERVERS();
         g_hash_table_steal(imap_servers, imap_server->key);
         g_free(imap_server->key);
         imap_server->key = g_strdup_printf("%s@%s", name, server->host);
         g_hash_table_insert(imap_servers, imap_server->key, imap_server);
-        pthread_mutex_unlock(&imap_servers_lock);
+        UNLOCK_SERVERS();
     }
     (parent_class)->set_username(server, name);
 }
@@ -134,12 +177,12 @@ libbalsa_imap_server_set_host(LibBalsaServer * server,
 {
     if(server->user && host) { /* we have been initialized... */
         LibBalsaImapServer *imap_server = LIBBALSA_IMAP_SERVER(server);
-        pthread_mutex_lock(&imap_servers_lock);
+        LOCK_SERVERS();
         g_hash_table_steal(imap_servers, imap_server->key);
         g_free(imap_server->key);
         imap_server->key = g_strdup_printf("%s@%s", server->user, host);
         g_hash_table_insert(imap_servers, imap_server->key, imap_server);
-        pthread_mutex_unlock(&imap_servers_lock);
+        UNLOCK_SERVERS();
     }
     (parent_class)->set_host(server, host, use_ssl);
 }
@@ -166,8 +209,11 @@ libbalsa_imap_server_class_init(LibBalsaImapServerClass * klass)
 static void
 libbalsa_imap_server_init(LibBalsaImapServer * imap_server)
 {
+    LIBBALSA_SERVER(imap_server)->protocol = "imap";
     imap_server->key = NULL;
+#if defined(BALSA_USE_THREADS)
     imap_server->lock = g_mutex_new();
+#endif
     imap_server->max_connections = MAX_CONNECTIONS_PER_SERVER;
     imap_server->used_connections = 0;
     imap_server->used_handles = NULL;
@@ -179,8 +225,8 @@ libbalsa_imap_server_init(LibBalsaImapServer * imap_server)
 #endif /* ENABLE_TOUCH_UI */
     imap_server->use_idle = TRUE;
     imap_server->connection_cleanup_id = 
-        g_timeout_add(CONNECTION_CLEANUP_POLL_PERIOD*1000,
-                      connection_cleanup, imap_server);
+        g_timeout_add_seconds(CONNECTION_CLEANUP_POLL_PERIOD,
+                              connection_cleanup, imap_server);
 }
 
 /* leave object in sane state (NULLified fields) */
@@ -195,17 +241,19 @@ libbalsa_imap_server_finalize(GObject * object)
     server = LIBBALSA_SERVER(object);
     imap_server = LIBBALSA_IMAP_SERVER(object);
 
-    pthread_mutex_lock(&imap_servers_lock);
+    LOCK_SERVERS();
     g_hash_table_remove(imap_servers, imap_server->key);
-    pthread_mutex_unlock(&imap_servers_lock);
+    UNLOCK_SERVERS();
     
     g_source_remove(imap_server->connection_cleanup_id);
 
 #if 0
-    g_mutex_lock(imap_server->lock);
+    LOCK_SERVER(imap_server);
 #endif
     libbalsa_imap_server_force_disconnect(imap_server);
+#if defined(BALSA_USE_THREADS)
     g_mutex_free(imap_server->lock);
+#endif
     g_free(imap_server->key); imap_server->key = NULL;
 
     G_OBJECT_CLASS(parent_class)->finalize(object);
@@ -316,7 +364,15 @@ lb_imap_server_info_new(LibBalsaServer *server)
     imap_handle_set_tls_mode(handle, mode);
     imap_handle_set_option(handle, IMAP_OPT_ANONYMOUS, server->try_anonymous);
     imap_handle_set_option(handle, IMAP_OPT_CLIENT_SORT, TRUE);
+#ifdef HAVE_GPGME
+    /* binary fetches change encoding and the checksums, and
+       signatures, disable them if we ever consider verifying message
+       integrity. */
+    imap_handle_set_option(handle, IMAP_OPT_BINARY, FALSE);
+#else
     imap_handle_set_option(handle, IMAP_OPT_BINARY, TRUE);
+#endif
+    imap_handle_set_option(handle, IMAP_OPT_COMPRESS, TRUE);
     imap_handle_set_option(handle, IMAP_OPT_IDLE,
                            LIBBALSA_IMAP_SERVER(server)->use_idle);
     return info;
@@ -326,6 +382,7 @@ lb_imap_server_info_new(LibBalsaServer *server)
 static void
 lb_imap_server_info_free(struct handle_info *info)
 {
+    imap_handle_force_disconnect(info->handle);
     g_object_unref(info->handle);
     g_free(info);
 }
@@ -341,7 +398,7 @@ lb_imap_server_cleanup(LibBalsaImapServer * imap_server)
     /* Quit if there is an action going on, eg. an connection is being
      * opened and the user is asked to confirm the certificate or
      * provide password, etc. */
-    if(!g_mutex_trylock(imap_server->lock))
+    if(!TRYLOCK_SERVER(imap_server))
         return; 
 
     idle_marker = time(NULL) - CONNECTION_CLEANUP_IDLE_TIME;
@@ -378,7 +435,7 @@ lb_imap_server_cleanup(LibBalsaImapServer * imap_server)
         }
     }
 
-    g_mutex_unlock(imap_server->lock);
+    UNLOCK_SERVER(imap_server);
 }
 
 static gboolean connection_cleanup(gpointer ptr)
@@ -409,15 +466,15 @@ static LibBalsaImapServer* get_or_create(const gchar *username,
     gchar *key;
 
     if (!imap_servers) {
-        pthread_mutex_lock(&imap_servers_lock);
+        LOCK_SERVERS();
         if (!imap_servers)
             imap_servers = g_hash_table_new(g_str_hash, g_str_equal);
-        pthread_mutex_unlock(&imap_servers_lock);
+        UNLOCK_SERVERS();
     }
 
     /* lookup username@host */
     key = g_strdup_printf("%s@%s", username, host);
-    pthread_mutex_lock(&imap_servers_lock);
+    LOCK_SERVERS();
     imap_server = g_hash_table_lookup(imap_servers, key);
     if (!imap_server) {
         imap_server = g_object_new(LIBBALSA_TYPE_IMAP_SERVER, NULL);
@@ -427,7 +484,7 @@ static LibBalsaImapServer* get_or_create(const gchar *username,
         g_free(key);
         g_object_ref(imap_server);
     }
-    pthread_mutex_unlock(&imap_servers_lock);
+    UNLOCK_SERVERS();
     return imap_server;
 }
 
@@ -495,18 +552,51 @@ libbalsa_imap_server_new_from_config(void)
     if(!d) imap_server->use_idle = !!d1;
     if (!server->passwd) {
         server->remember_passwd = libbalsa_conf_get_bool("RememberPasswd=false");
-        if(server->remember_passwd)
+        if(server->remember_passwd) {
+#if defined (HAVE_GNOME_KEYRING)
+	    GnomeKeyringResult r;
+	    server->passwd = NULL;
+	    r = gnome_keyring_find_password_sync(LIBBALSA_SERVER_KEYRING_SCHEMA,
+						 &server->passwd,
+						 "protocol", server->protocol,
+						 "server", server->host,
+						 "user", server->user,
+						 NULL);
+	    if(r != GNOME_KEYRING_RESULT_OK) {
+		gnome_keyring_free_password(server->passwd);
+		server->passwd = NULL;
+		printf("Keyring has no password for %s@%s\n",
+		       server->user, server->host);
+		server->passwd = libbalsa_conf_private_get_string("Password");
+		if (server->passwd != NULL) {
+		    gchar *buff = libbalsa_rot(server->passwd);
+		    libbalsa_free_password(server->passwd);
+		    server->passwd = buff;
+	            gnome_keyring_store_password_sync
+                        (LIBBALSA_SERVER_KEYRING_SCHEMA, NULL,
+                         _("Balsa passwords"), server->passwd,
+                         "protocol", server->protocol,
+                         "server", server->host,
+                         "user", server->user,
+                         NULL);
+		    /* We could in principle clear the password in the
+		       config file here but we do not for the backward
+		       compatibility. */
+		}
+	    }
+#else
             server->passwd = libbalsa_conf_private_get_string("Password");
+	    if (server->passwd != NULL) {
+		gchar *buff = libbalsa_rot(server->passwd);
+		libbalsa_free_password(server->passwd);
+		server->passwd = buff;
+	    }
+#endif
+	}
         if(server->passwd && server->passwd[0] == '\0') {
-            g_free(server->passwd);
+            libbalsa_free_password(server->passwd);
             server->passwd = NULL;
         }
-
-        if (server->passwd != NULL) {
-            gchar *buff = libbalsa_rot(server->passwd);
-            g_free(server->passwd);
-            server->passwd = buff;
-        }
     }
     return imap_server;
 }
@@ -535,7 +625,7 @@ handle_connection_error(int rc, struct handle_info *info,
     case IMAP_CONNECT_FAILED:
         g_set_error(err, LIBBALSA_MAILBOX_ERROR,
                     LIBBALSA_MAILBOX_NETWORK_ERROR,
-                    _("Cannot connect to the server"));
+                    _("Cannot connect to %s"), server->host);
         break;
     case IMAP_AUTH_CANCELLED:
         g_set_error(err, LIBBALSA_MAILBOX_ERROR,
@@ -574,7 +664,7 @@ libbalsa_imap_server_get_handle(LibBalsaImapServer *imap_server, GError **err)
     if (imap_server->offline_mode)
         return NULL;
 
-    g_mutex_lock(imap_server->lock);
+    LOCK_SERVER(imap_server);
     /* look for free connection */
     if (imap_server->free_handles) {
         GList *conn;
@@ -589,15 +679,15 @@ libbalsa_imap_server_get_handle(LibBalsaImapServer *imap_server, GError **err)
     /* create if used < max connections */
     if (!info
         && imap_server->used_connections < imap_server->max_connections) {
-        g_mutex_unlock(imap_server->lock);
+        UNLOCK_SERVER(imap_server);
         info = lb_imap_server_info_new(server);
-        g_mutex_lock(imap_server->lock);
+        LOCK_SERVER(imap_server);
         /* FIXME: after dropping and reacquiring the lock,
          * (imap_server->used_connections < imap_server->max_connections)
          * might no longer be true--do we care?
         if (imap_server->used_connections >= imap_server->max_connections) {
             lb_imap_server_info_free(info);
-            g_mutex_unlock(imap_server->lock);
+            UNLOCK_SERVER(imap_server);
             return NULL;
         }
          */
@@ -608,7 +698,7 @@ libbalsa_imap_server_get_handle(LibBalsaImapServer *imap_server, GError **err)
                                         REQ_SSL(server));
             if(rc != IMAP_SUCCESS) {
                 handle_connection_error(rc, info, server, err);
-                g_mutex_unlock(imap_server->lock);
+                UNLOCK_SERVER(imap_server);
                 return NULL;
             }
         }
@@ -617,7 +707,7 @@ libbalsa_imap_server_get_handle(LibBalsaImapServer *imap_server, GError **err)
                                                    info);
         imap_server->used_connections++;
     }
-    g_mutex_unlock(imap_server->lock);
+    UNLOCK_SERVER(imap_server);
 
     return info ? info->handle : NULL;
 }
@@ -647,7 +737,7 @@ libbalsa_imap_server_get_handle_with_user(LibBalsaImapServer *imap_server,
     if (imap_server->offline_mode)
         return NULL;
 
-    g_mutex_lock(imap_server->lock);
+    LOCK_SERVER(imap_server);
     /* look for free reusable connection */
     if (imap_server->free_handles) {
         GList *conn=NULL;
@@ -667,17 +757,17 @@ libbalsa_imap_server_get_handle_with_user(LibBalsaImapServer *imap_server,
      * those that do not SELECT any mailbox. */
     if (!info
         && imap_server->used_connections < imap_server->max_connections-1) {
-        g_mutex_unlock(imap_server->lock);
+        UNLOCK_SERVER(imap_server);
         info = lb_imap_server_info_new(server);
         if (!info)
             return NULL;
-        g_mutex_lock(imap_server->lock);
+        LOCK_SERVER(imap_server);
         /* FIXME: after dropping and reacquiring the lock,
          * (imap_server->used_connections < imap_server->max_connections)
          * might no longer be true--do we care?
         if (imap_server->used_connections >= imap_server->max_connections) {
             lb_imap_server_info_free(info);
-            g_mutex_unlock(imap_server->lock);
+            UNLOCK_SERVER(imap_server);
             return NULL;
         }
          */
@@ -695,7 +785,7 @@ libbalsa_imap_server_get_handle_with_user(LibBalsaImapServer *imap_server,
                     LIBBALSA_MAILBOX_TOOMANYOPEN_ERROR,
                     _("Exceeded the number of connections per server %s"),
                     server->host);
-        g_mutex_unlock(imap_server->lock);
+        UNLOCK_SERVER(imap_server);
         return NULL;
     }
 
@@ -704,7 +794,7 @@ libbalsa_imap_server_get_handle_with_user(LibBalsaImapServer *imap_server,
                                     REQ_SSL(server));
         if(rc != IMAP_SUCCESS) {
             handle_connection_error(rc, info, server, err);
-            g_mutex_unlock(imap_server->lock);
+            UNLOCK_SERVER(imap_server);
             return NULL;
         }
     }
@@ -713,7 +803,7 @@ libbalsa_imap_server_get_handle_with_user(LibBalsaImapServer *imap_server,
     imap_server->used_handles = g_list_prepend(imap_server->used_handles,
                                                info);
     imap_server->used_connections++;
-    g_mutex_unlock(imap_server->lock);
+    UNLOCK_SERVER(imap_server);
 
     return info->handle;
 }
@@ -733,7 +823,7 @@ void libbalsa_imap_server_release_handle(LibBalsaImapServer *imap_server,
     if (!handle)
         return;
 
-    g_mutex_lock(imap_server->lock);
+    LOCK_SERVER(imap_server);
     /* remove from used list */
     if (imap_server->used_handles) {
         GList *conn;
@@ -750,7 +840,7 @@ void libbalsa_imap_server_release_handle(LibBalsaImapServer *imap_server,
     /* add to free list */
         imap_server->free_handles = g_list_append(imap_server->free_handles,
                                                   info);
-    g_mutex_unlock(imap_server->lock);
+    UNLOCK_SERVER(imap_server);
 }
 
 /**
@@ -793,9 +883,10 @@ libbalsa_imap_server_has_persistent_cache(LibBalsaImapServer *srv)
  *
  * Forces a logout on all connections, used when cleaning up.
  **/
-void libbalsa_imap_server_force_disconnect(LibBalsaImapServer *imap_server)
+void
+libbalsa_imap_server_force_disconnect(LibBalsaImapServer *imap_server)
 {
-    g_mutex_lock(imap_server->lock);
+    LOCK_SERVER(imap_server);
     g_list_foreach(imap_server->used_handles,
                    (GFunc) lb_imap_server_info_free, NULL);
     g_list_free(imap_server->used_handles);
@@ -804,7 +895,7 @@ void libbalsa_imap_server_force_disconnect(LibBalsaImapServer *imap_server)
                    (GFunc) lb_imap_server_info_free, NULL);
     g_list_free(imap_server->free_handles);
     imap_server->free_handles = NULL;
-    g_mutex_unlock(imap_server->lock);
+    UNLOCK_SERVER(imap_server);
 }
 
 /**
@@ -824,10 +915,10 @@ static void close_all_connections_cb(gpointer key, gpointer value,
 void
 libbalsa_imap_server_close_all_connections(void)
 {
-    pthread_mutex_lock(&imap_servers_lock);
+    LOCK_SERVERS();
     if (imap_servers)
         g_hash_table_foreach(imap_servers, close_all_connections_cb, NULL);
-    pthread_mutex_unlock(&imap_servers_lock);
+    UNLOCK_SERVERS();
     libbalsa_imap_purge_temp_dir(0);
 }
 
@@ -843,10 +934,10 @@ libbalsa_imap_server_close_all_connections(void)
 gboolean libbalsa_imap_server_has_free_handles(LibBalsaImapServer *imap_server)
 {
     gboolean result;
-    g_mutex_lock(imap_server->lock);
+    LOCK_SERVER(imap_server);
     result = imap_server->used_connections < imap_server->max_connections
         || imap_server->free_handles;
-    g_mutex_unlock(imap_server->lock);
+    UNLOCK_SERVER(imap_server);
     return result;
 }
 
