@@ -1,5 +1,5 @@
 /* libimap library.
- * Copyright (C) 2003-2004 Pawel Salek.
+ * Copyright (C) 2003-2010 Pawel Salek.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -119,7 +119,7 @@ imap_check_capability(ImapMboxHandle* handle)
 {
   IMAP_REQUIRED_STATE3(handle, IMHS_CONNECTED, IMHS_AUTHENTICATED,
                        IMHS_SELECTED, FALSE);
-  if (imap_cmd_exec(handle, "CAPABILITY") != 0)
+  if (imap_cmd_exec(handle, "CAPABILITY") != IMR_OK)
     return FALSE;
 
   if (!(imap_mbox_handle_can_do(handle, IMCAP_IMAP4) ||
@@ -181,19 +181,18 @@ imap_mbox_handle_noop(ImapMboxHandle *handle)
 /* 6.3.1 SELECT Command
  * readonly_mbox can be NULL. */
 ImapResponse
-imap_mbox_select(ImapMboxHandle* handle, const char *mbox,
-                 gboolean *readonly_mbox)
+imap_mbox_select_unlocked(ImapMboxHandle* handle, const char *mbox,
+                          gboolean *readonly_mbox)
 {
   gchar* cmd, *mbx7;
   ImapResponse rc;
 
   IMAP_REQUIRED_STATE3(handle, IMHS_CONNECTED, IMHS_AUTHENTICATED,
                        IMHS_SELECTED, IMR_BAD);
-  HANDLE_LOCK(handle);
+
   if (handle->state == IMHS_SELECTED && strcmp(handle->mbox, mbox) == 0) {
     if(readonly_mbox)
       *readonly_mbox = handle->readonly_mbox;
-    HANDLE_UNLOCK(handle);
     return IMR_OK;
   }
   imap_mbox_resize_cache(handle, 0);
@@ -219,10 +218,20 @@ imap_mbox_select(ImapMboxHandle* handle, const char *mbox,
     mbox_view_dispose(&handle->mbox_view);
     g_signal_emit_by_name(G_OBJECT(handle), "exists-notify");
   }
-  HANDLE_UNLOCK(handle);
+
   return rc;
 }
 
+ImapResponse
+imap_mbox_select(ImapMboxHandle* handle, const char *mbox,
+                 gboolean *readonly_mbox)
+{
+  ImapResponse rc;
+  HANDLE_LOCK(handle);
+  rc = imap_mbox_select_unlocked(handle, mbox, readonly_mbox);
+  HANDLE_UNLOCK(handle);
+  return rc;
+}
 
 /* 6.3.2 EXAMINE Command */
 ImapResponse
@@ -317,15 +326,18 @@ imap_mbox_subscribe(ImapMboxHandle* handle,
 ImapResponse
 imap_mbox_list(ImapMboxHandle *handle, const char* what)
 {
+  ImapResponse rc;
   IMAP_REQUIRED_STATE2(handle,IMHS_AUTHENTICATED, IMHS_SELECTED, IMR_BAD);
+  HANDLE_LOCK(handle);
   {
   gchar *mbx7 = imap_utf8_to_mailbox(what);
   gchar *cmd = g_strdup_printf("LIST \"%s\" \"%%\"", mbx7);
-  ImapResponse rc = imap_cmd_exec(handle, cmd);
+  rc = imap_cmd_exec(handle, cmd);
   g_free(cmd);
   g_free(mbx7);
-  return rc;
   }
+  HANDLE_UNLOCK(handle);
+  return rc;
 }
 
 
@@ -333,14 +345,17 @@ imap_mbox_list(ImapMboxHandle *handle, const char* what)
 ImapResponse
 imap_mbox_lsub(ImapMboxHandle *handle, const char* what)
 {
+  ImapResponse rc;
   IMAP_REQUIRED_STATE2(handle,IMHS_AUTHENTICATED, IMHS_SELECTED, IMR_BAD);
+  HANDLE_LOCK(handle);
   {
   gchar *mbx7 = imap_utf8_to_mailbox(what);
   gchar *cmd = g_strdup_printf("LSUB \"%s\" \"%%\"", mbx7);
-  ImapResponse rc = imap_cmd_exec(handle, cmd);
+  rc = imap_cmd_exec(handle, cmd);
   g_free(mbx7); g_free(cmd);
-  return rc;
   }
+  HANDLE_UNLOCK(handle);
+  return rc;
 }
 
 
@@ -367,13 +382,15 @@ imap_mbox_status(ImapMboxHandle *r, const char*what,
     gchar *mbx7 = imap_utf8_to_mailbox(what);
     gchar *items = g_strjoinv(" ", (gchar**)&item_arr[0]);
     gchar *cmd = g_strdup_printf("STATUS \"%s\" (%s)", mbx7, items);
+    HANDLE_LOCK(r);
     g_hash_table_insert(r->status_resps, (gpointer)what, res);
     rc = imap_cmd_exec(r, cmd);
     g_hash_table_remove(r->status_resps, what);
+    HANDLE_UNLOCK(r);
     g_free(mbx7); g_free(cmd);
     g_free(items);
   }
-  return IMR_OK; 
+  return rc; 
 }
 /* 6.3.11 APPEND Command */
 static gchar*
@@ -382,11 +399,11 @@ enum_flag_to_str(ImapMsgFlags flg)
   GString *flags_str = g_string_new("");
   unsigned idx;
 
-  for(idx=0; idx < ELEMENTS(msg_flags); idx++) {
+  for(idx=0; idx < ELEMENTS(imap_msg_flags); idx++) {
     if((flg & (1<<idx)) == 0) continue;
     if(*flags_str->str) g_string_append_c(flags_str, ' ');
     g_string_append_c(flags_str, '\\');
-    g_string_append(flags_str, msg_flags[idx]);
+    g_string_append(flags_str, imap_msg_flags[idx]);
   }
   return g_string_free(flags_str, FALSE);
 }
@@ -493,7 +510,7 @@ imap_mbox_append_multi_real(ImapMboxHandle *handle,
   if(uid_sequence)
     uid_sequence->ranges = NULL;
 
-  imap_handle_idle_disable(handle);
+  if (!imap_handle_idle_disable(handle)) return IMR_SEVERED;
   for(msg_cnt=0;
       (msg_size = dump_cb(buf, sizeof(buf),
 			  IMA_STAGE_NEW_MSG, &flags, cb_arg)) >0;
@@ -540,7 +557,7 @@ imap_mbox_append_multi_real(ImapMboxHandle *handle,
 	rc = imap_cmd_step (handle, cmdno);
       } while (rc == IMR_UNTAGGED);
       if(rc != IMR_RESPOND) return rc;
-      while( (c=sio_getc(handle->sio)) != -1 && c != '\n');
+      EAT_LINE(handle, c);
       if(c == -1) {
 	imap_handle_disconnect(handle);
 	return IMR_SEVERED;
@@ -692,11 +709,15 @@ imap_mbox_expunge(ImapMboxHandle *handle)
 ImapResponse
 imap_mbox_expunge_a(ImapMboxHandle *handle)
 {
+  ImapResponse rc;
   IMAP_REQUIRED_STATE2(handle,IMHS_AUTHENTICATED, IMHS_SELECTED, IMR_BAD);
   /* extra care would be required to use this once since no other
      commands that use sequence numbers can be issued before this one
      finishes... */
-  return imap_cmd_issue(handle, "EXPUNGE");
+  HANDLE_LOCK(handle);
+  rc = imap_cmd_issue(handle, "EXPUNGE");
+  HANDLE_UNLOCK(handle);
+  return rc;
 }
 
 /* 6.4.4 SEARCH Command */
@@ -859,7 +880,9 @@ imap_assure_needed_flags(ImapMboxHandle *h, ImapMsgFlag needed_flags)
       case IMSGF_RECENT:   flg = "RECENT"; break;
       default: g_free(seqno); continue;
       }
-      if(!cmd) imap_handle_idle_disable(h);
+      if(!cmd) {
+        if (!imap_handle_idle_disable(h)) { rc = IMR_SEVERED; break; }
+      }
       cmd = g_strdup_printf(cmd_format, seqno, flg);
       g_free(seqno);
       flag[issued_cmd] = fnd.flag;
@@ -1009,9 +1032,10 @@ imap_mbox_handle_fetch_range(ImapMboxHandle* handle,
   return rc;
 }
 
-ImapResponse
-imap_mbox_handle_fetch_set(ImapMboxHandle* handle,
-                           unsigned *set, unsigned cnt, ImapFetchType ift)
+static ImapResponse
+imap_mbox_handle_fetch_set_unlocked(ImapMboxHandle* handle,
+				    unsigned *set, unsigned cnt,
+				    ImapFetchType ift)
 {
   gchar * seq;
   ImapResponse rc;
@@ -1029,15 +1053,24 @@ imap_mbox_handle_fetch_set(ImapMboxHandle* handle,
   if(seq) {
     const char* hdr[13];
     ic_construct_header_list(hdr, fd.fd.req_fetch_type);
-    HANDLE_LOCK(handle);
     rc = imap_mbox_handle_fetch(handle, seq, hdr);
     if(rc == IMR_OK) set_avail_headers(handle, seq, fd.fd.req_fetch_type);
-    HANDLE_UNLOCK(handle);
     g_free(seq);
   } else rc = IMR_OK;
   return rc;
 }
 
+ImapResponse
+imap_mbox_handle_fetch_set(ImapMboxHandle* handle,
+                           unsigned *set, unsigned cnt, ImapFetchType ift)
+{
+  ImapResponse rc;
+  HANDLE_LOCK(handle);
+  rc = imap_mbox_handle_fetch_set_unlocked(handle, set, cnt, ift);
+  HANDLE_UNLOCK(handle);
+  return rc;
+}
+
 static void
 write_nstring(unsigned seqno, ImapFetchBodyType body_type,
               const char *str, size_t len, void *fl)
@@ -1574,7 +1607,7 @@ imap_mbox_thread(ImapMboxHandle *h, const char *how, ImapSearchKey *filter)
 
     cmdno = imap_make_tag(tag);
     
-    imap_handle_idle_disable(h);
+    if (!imap_handle_idle_disable(h)) { rc = IMR_SEVERED; goto exit_cleanup; }
     sio_printf(h->sio, "%s THREAD %s UTF-8 ", tag, how);
     if(!filter)
       sio_write(h->sio, "ALL", 3);
@@ -1590,6 +1623,7 @@ imap_mbox_thread(ImapMboxHandle *h, const char *how, ImapSearchKey *filter)
     while(rc == IMR_UNTAGGED);
     imap_handle_idle_enable(h, 30);
   }
+ exit_cleanup:
   HANDLE_UNLOCK(h);
 
   return rc;
@@ -1775,8 +1809,8 @@ imap_mbox_sort_msgno_client(ImapMboxHandle *handle, ImapSortKey key,
     qsort(seqno_to_fetch, fetch_cnt, sizeof(unsigned), comp_unsigned);
     printf("Should the client side sorting code "
            "be sorry about your bandwidth usage?\n");
-    rc = imap_mbox_handle_fetch_set(handle, seqno_to_fetch,
-                                    fetch_cnt, fetch_type);
+    rc = imap_mbox_handle_fetch_set_unlocked(handle, seqno_to_fetch,
+					     fetch_cnt, fetch_type);
     if(rc != IMR_OK)
       return rc;
   }
@@ -1885,7 +1919,7 @@ imap_mbox_sort_filter(ImapMboxHandle *handle, ImapSortKey key, int ascending,
 
       cmdno =  imap_make_tag(tag);
       keystr = sort_code_to_string(key);
-      imap_handle_idle_disable(handle);
+      if (!imap_handle_idle_disable(handle)) { rc = IMR_SEVERED; goto cleanup; }
       sio_printf(handle->sio, "%s SORT (%s%s) UTF-8 ", tag,
                  ascending ? "" : "REVERSE ", keystr);
 
@@ -1935,7 +1969,7 @@ imap_mbox_sort_filter(ImapMboxHandle *handle, ImapSortKey key, int ascending,
       g_node_append_data(handle->thread_root,
                          GUINT_TO_POINTER(handle->mbox_view.arr[i]));
   }
-
+ cleanup:
   HANDLE_UNLOCK(handle);
 
   return rc;
