@@ -1,7 +1,7 @@
 #ifndef __IMAP_PRIVATE_H__
 #define __IMAP_PRIVATE_H__
 /* libimap library.
- * Copyright (C) 2003-2004 Pawel Salek.
+ * Copyright (C) 2003-2006 Pawel Salek.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,9 +21,35 @@
 
 #include <glib-object.h>
 
-#include <config.h>
+#include "config.h"
+
+#if defined(BALSA_USE_THREADS)
+#include <pthread.h>
+#define HANDLE_LOCK(h)   pthread_mutex_lock(&h->mutex)
+#define HANDLE_TRYLOCK(h) pthread_mutex_trylock(&h->mutex)
+#define HANDLE_UNLOCK(h) pthread_mutex_unlock(&h->mutex)
+#else
+#define HANDLE_LOCK(h) 
+#define HANDLE_TRYLOCK(h) 0
+#define HANDLE_UNLOCK(h)
+#endif
 
 #include "imap-commands.h"
+#include "imap_compress.h"
+
+typedef enum {
+  IMAP_BODY_TYPE_RFC822, /**< as fetched with RFC822 */
+  IMAP_BODY_TYPE_HEADER, /** header of the message: BODY[HEADER] */
+  IMAP_BODY_TYPE_TEXT,   /**< content of the message part: BODY[TEXT] */
+  IMAP_BODY_TYPE_BODY    /**< a part as fetched with BODY[x] */
+} ImapFetchBodyType;
+
+
+typedef void (*ImapFetchBodyInternalCb)(unsigned seqno,
+					ImapFetchBodyType body_type,
+					const char *buf,
+					size_t buflen, void* arg);
+
 
 struct _MboxView {
   unsigned *arr;
@@ -58,13 +84,25 @@ struct _ImapMboxHandle {
   ImapMessage **msg_cache;
   GArray       *flag_cache;
   MboxView mbox_view;
-  GHashTable *cmd_queue; /* A hash of commands we received return codes for.
-                          * We use the command number as the key. */
+  /** cmd_info is a list of commands that serves two-fold purpose. It
+      can contain task to execute when certain command completes. It
+      can be also used to store return codes in case they were
+      received asynchronously. */
+  GList *cmd_info;
   GSList *tasks;         /* a list of tasks to be executed after
                           * processing of current line is finished. */
-
   GNode *thread_root; /* deprecated! */
 
+  /** Compression data */
+  struct ImapCompressContext compress;
+
+  struct {
+    GList* src; /**< returned by COPY */
+    GList* dst; /**< returned by APPEND and COPY */
+    unsigned dst_uid_validity;
+    unsigned store_response:1;
+  } uidplus;
+
   /* BYE handling depends on the state */
   gboolean doing_logout;
   ImapInfoCb info_cb;
@@ -74,7 +112,7 @@ struct _ImapMboxHandle {
 
   ImapFlagsCb flags_cb;
   void *flags_arg;
-  ImapFetchBodyCb body_cb;
+  ImapFetchBodyInternalCb body_cb;
   void *body_arg;
 
   ImapMonitorCb monitor_cb;
@@ -82,16 +120,36 @@ struct _ImapMboxHandle {
 
   ImapSearchCb search_cb;
   void *search_arg;
+
+  GHashTable *status_resps; /* A hash of STATUS responses that we wait for */
+
+  GIOChannel *iochannel; /* IO channel used for monitoring the connection */
+#if defined(BALSA_USE_THREADS)
+  pthread_mutex_t mutex;
+#endif
+  guint idle_enable_id; /* callback to issue IDLE after a period of
+                           inactivity */
+  guint async_watch_id;  /* callback to process incoming data */
   ImapTlsMode tls_mode; /* disabled, enabled, required */
+  enum { IDLE_INACTIVE, IDLE_RESPONSE_PENDING, IDLE_ACTIVE }
+    idle_state; /*  IDLE State? */
+  unsigned op_cancelled:1; /* last op timed out and was cancelled by user */
+  unsigned readonly_mbox:1;
+  unsigned can_fetch_body:1; /* set for servers that always respond
+                              * correctly to FETCH x BODY[y]
+                              * requests. */
+  unsigned enable_anonymous:1;   /**< try anonymous if possible */
+  unsigned enable_binary:1;      /**< enable binary extension */
+  unsigned enable_client_sort:1; /**< client side sorting allowed */
+  unsigned enable_compress:1; /**< enable compress extension */
+  unsigned enable_idle:1;     /**< use IDLE - no problem with firewalls */
+
+  /* conditional stuff at the end for the safety. */
 #ifdef USE_TLS
   unsigned over_ssl:1; /* transmission is to be made over SSL-protected
                         * connection, usually to imaps port. */
   unsigned using_tls:1;
 #endif
-  unsigned readonly_mbox:1;
-  unsigned can_fetch_body:1; /* set for servers that always respond
-                              * correctly to FETCH x BODY[y]
-                              * requests. */
 };
 
 #define IMAP_MBOX_IS_DISCONNECTED(h)  ((h)->state == IMHS_DISCONNECTED)
@@ -111,15 +169,27 @@ struct _ImapMboxHandle {
 
 #define IS_ATOM_CHAR(c) (strchr("(){ %*\"\\]",(c))==NULL&&(c)>0x1f&&(c)!=0x7f)
 
-extern const char* msg_flags[6];
+#define EAT_LINE(h, c) while( (c=sio_getc(h->sio)) != -1 && c != '\n')
+
+extern const char* imap_msg_flags[6];
+
+ImapResponse imap_mbox_select_unlocked(ImapMboxHandle* handle, const char *mbox,
+                                       gboolean *readonly_mbox);
 
 void imap_mbox_resize_cache(ImapMboxHandle *h, unsigned new_size);
 
-ImapResponse imap_cmd_exec(ImapMboxHandle* handle, const char* cmd);
+ImapResponse imap_cmd_exec_cmdno(ImapMboxHandle* handle, const char* cmd,
+				 unsigned *cmdno);
+#define imap_cmd_exec(h, c) imap_cmd_exec_cmdno((h),(c),NULL)
+
+ImapResponse imap_cmd_issue(ImapMboxHandle* handle, const char* cmd);
 char* imap_mbox_gets(ImapMboxHandle *h, char* buf, size_t sz);
 
 ImapResponse imap_write_key(ImapMboxHandle *handle, ImapSearchKey *s,
                             unsigned cmdno, int use_literal);
+ImapResponse imap_search_exec_unlocked(ImapMboxHandle *h, gboolean uid, 
+				       ImapSearchKey *s,
+				       ImapSearchCb cb, void *cb_arg);
 ImapResponse imap_assure_needed_flags(ImapMboxHandle *h,
                                       ImapMsgFlag needed_flags);
 
@@ -130,12 +200,19 @@ int imap_setup_ssl(struct siobuf *sio, const char* host, SSL *ssl,
                    ImapUserCb user_cb, void *user_arg);
 #endif
 
+void imap_handle_disconnect(ImapMboxHandle *h);
 ImapConnectionState imap_mbox_handle_get_state(ImapMboxHandle *h);
 void imap_mbox_handle_set_state(ImapMboxHandle *h,
                                 ImapConnectionState newstate);
 #define imap_mbox_handle_set_msg(h,s) \
   do{g_free((h)->last_msg); (h)->last_msg = g_strdup(s); }while(0)
 
+typedef unsigned (*ImapCoalesceFunc)(int, void*);
+gchar* imap_coalesce_seq_range(int lo, int hi,
+			       ImapCoalesceFunc fun, void *data);
+unsigned imap_coalesce_func_simple(int i, unsigned msgno[]);
+gchar *imap_coalesce_set(int cnt, unsigned *seqnos);
+
 /* even more private functions */
 int imap_cmd_start(ImapMboxHandle* handle, const char* cmd, unsigned* cmdno);
 ImapResponse imap_cmd_step(ImapMboxHandle* handle, unsigned cmdno);
@@ -146,5 +223,5 @@ void mbox_view_append_no(MboxView *mv, unsigned seqno);
 
 int imap_socket_open(const char* host, const char *def_port);
 
-
+extern const char* imap_status_item_names[5];
 #endif /* __IMAP_PRIVATE_H__ */
