@@ -60,6 +60,8 @@ struct siobuf
     int milliseconds;		/* Timeout in ms */
 
     char *read_buffer;		/* client read buffer */
+    const char *read_buffer_start; /* client read buffer start, for
+                                      ungetc error checking. */
     char *read_position;	/* client read buffer pointer */
     int read_unread;		/* number of bytes unread in buffer */
 
@@ -83,6 +85,8 @@ struct siobuf
     void *user_data;
   };
 
+static void sio_flush_buffer (struct siobuf *sio);
+
 /* Attach bi-directional buffering to the socket descriptor.
  */
 struct siobuf *
@@ -373,7 +377,7 @@ sio_write (struct siobuf *sio, const void *bufp, int buflen)
 	  buf += sio->write_available;
 	  buflen -= sio->write_available;
 	}
-      sio_flush (sio);
+      sio_flush_buffer (sio);
       assert (sio->write_available > 0);
     }
   if (buflen > 0)
@@ -383,7 +387,7 @@ sio_write (struct siobuf *sio, const void *bufp, int buflen)
       sio->write_available -= buflen;
       /* If the buffer is exactly filled, flush it */
       if (sio->write_available == 0)
-	sio_flush (sio);
+	sio_flush_buffer (sio);
     }
 }
 
@@ -440,8 +444,8 @@ raw_write (struct siobuf *sio, const char *buf, int len)
       }
 }
 
-void
-sio_flush (struct siobuf *sio)
+static void
+sio_flush_buffer (struct siobuf *sio)
 {
   int length;
 
@@ -460,7 +464,7 @@ sio_flush (struct siobuf *sio)
   if (sio->encode_cb != NULL)
     {
       char *buf;
-      int len;
+      int len = 0;
 
       /* Rules for the encode callback.
 
@@ -470,8 +474,10 @@ sio_flush (struct siobuf *sio)
          callback must maintain its own buffer which must persist until
          the next call in the same thread.  The secarg argument may be
          used to maintain this buffer. */
-      (*sio->encode_cb) (&buf, &len, sio->write_buffer, length, sio->secarg);
-      raw_write (sio, buf, len);
+      while ((*sio->encode_cb) (&buf, &len, sio->write_buffer, 
+                                length, sio->secarg) >0) {
+        raw_write (sio, buf, len);
+      }
     }
   else
     raw_write (sio, sio->write_buffer, length);
@@ -489,6 +495,30 @@ sio_flush (struct siobuf *sio)
   sio->flush_mark = NULL;
 }
 
+void
+sio_flush (struct siobuf *sio)
+{
+  sio_flush_buffer (sio);
+  if (sio->encode_cb != NULL)
+    {
+      char *buf;
+      int len = 0;
+
+      /* Rules for the encode callback.
+
+         The output variables (here buf and len) may be set to the
+         write_buffer iff the encoding can be performed in place and
+         the result is shorter than the original data.  Otherwise the
+         callback must maintain its own buffer which must persist until
+         the next call in the same thread.  The secarg argument may be
+         used to maintain this buffer. */
+      while ((*sio->encode_cb) (&buf, &len, sio->write_buffer, 
+                                0, sio->secarg) >0) {
+        raw_write (sio, buf, len);
+      }
+    }
+}
+  
 void
 sio_mark (struct siobuf *sio)
 {
@@ -557,11 +587,8 @@ sio_fill (struct siobuf *sio)
 {
   assert (sio != NULL);
 
-  sio->read_unread = raw_read (sio, sio->read_buffer, sio->buffer_size);
-  if (sio->read_unread <= 0)
-    return 0;
 
-  if (sio->decode_cb != NULL)
+  if (sio->decode_cb != NULL) {
     /* Rules for the decode callback.
 
        The output variables (here buf and len) may be set to the
@@ -569,11 +596,32 @@ sio_fill (struct siobuf *sio)
        the result is shorter than the original data.  Otherwise the
        callback must maintain its own buffer which must persist until
        the next call in the same thread.  The secarg argument may be
-       used to maintain this buffer. */
-    (*sio->decode_cb) (&sio->read_position, &sio->read_unread,
-		       sio->read_buffer, sio->read_unread, sio->secarg);
-  else
+       used to maintain this buffer.
+       
+       Decode callback gets at most twice for each buffer: first time
+       with buflen == 0 to decode whatever is in the internal decode
+       buffer. If that call returns 0, actual data read is performed
+       and decode is given the second shot, when it isupposed to
+       return nonzero.
+       length value 0 means error.
+    */
+    while ((*sio->decode_cb) (&sio->read_position, &sio->read_unread,
+                               sio->read_buffer, sio->read_unread,
+                               sio->secarg) == 0) {
+      sio->read_unread = raw_read (sio, sio->read_buffer, sio->buffer_size);
+      if (sio->read_unread <= 0)
+        break;
+    }
+    sio->read_buffer_start = sio->read_position;
+    if (sio->read_unread <= 0)
+      return 0;
+  } else {
+    sio->read_unread = raw_read (sio, sio->read_buffer, sio->buffer_size);
+    if (sio->read_unread <= 0)
+      return 0;
     sio->read_position = sio->read_buffer;
+    sio->read_buffer_start = sio->read_position;
+ }
 
   if (sio->monitor_cb != NULL && sio->read_unread > 0)
     (*sio->monitor_cb) (sio->read_position, sio->read_unread,
@@ -620,7 +668,7 @@ sio_getc(struct siobuf *sio)
 int
 sio_ungetc(struct siobuf *sio)
 {
-  if(sio->read_position>sio->read_buffer) {
+  if(sio->read_position>sio->read_buffer_start) {
     sio->read_position--;
     sio->read_unread++;
     return 0;
