@@ -36,8 +36,8 @@
 #define g_marshal_value_peek_ulong(v)    (v)->data[0].v_ulong
 #define g_marshal_value_peek_int64(v)    (v)->data[0].v_int64
 #define g_marshal_value_peek_uint64(v)   (v)->data[0].v_uint64
-#define g_marshal_value_peek_enum(v)     (v)->data[0].v_int
-#define g_marshal_value_peek_flags(v)    (v)->data[0].v_uint
+#define g_marshal_value_peek_enum(v)     (v)->data[0].v_long
+#define g_marshal_value_peek_flags(v)    (v)->data[0].v_ulong
 #define g_marshal_value_peek_float(v)    (v)->data[0].v_float
 #define g_marshal_value_peek_double(v)   (v)->data[0].v_double
 #define g_marshal_value_peek_string(v)   (v)->data[0].v_pointer
@@ -51,10 +51,10 @@
 /* INT:OBJECT (./libbalsa-marshal.list:5) */
 void
 libbalsa_INT__OBJECT (GClosure     *closure,
-                      GValue       *return_value,
+                      GValue       *return_value G_GNUC_UNUSED,
                       guint         n_param_values,
                       const GValue *param_values,
-                      gpointer      invocation_hint,
+                      gpointer      invocation_hint G_GNUC_UNUSED,
                       gpointer      marshal_data)
 {
   typedef gint (*GMarshalFunc_INT__OBJECT) (gpointer     data1,
@@ -90,10 +90,10 @@ libbalsa_INT__OBJECT (GClosure     *closure,
 /* INT:OBJECT,OBJECT (./libbalsa-marshal.list:6) */
 void
 libbalsa_INT__OBJECT_OBJECT (GClosure     *closure,
-                             GValue       *return_value,
+                             GValue       *return_value G_GNUC_UNUSED,
                              guint         n_param_values,
                              const GValue *param_values,
-                             gpointer      invocation_hint,
+                             gpointer      invocation_hint G_GNUC_UNUSED,
                              gpointer      marshal_data)
 {
   typedef gint (*GMarshalFunc_INT__OBJECT_OBJECT) (gpointer     data1,
@@ -131,10 +131,10 @@ libbalsa_INT__OBJECT_OBJECT (GClosure     *closure,
 /* INT:POINTER,POINTER,POINTER (./libbalsa-marshal.list:8) */
 void
 libbalsa_INT__POINTER_POINTER_POINTER (GClosure     *closure,
-                                       GValue       *return_value,
+                                       GValue       *return_value G_GNUC_UNUSED,
                                        guint         n_param_values,
                                        const GValue *param_values,
-                                       gpointer      invocation_hint,
+                                       gpointer      invocation_hint G_GNUC_UNUSED,
                                        gpointer      marshal_data)
 {
   typedef gint (*GMarshalFunc_INT__POINTER_POINTER_POINTER) (gpointer     data1,
@@ -174,10 +174,10 @@ libbalsa_INT__POINTER_POINTER_POINTER (GClosure     *closure,
 /* POINTER:OBJECT (./libbalsa-marshal.list:11) */
 void
 libbalsa_POINTER__OBJECT (GClosure     *closure,
-                          GValue       *return_value,
+                          GValue       *return_value G_GNUC_UNUSED,
                           guint         n_param_values,
                           const GValue *param_values,
-                          gpointer      invocation_hint,
+                          gpointer      invocation_hint G_GNUC_UNUSED,
                           gpointer      marshal_data)
 {
   typedef gpointer (*GMarshalFunc_POINTER__OBJECT) (gpointer     data1,
@@ -213,10 +213,10 @@ libbalsa_POINTER__OBJECT (GClosure     *closure,
 /* POINTER:POINTER,POINTER (./libbalsa-marshal.list:12) */
 void
 libbalsa_POINTER__POINTER_POINTER (GClosure     *closure,
-                                   GValue       *return_value,
+                                   GValue       *return_value G_GNUC_UNUSED,
                                    guint         n_param_values,
                                    const GValue *param_values,
-                                   gpointer      invocation_hint,
+                                   gpointer      invocation_hint G_GNUC_UNUSED,
                                    gpointer      marshal_data)
 {
   typedef gpointer (*GMarshalFunc_POINTER__POINTER_POINTER) (gpointer     data1,
@@ -254,10 +254,10 @@ libbalsa_POINTER__POINTER_POINTER (GClosure     *closure,
 /* POINTER:VOID (./libbalsa-marshal.list:13) */
 void
 libbalsa_POINTER__VOID (GClosure     *closure,
-                        GValue       *return_value,
+                        GValue       *return_value G_GNUC_UNUSED,
                         guint         n_param_values,
                         const GValue *param_values,
-                        gpointer      invocation_hint,
+                        gpointer      invocation_hint G_GNUC_UNUSED,
                         gpointer      marshal_data)
 {
   typedef gpointer (*GMarshalFunc_POINTER__VOID) (gpointer     data1,
@@ -291,10 +291,10 @@ libbalsa_POINTER__VOID (GClosure     *closure,
 /* VOID:INT,INT,INT (./libbalsa-marshal.list:14) */
 void
 libbalsa_VOID__INT_INT_INT (GClosure     *closure,
-                            GValue       *return_value,
+                            GValue       *return_value G_GNUC_UNUSED,
                             guint         n_param_values,
                             const GValue *param_values,
-                            gpointer      invocation_hint,
+                            gpointer      invocation_hint G_GNUC_UNUSED,
                             gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__INT_INT_INT) (gpointer     data1,
@@ -330,10 +330,10 @@ libbalsa_VOID__INT_INT_INT (GClosure     *closure,
 /* VOID:INT,INT,INT,STRING (./libbalsa-marshal.list:15) */
 void
 libbalsa_VOID__INT_INT_INT_STRING (GClosure     *closure,
-                                   GValue       *return_value,
+                                   GValue       *return_value G_GNUC_UNUSED,
                                    guint         n_param_values,
                                    const GValue *param_values,
-                                   gpointer      invocation_hint,
+                                   gpointer      invocation_hint G_GNUC_UNUSED,
                                    gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__INT_INT_INT_STRING) (gpointer     data1,
@@ -368,13 +368,50 @@ libbalsa_VOID__INT_INT_INT_STRING (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,INT (./libbalsa-marshal.list:16) */
+/* VOID:OBJECT,BOOLEAN (./libbalsa-marshal.list:16) */
+void
+libbalsa_VOID__OBJECT_BOOLEAN (GClosure     *closure,
+                               GValue       *return_value G_GNUC_UNUSED,
+                               guint         n_param_values,
+                               const GValue *param_values,
+                               gpointer      invocation_hint G_GNUC_UNUSED,
+                               gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__OBJECT_BOOLEAN) (gpointer     data1,
+                                                     gpointer     arg_1,
+                                                     gboolean     arg_2,
+                                                     gpointer     data2);
+  register GMarshalFunc_VOID__OBJECT_BOOLEAN callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 3);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__OBJECT_BOOLEAN) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_object (param_values + 1),
+            g_marshal_value_peek_boolean (param_values + 2),
+            data2);
+}
+
+/* VOID:POINTER,INT (./libbalsa-marshal.list:17) */
 void
 libbalsa_VOID__POINTER_INT (GClosure     *closure,
-                            GValue       *return_value,
+                            GValue       *return_value G_GNUC_UNUSED,
                             guint         n_param_values,
                             const GValue *param_values,
-                            gpointer      invocation_hint,
+                            gpointer      invocation_hint G_GNUC_UNUSED,
                             gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__POINTER_INT) (gpointer     data1,
