@@ -20,9 +20,10 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
-
-#define _XOPEN_SOURCE 500
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "libbalsa.h"
 
 #include <glib.h>
 
@@ -48,12 +49,20 @@
 #if HAVE_GTKSOURCEVIEW
 #include <gtksourceview/gtksourceview.h>
 #include <gtksourceview/gtksourcebuffer.h>
-#include <gtksourceview/gtksourcetag.h>
-#include <gtksourceview/gtksourcetagstyle.h>
+/* note GtkSourceview 1 and 2 have a slightly different API */
+#if (HAVE_GTKSOURCEVIEW == 1)
+#  include <gtksourceview/gtksourcetag.h>
+#  include <gtksourceview/gtksourcetagstyle.h>
+#else
+#  include <gtksourceview/gtksourcelanguage.h>
+#  include <gtksourceview/gtksourcelanguagemanager.h>
+#  include <gtksourceview/gtksourcestylescheme.h>
+#  include <gtksourceview/gtksourcestyleschememanager.h>
+#endif
 #endif
 
-#include "libbalsa.h"
 #include "misc.h"
+#include "missing.h"
 #include <glib/gi18n.h>
 
 #ifdef BALSA_USE_THREADS
@@ -93,16 +102,28 @@ libbalsa_init(LibBalsaInformationFunc information_callback)
 
     libbalsa_real_information_func = information_callback;
 
-    g_mime_init(0);
+    g_mime_init(GMIME_ENABLE_RFC2047_WORKAROUNDS);
 
-    /* Register our types */
-    /* So that libbalsa_mailbox_new_from_config will work... */
+    GMIME_TYPE_DATA_WRAPPER;
+    GMIME_TYPE_FILTER;
+    GMIME_TYPE_FILTER_CRLF;
+    GMIME_TYPE_PARSER;
+    GMIME_TYPE_STREAM;
+    GMIME_TYPE_STREAM_BUFFER;
+    GMIME_TYPE_STREAM_MEM;
+    GMIME_TYPE_STREAM_NULL;
+
+    /* Register our types to avoid possible race conditions. See
+       output of "valgrind --tool=helgrind --log-file=balsa.log balsa"
+       Mailbox type registration is needed also for
+       libbalsa_mailbox_new_from_config() to work. */
     LIBBALSA_TYPE_MAILBOX_LOCAL;
     LIBBALSA_TYPE_MAILBOX_POP3;
     LIBBALSA_TYPE_MAILBOX_IMAP;
     LIBBALSA_TYPE_MAILBOX_MBOX;
     LIBBALSA_TYPE_MAILBOX_MH;
     LIBBALSA_TYPE_MAILBOX_MAILDIR;
+    LIBBALSA_TYPE_MESSAGE;
 
     LIBBALSA_TYPE_ADDRESS_BOOK_VCARD;
     LIBBALSA_TYPE_ADDRESS_BOOK_EXTERN;
@@ -558,7 +579,8 @@ ask_cert_real(void *data)
     label = gtk_label_new(str->str);
     g_string_free(str, TRUE);
     gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),
+    gtk_box_pack_start(GTK_BOX
+                       (gtk_dialog_get_content_area(GTK_DIALOG(dialog))),
                        label, TRUE, TRUE, 1);
     gtk_widget_show(label);
 
@@ -619,7 +641,12 @@ gboolean
 libbalsa_abort_on_timeout(const char *host)
 {  /* It appears not to be entirely thread safe... Some locks do not
       get released as they should be. */
-    return libbalsa_ask(ask_timeout_real, (void*)host) != 0; 
+    char *hostname;
+
+    hostname = g_alloca (strlen (host) + 1);
+    strcpy (hostname, host);
+    
+    return libbalsa_ask(ask_timeout_real, hostname) != 0; 
 }
 
 
@@ -742,7 +769,7 @@ libbalsa_threads_leave(void)
     self = pthread_self();
 
     if (libbalsa_threads_lock == 0 || self != libbalsa_threads_id) {
-	g_warning("Not holding gdk lock!!!");
+        g_warning("%s: Not holding gdk lock!!!", __func__);
 	return;
     }
 
@@ -826,9 +853,9 @@ libbalsa_get_image_from_face_header(const gchar * content, GError ** err)
     GtkWidget *image = NULL;
 
     stream = g_mime_stream_mem_new();
-    stream_filter = g_mime_stream_filter_new_with_stream(stream);
+    stream_filter = g_mime_stream_filter_new(stream);
 
-    filter = g_mime_filter_basic_new_type(GMIME_FILTER_BASIC_BASE64_DEC);
+    filter = g_mime_filter_basic_new(GMIME_CONTENT_ENCODING_BASE64, FALSE);
     g_mime_stream_filter_add(GMIME_STREAM_FILTER(stream_filter), filter);
     g_object_unref(filter);
 
@@ -929,12 +956,15 @@ libbalsa_get_image_from_x_face_header(const gchar * content, GError ** err)
 GtkWidget *
 libbalsa_source_view_new(gboolean highlight_phrases, GdkColor *q_colour)
 {
+    GtkSourceBuffer *sbuffer;
+    GtkWidget *sview;
+
+#if (HAVE_GTKSOURCEVIEW == 1)
+
     GtkTextTag * text_tag;
     GtkSourceTagStyle *tag_style;
     GtkSourceTagTable *tag_table;
     GSList *tag_list;
-    GtkSourceBuffer *sbuffer;
-    GtkWidget *sview;
 
     /* create the tag table */
     tag_list = NULL;
@@ -1007,6 +1037,59 @@ libbalsa_source_view_new(gboolean highlight_phrases, GdkColor *q_colour)
     gtk_source_buffer_set_highlight(sbuffer, highlight_phrases || q_colour);
     gtk_source_buffer_set_check_brackets(sbuffer, FALSE);
 
+#else /* (HAVE_GTKSOURCEVIEW == 1) */
+
+    static GtkSourceLanguageManager * lm = NULL;
+    static GtkSourceStyleScheme * scheme = NULL;
+    static GtkSourceLanguage * src_lang = NULL;
+
+    /* initialise the source language manager if necessary */
+    if (!lm) {
+	const gchar * const * lm_dpaths;
+
+	if ((lm = gtk_source_language_manager_new()) &&
+	    (lm_dpaths = gtk_source_language_manager_get_search_path(lm))) {
+	    gchar ** lm_rpaths;
+	    gint n;
+
+	    /* add the balsa share path to the language manager's paths - we
+	     * cannot simply replace it as it still wants to see the
+	     * RelaxNG schema... */
+	    for (n = 0; lm_dpaths[n]; n++);
+	    lm_rpaths = g_new0(gchar *, n + 2);
+	    for (n = 0; lm_dpaths[n]; n++)
+		lm_rpaths[n] = g_strdup(lm_dpaths[n]);
+	    lm_rpaths[n] = g_strdup(BALSA_DATA_PREFIX "/gtksourceview-2.0");
+	    gtk_source_language_manager_set_search_path(lm, lm_rpaths);
+	    g_strfreev(lm_rpaths);
+
+	    /* try to load the language */
+	    if ((src_lang =
+		 gtk_source_language_manager_get_language(lm, "balsa"))) {
+		GtkSourceStyleSchemeManager *smgr =
+		    gtk_source_style_scheme_manager_new();
+		gchar * sm_paths[] = {
+		    BALSA_DATA_PREFIX "/gtksourceview-2.0",
+		    NULL };
+	    
+		/* try to load the colouring scheme */
+		gtk_source_style_scheme_manager_set_search_path(smgr, sm_paths);
+		scheme = gtk_source_style_scheme_manager_get_scheme(smgr, "balsa-mail");
+	    }
+	}
+    }
+
+    /* create a new buffer and set the language and scheme */
+    sbuffer = gtk_source_buffer_new(NULL);
+    if (src_lang)
+	gtk_source_buffer_set_language(sbuffer, src_lang);
+    if (scheme)
+	gtk_source_buffer_set_style_scheme(sbuffer, scheme);
+    gtk_source_buffer_set_highlight_syntax(sbuffer, TRUE);
+    gtk_source_buffer_set_highlight_matching_brackets(sbuffer, FALSE);
+
+#endif /* (HAVE_GTKSOURCEVIEW == 1) */
+
     /* create & return the source view */
     sview = gtk_source_view_new_with_buffer(sbuffer);
     g_object_unref(sbuffer);
