@@ -20,7 +20,9 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -34,7 +36,7 @@
 #include "filter-funcs.h"
 #include "mailbox-filter.h"
 #include "misc.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 
 
 static LibBalsaMailboxClass *parent_class = NULL;
@@ -437,11 +439,7 @@ libbalsa_mailbox_local_load_config(LibBalsaMailbox * mailbox,
  */
 
 typedef struct {
-#if GLIB_CHECK_VERSION(2, 8, 0)
     GArray * array;
-#else                           /* GLIB_CHECK_VERSION(2, 8, 0) */
-    int fd;
-#endif                          /* GLIB_CHECK_VERSION(2, 8, 0) */
     guint (*fileno)(LibBalsaMailboxLocal * local, guint msgno);
     LibBalsaMailboxLocal *local;
 } LibBalsaMailboxLocalSaveTreeInfo;
@@ -474,11 +472,7 @@ lbm_local_save_tree_item(guint msgno, guint a,
         info.value.parent = a;
     }
 
-#if GLIB_CHECK_VERSION(2, 8, 0)
     return g_array_append_val(save_info->array, info) == NULL;
-#else                           /* GLIB_CHECK_VERSION(2, 8, 0) */
-    return write(save_info->fd, &info, sizeof info) != sizeof info;
-#endif                          /* GLIB_CHECK_VERSION(2, 8, 0) */
 }
 
 static gboolean
@@ -512,11 +506,7 @@ lbm_local_save_tree(LibBalsaMailboxLocal * local)
     LibBalsaMailbox *mailbox = LIBBALSA_MAILBOX(local);
     gchar *filename;
     LibBalsaMailboxLocalSaveTreeInfo save_info;
-#if GLIB_CHECK_VERSION(2, 8, 0)
-    GError *err;
-#else                           /* GLIB_CHECK_VERSION(2, 8, 0) */
-    gchar *template;
-#endif                          /* GLIB_CHECK_VERSION(2, 8, 0) */
+    GError *err = NULL;
 
     if (!mailbox->msg_tree || !mailbox->msg_tree_changed)
         return;
@@ -536,33 +526,16 @@ lbm_local_save_tree(LibBalsaMailboxLocal * local)
 
     save_info.fileno = LIBBALSA_MAILBOX_LOCAL_GET_CLASS(local)->fileno;
     save_info.local = local;
-#if GLIB_CHECK_VERSION(2, 8, 0)
     save_info.array =
         g_array_new(FALSE, FALSE, sizeof(LibBalsaMailboxLocalTreeInfo));
     lbm_local_save_tree_item(0, libbalsa_mailbox_get_total(mailbox),
                              &save_info);
-#else                           /* GLIB_CHECK_VERSION(2, 8, 0) */
-    template = g_strconcat(filename, ":XXXXXX", NULL);
-    save_info.fd = g_mkstemp(template);
-    if (save_info.fd < 0
-        || lbm_local_save_tree_item(0, libbalsa_mailbox_get_total(mailbox),
-                                    &save_info)) {
-        libbalsa_information(LIBBALSA_INFORMATION_WARNING,
-                             _("Failed to create temporary file \"%s\": %s"),
-                             template, strerror(errno));
-        g_free(template);
-        g_free(filename);
-        return;
-    }
-#endif                          /* GLIB_CHECK_VERSION(2, 8, 0) */
 
     /* Pre-order is required for the file to be created correctly. */
     g_node_traverse(mailbox->msg_tree, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
                     (GNodeTraverseFunc) lbm_local_save_tree_func,
                     &save_info);
 
-#if GLIB_CHECK_VERSION(2, 8, 0)
-    err = NULL;
     if (!g_file_set_contents(filename, save_info.array->data,
                              save_info.array->len *
                              sizeof(LibBalsaMailboxLocalTreeInfo), &err)) {
@@ -572,17 +545,6 @@ lbm_local_save_tree(LibBalsaMailboxLocal * local)
         g_error_free(err);
     }
     g_array_free(save_info.array, TRUE);
-#else                           /* GLIB_CHECK_VERSION(2, 8, 0) */
-    if (close(save_info.fd) != 0
-        || (unlink(filename) != 0 && errno != ENOENT)
-        || libbalsa_safe_rename(template, filename) != 0)
-        libbalsa_information(LIBBALSA_INFORMATION_WARNING,
-                             _("Failed to save cache file \"%s\": %s.  "
-                               "New version saved as \"%s\""),
-                             filename, strerror(errno), template);
-
-    g_free(template);
-#endif                          /* GLIB_CHECK_VERSION(2, 8, 0) */
     g_free(filename);
 }
 
@@ -592,6 +554,7 @@ lbm_local_restore_tree(LibBalsaMailboxLocal * local, guint * total)
     LibBalsaMailbox *mailbox = LIBBALSA_MAILBOX(local);
     gchar *filename;
     gchar *name;
+    struct stat st;
     gchar *contents;
     gsize length;
     GError *err = NULL;
@@ -603,7 +566,8 @@ lbm_local_restore_tree(LibBalsaMailboxLocal * local, guint * total)
     name = mailbox->name ? g_strdup(mailbox->name) :
         g_path_get_basename(libbalsa_mailbox_local_get_path(local));
 
-    if (!g_file_test(filename, G_FILE_TEST_IS_REGULAR)) {
+    if (stat(filename, &st) < 0
+        || st.st_mtime < libbalsa_mailbox_get_mtime(mailbox)) {
         /* No error, but we return FALSE so the caller can grab all the
          * message info needed to rethread from scratch. */
         if (libbalsa_mailbox_total_messages(mailbox) > 0)
@@ -843,7 +807,6 @@ message_match_real(LibBalsaMailbox *mailbox, guint msgno,
     LibBalsaMessage *message = NULL;
     gboolean match = FALSE;
     gboolean is_refed = FALSE;
-    gchar *str;
     LibBalsaMailboxIndexEntry *entry =
         g_ptr_array_index(mailbox->mindex, msgno-1);
     LibBalsaMailboxLocalInfo *info =
@@ -855,6 +818,8 @@ message_match_real(LibBalsaMailbox *mailbox, guint msgno,
      * fetch it here, and that will also populate entry and info. */
     if (!entry || !info) {
         message = libbalsa_mailbox_get_message(mailbox, msgno);
+        if (!message)
+            return FALSE;
         libbalsa_mailbox_local_cache_message(local, msgno, message);
         entry = g_ptr_array_index(mailbox->mindex, msgno-1);
         info  = g_ptr_array_index(local->threading_info, msgno - 1);
@@ -867,30 +832,36 @@ message_match_real(LibBalsaMailbox *mailbox, guint msgno,
 
     switch (cond->type) {
     case CONDITION_STRING:
-        if (CONDITION_CHKMATCH
-            (cond, (CONDITION_MATCH_CC | CONDITION_MATCH_BODY))) {
+        if (CONDITION_CHKMATCH(cond, (CONDITION_MATCH_TO |
+                                      CONDITION_MATCH_CC |
+                                      CONDITION_MATCH_BODY))) {
             if (!message)
                 message = libbalsa_mailbox_get_message(mailbox, msgno);
+            if (!message)
+                return FALSE;
             is_refed = libbalsa_message_body_ref(message, FALSE, FALSE);
             if (!is_refed) {
                 libbalsa_information(LIBBALSA_INFORMATION_ERROR,
                                      _("Unable to load message body to "
                                        "match filter"));
-                if (message)
-                    g_object_unref(message);
+                g_object_unref(message);
                 return FALSE;   /* We don't want to match if an error occurred */
             }
         }
 
         /* do the work */
 	if (CONDITION_CHKMATCH(cond,CONDITION_MATCH_TO)) {
-            if (!message)
-                message = libbalsa_mailbox_get_message(mailbox, msgno);
-            str = internet_address_list_to_string(message->headers->to_list,
-                                                  FALSE);
-            match = libbalsa_utf8_strstr(str,cond->match.string.string);
-            g_free(str);
-            if(match) break;
+            g_assert(is_refed);
+            if (message->headers->to_list) {
+                gchar *str =
+                    internet_address_list_to_string(message->headers->
+                                                    to_list, FALSE);
+                match =
+                    libbalsa_utf8_strstr(str, cond->match.string.string);
+                g_free(str);
+                if (match)
+                    break;
+            }
 	}
         if (CONDITION_CHKMATCH(cond, CONDITION_MATCH_FROM)) {
 	    if (libbalsa_utf8_strstr(info->sender,
@@ -908,12 +879,16 @@ message_match_real(LibBalsaMailbox *mailbox, guint msgno,
 	}
 	if (CONDITION_CHKMATCH(cond,CONDITION_MATCH_CC)) {
             g_assert(is_refed);
-            str =
-                internet_address_list_to_string(message->headers->cc_list,
-                                                FALSE);
-	    match = libbalsa_utf8_strstr(str,cond->match.string.string);
-	    g_free(str);
-	    if (match) break;
+            if (message->headers->cc_list) {
+                gchar *str =
+                    internet_address_list_to_string(message->headers->
+                                                    cc_list, FALSE);
+                match =
+                    libbalsa_utf8_strstr(str, cond->match.string.string);
+                g_free(str);
+                if (match)
+                    break;
+            }
 	}
 	if (CONDITION_CHKMATCH(cond,CONDITION_MATCH_US_HEAD)) {
             if (cond->match.string.user_header) {
@@ -921,6 +896,8 @@ message_match_real(LibBalsaMailbox *mailbox, guint msgno,
 
                 if (!message)
                     message = libbalsa_mailbox_get_message(mailbox, msgno);
+                if (!message)
+                    return FALSE;
                 header =
                     libbalsa_message_get_user_header(message,
                                                      cond->match.string.
@@ -940,9 +917,7 @@ message_match_real(LibBalsaMailbox *mailbox, guint msgno,
                 g_object_unref(message);
 		return FALSE; /* We don't want to match if an error occurred */
             }
-            body =
-                content2reply(message->body_list, NULL, 
-                              0, FALSE, FALSE, NULL, NULL);
+            body = content2reply(message->body_list, NULL, 0, FALSE, FALSE);
 	    if (body) {
 		if (body->str)
                     match = libbalsa_utf8_strstr(body->str,
@@ -1029,8 +1004,13 @@ libbalsa_mailbox_local_cache_message(LibBalsaMailboxLocal * local,
     info->message_id = g_strdup(message->message_id);
     info->refs_for_threading =
         libbalsa_message_refs_for_threading(message);
-    info->sender =
-        internet_address_list_to_string(message->headers->from, FALSE);
+
+    info->sender = NULL;
+    if (message->headers->from)
+        info->sender =
+            internet_address_list_to_string(message->headers->from, FALSE);
+    if (!info->sender)
+        info->sender = g_strdup("");
 }
 
 void
@@ -1179,6 +1159,7 @@ lbm_local_update_view_filter(LibBalsaMailbox * mailbox,
     LibBalsaProgress progress = LIBBALSA_PROGRESS_INIT;
     LibBalsaMailboxSearchIter *iter_view;
     guint msgno;
+    gboolean is_flag_only = TRUE;
 
     total = libbalsa_mailbox_total_messages(mailbox);
     if (view_filter
@@ -1188,6 +1169,7 @@ lbm_local_update_view_filter(LibBalsaMailbox * mailbox,
         text = g_strdup_printf(_("Filtering %s"), mailbox->name);
         libbalsa_progress_set_text(&progress, text, total);
         g_free(text);
+        is_flag_only = FALSE;
     }
 
     iter_view = libbalsa_mailbox_search_iter_new(view_filter);
@@ -1200,7 +1182,10 @@ lbm_local_update_view_filter(LibBalsaMailbox * mailbox,
     libbalsa_progress_set_text(&progress, NULL, 0);
     libbalsa_mailbox_search_iter_free(iter_view);
 
-    lbm_local_queue_save_tree(LIBBALSA_MAILBOX_LOCAL(mailbox));
+    /* If this is not a flags-only filter, the new mailbox tree is
+     * temporary, so we don't want to save it. */
+    if (is_flag_only)
+        lbm_local_queue_save_tree(LIBBALSA_MAILBOX_LOCAL(mailbox));
 }
 
 /*
@@ -1360,7 +1345,9 @@ libbalsa_mailbox_local_get_msg_part(LibBalsaMessage *msg,
 {
     g_return_val_if_fail(part->mime_part, FALSE);
 
-    return GMIME_IS_PART(part->mime_part);
+    return GMIME_IS_PART(part->mime_part)
+        || GMIME_IS_MULTIPART(part->mime_part)
+	|| GMIME_IS_MESSAGE_PART(part->mime_part);
 }
 
 /*--------------------------------*/
@@ -1384,6 +1371,7 @@ struct _ThreadingInfo {
     GHashTable *id_table;
     GHashTable *subject_table;
     GSList *unthreaded;
+    LibBalsaMailboxThreadingType type;
 };
 typedef struct _ThreadingInfo ThreadingInfo;
 
@@ -1953,6 +1941,7 @@ lbml_threading_simple(LibBalsaMailbox * mailbox,
 	g_node_traverse(msg_tree, G_POST_ORDER, G_TRAVERSE_ALL,
 			-1, (GNodeTraverseFunc) lbml_insert_message, &ti);
 
+    ti.type = type;
     g_node_traverse(msg_tree, G_POST_ORDER, G_TRAVERSE_ALL, -1,
 		    (GNodeTraverseFunc) lbml_thread_message, &ti);
 
@@ -1984,26 +1973,33 @@ lbml_insert_message(GNode * node, ThreadingInfo * ti)
 static gboolean
 lbml_thread_message(GNode * node, ThreadingInfo * ti)
 {
-    LibBalsaMailboxLocalInfo *info;
-    GNode *parent = NULL;
-
     if (!node->parent)
         return FALSE;
 
-    info = lbml_get_info(node, ti);
+    if (ti->type == LB_MAILBOX_THREADING_FLAT) {
+        if (node->parent != ti->mailbox->msg_tree)
+            libbalsa_mailbox_unlink_and_prepend(ti->mailbox, node,
+                                                ti->mailbox->msg_tree);
+    } else {
+        LibBalsaMailboxLocalInfo *info;
+        GList *refs;
+        GNode *parent = NULL;
 
-    if (info) {
-        GList *refs = info->refs_for_threading;
+        info = lbml_get_info(node, ti);
+        if (!info)
+            return FALSE;
+
+        refs = info->refs_for_threading;
         if (refs)
             parent = g_hash_table_lookup(ti->id_table,
                                          g_list_last(refs)->data);
-    }
 
-    if (!parent)
-        parent = ti->mailbox->msg_tree;
-    if (parent != node->parent && parent != node
-        && !g_node_is_ancestor(node, parent))
-        libbalsa_mailbox_unlink_and_prepend(ti->mailbox, node, parent);
+        if (!parent)
+            parent = ti->mailbox->msg_tree;
+        if (parent != node->parent && parent != node
+            && !g_node_is_ancestor(node, parent))
+            libbalsa_mailbox_unlink_and_prepend(ti->mailbox, node, parent);
+    }
 
     return FALSE;
 }
@@ -2126,8 +2122,8 @@ lbm_local_sync_queue(LibBalsaMailboxLocal * local)
 static void
 lbm_local_sort(LibBalsaMailbox * mailbox, GArray *sort_array)
 {
-    lbm_local_queue_save_tree(LIBBALSA_MAILBOX_LOCAL(mailbox));
     LIBBALSA_MAILBOX_CLASS(parent_class)->sort(mailbox, sort_array);
+    lbm_local_queue_save_tree(LIBBALSA_MAILBOX_LOCAL(mailbox));
 }
 
 #define FLAGS_REALLY_DIFFER(flags0, flags1) \
@@ -2145,7 +2141,7 @@ libbalsa_mailbox_local_messages_change_flags(LibBalsaMailbox * mailbox,
         LIBBALSA_MAILBOX_LOCAL_GET_CLASS(local)->get_info;
     guint i;
     guint changed = 0;
-
+    libbalsa_lock_mailbox(mailbox);
     for (i = 0; i < msgnos->len; i++) {
         guint msgno = g_array_index(msgnos, guint, i);
         LibBalsaMailboxLocalMessageInfo *msg_info;
@@ -2179,6 +2175,7 @@ libbalsa_mailbox_local_messages_change_flags(LibBalsaMailbox * mailbox,
         mailbox->unread_messages +=
             is_unread_undeleted - was_unread_undeleted;
     }
+    libbalsa_unlock_mailbox(mailbox);
 
     if (changed > 0) {
         libbalsa_mailbox_set_unread_messages_flag(mailbox,
