@@ -23,6 +23,8 @@
 #ifndef __LIBBALSA_MAILBOX_LOCAL_H__
 #define __LIBBALSA_MAILBOX_LOCAL_H__
 
+#include "libbalsa.h"
+
 #define LIBBALSA_TYPE_MAILBOX_LOCAL \
     (libbalsa_mailbox_local_get_type())
 #define LIBBALSA_MAILBOX_LOCAL(obj) \
@@ -44,6 +46,19 @@ GType libbalsa_mailbox_local_get_type(void);
 typedef struct _LibBalsaMailboxLocal LibBalsaMailboxLocal;
 typedef struct _LibBalsaMailboxLocalClass LibBalsaMailboxLocalClass;
 
+struct _LibBalsaMailboxLocalPool {
+    LibBalsaMessage * message;
+    guint pool_seqno;
+};
+typedef struct _LibBalsaMailboxLocalPool LibBalsaMailboxLocalPool;
+#define LBML_POOL_SIZE 32
+
+struct _LibBalsaMailboxLocalMessageInfo {
+    LibBalsaMessageFlag flags;          /* May have pseudo-flags */
+    LibBalsaMessage *message;
+};
+typedef struct _LibBalsaMailboxLocalMessageInfo LibBalsaMailboxLocalMessageInfo;
+
 struct _LibBalsaMailboxLocal {
     LibBalsaMailbox mailbox;
 
@@ -53,40 +68,50 @@ struct _LibBalsaMailboxLocal {
     guint sync_cnt; /* we do not want to rely on the time of last sync since
                      * some sync can be faster than others. Instead, we
                      * average the syncing time for mailbox. */
-    GArray *threading_info;
+    guint thread_id;    /* id of the idle mailbox thread job */
+    guint save_tree_id; /* id of the idle mailbox save-tree job */
+    GPtrArray *threading_info;
+    LibBalsaMailboxLocalPool message_pool[LBML_POOL_SIZE];
+    guint pool_seqno;
 };
 
 struct _LibBalsaMailboxLocalClass {
     LibBalsaMailboxClass klass;
 
-    LibBalsaMessage *(*load_message)(LibBalsaMailbox *mb, guint msgno);
-    void (*remove_files)(LibBalsaMailboxLocal *mb);
+    gint (*check_files)(const gchar * path, gboolean create);
+    void (*set_path)(LibBalsaMailboxLocal * local, const gchar * path);
+    void (*remove_files)(LibBalsaMailboxLocal * local);
+    guint (*fileno)(LibBalsaMailboxLocal * local, guint msgno);
+    LibBalsaMailboxLocalMessageInfo *(*get_info)(LibBalsaMailboxLocal * local,
+                                                 guint msgno);
 };
 
 GObject *libbalsa_mailbox_local_new(const gchar * path, gboolean create);
 gint libbalsa_mailbox_local_set_path(LibBalsaMailboxLocal * mailbox,
-				     const gchar * path);
+				     const gchar * path, gboolean create);
+void libbalsa_mailbox_local_set_threading_info(LibBalsaMailboxLocal *
+                                               local);
 
 #define libbalsa_mailbox_local_get_path(mbox) \
 	((const gchar *) (LIBBALSA_MAILBOX(mbox))->url+7)
 
 void libbalsa_mailbox_local_load_messages(LibBalsaMailbox * mailbox,
 					  guint last_msgno);
+void libbalsa_mailbox_local_cache_message(LibBalsaMailboxLocal * local,
+                                          guint msgno,
+                                          LibBalsaMessage * message);
 void libbalsa_mailbox_local_msgno_removed(LibBalsaMailbox * mailbox,
 					  guint msgno);
 void libbalsa_mailbox_local_remove_files(LibBalsaMailboxLocal *mailbox);
 
 /* Helpers for maildir and mh. */
-GMimeMessage *_libbalsa_mailbox_local_get_mime_message(LibBalsaMailbox *
+GMimeMessage *libbalsa_mailbox_local_get_mime_message(LibBalsaMailbox *
+						      mailbox,
+						      const gchar * name1,
+						      const gchar * name2);
+GMimeStream *libbalsa_mailbox_local_get_message_stream(LibBalsaMailbox *
 						       mailbox,
 						       const gchar * name1,
 						       const gchar * name2);
-GMimeStream *_libbalsa_mailbox_local_get_message_stream(LibBalsaMailbox *
-							mailbox,
-							const gchar * name1,
-							const gchar * name2);
-
-/* Queued sync. */
-void libbalsa_mailbox_local_queue_sync(LibBalsaMailboxLocal * local);
 
 #endif				/* __LIBBALSA_MAILBOX_LOCAL_H__ */
