@@ -20,9 +20,10 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
 
-#define _XOPEN_SOURCE 500
 
 #include <gmime/gmime-stream-fs.h>
 
@@ -44,7 +45,11 @@
 /* for mx_lock_file and mx_unlock_file */
 #include "mailbackend.h"
 #include "mime-stream-shared.h"
-#include "i18n.h"
+#include "missing.h"
+
+#include <glib/gi18n.h>
+
+/* #define DEBUG_SEEK TRUE */
 
 struct message_info {
     LibBalsaMailboxLocalMessageInfo local_info;
@@ -65,10 +70,12 @@ static LibBalsaMailboxLocalClass *parent_class = NULL;
 
 static void libbalsa_mailbox_mbox_class_init(LibBalsaMailboxMboxClass *klass);
 static void libbalsa_mailbox_mbox_init(LibBalsaMailboxMbox * mailbox);
+static void libbalsa_mailbox_mbox_dispose(GObject * object);
 
 static GMimeStream *libbalsa_mailbox_mbox_get_message_stream(LibBalsaMailbox *
 							     mailbox,
-							     guint msgno);
+							     guint msgno,
+							     gboolean peek);
 static gint lbm_mbox_check_files(const gchar * path, gboolean create);
 static void libbalsa_mailbox_mbox_remove_files(LibBalsaMailboxLocal *mailbox);
 
@@ -88,11 +95,10 @@ static gboolean
 libbalsa_mailbox_mbox_fetch_message_structure(LibBalsaMailbox * mailbox,
                                               LibBalsaMessage * message,
                                               LibBalsaFetchFlag flags);
-static gboolean libbalsa_mailbox_mbox_add_message(LibBalsaMailbox *
-                                                  mailbox,
-                                                  GMimeStream * stream,
-                                                  LibBalsaMessageFlag
-                                                  flags, GError ** err);
+static guint libbalsa_mailbox_mbox_add_messages(LibBalsaMailbox *mailbox,
+						LibBalsaAddMessageIterator m,
+						void *m_arg,
+						GError ** err);
 static guint
 libbalsa_mailbox_mbox_total_messages(LibBalsaMailbox * mailbox);
 #if BALSA_USE_THREADS
@@ -163,7 +169,7 @@ libbalsa_mailbox_mbox_class_init(LibBalsaMailboxMboxClass * klass)
 	libbalsa_mailbox_mbox_close_mailbox;
     libbalsa_mailbox_class->fetch_message_structure =
 	libbalsa_mailbox_mbox_fetch_message_structure;
-    libbalsa_mailbox_class->add_message = libbalsa_mailbox_mbox_add_message;
+    libbalsa_mailbox_class->add_messages = libbalsa_mailbox_mbox_add_messages;
     libbalsa_mailbox_class->total_messages =
 	libbalsa_mailbox_mbox_total_messages;
 #if BALSA_USE_THREADS
@@ -175,6 +181,7 @@ libbalsa_mailbox_mbox_class_init(LibBalsaMailboxMboxClass * klass)
 	libbalsa_mailbox_mbox_remove_files;
 
     libbalsa_mailbox_local_class->get_info = lbm_mbox_get_info;
+    object_class->dispose = libbalsa_mailbox_mbox_dispose;
 }
 
 
@@ -184,6 +191,13 @@ libbalsa_mailbox_mbox_init(LibBalsaMailboxMbox * mbox)
 {
 }
 
+static void
+libbalsa_mailbox_mbox_dispose(GObject * object)
+{
+    if(MAILBOX_OPEN(LIBBALSA_MAILBOX(object)))
+	libbalsa_mailbox_mbox_close_mailbox(LIBBALSA_MAILBOX(object), FALSE);
+}
+
 static gint
 lbm_mbox_check_files(const gchar * path, gboolean create)
 {
@@ -238,11 +252,22 @@ static gboolean
 lbm_mbox_stream_seek_to_message(GMimeStream * stream, off_t offset)
 {
     char buffer[5];
+    ssize_t nread = 0;
     gboolean retval;
 
     retval = g_mime_stream_seek(stream, offset, GMIME_STREAM_SEEK_SET) >= 0
-        && g_mime_stream_read(stream, buffer, sizeof buffer) == sizeof buffer
+        && (nread = g_mime_stream_read(stream, buffer, sizeof buffer))
+        == sizeof buffer
         && strncmp("From ", buffer, 5) == 0;
+#if DEBUG_SEEK
+    if (!retval) {
+        if (nread == sizeof buffer)
+            --nread;
+        buffer[nread] = 0;
+        g_print("%s at %ld failed: read %ld chars, saw \"%s\"\n", __func__,
+                (long) offset, (long) nread, buffer);
+    }
+#endif
 
     g_mime_stream_seek(stream, offset, GMIME_STREAM_SEEK_SET);
 
@@ -273,7 +298,7 @@ message_info_from_msgno(LibBalsaMailboxMbox * mbox, guint msgno)
 
 static GMimeStream *
 libbalsa_mailbox_mbox_get_message_stream(LibBalsaMailbox * mailbox,
-                                         guint msgno)
+                                         guint msgno, gboolean peek)
 {
     LibBalsaMailboxMbox *mbox;
     struct message_info *msg_info;
@@ -331,10 +356,10 @@ static void mbox_unlock(LibBalsaMailbox * mailbox, GMimeStream *stream)
  */
 static void
 lbm_mbox_header_cb(GMimeParser * parser, const char *header,
-                   const char *value, off_t offset,
-                   struct message_info **msg_info_p)
+                   const char *value, gint64 offset,
+                   gpointer user_data)
 {
-    struct message_info *msg_info = *msg_info_p;
+    struct message_info *msg_info = *(struct message_info **) user_data;
 
     if (g_ascii_strcasecmp(header, "Status") == 0 && msg_info->status < 0)
         msg_info->status = offset;
@@ -369,9 +394,9 @@ static void
 lbm_mbox_save(LibBalsaMailboxMbox * mbox)
 {
     gchar *filename;
-#if GLIB_CHECK_VERSION(2, 8, 0) && !defined(__APPLE__)
+#if !defined(__APPLE__)
     GError *err = NULL;
-#endif                          /* GLIB_CHECK_VERSION(2, 8, 0) */
+#endif                          /* !defined(__APPLE__) */
 
     if (!mbox->messages_info_changed)
         return;
@@ -385,10 +410,10 @@ lbm_mbox_save(LibBalsaMailboxMbox * mbox)
             g_array_sized_new(FALSE, FALSE, sizeof(struct message_info), 
                               mbox->msgno_2_msg_info->len);
         guint msgno;
-#if !GLIB_CHECK_VERSION(2, 8, 0) || defined(__APPLE__)
+#if defined(__APPLE__)
         gchar *template;
         gint fd;
-#endif                          /* GLIB_CHECK_VERSION(2, 8, 0) */
+#endif                          /* !defined(__APPLE__) */
 
         for (msgno = 1; msgno <= mbox->msgno_2_msg_info->len; msgno++) {
             struct message_info *msg_info =
@@ -396,7 +421,7 @@ lbm_mbox_save(LibBalsaMailboxMbox * mbox)
             g_array_append_val(messages_info, *msg_info);
         }
 
-#if GLIB_CHECK_VERSION(2, 8, 0) && !defined(__APPLE__)
+#if !defined(__APPLE__)
         if (!g_file_set_contents(filename, messages_info->data,
                                  messages_info->len
                                  * sizeof(struct message_info), &err)) {
@@ -405,7 +430,7 @@ lbm_mbox_save(LibBalsaMailboxMbox * mbox)
                                  filename, err->message);
             g_error_free(err);
         }
-#else                           /* GLIB_CHECK_VERSION(2, 8, 0) */
+#else                           /* !defined(__APPLE__) */
         template = g_strconcat(filename, ":XXXXXX", NULL);
         fd = g_mkstemp(template);
         if (fd < 0 || write(fd, messages_info->data,
@@ -429,7 +454,7 @@ lbm_mbox_save(LibBalsaMailboxMbox * mbox)
                                    "New version saved as \"%s\""),
                                  filename, strerror(errno), template);
         g_free(template);
-#endif                          /* GLIB_CHECK_VERSION(2, 8, 0) */
+#endif                          /* !defined(__APPLE__) */
         g_array_free(messages_info, TRUE);
     } else if (unlink(filename) < 0)
         libbalsa_information(LIBBALSA_INFORMATION_WARNING,
@@ -461,7 +486,6 @@ parse_mailbox(LibBalsaMailboxMbox * mbox)
     g_mime_parser_set_respect_content_length(gmime_parser, TRUE);
     g_mime_parser_set_header_regex(gmime_parser,
                                    "^Status|^X-Status|^MIME-Version",
-				   (GMimeParserHeaderRegexFunc)
 				   lbm_mbox_header_cb, &msg_info_p);
 
     libbalsa_mailbox_local_set_threading_info(local);
@@ -512,13 +536,13 @@ parse_mailbox(LibBalsaMailboxMbox * mbox)
         msg->mailbox = LIBBALSA_MAILBOX(mbox);
         msg->msgno = ++msgno;
 	/* We must drop the mime-stream lock to call
+         * libbalsa_mailbox_local_cache_message, which calls
 	 * libbalsa_mailbox_cache_message(), as it may grab the
 	 * gdk lock to emit gtk signals; we save and restore the current
 	 * stream position, in case someone changes it while we're not
 	 * holding the lock. */
         offset = g_mime_stream_tell(mbox->gmime_stream);
         libbalsa_mime_stream_shared_unlock(mbox->gmime_stream);
-        libbalsa_mailbox_cache_message(LIBBALSA_MAILBOX(mbox), msgno, msg);
         libbalsa_mailbox_local_cache_message(local, msgno, msg);
         libbalsa_mime_stream_shared_lock(mbox->gmime_stream);
         g_mime_stream_seek(mbox->gmime_stream, offset, GMIME_STREAM_SEEK_SET);
@@ -540,6 +564,7 @@ free_message_info(struct message_info *msg_info)
                                      (gpointer) & msg_info->local_info.message);
 	msg_info->local_info.message = NULL;
     }
+    g_free(msg_info);
 }
 
 static void
@@ -551,7 +576,6 @@ free_messages_info(GPtrArray * msgno_2_msg_info)
         struct message_info *msg_info =
             g_ptr_array_index(msgno_2_msg_info, i);
         free_message_info(msg_info);
-        g_free(msg_info);
     }
     g_ptr_array_free(msgno_2_msg_info, TRUE);
 }
@@ -560,20 +584,23 @@ static void
 lbm_mbox_restore(LibBalsaMailboxMbox * mbox)
 {
     gchar *filename;
+    struct stat st;
     gchar *contents;
     gsize length;
-    gboolean tmp;
     off_t end;
     struct message_info *msg_info;
     GMimeStream *mbox_stream;
 
     filename = lbm_mbox_get_cache_filename(mbox);
-    tmp = g_file_get_contents(filename, &contents, &length, NULL);
-    g_free(filename);
-
-    if (!tmp)
-        /* Nonexistent file is no error. */
+    if (stat(filename, &st) < 0 
+        || st.st_mtime < libbalsa_mailbox_get_mtime(LIBBALSA_MAILBOX(mbox))
+        || !g_file_get_contents(filename, &contents, &length, NULL)) {
+        /* No cache file, stale cache, or read error. */
+        g_free(filename);
         return;
+    }
+
+    g_free(filename);
 
     if (length < sizeof(struct message_info)) {
         /* Error: file always contains at least one record. */
@@ -697,6 +724,10 @@ libbalsa_mailbox_mbox_open(LibBalsaMailbox * mailbox, GError **err)
     }
 
     mbox->size = st.st_size;
+#if DEBUG_SEEK
+    g_print("%s %s set size from stat %d\n", __func__, mailbox->name,
+            mbox->size);
+#endif
     libbalsa_mailbox_set_mtime(mailbox, st.st_mtime);
     mbox->gmime_stream = gmime_stream;
 
@@ -967,6 +998,10 @@ libbalsa_mailbox_mbox_check(LibBalsaMailbox * mailbox)
 	/* First check--just cache the mtime and size. */
         libbalsa_mailbox_set_mtime(mailbox, st.st_mtime);
 	mbox->size = st.st_size;
+#if DEBUG_SEEK
+        g_print("%s %s set size from stat %d\n", __func__, mailbox->name,
+                mbox->size);
+#endif
 	return;
     }
     if (st.st_mtime == mtime && st.st_size == mbox->size)
@@ -980,6 +1015,10 @@ libbalsa_mailbox_mbox_check(LibBalsaMailbox * mailbox)
 								 path));
 	/* Cache the file size, so we don't check the next time. */
 	mbox->size = st.st_size;
+#if DEBUG_SEEK
+        g_print("%s %s set size from stat %d\n", __func__, mailbox->name,
+                mbox->size);
+#endif
 	return;
     }
 
@@ -1007,6 +1046,23 @@ libbalsa_mailbox_mbox_check(LibBalsaMailbox * mailbox)
     /* If Balsa appended a message, it was prefixed with "\nFrom ", so
      * we first check one byte beyond the end of the last message: */
     start = mbox->size + 1;
+#if DEBUG_SEEK
+    g_print("%s %s looking where to start parsing.\n",
+              __func__, mailbox->name);
+    if (!lbm_mbox_stream_seek_to_message(mbox_stream, start)) {
+        g_print(" did not find a message at offset %ld\n", (long) start);
+        --start;
+        if (lbm_mbox_stream_seek_to_message(mbox_stream, start))
+            g_print(" found a message at offset %ld\n", (long) start);
+        else
+            g_print(" did not find a message at offset %ld\n", (long) start);
+    } else
+        g_print(" found a message at offset %ld\n", (long) start);
+#else
+    if (!lbm_mbox_stream_seek_to_message(mbox_stream, start))
+        /* Sometimes we seem to be off by 1: */
+        --start;
+#endif
 
     while ((msgno = mbox->msgno_2_msg_info->len) > 0) {
 	off_t offset;
@@ -1017,6 +1073,9 @@ libbalsa_mailbox_mbox_check(LibBalsaMailbox * mailbox)
 	     * the first new message--start parsing here. */
             break;
 
+#if DEBUG_SEEK
+        g_print(" backing up over message %d\n", msgno);
+#endif
 	/* Back up over this message and try again. */
         msg_info = message_info_from_msgno(mbox, msgno);
         start = msg_info->start;
@@ -1048,6 +1107,10 @@ libbalsa_mailbox_mbox_check(LibBalsaMailbox * mailbox)
 #endif
     parse_mailbox(mbox);
     mbox->size = g_mime_stream_tell(mbox_stream);
+#if DEBUG_SEEK
+    g_print("%s %s set size from tell %d\n", __func__, mailbox->name,
+            mbox->size);
+#endif
     libbalsa_mime_stream_shared_unlock(mbox_stream);
     mbox_unlock(mailbox, mbox_stream);
     libbalsa_mailbox_local_load_messages(mailbox, msgno);
@@ -1065,16 +1128,16 @@ libbalsa_mailbox_mbox_close_mailbox(LibBalsaMailbox * mailbox,
     if (mbox->msgno_2_msg_info->len != len)
         libbalsa_mailbox_changed(mailbox);
 
+    if (LIBBALSA_MAILBOX_CLASS(parent_class)->close_mailbox)
+        LIBBALSA_MAILBOX_CLASS(parent_class)->close_mailbox(mailbox,
+                                                            expunge);
+
+    /* Now it's safe to close the stream and free the message info. */
     if (mbox->gmime_stream) {
         g_object_unref(mbox->gmime_stream);
         mbox->gmime_stream = NULL;
     }
 
-    if (LIBBALSA_MAILBOX_CLASS(parent_class)->close_mailbox)
-        LIBBALSA_MAILBOX_CLASS(parent_class)->close_mailbox(mailbox,
-                                                            expunge);
-
-    /* Now it's safe to free the message info. */
     free_messages_info(mbox->msgno_2_msg_info);
     mbox->msgno_2_msg_info = NULL;
 }
@@ -1087,7 +1150,7 @@ lbm_mbox_get_mime_message(LibBalsaMailbox * mailbox,
     GMimeParser *parser;
     GMimeMessage *mime_message;
 
-    stream = libbalsa_mailbox_mbox_get_message_stream(mailbox, msgno);
+    stream = libbalsa_mailbox_mbox_get_message_stream(mailbox, msgno, TRUE);
     if (!stream)
 	return NULL;
     libbalsa_mime_stream_shared_lock(stream);
@@ -1541,6 +1604,10 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
         g_warning("mbox_sync: message not in expected position.\n");
     else if (g_mime_stream_write_to_stream(temp_stream, mbox_stream) != -1) {
         mbox->size = g_mime_stream_tell(mbox_stream);
+#if DEBUG_SEEK
+        g_print("%s %s set size from tell %d\n", __func__, mailbox->name,
+                mbox->size);
+#endif
         if (ftruncate(GMIME_STREAM_FS(mbox_stream)->fd, mbox->size) == 0)
             save_failed = FALSE;
     }
@@ -1609,7 +1676,6 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
     g_mime_parser_set_respect_content_length(gmime_parser, TRUE);
     g_mime_parser_set_header_regex(gmime_parser,
                                    "^Status|^X-Status|^MIME-Version",
-				   (GMimeParserHeaderRegexFunc)
 				   lbm_mbox_header_cb, &msg_info);
     for (j = first; j < mbox->msgno_2_msg_info->len; ) {
 	GMimeMessage *mime_msg;
@@ -1634,10 +1700,12 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
 	}
 	if (msg_info->local_info.message)
 	    msg_info->local_info.message->msgno = j + 1;
-	j++;
 
 	msg_info->status = msg_info->x_status = msg_info->mime_version = -1;
 	mime_msg = g_mime_parser_construct_message(gmime_parser);
+        if (!mime_msg)
+            /* Try to recover */
+            continue;
         msg_info->start = g_mime_parser_get_from_offset(gmime_parser);
 
 	/* Make sure we don't have offsets for any encapsulated headers. */
@@ -1649,11 +1717,16 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
 	    msg_info->mime_version = -1;
 
 	from = g_mime_parser_get_from(gmime_parser);
+        if (!from) {
+            /* Try to recover */
+            g_object_unref(mime_msg);
+            continue;
+        }
+
         msg_info->from_len = strlen(from) + 1;
         g_free(from);
 	msg_info->end = g_mime_parser_tell(gmime_parser);
 	msg_info->orig_flags = REAL_FLAGS(msg_info->local_info.flags);
-	g_assert(mime_msg != NULL);
 	g_assert(mime_msg->mime_part != NULL);
 	if (!msg_info->local_info.message || !msg_info->local_info.message->mime_msg)
 	    g_object_unref(mime_msg);
@@ -1666,6 +1739,8 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
 	    libbalsa_message_body_set_mime_body(msg_info->local_info.message->body_list,
 						mime_msg->mime_part);
 	}
+
+	j++;
     }
     libbalsa_mime_stream_shared_unlock(mbox_stream);
     mbox->msgno_2_msg_info->len = j;
@@ -1727,7 +1802,7 @@ lbm_mbox_message_new(GMimeMessage * mime_message,
 
     message = libbalsa_message_new();
 
-    header = g_mime_message_get_header (mime_message, "Status");
+    header = g_mime_object_get_header (GMIME_OBJECT(mime_message), "Status");
     if (header) {
 	if (strchr(header, 'R') == NULL) /* not found == not READ */
 	    flags |= LIBBALSA_MESSAGE_FLAG_NEW;
@@ -1737,7 +1812,7 @@ lbm_mbox_message_new(GMimeMessage * mime_message,
 	    flags |= LIBBALSA_MESSAGE_FLAG_RECENT;
     } else
 	    flags |= LIBBALSA_MESSAGE_FLAG_NEW |  LIBBALSA_MESSAGE_FLAG_RECENT;
-    header = g_mime_message_get_header (mime_message, "X-Status");
+    header = g_mime_object_get_header (GMIME_OBJECT(mime_message), "X-Status");
     if (header) {
 	if (strchr(header, 'D') != NULL) /* found == DELETED */
 	    flags |= LIBBALSA_MESSAGE_FLAG_DELETED;
@@ -1761,45 +1836,57 @@ static void update_message_status_headers(GMimeMessage *message,
     /* Create headers with spaces in place of flags, if necessary, so we
      * can later update them in place. */
     lbm_mbox_status_hdr(flags, 2, new_header);
-    g_mime_message_set_header(message, "Status", new_header->str);
+    g_mime_object_set_header(GMIME_OBJECT(message), "Status", new_header->str);
     g_string_truncate(new_header, 0);
     lbm_mbox_x_status_hdr(flags, 3, new_header);
-    g_mime_message_set_header(message, "X-Status", new_header->str);
+    g_mime_object_set_header(GMIME_OBJECT(message), "X-Status", new_header->str);
     g_string_free(new_header, TRUE);
 }
 
+#if defined(GMIME_MAJOR_VERSION)
+# if GMIME_CHECK_VERSION(2, 6, 0)
+#  define HAVE_GMIME_2_6 TRUE
+# endif                         /* GMIME_CHECK_VERSION(2, 6, 0) */
+#endif                          /* defined(GMIME_MAJOR_VERSION) */
+
+#if !defined(HAVE_GMIME_2_6)
 /*
  * Encode text parts as quoted-printable.
  */
 static void
-lbm_mbox_prepare_object(GMimeObject * mime_part)
+lbm_mbox_prepare_object(GMimeObject * object)
 {
-    g_mime_object_remove_header(mime_part, "Content-Length");
+    g_mime_object_remove_header(object, "Content-Length");
 
-    if (GMIME_IS_MULTIPART(mime_part)) {
-        if (GMIME_IS_MULTIPART_SIGNED(mime_part)
-            || GMIME_IS_MULTIPART_ENCRYPTED(mime_part))
-            /* Do not break crypto. */
-            return;
+    if (GMIME_IS_MULTIPART(object)) {
+        /* Do not break crypto */
+        if (!(GMIME_IS_MULTIPART_SIGNED(object) ||
+              GMIME_IS_MULTIPART_ENCRYPTED(object))) {
+            GMimeMultipart *multipart = (GMimeMultipart *) object;
+            gint i, count = g_mime_multipart_get_count(multipart);
 
-        g_mime_multipart_foreach((GMimeMultipart *) mime_part,
-                                 (GMimePartFunc) lbm_mbox_prepare_object,
-                                 NULL);
-    } else if (GMIME_IS_MESSAGE_PART(mime_part))
+            for (i = 0; i < count; ++i)
+                lbm_mbox_prepare_object(g_mime_multipart_get_part
+                                        (multipart, i));
+        }
+    } else if (GMIME_IS_MESSAGE_PART(object))
         lbm_mbox_prepare_object(GMIME_OBJECT
-                                (((GMimeMessagePart *) mime_part)->
-                                 message));
-    else if (GMIME_IS_MESSAGE(mime_part))
-        lbm_mbox_prepare_object(((GMimeMessage *) mime_part)->mime_part);
-    else if (!GMIME_IS_MESSAGE_PARTIAL(mime_part)) {
-        GMimePartEncodingType encoding;
-        const GMimeContentType *mime_type;
+                                (((GMimeMessagePart *) object)->message));
+    else if (GMIME_IS_MESSAGE(object))
+        lbm_mbox_prepare_object(((GMimeMessage *) object)->mime_part);
+    else if (GMIME_IS_PART(object)) {
+        GMimePart *mime_part = (GMimePart *) object;
+        GMimeContentEncoding encoding;
+        GMimeContentType *mime_type;
 
-        encoding = g_mime_part_get_encoding(GMIME_PART(mime_part));
-        if (encoding == GMIME_PART_ENCODING_BASE64)
+        if (GMIME_IS_MESSAGE_PARTIAL(mime_part))
             return;
 
-        mime_type = g_mime_object_get_content_type(mime_part);
+        encoding = g_mime_part_get_content_encoding(mime_part);
+        if (encoding == GMIME_CONTENT_ENCODING_BASE64)
+            return;
+
+        mime_type = g_mime_object_get_content_type(object);
         if (g_mime_content_type_is_type(mime_type, "text", "plain")) {
             const gchar *format =
                 g_mime_content_type_get_parameter(mime_type, "format");
@@ -1808,10 +1895,11 @@ lbm_mbox_prepare_object(GMimeObject * mime_part)
                 return;
         }
 
-        g_mime_part_set_encoding(GMIME_PART(mime_part),
-                                 GMIME_PART_ENCODING_QUOTEDPRINTABLE);
+        g_mime_part_set_content_encoding
+            (mime_part, GMIME_CONTENT_ENCODING_QUOTEDPRINTABLE);
     }
 }
+#endif                          /* defined(HAVE_GMIME_2_6) */
 
 static GMimeObject *
 lbm_mbox_armored_object(GMimeStream * stream)
@@ -1822,7 +1910,12 @@ lbm_mbox_armored_object(GMimeStream * stream)
     parser = g_mime_parser_new_with_stream(stream);
     object = GMIME_OBJECT(g_mime_parser_construct_message(parser));
     g_object_unref(parser);
+
+#if defined(HAVE_GMIME_2_6)
+    g_mime_object_encode(object, GMIME_ENCODING_CONSTRAINT_7BIT);
+#else                           /* defined(HAVE_GMIME_2_6) */
     lbm_mbox_prepare_object(object);
+#endif                          /* defined(HAVE_GMIME_2_6) */
 
     return object;
 }
@@ -1833,10 +1926,10 @@ lbm_mbox_armored_stream(GMimeStream * stream)
     GMimeStream *fstream;
     GMimeFilter *filter;
     
-    fstream = g_mime_stream_filter_new_with_stream(stream);
+    fstream = g_mime_stream_filter_new(stream);
 
-    filter = g_mime_filter_crlf_new(GMIME_FILTER_CRLF_DECODE,
-				    GMIME_FILTER_CRLF_MODE_CRLF_ONLY);
+    filter = g_mime_filter_crlf_new(FALSE,
+				    FALSE);
     g_mime_stream_filter_add(GMIME_STREAM_FILTER(fstream), filter);
     g_object_unref(filter);
 
@@ -1957,6 +2050,27 @@ libbalsa_mailbox_mbox_add_message(LibBalsaMailbox * mailbox,
     return retval >= 0;
 }
 
+static guint
+libbalsa_mailbox_mbox_add_messages(LibBalsaMailbox * mailbox,
+				   LibBalsaAddMessageIterator msg_iterator,
+				   void *arg,
+				   GError **err)
+{
+    LibBalsaMessageFlag flag;
+    GMimeStream *stream;
+
+    guint cnt = 0;
+    while( msg_iterator(&flag, &stream, arg) ) {
+	gboolean success =
+	    libbalsa_mailbox_mbox_add_message(mailbox, stream, flag, err);
+	g_object_unref(stream);
+	if(!success)
+	    break;
+	cnt++;
+    }
+    return cnt;
+}
+
 static guint
 libbalsa_mailbox_mbox_total_messages(LibBalsaMailbox * mailbox)
 {
