@@ -20,7 +20,9 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
 
 #include <string.h>
 #include <ctype.h>
@@ -80,33 +82,34 @@ process_mime_part(LibBalsaMessage * message, LibBalsaMessageBody * body,
 	if (!res)
 	    return NULL;
 
-#ifdef HAVE_GTKHTML
+#ifdef HAVE_HTML_WIDGET
 	if (html_type) {
 	    allocated = libbalsa_html_filter(html_type, &res, allocated);
 	    libbalsa_html_to_string(&res, allocated);
 	}
-#endif /* HAVE_GTKHTML */
+#endif /* HAVE_HTML_WIDGET */
 
-	if (llen > 0) {
-            if (flow && libbalsa_message_body_is_flowed(body)) {
-                /* we're making a `format=flowed' message, and the
-                 * message we're quoting was flowed
-                 *
-                 * we'll assume it's going to the screen */
-		gboolean delsp = libbalsa_message_body_is_delsp(body);
+        if (flow && libbalsa_message_body_is_flowed(body)) {
+            /* we're making a `format=flowed' message, and the
+             * message we're quoting was flowed
+             *
+             * we'll assume it's going to the screen */
+            gboolean delsp = libbalsa_message_body_is_delsp(body);
 
-		reply =
-		    libbalsa_process_text_rfc2646(res, G_MAXINT, FALSE,
-						  TRUE,
-						  reply_prefix_str != NULL,
-						  delsp);
-                g_free(res);
-                break;
-            }
+            reply =
+                libbalsa_process_text_rfc2646(res, G_MAXINT, FALSE, TRUE,
+                                              reply_prefix_str != NULL,
+                                              delsp);
+            g_free(res);
+            break;
+        }
+
+        if (llen > 0) {
 	    if (reply_prefix_str)
 		llen -= strlen(reply_prefix_str);
 	    libbalsa_wrap_string(res, llen);
 	}
+
         if (reply_prefix_str || flow) {
 	    gchar *str, *ptr;
 	    /* prepend the prefix to all the lines */
@@ -397,12 +400,7 @@ dowrap_rfc2646(GList * list, gint width, gboolean to_screen,
                 first_word = FALSE;
                 line_break = str;
             }                   /* end of loop over words */
-            /* 
-             * make sure that a line ending in whitespace ends in an
-             * actual ' ' 
-             * */
-            if (str > start && isspace((int)str[-1]) && str[-1] != ' ')
-                g_string_append_c(result, ' ');
+
             if (*str) {         /* line separator */
                 if (to_screen || str == start)
 		    g_string_append_c(result, '\n');
@@ -487,7 +485,6 @@ libbalsa_wrap_view(GtkTextView * view, gint length)
     GtkTextBuffer *buffer = gtk_text_view_get_buffer(view);
     GtkTextTagTable *table = gtk_text_buffer_get_tag_table(buffer);
     GtkTextTag *url_tag = gtk_text_tag_table_lookup(table, "url");
-    GtkTextTag *soft_tag = gtk_text_tag_table_lookup(table, "soft");
     GtkTextIter iter;
     PangoContext *context = gtk_widget_get_pango_context(GTK_WIDGET(view));
     PangoLanguage *language = pango_context_get_language(context);
@@ -571,8 +568,7 @@ libbalsa_wrap_view(GtkTextView * view, gint length)
                 gtk_text_buffer_delete(buffer, &start, &iter);
 	    }
 
-	    gtk_text_buffer_insert_with_tags(buffer, &iter, "\n", 1,
-					     soft_tag, NULL);
+	    gtk_text_buffer_insert(buffer, &iter, "\n", 1);
 	    if (quote_string)
 		gtk_text_buffer_insert_with_tags(buffer, &iter,
 						 quote_string, -1,
@@ -667,74 +663,42 @@ is_in_url(GtkTextIter * iter, gint offset, GtkTextTag * url_tag)
  * num_paras < 0, process the whole buffer. */
 
 /* Forward references: */
-static gboolean prescanner(const gchar * p);
+static gboolean prescanner(const gchar * p, guint len);
 static void mark_urls(GtkTextBuffer * buffer, GtkTextIter * iter,
                       GtkTextTag * tag, const gchar * p);
+#if USE_GREGEX
+static GRegex *get_url_reg(void);
+#else                           /* USE_GREGEX */
 static regex_t *get_url_reg(void);
+#endif                          /* USE_GREGEX */
 
 void
 libbalsa_unwrap_buffer(GtkTextBuffer * buffer, GtkTextIter * iter,
 		       gint num_paras)
 {
     GtkTextTagTable *table = gtk_text_buffer_get_tag_table(buffer);
-    GtkTextTag *soft_tag = gtk_text_tag_table_lookup(table, "soft");
     GtkTextTag *url_tag = gtk_text_tag_table_lookup(table, "url");
 
     /* Check whether the previous line flowed into this one. */
     gtk_text_iter_set_line_offset(iter, 0);
-    if (gtk_text_iter_ends_tag(iter, soft_tag))
-	gtk_text_iter_backward_line(iter);
 
     for (; num_paras; num_paras--) {
 	gint quote_depth;
-	gint qd;
 	GtkTextIter start;
 	gchar *line;
 
 	gtk_text_iter_set_line_offset(iter, 0);
 	quote_depth = get_quote_depth(iter, NULL);
 
-	for (;;) {
-	    gchar *quote_string;
-	    gboolean stuffed;
-
-	    /* Move to the end of the line, if not there already. */
-	    if (!gtk_text_iter_ends_line(iter)
-		&& !gtk_text_iter_forward_to_line_end(iter))
-		return;
-	    /* Save this iter as the start of a possible deletion. */
-	    start = *iter;
-	    /* Move to the start of the next line. */
-	    if (!gtk_text_iter_forward_line(iter))
-		return;
-
-	    qd = get_quote_depth(iter, &quote_string);
-	    stuffed = quote_string && quote_string[qd];
-	    g_free(quote_string);
-
-	    if (gtk_text_iter_has_tag(&start, soft_tag)) {
-		if (qd != quote_depth) {
-		    /* Broken: use quote-depth-wins. */
-		    GtkTextIter end = start;
-		    gtk_text_iter_forward_to_tag_toggle(&end, soft_tag);
-		    gtk_text_buffer_remove_tag(buffer, soft_tag,
-					       &start, &end);
-		    break;
-		}
-	    } else
-		/* Hard newline. */
-		break;
-
-	    if (qd > 0 && !stuffed) {
-		/* User deleted the space following the '>' chars; we'll
-		 * delete the space at the end of the previous line, if
-		 * there was one. */
-		gtk_text_iter_backward_char(&start);
-		if (gtk_text_iter_get_char(&start) != ' ')
-		    gtk_text_iter_forward_char(&start);
-	    }
-	    gtk_text_buffer_delete(buffer, &start, iter);
-	}
+	/* Move to the end of the line, if not there already. */
+	if (!gtk_text_iter_ends_line(iter)
+	    && !gtk_text_iter_forward_to_line_end(iter))
+	    return;
+	/* Save this iter as the start of a possible deletion. */
+	start = *iter;
+	/* Move to the start of the next line. */
+	if (!gtk_text_iter_forward_line(iter))
+	    return;
 
 	if (num_paras < 0) {
 	    /* This is a wrap_body call, not a continuous wrap, so we'll
@@ -749,10 +713,11 @@ libbalsa_unwrap_buffer(GtkTextBuffer * buffer, GtkTextIter * iter,
 		  && gtk_text_iter_get_char(&tmp_iter) == '-'
 		  && gtk_text_iter_backward_char(&tmp_iter)
 		  && gtk_text_iter_get_char(&tmp_iter) == '-'
-		  && gtk_text_iter_get_line_offset(&tmp_iter) == qd)) {
+		  && gtk_text_iter_get_line_offset(&tmp_iter) ==
+		  quote_depth)) {
 		*iter = start;
 		while (gtk_text_iter_get_line_offset(&start) >
-		       (qd ? qd + 1 : 0)) {
+		       (quote_depth ? quote_depth + 1 : 0)) {
 		    gtk_text_iter_backward_char(&start);
 		    if (gtk_text_iter_get_char(&start) != ' ') {
 			gtk_text_iter_forward_char(&start);
@@ -766,7 +731,7 @@ libbalsa_unwrap_buffer(GtkTextBuffer * buffer, GtkTextIter * iter,
 	}
 
 	line = get_line(buffer, &start);
-	if (prescanner(line))
+	if (prescanner(line, strlen(line)))
 	    mark_urls(buffer, &start, url_tag, line);
 	g_free(line);
     }
@@ -778,6 +743,31 @@ mark_urls(GtkTextBuffer * buffer, GtkTextIter * iter, GtkTextTag * tag,
           const gchar * line)
 {
     const gchar *p = line;
+    const gchar * const line_end = line + strlen(line);
+#if USE_GREGEX
+    GRegex *url_reg = get_url_reg();
+    GMatchInfo *url_match;
+    GtkTextIter start = *iter;
+    GtkTextIter end = *iter;
+
+    while (g_regex_match(url_reg, p, 0, &url_match)) {
+        gint start_pos, end_pos;
+
+        if (g_match_info_fetch_pos(url_match, 0, &start_pos, &end_pos)) {
+            glong offset = g_utf8_pointer_to_offset(line, p + start_pos);
+            gtk_text_iter_set_line_offset(&start, offset);
+            offset = g_utf8_pointer_to_offset(line, p + end_pos);
+            gtk_text_iter_set_line_offset(&end, offset);
+            gtk_text_buffer_apply_tag(buffer, tag, &start, &end);
+
+            p += end_pos;
+        }
+        if (!prescanner(p, line_end - p))
+            break;
+        g_match_info_free(url_match);
+    }
+    g_match_info_free(url_match);
+#else                          /* USE_GREGEX */
     regex_t *url_reg = get_url_reg();
     regmatch_t url_match;
     GtkTextIter start = *iter;
@@ -791,23 +781,10 @@ mark_urls(GtkTextBuffer * buffer, GtkTextIter * iter, GtkTextTag * tag,
         gtk_text_buffer_apply_tag(buffer, tag, &start, &end);
 
         p += url_match.rm_eo;
-        if (!prescanner(p))
+        if (!prescanner(p, line_end - p))
             break;
     }
-}
-
-/* Prepare the buffer for sending with DelSp=Yes. */
-void
-libbalsa_prepare_delsp(GtkTextBuffer * buffer)
-{
-    GtkTextTagTable *table = gtk_text_buffer_get_tag_table(buffer);
-    GtkTextTag *soft_tag = gtk_text_tag_table_lookup(table, "soft");
-    GtkTextIter iter;
-
-    gtk_text_buffer_get_start_iter(buffer, &iter);
-    while (gtk_text_iter_forward_to_line_end(&iter))
-	if (gtk_text_iter_has_tag(&iter, soft_tag))
-	    gtk_text_buffer_insert(buffer, &iter, " ", 1);
+#endif                          /* USE_GREGEX */
 }
 
 /*
@@ -826,14 +803,11 @@ libbalsa_prepare_delsp(GtkTextBuffer * buffer)
  * returns TRUE if the line may contain an URL.
  */
 static gboolean
-prescanner(const gchar * s)
+prescanner(const gchar * s, guint len)
 {
-    gint left = strlen(s) - 6;
+    gint left = len - 5;
 
-    if (left <= 0)
-        return FALSE;
-
-    while (left--) {
+    while (--left > 0) {
         switch (tolower(*s++)) {
         case 'f':              /* ftp:/, ftps: */
             if (tolower(*s) == 't' &&
@@ -868,6 +842,73 @@ prescanner(const gchar * s)
     return FALSE;
 }
 
+#if USE_GREGEX
+struct url_regex_info {
+    GRegex *url_reg;
+    const gchar *str;
+    const gchar *func;
+    const gchar *msg;
+};
+
+static GRegex *
+get_url_helper(struct url_regex_info *info)
+{
+    if (!info->url_reg) {
+        GError *err = NULL;
+
+        info->url_reg = g_regex_new(info->str, G_REGEX_CASELESS, 0, &err);
+        if (err) {
+            g_warning("%s %s: %s", info->func, info->msg, err->message);
+            g_error_free(err);
+        }
+    }
+
+    return info->url_reg;
+}
+
+static GRegex *
+get_url_reg(void)
+{
+    static struct url_regex_info info = {
+        NULL,
+        "(((https?|ftps?|nntp)://)|(mailto:|news:))"
+            "(%[0-9A-F]{2}|[-_.!~*';/?:@&=+$,#[:alnum:]])+",
+        __func__,
+        "url regex compilation failed"
+    };
+
+    return get_url_helper(&info);
+}
+
+static GRegex *
+get_ml_url_reg(void)
+{
+    static struct url_regex_info info = {
+        NULL,
+        "("
+        "%[0-9A-F]{2}|[-_.!~*';/?:@&=+$,#[:alnum:]]|[ \t]*[\r\n]+[ \t>]*"
+        ")+"
+        "(%[0-9A-F]{2}|[-_.!~*';/?:@&=+$,#[:alnum:]])>",
+        __func__,
+        "multiline url regex compilation failed"
+    };
+
+    return get_url_helper(&info);
+}
+
+static GRegex *
+get_ml_flowed_url_reg(void)
+{
+    static struct url_regex_info info = {
+        NULL,
+        "(%[0-9A-F]{2}|[-_.!~*';/?:@&=+$,#[:alnum:]]|[ \t]+)+>",
+        __func__,
+        "multiline url regex compilation failed"
+    };
+
+    return get_url_helper(&info);
+}
+#else                           /* USE_GREGEX */
 static regex_t *
 get_url_reg(void)
 {
@@ -896,7 +937,10 @@ get_ml_url_reg(void)
     if (!url_reg) {
         /* one-time compilation of a constant url_str expression */
         static const char url_str[] =
-	    "(%[0-9A-F]{2}|[-_.!~*';/?:@&=+$,#[:alnum:]]|[ \t]*[\r\n]+[ \t]*)+>";
+            "("
+            "%[0-9A-F]{2}|[-_.!~*';/?:@&=+$,#[:alnum:]]|[ \t]*[\r\n]+[ \t>]*"
+            ")+"
+            "(%[0-9A-F]{2}|[-_.!~*';/?:@&=+$,#[:alnum:]])>";
 
 	url_reg = g_new(regex_t, 1);
         if (regcomp(url_reg, url_str, REG_EXTENDED | REG_ICASE) != 0)
@@ -925,163 +969,255 @@ get_ml_flowed_url_reg(void)
     
     return url_reg;
 }
+#endif                          /* USE_GREGEX */
 
 gboolean
 libbalsa_insert_with_url(GtkTextBuffer * buffer,
                          const char *chars,
-			 const char *all_chars,
+                         guint len,
                          GtkTextTag * tag,
                          LibBalsaUrlInsertInfo *url_info)
 {
-    gchar *p, *buf;
-    const gchar *all_p;
     GtkTextIter iter;
+    GtkTextTagTable *table = gtk_text_buffer_get_tag_table(buffer);
+    GtkTextTag *url_tag = gtk_text_tag_table_lookup(table, "url");
+    gboolean match;
+#if USE_GREGEX
+    gint start_pos, end_pos;
+    GRegex *url_reg;
+    GMatchInfo *url_match;
+#else                           /* USE_GREGEX */
+    regex_t *url_reg;
+    regmatch_t url_match;
+#endif                          /* USE_GREGEX */
+    const gchar * const line_end = chars + len;
 
-    buf = p = g_strdup(chars);
-    all_p = all_chars;
     gtk_text_buffer_get_iter_at_mark(buffer, &iter,
                                      gtk_text_buffer_get_insert(buffer));
 
-    /* if there shouldn't be a callback for URL's we don't need to detect
-       them... */
-    if (url_info) {
-	GtkTextTagTable *table = gtk_text_buffer_get_tag_table(buffer);
-	GtkTextTag *url_tag = gtk_text_tag_table_lookup(table, "url");
+    if (url_info->ml_url_buffer) {
+        const gchar *url_end;
+        gchar *url, *q, *r;
 
-	if (url_info->ml_url) {
-	    gchar *url_end = strchr(p, '>');
+        if (!(url_end = strchr(chars, '>')) || url_end >= line_end) {
+            g_string_append_len(url_info->ml_url_buffer, chars,
+                                line_end - chars);
+            g_string_append_c(url_info->ml_url_buffer, '\n');
+            return TRUE;
+        }
 
-	    if (url_end) {
-		url_info->ml_url_buffer =
-		    g_string_append_len(url_info->ml_url_buffer, p, url_end - p);
-		gtk_text_buffer_insert_with_tags(buffer, &iter,
-						 url_info->ml_url_buffer->str, -1,
-						 url_tag, tag, NULL);
-		if (url_info->callback)
-		    url_info->callback(buffer, &iter, url_info->ml_url, url_info->callback_data);
-		g_string_free(url_info->ml_url_buffer, TRUE);
-		g_free(url_info->ml_url);
-		url_info->ml_url_buffer = NULL;
-		url_info->ml_url = NULL;
-		p = url_end;
-	    } else {
-		url_info->ml_url_buffer =
-		    g_string_append(url_info->ml_url_buffer, p);
-		url_info->ml_url_buffer =
-		    g_string_append_c(url_info->ml_url_buffer, '\n');
-		return TRUE;
-	    }
-	}
-
-	if (prescanner(p)) {
-	    gint offset = 0;
-	    regex_t *url_reg = get_url_reg();
-	    regmatch_t url_match;
-	    gint match = regexec(url_reg, p, 1, &url_match, 0);
-
-	    while (!match) {
-		gchar *buf;
-		gchar *spc;
-
-		if (url_match.rm_so) {
-		    /* check if we hit a multi-line URL... (see RFC 1738) */
-		    if (all_p && (p[url_match.rm_so - 1] == '<' ||
-				  (url_match.rm_so > 4 &&
-				   !g_ascii_strncasecmp(p + url_match.rm_so - 5, "<URL:", 5)))) {
-			/* if the input is flowed, we will see a space at
-			 * url_match.rm_eo - in this case the complete remainder
-			 * of the ml uri should be in the passed buffer... */
-			if (url_info && url_info->buffer_is_flowed &&
-			    *(p + url_match.rm_eo) == ' ') {
-			    regex_t *ml_flowed_url_reg = get_ml_flowed_url_reg();
-			    regmatch_t ml_url_match;
-
-			    if (!regexec(ml_flowed_url_reg,
-					 all_chars + offset + url_match.rm_eo, 1,
-					 &ml_url_match, 0) && ml_url_match.rm_so == 0)
-				url_match.rm_eo += ml_url_match.rm_eo - 1;
-			} else if (!strchr(p + url_match.rm_eo, '>')) {
-			    regex_t *ml_url_reg = get_ml_url_reg();
-			    regmatch_t ml_url_match;
-		    
-			    if (!regexec(ml_url_reg,
-					 all_chars + offset + url_match.rm_eo, 1,
-					 &ml_url_match, 0) && ml_url_match.rm_so == 0) {
-				GString *ml_url = g_string_new("");
-				const gchar *ml_p = all_chars + offset + url_match.rm_so;
-				gint ml_cnt =
-				    url_match.rm_eo - url_match.rm_so + ml_url_match.rm_eo - 1;
-
-				for (; ml_cnt; (ml_p++, ml_cnt--))
-				    if (*ml_p > ' ')
-					ml_url = g_string_append_c(ml_url, *ml_p);
-				url_info->ml_url = ml_url->str;
-				g_string_free(ml_url, FALSE);
-			    }
-			}
-		    }
-
-		    buf = g_strndup(p, url_match.rm_so);
-		    gtk_text_buffer_insert_with_tags(buffer, &iter,
-						     buf, -1, tag, NULL);
-		    g_free(buf);
-		}
-
-		if (url_info->ml_url) {
-		    url_info->ml_url_buffer = g_string_new(p + url_match.rm_so);
-		    url_info->ml_url_buffer =
-			g_string_append_c(url_info->ml_url_buffer, '\n');
-		    return TRUE;
-		}
-
-		/* add the url - it /may/ contain spaces if the text is flowed */
-		buf = g_strndup(p + url_match.rm_so,
-				url_match.rm_eo - url_match.rm_so);
-		if ((spc = strchr(buf, ' '))) {
-		    GString *uri_real = g_string_new("");
-		    gchar * p = buf;
-
-		    while (spc) {
-			*spc = '\n';
-			g_string_append_len(uri_real, p, spc - p);
-			p = spc + 1;
-			spc = strchr(p, ' ');
-		    }
-		    g_string_append(uri_real, p);
-		    gtk_text_buffer_insert_with_tags(buffer, &iter, buf, -1,
-						     url_tag, tag, NULL);
-		    if (url_info->callback)
-			url_info->callback(buffer, &iter, uri_real->str, url_info->callback_data);
-		    g_string_free(uri_real, TRUE);
-		} else {
-		    gtk_text_buffer_insert_with_tags(buffer, &iter, buf, -1,
-						     url_tag, tag, NULL);
-
-		    /* remember the URL and its position within the text */
-		    if (url_info->callback)
-			url_info->callback(buffer, &iter, buf, url_info->callback_data);
-		}
-		g_free(buf);
-
-		p += url_match.rm_eo;
-		offset += url_match.rm_eo;
-		if (prescanner(p))
-		    match = regexec(url_reg, p, 1, &url_match, 0);
-		else
-		    match = -1;
-	    }
-	}
+        g_string_append_len(url_info->ml_url_buffer, chars,
+                            url_end - chars);
+        gtk_text_buffer_insert_with_tags(buffer, &iter,
+                                         url_info->ml_url_buffer->str,
+                                         url_info->ml_url_buffer->len,
+                                         url_tag, tag, NULL);
+        q = url = g_new(gchar, url_info->ml_url_buffer->len);
+        for (r = url_info->ml_url_buffer->str; *r; r++)
+            if (*r > ' ')
+                *q++ = *r;
+        url_info->callback(buffer, &iter, url, q - url,
+                           url_info->callback_data);
+        g_free(url);
+        g_string_free(url_info->ml_url_buffer, TRUE);
+        url_info->ml_url_buffer = NULL;
+        chars = url_end;
     }
 
-    if (*p)
-        gtk_text_buffer_insert_with_tags(buffer, &iter, p, -1, tag, NULL);
-    g_free(buf);
+    if (!prescanner(chars, line_end - chars)) {
+        gtk_text_buffer_insert_with_tags(buffer, &iter, chars,
+                                         line_end - chars, tag, NULL);
+        return FALSE;
+    }
+
+#if USE_GREGEX
+    url_reg = get_url_reg();
+    match = g_regex_match(url_reg, chars, 0, &url_match)
+        && g_match_info_fetch_pos(url_match, 0, &start_pos, &end_pos)
+        && chars + start_pos < line_end;
+
+    while (match) {
+        gchar *spc;
+
+        gtk_text_buffer_insert_with_tags(buffer, &iter, chars,
+                                         start_pos, tag, NULL);
+
+        /* check if we hit a multi-line URL... (see RFC 1738) */
+        if ((start_pos > 0 && (chars[start_pos - 1] == '<')) ||
+            (start_pos > 4 &&
+             !g_ascii_strncasecmp(chars + start_pos - 5, "<URL:", 5))) {
+            GMatchInfo *ml_url_match;
+            gint ml_start_pos, ml_end_pos;
+
+            /* if the input is flowed, we may see a space at
+             * url_match.rm_eo - in this case the complete remainder
+             * of the ml uri should be in the passed buffer... */
+            if (url_info->buffer_is_flowed && chars[end_pos] == ' ') {
+                if (g_regex_match(get_ml_flowed_url_reg(), chars + end_pos,
+                                  0, &ml_url_match)
+                    && g_match_info_fetch_pos(ml_url_match, 0,
+                                              &ml_start_pos, &ml_end_pos)
+                    && ml_start_pos == 0)
+                    end_pos += ml_end_pos - 1;
+                g_match_info_free(ml_url_match);
+            } else if (chars[end_pos] != '>') {
+                if (g_regex_match(get_ml_url_reg(), chars + end_pos,
+                                  0, &ml_url_match)
+                    && g_match_info_fetch_pos(ml_url_match, 0,
+                                              &ml_start_pos, NULL)
+                    && ml_start_pos == 0) {
+                    chars += start_pos;
+                    url_info->ml_url_buffer =
+                        g_string_new_len(chars, line_end - chars);
+                    g_string_append_c(url_info->ml_url_buffer, '\n');
+                }
+                g_match_info_free(ml_url_match);
+                if (url_info->ml_url_buffer)
+                    return TRUE;
+            }
+        }
+
+        /* add the url - it /may/ contain spaces if the text is flowed */
+        if ((spc = strchr(chars + start_pos, ' ')) && spc < chars + end_pos) {
+            GString *uri_real = g_string_new("");
+            gchar *q, *buf;
+
+            q = buf = g_strndup(chars + start_pos, end_pos - start_pos);
+            spc = buf + (spc - (chars + start_pos));
+            do {
+                *spc = '\n';
+                g_string_append_len(uri_real, q, spc - q);
+                q = spc + 1;
+            } while ((spc = strchr(q, ' ')));
+            g_string_append(uri_real, q);
+            gtk_text_buffer_insert_with_tags(buffer, &iter, buf, -1,
+                                             url_tag, tag, NULL);
+            g_free(buf);
+            url_info->callback(buffer, &iter,
+                               uri_real->str, uri_real->len,
+                               url_info->callback_data);
+            g_string_free(uri_real, TRUE);
+        } else {
+            gtk_text_buffer_insert_with_tags(buffer, &iter,
+                                             chars + start_pos,
+                                             end_pos - start_pos,
+                                             url_tag, tag, NULL);
+
+            /* remember the URL and its position within the text */
+            url_info->callback(buffer, &iter, chars + start_pos,
+                               end_pos - start_pos,
+                               url_info->callback_data);
+        }
+
+        chars += end_pos;
+        if (prescanner(chars, line_end - chars)) {
+            g_match_info_free(url_match);
+            match = g_regex_match(url_reg, chars, 0, &url_match)
+                && g_match_info_fetch_pos(url_match, 0, &start_pos,
+                                          &end_pos)
+                && chars + start_pos < line_end;
+        } else
+            match = FALSE;
+    }
+    g_match_info_free(url_match);
+#else                           /* USE_GREGEX */
+    url_reg = get_url_reg();
+    match = regexec(url_reg, chars, 1, &url_match, 0) == 0
+        && chars + url_match.rm_so < line_end;
+
+    while (match) {
+        gchar *spc;
+        gint start_pos, end_pos;
+
+        start_pos = url_match.rm_so;
+        end_pos = url_match.rm_eo;
+
+        gtk_text_buffer_insert_with_tags(buffer, &iter, chars,
+                                         start_pos, tag, NULL);
+
+        /* check if we hit a multi-line URL... (see RFC 1738) */
+        if ((start_pos > 0 && (chars[start_pos - 1] == '<')) ||
+            (start_pos > 4 &&
+             !g_ascii_strncasecmp(chars + start_pos - 5, "<URL:", 5))) {
+            regex_t *ml_url_reg;
+            regmatch_t ml_url_match;
+
+            /* if the input is flowed, we may see a space at
+             * url_match.rm_eo - in this case the complete remainder
+             * of the ml uri should be in the passed buffer... */
+            if (url_info->buffer_is_flowed && chars[end_pos] == ' ') {
+                ml_url_reg = get_ml_flowed_url_reg();
+                if (!regexec(ml_url_reg, chars + end_pos, 1,
+                             &ml_url_match, 0)
+                    && ml_url_match.rm_so == 0)
+                    end_pos += ml_url_match.rm_eo - 1;
+            } else if (chars[end_pos] != '>') {
+                ml_url_reg = get_ml_url_reg();
+                if (!regexec(ml_url_reg, chars + end_pos, 1,
+                             &ml_url_match, 0)
+                    && ml_url_match.rm_so == 0) {
+                    chars += start_pos;
+                    url_info->ml_url_buffer =
+                        g_string_new_len(chars, line_end - chars);
+                    g_string_append_c(url_info->ml_url_buffer, '\n');
+                    return TRUE;
+                }
+            }
+        }
+
+        /* add the url - it /may/ contain spaces if the text is flowed */
+        if ((spc = strchr(chars + start_pos, ' ')) && spc < chars + end_pos) {
+            GString *uri_real = g_string_new("");
+            gchar *q, *buf;
+
+            q = buf = g_strndup(chars + start_pos, end_pos - start_pos);
+            spc = buf + (spc - (chars + start_pos));
+            do {
+                *spc = '\n';
+                g_string_append_len(uri_real, q, spc - q);
+                q = spc + 1;
+            } while ((spc = strchr(q, ' ')));
+            g_string_append(uri_real, q);
+            gtk_text_buffer_insert_with_tags(buffer, &iter, buf, -1,
+                                             url_tag, tag, NULL);
+            g_free(buf);
+            url_info->callback(buffer, &iter,
+                               uri_real->str, uri_real->len,
+                               url_info->callback_data);
+            g_string_free(uri_real, TRUE);
+        } else {
+            gtk_text_buffer_insert_with_tags(buffer, &iter,
+                                             chars + start_pos,
+                                             end_pos - start_pos,
+                                             url_tag, tag, NULL);
+
+            /* remember the URL and its position within the text */
+            url_info->callback(buffer, &iter, chars + start_pos,
+                               end_pos - start_pos,
+                               url_info->callback_data);
+        }
+
+        chars += end_pos;
+        if (prescanner(chars, line_end - chars))
+            match = regexec(url_reg, chars, 1, &url_match, 0) == 0
+                && chars + url_match.rm_so < line_end;
+        else
+            match = FALSE;
+    }
+#endif                          /* USE_GREGEX */
+
+    gtk_text_buffer_insert_with_tags(buffer, &iter, chars,
+                                     line_end - chars, tag, NULL);
 
     return FALSE;
 }
 
 void
+#if USE_GREGEX
+libbalsa_unwrap_selection(GtkTextBuffer * buffer, GRegex * rex)
+#else                           /* USE_GREGEX */
 libbalsa_unwrap_selection(GtkTextBuffer * buffer, regex_t * rex)
+#endif                          /* USE_GREGEX */
 {
     GtkTextIter start, end;
     gchar *line;
@@ -1152,6 +1288,34 @@ libbalsa_unwrap_selection(GtkTextBuffer * buffer, regex_t * rex)
     }
 }
 
+#if USE_GREGEX
+gboolean
+libbalsa_match_regex(const gchar * line, GRegex * rex, guint * count,
+                     guint * index)
+{
+    GMatchInfo *rm;
+    gint c;
+    const gchar *p;
+    gint end_pos;
+
+    c = 0;
+    for (p = line;
+         g_regex_match(rex, p, 0, &rm)
+         && g_match_info_fetch_pos(rm, 0, NULL, &end_pos)
+         && end_pos > 0;
+         p += end_pos) {
+        c++;
+        g_match_info_free(rm);
+    }
+    g_match_info_free(rm);
+
+    if (count)
+        *count = c;
+    if (index)
+        *index = p - line;
+    return c > 0;
+}
+#else                           /* USE_GREGEX */
 gboolean
 libbalsa_match_regex(const gchar * line, regex_t * rex, guint * count,
 		     guint * index)
@@ -1161,7 +1325,8 @@ libbalsa_match_regex(const gchar * line, regex_t * rex, guint * count,
     const gchar *p;
 
     c = 0;
-    for (p = line; regexec(rex, p, 1, &rm, 0) == 0; p += rm.rm_eo)
+    for (p = line; !regexec(rex, p, 1, &rm, 0) && rm.rm_eo > 0;
+         p += rm.rm_eo)
 	c++;
     if (count)
 	*count = c;
@@ -1169,4 +1334,172 @@ libbalsa_match_regex(const gchar * line, regex_t * rex, guint * count,
 	*index = p - line;
     return c > 0;
 }
+#endif                          /* USE_GREGEX */
 
+
+GString *
+libbalsa_html_encode_hyperlinks(GString * paragraph)
+{
+    GString * retval;
+    gchar * p;
+#if USE_GREGEX
+    GRegex *url_reg = get_url_reg();
+    GMatchInfo *url_match;
+#else
+    regex_t *url_reg = get_url_reg();
+    regmatch_t url_match;
+#endif
+    gboolean match;
+    gchar * markup;
+
+    /* check for any url */
+    if (!prescanner(paragraph->str, paragraph->len)) {
+        markup = g_markup_escape_text(paragraph->str, -1);
+        g_string_assign(paragraph, markup);
+        g_free(markup);
+        return paragraph;
+    }
+
+    /* got some url's... */
+    retval = g_string_new("");
+    p = paragraph->str;
+
+#if USE_GREGEX
+    match = g_regex_match(url_reg, p, 0, &url_match);
+#else
+    match = regexec(url_reg, p, 1, &url_match, 0) == 0;
+#endif
+
+    while (match) {
+        gint start_pos, end_pos;
+
+#if USE_GREGEX
+        if (!g_match_info_fetch_pos(url_match, 0, &start_pos, &end_pos))
+            break;
+#else
+        start_pos = url_match.rm_so;
+        end_pos   = url_match.rm_eo;
+#endif
+
+        /* add the url to the result */
+        if (start_pos > 0) {
+            markup = g_markup_escape_text(p, start_pos);
+            retval = g_string_append(retval, markup);
+            g_free(markup);
+        }
+        retval = g_string_append(retval, "<a href=\"");
+        retval = g_string_append_len(retval, p + start_pos, end_pos - start_pos);
+        retval = g_string_append(retval, "\">");
+        retval = g_string_append_len(retval, p + start_pos, end_pos - start_pos);
+        retval = g_string_append(retval, "</a>");
+
+        /* find next (if any) */
+        p += end_pos;
+        if (prescanner(p, paragraph->len - (p - paragraph->str))) {
+#if USE_GREGEX
+            g_match_info_free(url_match);
+            match = g_regex_match(url_reg, p, 0, &url_match);
+#else
+            match = regexec(url_reg, p, 1, &url_match, 0) == 0;
+#endif
+        } else
+            match = FALSE;
+    }
+#if USE_GREGEX
+    g_match_info_free(url_match);
+#endif                          /* USE_GREGEX */
+
+    /* copy remainder */
+    if (*p != '\0') {
+        markup = g_markup_escape_text(p, -1);
+        retval = g_string_append(retval, markup);
+        g_free(markup);
+    }
+
+    /* done - free original, return new */
+    g_string_free(paragraph, TRUE);
+    return retval;
+}
+
+
+gchar *
+libbalsa_text_to_html(const gchar * title, const gchar * body, const gchar * lang)
+{
+    GString * html_body =
+        g_string_new("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">\n");
+    gchar * html_subject;
+    const gchar * start = body;
+    gchar * html_lang;
+    gchar * p;
+
+    /* set the html header, including the primary language and the title */
+    if (lang) {
+        html_lang = g_strdup(lang);
+        if ((p = strchr(html_lang, '_')))
+            *p = '-';
+    } else
+        html_lang = g_strdup("x-unknown");
+    html_subject = g_markup_escape_text(title, -1);
+    g_string_append_printf(html_body, 
+                           "<html lang=\"%s\"><head>\n"
+                           "<title>%s</title>\n"
+                           "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n"
+                           "<style type=\"text/css\">\n"
+                           "  p { margin-top: 0px; margin-bottom: 0px; }\n"
+                           "</style></head>\n"
+                           "<body>\n", html_lang, html_subject);
+    g_free(html_subject);
+    g_free(html_lang);
+
+    /* add the lines of the message body */
+    while (*start) {
+        const gchar * eol = strchr(start, '\n');
+        const gchar * p = start;
+        PangoDirection direction = PANGO_DIRECTION_NEUTRAL;
+        GString * html;
+        gsize idx;
+
+        if (!eol)
+            eol = start + strlen(start);
+
+        /* find the first real char to determine the paragraph direction */
+        while (p < eol && direction == PANGO_DIRECTION_NEUTRAL) {
+            direction = pango_unichar_direction(g_utf8_get_char(p));
+            p = g_utf8_next_char(p);
+        }
+
+        /* html escape the line */
+        html = g_string_new_len(start, eol - start);
+
+        /* encode hyperlinks */
+        html = libbalsa_html_encode_hyperlinks(html);
+
+        /* replace a series of n spaces by (n - 1) &nbsp; and one space */
+        idx = 0;
+        while (idx < html->len) {
+            if (html->str[idx] == ' ' && (idx == 0 || html->str[idx + 1] == ' ')) {
+                html->str[idx++] = '&';
+                html = g_string_insert(html, idx, "nbsp;");
+                idx += 5;
+            } else
+                idx = g_utf8_next_char(html->str + idx) - html->str;
+        }
+
+        /* append the paragraph, always stating the proper direction */
+        g_string_append_printf(html_body, "<p dir=\"%s\">%s</p>\n",
+                               direction == PANGO_DIRECTION_RTL ? "rtl" : "ltr",
+                               *html->str ? html->str : "&nbsp;");
+        g_string_free(html, TRUE);
+
+        /* next line */
+        start = eol;
+        if (*start)
+            start++;
+    }
+
+    /* close the html context */
+    html_body = g_string_append(html_body, "</body></html>\n");
+    
+    /* return the utf-8 encoded text/html */
+    return g_string_free(html_body, FALSE);
+}
