@@ -24,10 +24,12 @@
    this file can be linked against the address book program balsa-ab
    without introducing extra dependencies. External library
    dependencies should go to libbalsa.c */
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "misc.h"
 
 #define _SVID_SOURCE           1
-#define _XOPEN_SOURCE          500
 #include <ctype.h>
 #include <dirent.h>
 #include <errno.h>
@@ -41,7 +43,6 @@
 
 #include "libbalsa.h"
 #include "libbalsa_private.h"
-#include "misc.h"
 #include "html.h"
 #include <glib/gi18n.h>
 
@@ -151,8 +152,8 @@ libbalsa_urlencode(const gchar* str)
        } else if (!isalnum(*x) && strchr("_-.", *x) == NULL) {
            /* Allow only alnum chars and '_', '-', '.'; escape the rest */
            *x++ = '%';
-           *x++ = hexchars[*str >> 4];
-           *x = hexchars[*str & 0x0F];
+           *x++ = hexchars[(*str >> 4) & 0x0F];
+           *x = hexchars[(*str) & 0x0F];
        }
     }
     
@@ -166,7 +167,7 @@ libbalsa_urldecode(const gchar * str)
     gchar *retval;
     gchar *x;
 
-    retval = g_new(char, strlen(str));
+    retval = g_new(char, strlen(str)+1);
 
     for (x = retval; *str != '\0'; str++, x++) {
 	*x = *str;
@@ -486,7 +487,6 @@ LibBalsaCodeset
 libbalsa_set_fallback_codeset(LibBalsaCodeset codeset)
 {
     LibBalsaCodeset ret = sanitize_fallback_codeset;
-#if defined(HAVE_GMIME_2_2_7)
     const gchar *charsets[] = {
         "UTF-8",
         libbalsa_get_codeset_name(NULL, codeset),
@@ -494,8 +494,6 @@ libbalsa_set_fallback_codeset(LibBalsaCodeset codeset)
     };
 
     g_mime_set_user_charsets(charsets);
-    /* GMime will free the strings. */
-#endif                          /* HAVE_GMIME_2_2_7 */
 
     sanitize_fallback_codeset = codeset;
     return ret;
@@ -1036,22 +1034,6 @@ libbalsa_unlock_file (const char *path, int fd, int dot)
     return 0;
 }
 
-#if !GLIB_CHECK_VERSION(2, 2, 0)
-gboolean
-libbalsa_str_has_prefix(const gchar * str, const gchar * prefix)
-{
-    g_return_val_if_fail(str != NULL, FALSE);
-    g_return_val_if_fail(prefix != NULL, FALSE);
-
-    while (*prefix == *str && *prefix) {
-	++prefix;
-	++str;
-    }
-
-    return *prefix == '\0';
-}
-#endif				/* !GLIB_CHECK_VERSION(2, 2, 0) */
-
 
 /* libbalsa_ia_rfc2821_equal
    compares two addresses according to rfc2821: local-part@domain is equal,
@@ -1062,23 +1044,25 @@ libbalsa_ia_rfc2821_equal(const InternetAddress * a,
 			  const InternetAddress * b)
 {
     const gchar *a_atptr, *b_atptr;
+    const gchar *a_addr, *b_addr;
     gint a_atpos, b_atpos;
 
-    if (!a || !b || a->type != INTERNET_ADDRESS_NAME ||
-	b->type != INTERNET_ADDRESS_NAME)
+    if (!INTERNET_ADDRESS_IS_MAILBOX(a) || !INTERNET_ADDRESS_IS_MAILBOX(b))
         return FALSE;
 
     /* first find the "@" in the two addresses */
-    a_atptr = strchr(a->value.addr, '@');
-    b_atptr = strchr(b->value.addr, '@');
+    a_addr = INTERNET_ADDRESS_MAILBOX(a)->addr;
+    b_addr = INTERNET_ADDRESS_MAILBOX(b)->addr;
+    a_atptr = strchr(a_addr, '@');
+    b_atptr = strchr(b_addr, '@');
     if (!a_atptr || !b_atptr)
         return FALSE;
-    a_atpos = a_atptr - a->value.addr;
-    b_atpos = b_atptr - b->value.addr;
+    a_atpos = a_atptr - a_addr;
+    b_atpos = b_atptr - b_addr;
 
     /* now compare the strings */
     if (!a_atpos || !b_atpos || a_atpos != b_atpos || 
-        strncmp(a->value.addr, b->value.addr, a_atpos) ||
+        strncmp(a_addr, b_addr, a_atpos) ||
         g_ascii_strcasecmp(a_atptr, b_atptr))
         return FALSE;
     else
@@ -1156,11 +1140,8 @@ libbalsa_create_entry(GtkWidget * table, GCallback changed_func,
     gtk_table_attach(GTK_TABLE(table), entry, 1, 2, row, row + 1,
                      GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
 
-    if (initval) {
-        gint zero = 0;
-
-        gtk_editable_insert_text(GTK_EDITABLE(entry), initval, -1, &zero);
-    }
+    if (initval)
+        gtk_entry_set_text(GTK_ENTRY(entry), initval);
 
     gtk_label_set_mnemonic_widget(GTK_LABEL(hotlabel), entry);
 
@@ -1177,7 +1158,7 @@ libbalsa_create_entry(GtkWidget * table, GCallback changed_func,
 static void
 lb_create_size_group_func(GtkWidget * widget, gpointer data)
 {
-    if (GTK_IS_LABEL(widget) && GTK_IS_TABLE(widget->parent))
+    if (GTK_IS_LABEL(widget) && GTK_IS_TABLE(gtk_widget_get_parent(widget)))
         gtk_size_group_add_widget(GTK_SIZE_GROUP(data), widget);
     else if (GTK_IS_CONTAINER(widget))
         gtk_container_foreach(GTK_CONTAINER(widget),
