@@ -18,7 +18,10 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
  * 02111-1307, USA.
  */
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "rfc3156.h"
 
 #ifdef HAVE_GPGME
 
@@ -27,10 +30,10 @@
 #include <pwd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <stdlib.h>
 
 #include "libbalsa.h"
 #include "libbalsa_private.h"
-#include "rfc3156.h"
 
 #include "gmime-gpgme-context.h"
 #include "gmime-gpgme-signature.h"
@@ -45,8 +48,12 @@
 #  include "misc.h"
 #endif
 
+#if HAVE_MACOSX_DESKTOP
+#  include "macosx-helpers.h"
+#endif
+
 #include "padlock-keyhole.xpm"
-#include "i18n.h"
+#include <glib/gi18n.h>
 
 
 /* local prototypes */
@@ -64,6 +71,8 @@ static gboolean accept_low_trust_key(const gchar * name,
 				     GMimeGpgmeContext * ctx);
 static gboolean gpg_updates_trustdb(void);
 static gchar *fix_EMail_info(gchar * str);
+static gboolean have_pub_key_for(gpgme_ctx_t gpgme_ctx,
+				 InternetAddressList * recipients);
 
 
 /* ==== public functions =================================================== */
@@ -111,7 +120,7 @@ libbalsa_check_crypto_engine(gpgme_protocol_t protocol)
 	g_string_append_printf(message,
 			       _("\nDisable support for protocol %s."),
 			       gpgme_get_protocol_name(protocol));
-	libbalsa_information(LIBBALSA_INFORMATION_ERROR, message->str);
+	libbalsa_information(LIBBALSA_INFORMATION_ERROR, "%s", message->str);
 	g_string_free(message, TRUE);
 	return FALSE;
     } else
@@ -126,20 +135,57 @@ body_is_type(LibBalsaMessageBody * body, const gchar * type,
     gboolean retval;
 
     if (body->mime_part) {
-	const GMimeContentType *content_type =
+	GMimeContentType *content_type =
 	    g_mime_object_get_content_type(body->mime_part);
 	retval = g_mime_content_type_is_type(content_type, type, sub_type);
     } else {
 	GMimeContentType *content_type =
 	    g_mime_content_type_new_from_string(body->content_type);
 	retval = g_mime_content_type_is_type(content_type, type, sub_type);
-	g_mime_content_type_destroy(content_type);
+	g_object_unref(content_type);
     }
 
     return retval;
 }
 
 
+/* return TRUE if we can encrypt for every recipient in the recipients list
+ * using protocol */
+gboolean
+libbalsa_can_encrypt_for_all(InternetAddressList * recipients,
+			     gpgme_protocol_t protocol)
+{
+    gpgme_ctx_t gpgme_ctx;
+    gboolean result;
+
+    /* silent paranoia checks */
+    if (!recipients)
+	return TRUE;  /* we can of course encrypt for nobody... */
+#ifndef HAVE_SMIME
+    if (protocol == GPGME_PROTOCOL_OpenPGP)
+	return FALSE;
+#endif
+
+    /* check if gpg is currently available */
+    if (protocol == GPGME_PROTOCOL_OpenPGP && gpg_updates_trustdb())
+	return FALSE;
+
+    /* create the gpgme context and set the protocol */
+    if (gpgme_new(&gpgme_ctx) != GPG_ERR_NO_ERROR)
+	return FALSE;
+    if (gpgme_set_protocol(gpgme_ctx, protocol) != GPG_ERR_NO_ERROR) {
+	gpgme_release(gpgme_ctx);
+	return FALSE;
+    }
+
+    /* loop over all recipients and try to find valid keys */
+    result = have_pub_key_for(gpgme_ctx, recipients);
+    gpgme_release(gpgme_ctx);
+
+    return result;
+}
+
+
 /*
  * Check if body (and eventually its subparts) are RFC 2633 or RFC 3156 signed
  * or encrypted.
@@ -350,6 +396,7 @@ libbalsa_encrypt_mime_object(GMimeObject ** content, GList * rfc822_for,
 	result = 
 	    g_mime_multipart_encrypted_encrypt(mpe, *content,
 					       GMIME_CIPHER_CONTEXT(ctx),
+                                               FALSE, NULL,
 					       recipients, error);
     }
 #ifdef HAVE_SMIME
@@ -466,7 +513,7 @@ libbalsa_body_check_signature(LibBalsaMessageBody * body,
 
     /* check if the body is really a multipart/signed */
     if (!GMIME_IS_MULTIPART_SIGNED(body->mime_part)
-        || (g_mime_multipart_get_number
+        || (g_mime_multipart_get_count
             (((GMimeMultipart *) body->mime_part))) < 2)
         return FALSE;
     if (body->parts->next->sig_info)
@@ -661,7 +708,6 @@ libbalsa_body_decrypt(LibBalsaMessageBody * body,
 #endif
 
     libbalsa_message_body_set_mime_body(body, mime_obj);
-    g_object_unref(G_OBJECT(mime_obj));
     if (ctx->sig_state && ctx->sig_state->status != GPG_ERR_NOT_SIGNED) {
 	g_object_ref(ctx->sig_state);
 	body->sig_info = ctx->sig_state;
@@ -878,7 +924,7 @@ libbalsa_rfc2440_decrypt(GMimePart * part, GMimeGpgmeSigstat ** sig_info,
     }
 
     /* decrypt */
-    if (g_mime_part_rfc2440_decrypt(part, ctx, &error) == -1) {
+    if (g_mime_part_rfc2440_decrypt(part, ctx, &error) == NULL) {
 	if (error) {
 	    if (error->code != GPG_ERR_CANCELED)
 		libbalsa_information(LIBBALSA_INFORMATION_ERROR, "%s: %s",
@@ -981,14 +1027,22 @@ libbalsa_gpgme_sig_protocol_name(gpgme_protocol_t protocol)
     }
 }
 
+#define APPEND_TIMET(str,fmt,t)						\
+    do {                                                                \
+        if (t) {                                                        \
+            gchar * _tbuf = libbalsa_date_to_utf8(&t, date_string);     \
+            g_string_append_printf(str, fmt, _tbuf);      \
+            g_free(_tbuf);                                              \
+        }                                                               \
+    } while (0)
+
 gchar *
 libbalsa_signature_info_to_gchar(GMimeGpgmeSigstat * info,
 				 const gchar * date_string)
 {
     GString *msg;
     gchar *retval;
-    struct tm date;
-    char buf[128];
+
     g_return_val_if_fail(info != NULL, NULL);
     g_return_val_if_fail(date_string != NULL, NULL);
     msg = g_string_new(libbalsa_gpgme_sig_protocol_name(info->protocol));
@@ -1005,11 +1059,7 @@ libbalsa_signature_info_to_gchar(GMimeGpgmeSigstat * info,
     } else if (info->sign_email && strlen(info->sign_email))
 	g_string_append_printf(msg, _("\nMail address: %s"),
 			       info->sign_email);
-    if (info->sign_time) {
-	localtime_r(&info->sign_time, &date);
-	strftime(buf, sizeof(buf), date_string, &date);
-	g_string_append_printf(msg, _("\nSigned on: %s"), buf);
-    }
+    APPEND_TIMET(msg, _("\nSigned on: %s"), info->sign_time);
     g_string_append_printf(msg, _("\nUser ID validity: %s"),
 			   libbalsa_gpgme_validity_to_gchar(info->
 							    validity));
@@ -1020,16 +1070,10 @@ libbalsa_signature_info_to_gchar(GMimeGpgmeSigstat * info,
     if (info->fingerprint)
 	g_string_append_printf(msg, _("\nKey fingerprint: %s"),
 			       info->fingerprint);
-    if (info->key_created) {
-	localtime_r(&info->key_created, &date);
-	strftime(buf, sizeof(buf), date_string, &date);
-	g_string_append_printf(msg, _("\nSubkey created on: %s"), buf);
-    }
-    if (info->key_expires) {
-	localtime_r(&info->key_expires, &date);
-	strftime(buf, sizeof(buf), date_string, &date);
-	g_string_append_printf(msg, _("\nSubkey expires on: %s"), buf);
-    }
+    /* Subkey creation date */
+    APPEND_TIMET(msg, _("\nSubkey created on: %s"), info->key_created);
+    /* Subkey expiration date */
+    APPEND_TIMET(msg, _("\nSubkey expires on: %s"), info->key_expires);
     if (info->key_revoked || info->key_expired || info->key_disabled ||
        info->key_invalid) {
        GString * attrs = g_string_new("");
@@ -1084,7 +1128,7 @@ libbalsa_signature_info_to_gchar(GMimeGpgmeSigstat * info,
 #include <sys/wait.h>
 #include <fcntl.h>
 
-/* run gpg asynchronously to import a key */
+/* run gpg asynchronously to import or update a key */
 typedef struct _spawned_gpg_T {
     gint child_pid;
     gint standard_error;
@@ -1095,7 +1139,8 @@ typedef struct _spawned_gpg_T {
 static gboolean check_gpg_child(gpointer data);
 
 gboolean
-gpg_run_import_key(const gchar * fingerprint, GtkWindow * parent)
+gpg_keyserver_op(const gchar * fingerprint, gpg_keyserver_action_t action,
+                 GtkWindow * parent)
 {
     gchar **argv;
     spawned_gpg_T *spawned_gpg;
@@ -1105,7 +1150,16 @@ gpg_run_import_key(const gchar * fingerprint, GtkWindow * parent)
     argv = g_new(gchar *, 5);
     argv[0] = g_strdup(GPG_PATH);
     argv[1] = g_strdup("--no-greeting");
-    argv[2] = g_strdup("--recv-keys");
+    switch (action) {
+    case GPG_KEYSERVER_IMPORT:
+        argv[2] = g_strdup("--recv-keys");
+        break;
+    case GPG_KEYSERVER_UPDATE:
+        argv[2] = g_strdup("--refresh-keys");
+        break;
+    default:
+        g_assert_not_reached();
+    }
     argv[3] = g_strdup(fingerprint);
     argv[4] = NULL;
     spawned_gpg = g_new0(spawned_gpg_T, 1);
@@ -1119,7 +1173,7 @@ gpg_run_import_key(const gchar * fingerprint, GtkWindow * parent)
     if (spawnres == FALSE) {
 	libbalsa_information(LIBBALSA_INFORMATION_ERROR,
 			     _
-			     ("Could not launch %s to get the public key %s."),
+			     ("Could not launch %s to query the public key %s."),
 			     GPG_PATH, fingerprint);
 	g_free(spawned_gpg);
 	return FALSE;
@@ -1169,15 +1223,18 @@ check_gpg_child(gpointer data)
 				   GTK_DIALOG_DESTROY_WITH_PARENT,
 				   GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
 				   _
-				   ("Running gpg failed with return value %d:\n%s"),
-				   WEXITSTATUS(status), gpg_message);
+				   ("Running %s failed with return value %d:\n%s"),
+				   GPG_PATH, WEXITSTATUS(status), gpg_message);
     else
 	dialog =
 	    gtk_message_dialog_new(spawned_gpg->parent,
 				   GTK_DIALOG_DESTROY_WITH_PARENT,
 				   GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
-				   _("Running gpg successful:\n%s"),
-				   gpg_message);
+				   _("Running %s successful:\n%s"),
+				   GPG_PATH, gpg_message);
+#if HAVE_MACOSX_DESKTOP
+    libbalsa_macosx_menu_for_parent(dialog, spawned_gpg->parent);
+#endif
     g_free(gpg_message);
     g_string_free(spawned_gpg->stderr_buf, TRUE);
     g_free(spawned_gpg);
@@ -1323,8 +1380,13 @@ select_key_from_list(const gchar * name, gboolean is_secret,
 					 GTK_STOCK_OK, GTK_RESPONSE_OK,
 					 GTK_STOCK_CANCEL,
 					 GTK_RESPONSE_CANCEL, NULL);
+#if HAVE_MACOSX_DESKTOP
+    libbalsa_macosx_menu_for_parent(dialog, parent);
+#endif
     vbox = gtk_vbox_new(FALSE, 12);
-    gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), vbox);
+    gtk_container_add(GTK_CONTAINER
+                      (gtk_dialog_get_content_area(GTK_DIALOG(dialog))),
+                      vbox);
     gtk_container_set_border_width(GTK_CONTAINER(vbox), 12);
     if (is_secret)
 	prompt =
@@ -1433,7 +1495,7 @@ select_key_from_list(const gchar * name, gboolean is_secret,
 
     gtk_container_add(GTK_CONTAINER(scrolled_window), tree_view);
     gtk_window_set_default_size(GTK_WINDOW(dialog), 500, 300);
-    gtk_widget_show_all(GTK_DIALOG(dialog)->vbox);
+    gtk_widget_show_all(gtk_dialog_get_content_area(GTK_DIALOG(dialog)));
 
     if (gtk_dialog_run(GTK_DIALOG(dialog)) != GTK_RESPONSE_OK)
 	use_key = NULL;
@@ -1477,6 +1539,9 @@ accept_low_trust_key(const gchar * name, gpgme_user_id_t uid,
 					   message1,
 					   message2,
 					   _("Use this key anyway?"));
+#if HAVE_MACOSX_DESKTOP
+    libbalsa_macosx_menu_for_parent(dialog, parent);
+#endif
 			      
     /* ask the user */
     result = gtk_dialog_run(GTK_DIALOG(dialog));
@@ -1506,9 +1571,14 @@ get_passphrase_real(GMimeGpgmeContext * ctx, const gchar * uid_hint,
 					 GTK_STOCK_OK, GTK_RESPONSE_OK,
 					 GTK_STOCK_CANCEL,
 					 GTK_RESPONSE_CANCEL, NULL);
+#if HAVE_MACOSX_DESKTOP
+    libbalsa_macosx_menu_for_parent(dialog, parent);
+#endif
     hbox = gtk_hbox_new(FALSE, 12);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 12);
-    gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), hbox);
+    gtk_container_add(GTK_CONTAINER
+                      (gtk_dialog_get_content_area(GTK_DIALOG(dialog))),
+                      hbox);
 
     vbox = gtk_vbox_new(FALSE, 12);
     gtk_container_add(GTK_CONTAINER(hbox), vbox);
@@ -1535,7 +1605,7 @@ get_passphrase_real(GMimeGpgmeContext * ctx, const gchar * uid_hint,
     entry = gtk_entry_new();
     gtk_container_add(GTK_CONTAINER(vbox), entry);
 
-    gtk_widget_show_all(GTK_WIDGET(GTK_DIALOG(dialog)->vbox));
+    gtk_widget_show_all(gtk_dialog_get_content_area(GTK_DIALOG(dialog)));
     gtk_entry_set_width_chars(GTK_ENTRY(entry), 40);
     gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
 
@@ -1691,4 +1761,55 @@ gpg_updates_trustdb(void)
 	return FALSE;
 }
 
+
+/* check if the context contains a public key for the passed recipients */
+#define KEY_IS_OK(k)   (!((k)->expired || (k)->revoked || \
+                          (k)->disabled || (k)->invalid))
+static gboolean
+have_pub_key_for(gpgme_ctx_t gpgme_ctx, InternetAddressList * recipients)
+{
+    gpgme_key_t key;
+    gboolean result = TRUE;
+    time_t now = time(NULL);
+    gint i;
+
+    for (i = 0; result && i < internet_address_list_length(recipients);
+         i++) {
+        InternetAddress *ia =
+            internet_address_list_get_address(recipients, i);
+
+	/* check all entries in the list, handle groups recursively */
+	if (INTERNET_ADDRESS_IS_GROUP(ia))
+	    result =
+                have_pub_key_for(gpgme_ctx,
+                                 INTERNET_ADDRESS_GROUP(ia)->members);
+	else {
+	    if (gpgme_op_keylist_start(gpgme_ctx,
+                                       INTERNET_ADDRESS_MAILBOX(ia)->addr,
+                                       FALSE) != GPG_ERR_NO_ERROR)
+		return FALSE;
+
+	    result = FALSE;
+	    while (!result &&
+		   gpgme_op_keylist_next(gpgme_ctx, &key) == GPG_ERR_NO_ERROR) {
+		/* check if this key and the relevant subkey are usable */
+		if (KEY_IS_OK(key)) {
+		    gpgme_subkey_t subkey = key->subkeys;
+
+		    while (subkey && !subkey->can_encrypt)
+			subkey = subkey->next;
+
+		    if (subkey && KEY_IS_OK(subkey) && 
+			(subkey->expires == 0 || subkey->expires > now))
+			result = TRUE;
+		}
+		gpgme_key_unref(key);
+	    }
+	    gpgme_op_keylist_end(gpgme_ctx);
+	}
+    }
+
+    return result;
+}
+
 #endif				/* HAVE_GPGME */
