@@ -19,25 +19,27 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
  * 02111-1307, USA.
  */
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+
 #if ENABLE_ESMTP
 /*
  * LibBalsaSmtpServer is a subclass of LibBalsaServer.
  */
+#include "server.h"
 
 #include <string.h>
-#include <libesmtp.h>
-
-#ifdef HAVE_GNOME
-#include <gnome.h>
-#endif                          /* HAVE_GNOME */
 
 #include "libbalsa.h"
-#include "server.h"
 #include "smtp-server.h"
 #include "libbalsa-conf.h"
 #include "misc.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
+
+#if HAVE_MACOSX_DESKTOP
+#  include "macosx-helpers.h"
+#endif
 
 static LibBalsaServerClass *parent_class = NULL;
 
@@ -105,8 +107,15 @@ authinteract(auth_client_request_t request, char **result, int fields,
     int i;
 
     for (i = 0; i < fields; i++) {
-        if (request[i].flags & AUTH_PASS)
+        if (request[i].flags & AUTH_PASS) {
+	    /* We need to return a const pointer, this is why we
+	       ignore the result from get_password, and take and
+	       advantage of the fact that this function sets the
+	       passwd field of the server. */
+	    if(!server->passwd)
+		g_free(libbalsa_server_get_password(server, NULL));
             result[i] = server->passwd;
+	}
         else if (request[i].flags & AUTH_USER)
             result[i] = (server->user
                          && *server->user) ? server->user : NULL;
@@ -140,6 +149,7 @@ tlsinteract(char *buf, int buflen, int rwflag, void *arg)
 static void
 libbalsa_smtp_server_init(LibBalsaSmtpServer * smtp_server)
 {
+    LIBBALSA_SERVER(smtp_server)->protocol = "smtp";
     smtp_server->authctx = auth_create_context();
     auth_set_mechanism_flags(smtp_server->authctx, AUTH_PLUGIN_PLAIN, 0);
     auth_set_interact_cb(smtp_server->authctx, authinteract, smtp_server);
@@ -200,6 +210,9 @@ libbalsa_smtp_server_new(void)
 
     smtp_server = g_object_new(LIBBALSA_TYPE_SMTP_SERVER, NULL);
 
+    /* Change the default. */
+    LIBBALSA_SERVER(smtp_server)->remember_passwd = TRUE;
+
     return smtp_server;
 }
 
@@ -231,8 +244,6 @@ libbalsa_smtp_server_new_from_config(const gchar * name)
 void
 libbalsa_smtp_server_save_config(LibBalsaSmtpServer * smtp_server)
 {
-    /* FIXME: isn't it a bit of a brute force? */
-    LIBBALSA_SERVER(smtp_server)->remember_passwd = TRUE;
     libbalsa_server_save_config(LIBBALSA_SERVER(smtp_server));
 
 #if HAVE_SMTP_TLS_CLIENT_CERTIFICATE
@@ -396,28 +407,23 @@ smtp_server_tls_widget(LibBalsaSmtpServer * smtp_server)
 }
 #endif                          /* HAVE_SMTP_TLS_CLIENT_CERTIFICATE */
 
-static const gchar smtp_server_section[] = "smtp-server-config";
-
 static void
 smtp_server_response(GtkDialog * dialog, gint response,
                      struct smtp_server_dialog_info *sdi)
 {
     LibBalsaServer *server = LIBBALSA_SERVER(sdi->smtp_server);
-#ifdef HAVE_GNOME
     GError *error = NULL;
-#endif                          /* HAVE_GNOME */
 
     switch (response) {
     case GTK_RESPONSE_HELP:
-#ifdef HAVE_GNOME
-        gnome_help_display("balsa", smtp_server_section, &error);
+        gtk_show_uri(NULL, "ghelp:balsa?smtp-server-config",
+                     gtk_get_current_event_time(), &error);
         if (error) {
             libbalsa_information(LIBBALSA_INFORMATION_WARNING,
-                                 _("Error displaying %s: %s\n"),
-                                 smtp_server_section, error->message);
+                                 _("Error displaying server help: %s\n"),
+                                 error->message);
             g_error_free(error);
         }
-#endif                          /* HAVE_GNOME */
         return;
     case GTK_RESPONSE_OK:
         libbalsa_smtp_server_set_name(sdi->smtp_server,
@@ -516,7 +522,7 @@ libbalsa_smtp_server_dialog(LibBalsaSmtpServer * smtp_server,
     sdi = g_object_get_data(G_OBJECT(smtp_server),
                             LIBBALSA_SMTP_SERVER_DIALOG_KEY);
     if (sdi) {
-        gdk_window_raise(sdi->dialog->window);
+        gtk_window_present(GTK_WINDOW(sdi->dialog));
         return;
     }
 
@@ -536,6 +542,9 @@ libbalsa_smtp_server_dialog(LibBalsaSmtpServer * smtp_server,
                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                     GTK_STOCK_HELP, GTK_RESPONSE_HELP,
                                     NULL);
+#if HAVE_MACOSX_DESKTOP
+    libbalsa_macosx_menu_for_parent(dialog, parent);
+#endif
     g_object_weak_ref(G_OBJECT(dialog),
 		    (GWeakNotify) smtp_server_weak_notify, sdi);
     g_signal_connect(G_OBJECT(dialog), "response",
@@ -548,7 +557,9 @@ libbalsa_smtp_server_dialog(LibBalsaSmtpServer * smtp_server,
 #define HIG_PADDING 12
     table = libbalsa_create_table(6, 2);
     gtk_container_set_border_width(GTK_CONTAINER(table), HIG_PADDING);
-    gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), table);
+    gtk_container_add(GTK_CONTAINER
+                      (gtk_dialog_get_content_area(GTK_DIALOG(dialog))),
+                      table);
 
     row = 0;
     smtp_server_add_widget(table, row, _("_Descriptive Name:"),
