@@ -19,7 +19,10 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "assistant_helper.h"
 
 #include <errno.h>
 #include <sys/types.h>
@@ -29,8 +32,7 @@
 #include <unistd.h>
 #include <gtk/gtk.h>
 
-#include "assistant_helper.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 #include "libbalsa.h"
 #include "url.h"
 
@@ -109,7 +111,7 @@ entry_changed_cb(GtkEntry * entry, EntryData * ed)
 {
     g_assert(ed != NULL);
 
-    if (entry->text_length) {
+    if (gtk_entry_get_text_length(entry)) {
         ed->master->setbits |= (1 << ed->num);
     } else {
         ed->master->setbits &= ~(1 << ed->num);
@@ -118,8 +120,13 @@ entry_changed_cb(GtkEntry * entry, EntryData * ed)
     /* The stuff below is only when we are displayed... which is not
      * always the case.
      */
+#if GTK_CHECK_VERSION(2, 18, 0)
+    if (!gtk_widget_get_visible(GTK_WIDGET(entry)))
+        return;
+#else                           /* GTK_CHECK_VERSION(2, 18, 0) */
     if (!GTK_WIDGET_VISIBLE(GTK_WIDGET(entry)))
         return;
+#endif                          /* GTK_CHECK_VERSION(2, 18, 0) */
 
     if (GTK_IS_ASSISTANT(ed->druid)) {
         /* Don't let them continue unless all entries have something. */
@@ -211,9 +218,10 @@ balsa_init_create_to_directory(const gchar * dir, gchar ** complaint)
             if (stat(sofar, &sb) < 0) {
                 if (mkdir(sofar, S_IRUSR | S_IWUSR | S_IXUSR) < 0) {
                     (*complaint) =
-                        g_strdup_printf(_
-                                        ("Couldn't create a directory: mkdir() failed on pathname \"%s\"."),
-                                        sofar);
+                        g_strdup_printf(_("Couldn't create a directory:"
+                                          " mkdir() failed on pathname \"%s\","
+                                          " with error \"%s\"."),
+                                        sofar, g_strerror(errno));
                     g_free(sofar);
                     return TRUE;
                 }
