@@ -19,8 +19,9 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
-
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
 #include "assistant_page_user.h"
 
 #include <sys/types.h>
@@ -141,7 +142,6 @@ balsa_druid_page_user_init(BalsaDruidPageUser * user,
                                 _("_Remember your password:"),
                                 remember_passwd, druid,
                                 &(user->remember_passwd));
-    gtk_table_set_row_spacing(table, row++, 10);
 
 #if !defined(ENABLE_TOUCH_UI)
     preset = g_strconcat(g_get_home_dir(), "/mail", NULL);
@@ -246,6 +246,7 @@ balsa_druid_page_user_next(GtkAssistant * druid, GtkWidget * page,
     const gchar *host, *mailbox;
     gchar *uhoh;
     LibBalsaIdentity *ident;
+    InternetAddress *ia;
 #if ENABLE_ESMTP
     LibBalsaSmtpServer *smtp_server;
 #endif /* ENABLE_ESMTP */
@@ -292,9 +293,9 @@ balsa_druid_page_user_next(GtkAssistant * druid, GtkWidget * page,
     } else {
         ident = balsa_app.current_ident;
     }
-    internet_address_set_name(ident->ia, mailbox);
-    internet_address_set_addr(ident->ia,
-                              gtk_entry_get_text(GTK_ENTRY(user->email)));
+    
+    ia = internet_address_mailbox_new (mailbox, gtk_entry_get_text(GTK_ENTRY(user->email)));
+    libbalsa_identity_set_address (ident, ia);
 
     /* outgoing mail */
 #if ENABLE_ESMTP
