@@ -1,6 +1,6 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2002 Stuart Parmenter and others,
+ * Copyright (C) 1997-2008 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,18 +19,30 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
 
+#include <stdlib.h>
 #include <string.h>
+#include <gdk/gdkkeysyms.h>
+#include <gtk/gtk.h>
+#if HAVE_GNOME
 #include <gnome.h>
+#endif
 #ifdef GTKHTML_HAVE_GCONF
 # include <gconf/gconf.h>
 #endif
 
+
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
 #endif
 
+#if HAVE_MACOSX_DESKTOP
+#  include <ige-mac-integration.h>
+#endif
+
 #include "address-book.h"
 #include "address-book-vcard.h"
 #include "address-book-extern.h"
@@ -41,13 +53,17 @@
 #if HAVE_SQLITE
 #include "address-book-gpe.h"
 #endif /* HAVE_SQLITE */
+#if HAVE_RUBRICA
+#include "address-book-rubrica.h"
+#endif /* HAVE_RUBRICA */
 #include "address-book-config.h"
 #include "libbalsa-conf.h"
 #include "libbalsa.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 
 struct ABMainWindow {
     GtkWindow *window;
+    GtkWidget *notebook; /**< notebook containing browse and edit pages. */
     GtkWidget *entry_list; /* GtkTreeView widget */
     GtkWidget *apply_button, *remove_button, *cancel_button;
     GtkWidget *edit_widget;
@@ -66,11 +82,13 @@ struct ABMainWindow {
 
 static void bab_cleanup(void);
 
+#if HAVE_GNOME
 static gint bab_save_session(GnomeClient * client, gint phase,
                              GnomeSaveStyle save_style, gint is_shutdown,
                              GnomeInteractStyle interact_style, gint is_fast,
                              gpointer client_data);
 static gint bab_kill_session(GnomeClient * client, gpointer client_data);
+#endif
 
 static void ab_set_edit_widget(LibBalsaAddress * address,
                                gboolean can_remove);
@@ -96,10 +114,15 @@ bab_config_init(const gchar * group, const gchar * value, gpointer data)
     return FALSE;
 }
 
-static void ab_warning(const char *fmt, ...);
+static void ab_warning(const char *fmt, ...)
+#ifdef __GNUC__
+    __attribute__ ((format (printf, 1, 2)))
+#endif
+;
 
 enum {
     LIST_COLUMN_NAME,
+    LIST_COLUMN_ADDRSPEC,
     LIST_COLUMN_ADDRESS,
     N_COLUMNS
 };
@@ -130,6 +153,7 @@ bab_load_cb(LibBalsaAddressBook *libbalsa_ab,
         /* GtkListStore refs address, and unrefs it when cleared  */
         gtk_list_store_set(GTK_LIST_STORE(model), &iter,
                            LIST_COLUMN_NAME, address->full_name,
+                           LIST_COLUMN_ADDRSPEC, address_string,
                            LIST_COLUMN_ADDRESS, address,
                            -1);
 
@@ -143,6 +167,7 @@ bab_load_cb(LibBalsaAddressBook *libbalsa_ab,
              * cleared */
             gtk_list_store_set(GTK_LIST_STORE(model), &iter,
                                LIST_COLUMN_NAME, address->full_name,
+                               LIST_COLUMN_ADDRSPEC, address_list->data,
                                LIST_COLUMN_ADDRESS, address,
                                -1);
 
@@ -194,6 +219,10 @@ bab_window_set_title(LibBalsaAddressBook * address_book)
     else if (LIBBALSA_IS_ADDRESS_BOOK_GPE(address_book))
         type = "GPE";
 #endif
+#if HAVE_RUBRICA
+    else if (LIBBALSA_IS_ADDRESS_BOOK_RUBRICA(address_book))
+        type = "Rubrica";
+#endif /* HAVE_RUBRICA */
 
     title =
         g_strconcat(type, _(" address book: "), address_book->name, NULL);
@@ -380,6 +409,16 @@ file_new_gpe_cb(GtkAction * action, gpointer user_data)
 }
 #endif /* HAVE_SQLITE */
 
+#if HAVE_RUBRICA
+static void
+file_new_rubrica_cb(GtkAction * action, gpointer user_data)
+{
+    balsa_address_book_config_new_from_type
+        (LIBBALSA_TYPE_ADDRESS_BOOK_RUBRICA, address_book_change,
+         contacts_app.window);
+}
+#endif /* HAVE_RUBRICA */
+
 static void
 file_properties_cb(GtkAction * action, gpointer user_data)
 {
@@ -447,26 +486,74 @@ file_delete_cb(GtkAction * action, gpointer user_data)
 }
 
 static void
-edit_new_person_cb(GtkAction * action, gpointer user_data)
+edit_new_entry_cb(GtkAction * action, gpointer user_data)
 {
     GtkTreeSelection *selection;
 
     contacts_app.displayed_address = NULL;
+    gtk_notebook_set_current_page(GTK_NOTEBOOK(contacts_app.notebook), 1);
     ab_set_edit_widget(NULL, FALSE);
     gtk_widget_set_sensitive(contacts_app.remove_button, FALSE);
     selection = gtk_tree_view_get_selection(GTK_TREE_VIEW
                                             (contacts_app.entry_list));
     gtk_tree_selection_unselect_all(selection);
-    gtk_widget_grab_focus(GTK_CONTAINER(contacts_app.edit_widget)->
-                          focus_child);
+    gtk_widget_grab_focus(gtk_container_get_focus_child
+                          (GTK_CONTAINER(contacts_app.edit_widget)));
+}
+
+static LibBalsaABErr
+ab_remove_address(LibBalsaAddress* address)
+{
+    LibBalsaABErr err = LBABERR_OK;
+
+    g_return_val_if_fail(address, err);
+
+    libbalsa_address_book_remove_address(contacts_app.address_book, address);
+
+    if(err == LBABERR_OK) {
+        GtkTreeIter       iter;
+        GtkTreeSelection *selection;
+        GtkTreeView  *v = GTK_TREE_VIEW(contacts_app.entry_list);
+        GtkTreeModel *m;
+        selection       = gtk_tree_view_get_selection(v);
+        if(gtk_tree_selection_get_selected(selection, &m, &iter))
+            gtk_list_store_remove(GTK_LIST_STORE(m), &iter);
+	if(address == contacts_app.displayed_address) {
+	    ab_clear_edit_widget();
+	    contacts_app.displayed_address = NULL;
+	}
+    } else 
+        ab_warning("Cannot remove: %s\n",
+                   libbalsa_address_book_strerror(contacts_app.address_book,
+                                                  err));
+    return err;
+}
+
+static void
+edit_delete_entry_cb(GtkAction * action, gpointer user_data)
+{
+    GtkTreeView  *v = GTK_TREE_VIEW(contacts_app.entry_list);
+    GtkTreeSelection *selection = gtk_tree_view_get_selection(v);
+    GtkTreeModel *model;
+    GtkTreeIter   iter;
+
+    if(gtk_tree_selection_get_selected(selection, &model, &iter)) {
+	GValue gv = {0,};
+	LibBalsaAddress *address;
+
+	gtk_tree_model_get_value(model, &iter, LIST_COLUMN_ADDRESS, &gv);
+	address = LIBBALSA_ADDRESS(g_value_get_object(&gv));
+	if (address)
+	    ab_remove_address(address);
+    }
 }
 
 /* Normal items */
 static GtkActionEntry entries[] = {
-    {"FileMenu", NULL, "_File"},
-    {"EntryMenu", NULL, "_Entry"},
-    {"HelpMenu", NULL, "_Help"},
-    {"New", GTK_STOCK_NEW, "_New"},
+    {"FileMenu", NULL, N_("_File")},
+    {"EntryMenu", NULL, N_("_Entry")},
+    {"HelpMenu", NULL, N_("_Help")},
+    {"New", GTK_STOCK_NEW, N_("_New")},
     {"NewVcard", NULL, N_("VCard Address Book (GnomeCard)"), NULL, NULL,
      G_CALLBACK(file_new_vcard_cb)},
     {"NewExtern", NULL, N_("External query (a program)"), NULL, NULL,
@@ -481,23 +568,21 @@ static GtkActionEntry entries[] = {
     {"NewGpe", NULL, N_("GPE Address Book"), NULL, NULL,
      G_CALLBACK(file_new_gpe_cb)},
 #endif /* HAVE_SQLITE */
-    {"Properties", GTK_STOCK_PROPERTIES, "_Properties", NULL,
-     "Edit address book properties", G_CALLBACK(file_properties_cb)},
-    {"Delete", GTK_STOCK_DELETE, "_Delete", NULL,
-     "Delete address book", G_CALLBACK(file_delete_cb)},
-    {"Quit", GTK_STOCK_QUIT, "_Quit", NULL, "Exit the program",
+#if HAVE_RUBRICA
+    {"NewRubrica", NULL, N_("Rubrica Address Book"), NULL, NULL,
+     G_CALLBACK(file_new_rubrica_cb)},
+#endif /* HAVE_RUBRICA */
+    {"Properties", GTK_STOCK_PROPERTIES, N_("_Properties"), NULL,
+     N_("Edit address book properties"), G_CALLBACK(file_properties_cb)},
+    {"Delete", GTK_STOCK_DELETE, N_("_Delete"), NULL,
+     N_("Delete address book"), G_CALLBACK(file_delete_cb)},
+    {"Quit", GTK_STOCK_QUIT, N_("_Quit"), NULL, N_("Exit the program"),
      gtk_main_quit},
-    {"NewPerson", GTK_STOCK_NEW, "_New Person", "<shift><control>N",
-     "Add new person", G_CALLBACK(edit_new_person_cb)},
-    {"NewGroup", GTK_STOCK_NEW, "New _Group", "<control>G",
-     "Add new group", NULL},
-    {"About",
-#if GTK_CHECK_VERSION(2, 6, 0)
-     GTK_STOCK_ABOUT,
-#else
-     GNOME_STOCK_ABOUT,
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
-     "_About", NULL, NULL, NULL}
+    {"NewEntry", GTK_STOCK_NEW, N_("_New Entry"), "<shift><control>N",
+     N_("Add new entry"), G_CALLBACK(edit_new_entry_cb)},
+    {"DeleteEntry", GTK_STOCK_NEW, N_("_Delete Entry"), NULL,
+     N_("Delete entry"), G_CALLBACK(edit_delete_entry_cb)},
+    {"About", GTK_STOCK_ABOUT, N_("_About"), NULL, NULL, NULL}
 };
 
 static const char *ui_description =
@@ -514,6 +599,9 @@ static const char *ui_description =
 #if HAVE_SQLITE
 "        <menuitem action='NewGpe'/>"
 #endif /* HAVE_SQLITE */
+#if HAVE_RUBRICA
+"        <menuitem action='NewRubrica'/>"
+#endif /* HAVE_RUBRICA */
 "      </menu>"
 "      <menuitem action='Properties'/>"
 "      <menuitem action='Delete'/>"
@@ -522,8 +610,8 @@ static const char *ui_description =
 "      <separator/>"
 "    </menu>"
 "    <menu action='EntryMenu'>"
-"      <menuitem action='NewPerson'/>"
-"      <menuitem action='NewGroup'/>"
+"      <menuitem action='NewEntry'/>"
+"      <menuitem action='DeleteEntry'/>"
 "    </menu>"
 "    <menu action='HelpMenu'>"
 "      <menuitem action='About'/>"
@@ -538,11 +626,15 @@ get_main_menu(GtkWidget * window, GtkWidget ** menubar,
     GtkActionGroup *action_group;
     GtkUIManager *ui_manager;
     GtkAccelGroup *accel_group;
-    GError *error;
+    GError *error = NULL;
     GList *ab;
+#if HAVE_MACOSX_DESKTOP
+    IgeMacMenuGroup *group;
+#endif
 
     contacts_app.action_group = action_group =
         gtk_action_group_new("MenuActions");
+    gtk_action_group_set_translation_domain(action_group, NULL);
     gtk_action_group_add_actions(action_group, entries,
                                  G_N_ELEMENTS(entries), window);
 
@@ -552,7 +644,6 @@ get_main_menu(GtkWidget * window, GtkWidget ** menubar,
     accel_group = gtk_ui_manager_get_accel_group(ui_manager);
     gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);
 
-    error = NULL;
     if (!gtk_ui_manager_add_ui_from_string(ui_manager, ui_description,
                                            -1, &error)) {
         g_message("building menus failed: %s", error->message);
@@ -562,12 +653,25 @@ get_main_menu(GtkWidget * window, GtkWidget ** menubar,
 
     for (ab = address_books; ab; ab = ab->next)
         add_address_book(LIBBALSA_ADDRESS_BOOK(ab->data));
+    
+#if HAVE_MACOSX_DESKTOP
+    ige_mac_menu_set_menu_bar(GTK_MENU_SHELL(gtk_ui_manager_get_widget(ui_manager, "/MainMenu")));
+    ige_mac_menu_set_quit_menu_item(GTK_MENU_ITEM(gtk_ui_manager_get_widget(ui_manager, "/MainMenu/FileMenu/Quit")));
+
+    group = ige_mac_menu_add_app_menu_group();
+    ige_mac_menu_add_app_menu_item(group,
+				   GTK_MENU_ITEM(gtk_ui_manager_get_widget(ui_manager, "/MainMenu/HelpMenu/About")), 
+                                   NULL);
+#endif
 
     if (menubar)
         /* Finally, return the actual menu bar created by the UIManager. */
         *menubar = gtk_ui_manager_get_widget(ui_manager, "/MainMenu");
 }
 
+static void
+list_row_activated_cb(GtkTreeView *tview, gpointer data);
+
 static void
 ab_set_edit_widget(LibBalsaAddress * address, gboolean can_remove)
 {
@@ -575,7 +679,7 @@ ab_set_edit_widget(LibBalsaAddress * address, gboolean can_remove)
     gtk_widget_show_all(contacts_app.edit_widget);
     gtk_widget_set_sensitive(contacts_app.apply_button, FALSE);
     gtk_widget_set_sensitive(contacts_app.remove_button, can_remove);
-    gtk_widget_set_sensitive(contacts_app.cancel_button, !address);
+    gtk_widget_set_sensitive(contacts_app.cancel_button, TRUE);
 }
 
 static void
@@ -585,16 +689,22 @@ ab_clear_edit_widget(void)
     gtk_widget_set_sensitive(contacts_app.apply_button,  FALSE);
     gtk_widget_set_sensitive(contacts_app.remove_button, FALSE);
     gtk_widget_set_sensitive(contacts_app.cancel_button, FALSE);
+    contacts_app.displayed_address = NULL;
+    gtk_notebook_set_current_page(GTK_NOTEBOOK(contacts_app.notebook), 0);
 }
 
 static void
 list_selection_changed_cb(GtkTreeSelection *selection, gpointer data)
 {
+#if 0
     GtkTreeIter iter;
     GtkTreeModel *model;
     GValue gv = {0,};
     LibBalsaAddress *address;
-
+    if(contacts_app.edit_widget &&
+       GTK_WIDGET_VISIBLE(contacts_app.edit_widget) &&
+       GTK_WIDGET_IS_SENSITIVE(contacts_app.edit_widget))
+	return;
     if(!gtk_tree_selection_get_selected(selection, &model, &iter))
         return;
     gtk_tree_model_get_value(model, &iter, LIST_COLUMN_ADDRESS, &gv);
@@ -602,13 +712,74 @@ list_selection_changed_cb(GtkTreeSelection *selection, gpointer data)
     if (address) {
         if (address != contacts_app.displayed_address)
             ab_set_edit_widget(address, TRUE);
+	/* gtk_widget_set_sensitive(contacts_app.edit_widget, FALSE); */
+    } else
+        ab_clear_edit_widget();
+    g_value_unset(&gv);
+    contacts_app.displayed_address = address;
+#endif
+}
+
+static void
+list_row_activated_cb(GtkTreeView *tree, gpointer data)
+{
+    GtkTreeIter iter;
+    GtkTreeModel *model;
+    GtkTreeSelection *selection;
+    GValue gv = {0,};
+    LibBalsaAddress *address;
+
+    selection = gtk_tree_view_get_selection(tree);
+    if(!gtk_tree_selection_get_selected(selection, &model, &iter))
+        return;
+    gtk_tree_model_get_value(model, &iter, LIST_COLUMN_ADDRESS, &gv);
+    address = LIBBALSA_ADDRESS(g_value_get_object(&gv));
+    if (address) {
+        if (address != contacts_app.displayed_address)
+            ab_set_edit_widget(address, TRUE);
+	printf("Switch page..\n");
+	gtk_notebook_set_current_page(GTK_NOTEBOOK(contacts_app.notebook), 1);
+	/* gtk_widget_set_sensitive(contacts_app.edit_widget, TRUE); */
+	contacts_app.displayed_address = address;
     } else
         ab_clear_edit_widget();
     g_value_unset(&gv);
     contacts_app.displayed_address = address;
 }
 
+static void 
+addrlist_drag_get_cb(GtkWidget* widget, GdkDragContext* drag_context, 
+                     GtkSelectionData* sel_data, guint target_type,
+                     guint time, gpointer user_data)
+{ 
+    GtkTreeView *addrlist;
+    GtkTreeModel *model;
+    GtkTreeSelection *selection;
+    GtkTreeIter iter;
+    LibBalsaAddress *address;
+    GValue gv = {0,};
 
+    g_return_if_fail (widget != NULL);
+    addrlist = GTK_TREE_VIEW(widget);
+
+    switch (target_type) {
+    case LIBBALSA_ADDRESS_TRG_ADDRESS:
+        selection = gtk_tree_view_get_selection(addrlist);
+        if(!gtk_tree_selection_get_selected(selection, &model, &iter))
+            return;
+        gtk_tree_model_get_value(model, &iter, LIST_COLUMN_ADDRESS, &gv);
+        address = LIBBALSA_ADDRESS(g_value_get_object(&gv));
+        gtk_selection_data_set(sel_data,
+                               gtk_selection_data_get_target(sel_data),
+                               8, (const guchar *) &address,
+                               sizeof(LibBalsaAddress*));
+        break;
+    case LIBBALSA_ADDRESS_TRG_STRING:
+        g_print("Text/plain cannot be sent.\n");
+        break;
+    default: g_print("Do not know what to do!\n");
+    }
+}
 
 static GtkWidget *
 bab_window_list_new(gpointer cb_data)
@@ -622,6 +793,7 @@ bab_window_list_new(gpointer cb_data)
     store =
         gtk_list_store_new(N_COLUMNS,
                            G_TYPE_STRING,   /* LIST_COLUMN_NAME           */
+                           G_TYPE_STRING,   /* LIST_COLUMN_ADDRSPEC       */
                            G_TYPE_OBJECT);  /* LIST_COLUMN_ADDRESS        */
     /*
     gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(store), 0,
@@ -633,20 +805,36 @@ bab_window_list_new(gpointer cb_data)
 
     tree = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
     g_object_unref(store);
+    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree));
+    gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
+    g_signal_connect(G_OBJECT(selection), "changed", 
+                     G_CALLBACK(list_selection_changed_cb), cb_data);
+    g_signal_connect(G_OBJECT(tree), "row-activated", 
+                     G_CALLBACK(list_row_activated_cb), cb_data);
 
     renderer = gtk_cell_renderer_text_new();
     column =
-        gtk_tree_view_column_new_with_attributes(_("Name"),
+        gtk_tree_view_column_new_with_attributes(_("_Name"),
                                                  renderer,
                                                  "text",
                                                  LIST_COLUMN_NAME, NULL);
     gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
 
-    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree));
-    gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);
 
-    g_signal_connect(G_OBJECT(selection), "changed", 
-                     G_CALLBACK(list_selection_changed_cb), cb_data);
+    gtk_drag_source_set(GTK_WIDGET(tree), 
+                        GDK_BUTTON1_MASK,
+                        libbalsa_address_target_list, 2,
+                        GDK_ACTION_COPY);
+    g_signal_connect(G_OBJECT(tree), "drag-data-get",
+                     G_CALLBACK(addrlist_drag_get_cb), NULL);
+
+    renderer = gtk_cell_renderer_text_new();
+    column =
+        gtk_tree_view_column_new_with_attributes(_("_Address"),
+                                                 renderer,
+                                                 "text",
+                                                 LIST_COLUMN_ADDRSPEC, NULL);
+    gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
 
     gtk_widget_show(tree);
     return tree;
@@ -683,7 +871,7 @@ apply_button_cb(GtkWidget * w, gpointer data)
 
         gtk_widget_set_sensitive(contacts_app.apply_button,  FALSE);
         gtk_widget_set_sensitive(contacts_app.remove_button, TRUE);
-        gtk_widget_set_sensitive(contacts_app.cancel_button, FALSE);
+        gtk_widget_set_sensitive(contacts_app.cancel_button, TRUE);
 
         /* We need to reload the book; if we modified an existing
          * address, we really need only to reload that one address, but
@@ -701,6 +889,8 @@ apply_button_cb(GtkWidget * w, gpointer data)
         }
         if (!gtk_tree_selection_get_selected(selection, NULL, NULL))
             ab_clear_edit_widget();
+	gtk_notebook_set_current_page(GTK_NOTEBOOK(contacts_app.notebook), 0);
+	
     } else 
         ab_warning(contacts_app.displayed_address ?
                    "Cannot modify: %s\n" : "Cannot add: %s\n",
@@ -712,35 +902,13 @@ apply_button_cb(GtkWidget * w, gpointer data)
 static void
 remove_button_cb(GtkWidget *w, gpointer data)
 {
-    LibBalsaABErr err = LBABERR_OK;
-
-    if (contacts_app.displayed_address)
-        libbalsa_address_book_remove_address(contacts_app.address_book,
-                                             contacts_app.displayed_address);
-    if(err == LBABERR_OK) {
-        GtkTreeIter       iter;
-        GtkTreeSelection *selection;
-        GtkTreeView  *v = GTK_TREE_VIEW(contacts_app.entry_list);
-        GtkTreeModel *m;
-        selection       = gtk_tree_view_get_selection(v);
-        if(gtk_tree_selection_get_selected(selection, &m, &iter))
-            gtk_list_store_remove(GTK_LIST_STORE(m), &iter);
-        ab_clear_edit_widget();
-        contacts_app.displayed_address = NULL;
-    } else 
-        ab_warning("Cannot remove: %s\n",
-                   libbalsa_address_book_strerror(contacts_app.address_book,
-                                                  err));
+    ab_remove_address(contacts_app.displayed_address);
 }
 
 static void
 cancel_button_cb(GtkWidget * w, gpointer data)
 {
-    struct ABMainWindow *abmw = (struct ABMainWindow *) data;
-    if (abmw->displayed_address)
-        ab_set_edit_widget(abmw->displayed_address, TRUE);
-    else
-        ab_clear_edit_widget();
+    ab_clear_edit_widget();
 }
 
 static GtkWidget*
@@ -813,56 +981,80 @@ bab_get_filter_box(void)
                              button);
     return search_hbox;
 }
+static gboolean
+ew_key_pressed(GtkEntry * entry, GdkEventKey * event, struct ABMainWindow *abmw)
+{
+    if (event->keyval != GDK_Escape)
+	return FALSE;
+    gtk_button_clicked(GTK_BUTTON(abmw->cancel_button));
+    return TRUE;
+}
 
 static GtkWidget*
 bab_window_new()
 {
-    GtkWidget* menubar = NULL, *main_vbox, *cont_box, *vbox, *scroll;
-    GtkWidget *wnd = gnome_app_new("Contacts", "Contacts");
-    GtkWidget *edit_box;
+    GtkWidget* menubar = NULL, *main_vbox, *scroll;
+    GtkWidget *wnd = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+    GtkWidget *browse_widget, *edit_widget;
+
+    gtk_window_set_title(GTK_WINDOW(wnd), "Contacts");
 
-    get_main_menu(GTK_WIDGET(wnd), &menubar, contacts_app.address_book_list);
-    if (menubar)
-	gnome_app_set_menus(GNOME_APP(wnd), GTK_MENU_BAR(menubar));
     /* main vbox */
     main_vbox = gtk_vbox_new(FALSE, 1);
+    gtk_container_add(GTK_CONTAINER(wnd), main_vbox);
+
+    get_main_menu(GTK_WIDGET(wnd), &menubar, contacts_app.address_book_list);
+#ifndef HAVE_MACOSX_DESKTOP
+    if (menubar)
+        gtk_box_pack_start(GTK_BOX(main_vbox),
+                           menubar, FALSE, FALSE, 1);
+#endif
+
+    contacts_app.notebook = gtk_notebook_new();
+    gtk_box_pack_start(GTK_BOX(main_vbox),
+		       contacts_app.notebook, TRUE, TRUE, 1);
+
+    browse_widget = gtk_vbox_new(FALSE, 1);
 
     /* Entry widget for finding an address */
-    gtk_box_pack_start(GTK_BOX(main_vbox),
+    gtk_box_pack_start(GTK_BOX(browse_widget),
                        bab_get_filter_box(), FALSE, FALSE, 1);
  
-    cont_box = gtk_hbox_new(FALSE, 1);
-    gtk_box_pack_start(GTK_BOX(main_vbox), cont_box, TRUE,TRUE, 1);
-
     scroll = gtk_scrolled_window_new(NULL, NULL);
     gtk_widget_show(scroll);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
 				   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-    gtk_box_pack_start(GTK_BOX(cont_box), scroll, TRUE, TRUE, 1);
+    gtk_box_pack_start(GTK_BOX(browse_widget), scroll, TRUE, TRUE, 1);
     
     contacts_app.entry_list = bab_window_list_new(&contacts_app);
     gtk_container_add(GTK_CONTAINER(scroll), contacts_app.entry_list);
 
-    vbox = gtk_vbox_new(FALSE, 1);
-    gtk_box_pack_start(GTK_BOX(cont_box), vbox, FALSE, FALSE, 1);
-    edit_box = gtk_vbox_new(FALSE, 1);
+    gtk_notebook_append_page(GTK_NOTEBOOK(contacts_app.notebook), browse_widget,
+			     gtk_label_new("Browse"));
+
+    edit_widget = gtk_vbox_new(FALSE, 1);
     contacts_app.edit_widget = 
         libbalsa_address_get_edit_widget(NULL, contacts_app.entries,
                                          G_CALLBACK(address_changed_cb),
                                          &contacts_app);
-    gtk_box_pack_start(GTK_BOX(edit_box), contacts_app.edit_widget,
+    gtk_box_pack_start(GTK_BOX(edit_widget), contacts_app.edit_widget,
                        FALSE, FALSE, 1);
-    gtk_box_pack_start(GTK_BOX(vbox), edit_box, TRUE, TRUE, 1);
-    gtk_box_pack_start(GTK_BOX(vbox),
+    gtk_box_pack_start(GTK_BOX(edit_widget),
                        bab_get_edit_button_box(&contacts_app),
                        FALSE, FALSE, 1);
+
+    gtk_notebook_append_page(GTK_NOTEBOOK(contacts_app.notebook), edit_widget,
+			     gtk_label_new("Edit"));
+
     /*
     g_signal_connect(G_OBJECT(find_entry), "changed",
 		     G_CALLBACK(balsa_ab_window_find), ab);
     */
+    g_signal_connect(wnd, "key-press-event",
+		     G_CALLBACK(ew_key_pressed), &contacts_app);
     gtk_window_set_default_size(GTK_WINDOW(wnd), 500, 400);
-    gnome_app_set_contents(GNOME_APP(wnd), main_vbox);
 
+    gtk_widget_show_all(wnd);
     return wnd;
 }
 
@@ -884,7 +1076,8 @@ ab_warning(const char *fmt, ...)
     va_end(va_args);
     d = gtk_message_dialog_new(GTK_WINDOW(ab_window),
                                GTK_DIALOG_DESTROY_WITH_PARENT,
-                               GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, msg);
+                               GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+                               "%s", msg);
     gtk_dialog_run(GTK_DIALOG(d));
     gtk_widget_destroy(d);
 }
@@ -901,6 +1094,9 @@ bab_init(void)
 #if HAVE_SQLITE
     LIBBALSA_TYPE_ADDRESS_BOOK_GPE;
 #endif
+#if HAVE_RUBRICA
+    LIBBALSA_TYPE_ADDRESS_BOOK_RUBRICA;
+#endif /* HAVE_RUBRICA */
     memset(&contacts_app, 0, sizeof(contacts_app));
 }
 
@@ -913,9 +1109,8 @@ information_real(void)
 int
 main(int argc, char *argv[])
 {
+#if HAVE_GNOME
     GnomeClient *client;
-#ifdef GTKHTML_HAVE_GCONF
-    GError *gconf_error;
 #endif
 
 #ifdef ENABLE_NLS
@@ -928,6 +1123,7 @@ main(int argc, char *argv[])
 
     /* FIXME: do we need to allow a non-GUI mode? */
     gtk_init_check(&argc, &argv);
+#if HAVE_GNOME
     gnome_program_init(PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv,
 #ifndef GNOME_PARAM_GOPTION_CONTEXT
                        GNOME_PARAM_POPT_TABLE, NULL,
@@ -938,15 +1134,27 @@ main(int argc, char *argv[])
                        GNOME_PARAM_GOPTION_CONTEXT, NULL,
                        GNOME_PARAM_NONE);
 #endif
+#endif
 
 #ifdef GTKHTML_HAVE_GCONF
-    if (!gconf_init(argc, argv, &gconf_error))
-	g_error_free(gconf_error);
-    gconf_error = NULL;
+    gconf_init(argc, argv, NULL);
 #endif
 
     bab_init();
-    libbalsa_init((LibBalsaInformationFunc) information_real);
+    LIBBALSA_TYPE_ADDRESS_BOOK_VCARD;
+    LIBBALSA_TYPE_ADDRESS_BOOK_EXTERN;
+    LIBBALSA_TYPE_ADDRESS_BOOK_LDIF;
+#if ENABLE_LDAP
+    LIBBALSA_TYPE_ADDRESS_BOOK_LDAP;
+#endif
+#if HAVE_SQLITE
+    LIBBALSA_TYPE_ADDRESS_BOOK_GPE;
+#endif
+#if HAVE_RUBRICA
+    LIBBALSA_TYPE_ADDRESS_BOOK_RUBRICA;
+#endif /* HAVE_RUBRICA */
+    libbalsa_real_information_func = (LibBalsaInformationFunc)information_real;
+    g_mime_init(GMIME_ENABLE_RFC2047_WORKAROUNDS);
 
     /* load address book data */
     libbalsa_conf_push_group("Globals");
@@ -964,11 +1172,13 @@ main(int argc, char *argv[])
                      G_CALLBACK(bab_delete_ok), NULL);
 
     /* session management */
+#if HAVE_GNOME
     client = gnome_master_client();
     g_signal_connect(G_OBJECT(client), "save_yourself",
 		     G_CALLBACK(bab_save_session), argv[0]);
     g_signal_connect(G_OBJECT(client), "die",
 		     G_CALLBACK(bab_kill_session), NULL);
+#endif
 
     gtk_widget_show_all(ab_window);
     gtk_widget_hide(contacts_app.edit_widget);
@@ -979,7 +1189,11 @@ main(int argc, char *argv[])
     gdk_threads_enter();
     gtk_main();
     gdk_threads_leave();
-    
+  
+    /* Proper shutdown here */
+    g_list_foreach(contacts_app.address_book_list, (GFunc)g_object_unref, NULL);
+    g_list_free(contacts_app.address_book_list);
+
     return 0;
 }
 
@@ -987,10 +1201,13 @@ main(int argc, char *argv[])
 static void
 bab_cleanup(void)
 {
+#if HAVE_GNOME
     gnome_sound_shutdown();
+#endif
     gtk_main_quit();
 }
 
+#if HAVE_GNOME
 static gint
 bab_kill_session(GnomeClient * client, gpointer client_data)
 {
@@ -1020,3 +1237,4 @@ bab_save_session(GnomeClient * client, gint phase,
 
     return TRUE;
 }
+#endif /* HAVE_GNOME */
