@@ -19,13 +19,17 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "ab-window.h"
 
-#include <gnome.h>
 #include <string.h>
+#include <glib/gi18n.h>
+#include "address-view.h"
 #include "balsa-app.h"
 #include "sendmsg-window.h"
-#include "ab-window.h"
+#include "save-restore.h"
 
 enum {
     LIST_COLUMN_NAME,
@@ -51,8 +55,7 @@ static void balsa_ab_window_dist_mode_toggled(GtkWidget * w,
 						 BalsaAbWindow *ab);
 static void balsa_ab_window_menu_changed(GtkWidget * widget, 
 					    BalsaAbWindow *ab);
-static void balsa_ab_window_run_gnomecard(GtkWidget * widget,
-                                             gpointer data);
+static void balsa_ab_window_run_editor(GtkWidget * widget, gpointer data);
 static void balsa_ab_window_response_cb(BalsaAbWindow *ab, gint resp);
 static void balsa_ab_window_find(GtkWidget * group_entry,
                                     BalsaAbWindow *ab);
@@ -84,10 +87,10 @@ struct _BalsaAbWindowClass
     GtkDialogClass parent_class;
 };
 
-GtkType
+GType
 balsa_ab_window_get_type(void)
 {
-    static GtkType ab_type = 0;
+    static GType ab_type = 0;
 
     if ( !ab_type ) {
 	static const GTypeInfo ab_info = {
@@ -124,13 +127,15 @@ balsa_ab_window_new(gboolean composing, GtkWindow* parent)
                                GTK_STOCK_OK,     GTK_RESPONSE_OK,
                                GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                NULL);
-	gtk_widget_show(GTK_WIDGET(BALSA_AB_WINDOW(ret)->send_to_box));
+	gtk_widget_show(GTK_WIDGET(BALSA_AB_WINDOW(ret)->send_to_label));
+	gtk_widget_show(GTK_WIDGET(BALSA_AB_WINDOW(ret)->send_to_list));
 	gtk_widget_show(GTK_WIDGET(BALSA_AB_WINDOW(ret)->arrow_box));
     } else {
 	gtk_dialog_add_buttons(GTK_DIALOG(ret),
                                GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                                NULL);
-	gtk_widget_hide(GTK_WIDGET(BALSA_AB_WINDOW(ret)->send_to_box));
+	gtk_widget_hide(GTK_WIDGET(BALSA_AB_WINDOW(ret)->send_to_label));
+	gtk_widget_hide(GTK_WIDGET(BALSA_AB_WINDOW(ret)->send_to_list));
 	gtk_widget_hide(GTK_WIDGET(BALSA_AB_WINDOW(ret)->arrow_box));
     }
     if(parent) gtk_window_set_transient_for(GTK_WINDOW(ret), parent);
@@ -188,35 +193,56 @@ balsa_ab_window_list(BalsaAbWindow * ab, GCallback row_activated_cb)
     return tree;
 }
 
+static void
+balsa_ab_window_load_books(BalsaAbWindow * ab)
+{
+    GList *ab_list;
+    guint offset;
+
+    for (ab_list = balsa_app.address_book_list, offset = 0; ab_list;
+         ab_list = ab_list->next, ++offset) {
+        LibBalsaAddressBook *address_book = ab_list->data;
+
+        gtk_combo_box_append_text(GTK_COMBO_BOX(ab->combo_box),
+                                  address_book->name);
+
+        if (ab->current_address_book == NULL)
+            ab->current_address_book = address_book;
+        if (address_book == ab->current_address_book)
+            gtk_combo_box_set_active(GTK_COMBO_BOX(ab->combo_box), offset);
+    }
+}
+
 static void
 balsa_ab_window_init(BalsaAbWindow *ab)
 {
     GtkWidget *find_label,
-	*vbox, *vbox2,
+	*vbox,
 	*w,
+	*table,
 	*hbox,
 	*box2,
 	*scrolled_window,
-#if GTK_CHECK_VERSION(2, 4, 0)
-	*combo_box,
-#else
-	*ab_option, *ab_menu, *menu_item,
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
-	*frame, *label;
-    GList *ab_list;
-    LibBalsaAddressBook *address_book;
-    guint default_offset = 0;
+	*frame;
 
     ab->current_address_book = NULL;
+
+    g_return_if_fail(balsa_app.address_book_list);
     gtk_window_set_title(GTK_WINDOW(ab), _("Address Book"));
 
     g_signal_connect(G_OBJECT(ab), "response",
 		     G_CALLBACK(balsa_ab_window_response_cb), NULL);
 
-    vbox = GTK_DIALOG(ab)->vbox;
+    vbox = gtk_dialog_get_content_area(GTK_DIALOG(ab));
 
     gtk_window_set_wmclass(GTK_WINDOW(ab), "addressbook", "Balsa");
 
+    /* hig defaults */
+    gtk_container_set_border_width(GTK_CONTAINER(ab), 6);
+    gtk_dialog_set_has_separator(GTK_DIALOG(ab), FALSE);
+    gtk_box_set_spacing(GTK_BOX(vbox), 12);
+    
+
     /* The main address list */
     ab->address_list =
         balsa_ab_window_list(ab,
@@ -230,63 +256,27 @@ balsa_ab_window_init(BalsaAbWindow *ab)
                                 (balsa_ab_window_select_recipient));
 
     /* The address book selection menu */
-#if GTK_CHECK_VERSION(2, 4, 0)
-    combo_box = gtk_combo_box_new_text();
-#else
-    ab_menu = gtk_menu_new();
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
+    ab->combo_box = gtk_combo_box_new_text();
 
     ab->current_address_book = balsa_app.default_address_book;
 
-    ab_list = balsa_app.address_book_list;
-    while (ab_list) {
-	address_book = LIBBALSA_ADDRESS_BOOK(ab_list->data);
-	if (ab->current_address_book == NULL)
-	    ab->current_address_book = address_book;
-	
-#if GTK_CHECK_VERSION(2, 4, 0)
-	gtk_combo_box_append_text(GTK_COMBO_BOX(combo_box),
-                                  address_book->name);
-	if (address_book == balsa_app.default_address_book)
-	    gtk_combo_box_set_active(GTK_COMBO_BOX(combo_box),
-                                     default_offset);
-#else
-	menu_item = gtk_menu_item_new_with_label(address_book->name);
-	gtk_widget_show(menu_item);
-	gtk_menu_shell_append(GTK_MENU_SHELL(ab_menu), menu_item);
-	
-	g_object_set_data(G_OBJECT(menu_item), "address-book",
-			  address_book);
-	g_signal_connect(G_OBJECT(menu_item), "activate",
-			 G_CALLBACK(balsa_ab_window_menu_changed), ab);
-	
-	if (address_book == balsa_app.default_address_book)
-	    gtk_menu_set_active(GTK_MENU(ab_menu), default_offset);
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
-	
-	default_offset++;
-	
-	ab_list = g_list_next(ab_list);
-    }
-#if GTK_CHECK_VERSION(2, 4, 0)
-    g_signal_connect(combo_box, "changed",
+    balsa_ab_window_load_books(ab);
+
+    g_signal_connect(ab->combo_box, "changed",
                      G_CALLBACK(balsa_ab_window_menu_changed), ab);
     if (balsa_app.address_book_list->next)
 	/* More than one address book. */
-	gtk_widget_show(combo_box);
+	gtk_widget_show(ab->combo_box);
 
-    gtk_box_pack_start(GTK_BOX(vbox), combo_box, FALSE, FALSE, 0);
-#else
-    gtk_widget_show(ab_menu);
-
-    ab_option = gtk_option_menu_new();
-    gtk_option_menu_set_menu(GTK_OPTION_MENU(ab_option), ab_menu);
-    if(g_list_length(balsa_app.address_book_list)>1)
-        gtk_widget_show(ab_option);
-
-    gtk_box_pack_start(GTK_BOX(vbox), ab_option, FALSE, FALSE, 0);
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
+    gtk_box_pack_start(GTK_BOX(vbox), ab->combo_box, FALSE, FALSE, 0);
 
+    /* layout table */
+    table = gtk_table_new(3, 3, FALSE);
+    gtk_table_set_row_spacings(GTK_TABLE(table), 6);
+    gtk_box_pack_start(GTK_BOX(vbox), table, TRUE, TRUE, 0);
+    gtk_widget_show(table);
+    
+    /* -- table column 1 -- */
     /* Entry widget for finding an address */
     find_label = gtk_label_new_with_mnemonic(_("_Search for Name:"));
     gtk_widget_show(find_label);
@@ -296,20 +286,11 @@ balsa_ab_window_init(BalsaAbWindow *ab)
     gtk_label_set_mnemonic_widget(GTK_LABEL(find_label), ab->filter_entry);
     g_signal_connect(G_OBJECT(ab->filter_entry), "changed",
 		     G_CALLBACK(balsa_ab_window_find), ab);
-    
-    /* Horizontal layout */
-    hbox = gtk_hbox_new(FALSE, 1);
-    gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-    gtk_widget_show(hbox);
 
-    /* Column for address list */
-    vbox2 = gtk_vbox_new(FALSE, 1);
-    gtk_box_pack_start(GTK_BOX(hbox), vbox2, TRUE, TRUE, 1);
-    gtk_widget_show(vbox2);
-
-    /* Pack the find stuff into a box */
+    /* Pack the find stuff into the table */
     box2 = gtk_hbox_new(FALSE, 1);
-    gtk_box_pack_start(GTK_BOX(vbox2), box2, FALSE, FALSE, 1);
+    gtk_table_attach(GTK_TABLE(table), box2, 0, 1, 0, 1,
+		     GTK_FILL, GTK_FILL, 0, 0);
     gtk_box_pack_start(GTK_BOX(box2), find_label, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(box2), ab->filter_entry, TRUE, TRUE, 0);
     gtk_widget_show(GTK_WIDGET(box2));
@@ -320,14 +301,39 @@ balsa_ab_window_init(BalsaAbWindow *ab)
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
 				   GTK_POLICY_AUTOMATIC,
 				   GTK_POLICY_AUTOMATIC);
-    gtk_box_pack_start(GTK_BOX(vbox2), scrolled_window, TRUE, TRUE, 0);
+    gtk_table_attach_defaults(GTK_TABLE(table), scrolled_window, 0, 1, 1, 2);
     gtk_widget_show(scrolled_window);
     gtk_container_add(GTK_CONTAINER(scrolled_window), ab->address_list);
     gtk_widget_set_size_request(scrolled_window, 300, 250);
 
+    /* Buttons ... */
+    hbox = gtk_hbutton_box_new();
+    gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_SPREAD);
+    gtk_table_attach(GTK_TABLE(table), hbox, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
+    gtk_widget_show(GTK_WIDGET(hbox));
+
+    w = balsa_stock_button_with_label(GTK_STOCK_OPEN,
+                                      _("Run Editor"));
+    g_signal_connect(w, "clicked",
+                     G_CALLBACK(balsa_ab_window_run_editor), NULL);
+    gtk_container_add(GTK_CONTAINER(hbox), w);
+    gtk_widget_show(GTK_WIDGET(w));
+
+    w = balsa_stock_button_with_label(GTK_STOCK_ADD, 
+                                      _("_Re-Import"));
+    g_signal_connect(G_OBJECT(w), "clicked",
+                     G_CALLBACK(balsa_ab_window_reload),
+		       ab);
+    gtk_container_add(GTK_CONTAINER(hbox), w);
+    gtk_widget_show(w);
+
+    balsa_ab_window_load(ab);
+
+    /* -- table column 2 -- */
     /* Column for arrows in compose mode */
     ab->arrow_box = gtk_vbox_new(FALSE, 5);
-    gtk_box_pack_start(GTK_BOX(hbox), ab->arrow_box, FALSE, FALSE, 1);
+    gtk_table_attach(GTK_TABLE(table), ab->arrow_box, 1, 2, 1, 2,
+		     GTK_FILL, GTK_EXPAND | GTK_FILL, 6, 0);
     gtk_widget_show(ab->arrow_box);
     
     w = balsa_stock_button_with_label(GTK_STOCK_GO_FORWARD, "");
@@ -344,46 +350,21 @@ balsa_ab_window_init(BalsaAbWindow *ab)
 		     G_CALLBACK(balsa_ab_window_remove_from_recipient_list),
 		       ab);
     
-    /* Column for selected addresses in compose mode */
-    ab->send_to_box = gtk_vbox_new(FALSE, 1);
-    gtk_box_pack_start(GTK_BOX(hbox), ab->send_to_box, TRUE, TRUE, 1);
+    /* -- table column 3 -- */
+    /* label for selected addresses in compose mode */
+    ab->send_to_label = gtk_label_new(_("Send-To"));
+    gtk_widget_show(ab->send_to_label);
+    gtk_table_attach(GTK_TABLE(table), ab->send_to_label, 2, 3, 0, 1,
+		     GTK_FILL, GTK_FILL, 0, 0);
     
-    label = gtk_label_new(_("Send-To"));
-    gtk_widget_show(label);
-    gtk_box_pack_start(GTK_BOX(ab->send_to_box), label,
-		       FALSE, FALSE, 1);
-    
-    scrolled_window = gtk_scrolled_window_new(NULL, NULL);
-    gtk_widget_show(scrolled_window);
-    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
+    /* list for selected addresses in compose mode */
+    ab->send_to_list = gtk_scrolled_window_new(NULL, NULL);
+    gtk_widget_show(ab->send_to_list);
+    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(ab->send_to_list),
 				   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-    gtk_box_pack_start(GTK_BOX(ab->send_to_box), scrolled_window, FALSE, FALSE, 0);
-    gtk_container_add(GTK_CONTAINER(scrolled_window), ab->recipient_list);
-    gtk_widget_set_size_request(scrolled_window, 300, 250);
-
-    /* Buttons ... */
-    hbox = gtk_hbutton_box_new();
-    gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_SPREAD);
-    gtk_box_pack_start(GTK_BOX(vbox2), hbox, FALSE, FALSE, 0);
-    gtk_widget_show(GTK_WIDGET(hbox));
-
-    w = balsa_stock_button_with_label(GTK_STOCK_OPEN,
-                                      _("Run GnomeCard"));
-    g_signal_connect(G_OBJECT(w), "clicked",
-		     G_CALLBACK(balsa_ab_window_run_gnomecard),
-                       NULL);
-    gtk_container_add(GTK_CONTAINER(hbox), w);
-    gtk_widget_show(GTK_WIDGET(w));
-
-    w = balsa_stock_button_with_label(GTK_STOCK_ADD, 
-                                      _("_Re-Import"));
-    g_signal_connect(G_OBJECT(w), "clicked",
-                     G_CALLBACK(balsa_ab_window_reload),
-		       ab);
-    gtk_container_add(GTK_CONTAINER(hbox), w);
-    gtk_widget_show(w);
-
-    balsa_ab_window_load(ab);
+    gtk_table_attach_defaults(GTK_TABLE(table), ab->send_to_list, 2, 3, 1, 2);
+    gtk_container_add(GTK_CONTAINER(ab->send_to_list), ab->recipient_list);
+    gtk_widget_set_size_request(ab->send_to_list, 300, 250);
 
     /* mode switching stuff */
     frame = gtk_frame_new(_("Treat multiple addresses as:"));
@@ -430,11 +411,17 @@ balsa_ab_window_class_init(BalsaAbWindowClass *klass)
   Runs gnome card
 */
 static void
-balsa_ab_window_run_gnomecard(GtkWidget * widget, gpointer data)
+balsa_ab_window_run_editor(GtkWidget * widget, gpointer data)
 {
-    char *argv[] = { "gnomecard" };
+    char *argv[] = { "balsa-ab", NULL };
+    GError * err = NULL;
 
-    gnome_execute_async(NULL, 1, argv);
+    if (!g_spawn_async(NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
+                       NULL, NULL, NULL, &err))
+        balsa_information(LIBBALSA_INFORMATION_WARNING,
+                          _("Could not launch %s: %s"), argv[0],
+                          err ? err->message : "Unknown error");
+    g_clear_error(&err);
 }
 
 /*
@@ -603,15 +590,13 @@ balsa_ab_window_activate_address(GtkTreeView * view,
         gint which_multiple;
         gchar *addr;
 
-        snd = sendmsg_window_new(GTK_WIDGET(balsa_app.main_window), NULL,
-                                 SEND_NORMAL);
         gtk_tree_model_get_iter(model, &iter, path);
         gtk_tree_model_get(model, &iter,
                            LIST_COLUMN_ADDRESS, &address,
                            LIST_COLUMN_WHICH, &which_multiple, -1);
         addr = libbalsa_address_to_gchar(address, which_multiple);
 	g_object_unref(G_OBJECT(address));
-        gtk_entry_set_text(GTK_ENTRY(snd->to[1]), addr);
+        snd = sendmsg_window_compose_with_address(addr);
         g_free(addr);
 
         gtk_widget_grab_focus(snd->subject[1]);
@@ -673,10 +658,41 @@ balsa_ab_window_reload(GtkWidget *w, BalsaAbWindow *ab)
     balsa_ab_window_load(ab);
 }
 
-
 /*
- * Loads the addressbooks into a clist.  
+ * Load the current addressbook.
  */
+static void
+balsa_ab_window_set_title(BalsaAbWindow *ab)
+{
+    LibBalsaAddressBook *address_book = ab->current_address_book;
+    const gchar *type = "";
+    gchar *title;
+
+    if (LIBBALSA_IS_ADDRESS_BOOK_VCARD(address_book))
+        type = "vCard";
+    else if (LIBBALSA_IS_ADDRESS_BOOK_EXTERN(address_book))
+        type = "External query";
+    else if (LIBBALSA_IS_ADDRESS_BOOK_LDIF(address_book))
+        type = "LDIF";
+#if ENABLE_LDAP
+    else if (LIBBALSA_IS_ADDRESS_BOOK_LDAP(address_book))
+        type = "LDAP";
+#endif
+#if HAVE_SQLITE
+    else if (LIBBALSA_IS_ADDRESS_BOOK_GPE(address_book))
+        type = "GPE";
+#endif
+#if HAVE_RUBRICA
+    else if (LIBBALSA_IS_ADDRESS_BOOK_RUBRICA(address_book))
+        type = "Rubrica";
+#endif
+
+    title =
+        g_strconcat(type, _(" address book: "), address_book->name, NULL);
+    gtk_window_set_title(GTK_WINDOW(ab), title);
+    g_free(title);
+}
+
 static void
 balsa_ab_window_load(BalsaAbWindow *ab)
 {
@@ -692,11 +708,10 @@ balsa_ab_window_load(BalsaAbWindow *ab)
 	return;
 
     filter = gtk_entry_get_text(GTK_ENTRY(ab->filter_entry));
-    if( (err=libbalsa_address_book_load(ab->current_address_book, 
-                                        filter,
-                                        (LibBalsaAddressBookLoadFunc)
-                                        balsa_ab_window_load_cb,
-                                        ab)) != LBABERR_OK) {
+    err = libbalsa_address_book_load(ab->current_address_book, filter,
+                                     (LibBalsaAddressBookLoadFunc)
+                                     balsa_ab_window_load_cb, ab);
+    if (err != LBABERR_OK && err != LBABERR_CANNOT_READ) {
 	const gchar *desc =
 	    libbalsa_address_book_strerror(ab->current_address_book, err);
         balsa_information_parented(GTK_WINDOW(ab),
@@ -704,6 +719,7 @@ balsa_ab_window_load(BalsaAbWindow *ab)
 				   _("Error opening address book '%s':\n%s"),
 				   ab->current_address_book->name, desc);
     }
+    balsa_ab_window_set_title(ab);
 }
 
 /*
@@ -841,17 +857,9 @@ static void
 balsa_ab_window_menu_changed(GtkWidget * widget, BalsaAbWindow *ab)
 {
     LibBalsaAddressBook *addr;
-
-#if GTK_CHECK_VERSION(2, 4, 0)
-    addr =
-        LIBBALSA_ADDRESS_BOOK(g_list_nth_data
-                              (balsa_app.address_book_list,
-                               gtk_combo_box_get_active(GTK_COMBO_BOX
-                                                        (widget))));
-#else
-    addr = LIBBALSA_ADDRESS_BOOK(g_object_get_data
-                                 (G_OBJECT(widget), "address-book"));
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
+    int active = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
+    addr = LIBBALSA_ADDRESS_BOOK(g_list_nth_data
+                                 (balsa_app.address_book_list, active));
     g_assert(addr != NULL);
 
     ab->current_address_book = addr;
