@@ -19,17 +19,19 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "address-book-config.h"
 
 #include <gtk/gtk.h>
-#if !GTK_CHECK_VERSION(2, 6, 0)
-#undef GTK_DISABLE_DEPRECATED
-#include <gnome.h>
-#endif
 
 #include "balsa-app.h"
-#include "address-book-config.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
+
+#if HAVE_MACOSX_DESKTOP
+#  include "macosx-helpers.h"
+#endif
 
 typedef struct _AddressBookConfig AddressBookConfig;
 struct _AddressBookConfig {
@@ -66,7 +68,8 @@ struct _AddressBookConfig {
 };
 
 static GtkWidget *create_dialog_from_type(AddressBookConfig * abc);
-static GtkWidget *create_local_dialog(AddressBookConfig * abc);
+static GtkWidget *create_local_dialog(AddressBookConfig * abc,
+                                      const gchar * type);
 static GtkWidget *create_externq_dialog(AddressBookConfig * abc);
 #ifdef ENABLE_LDAP
 static GtkWidget *create_ldap_dialog(AddressBookConfig * abc);
@@ -74,6 +77,9 @@ static GtkWidget *create_ldap_dialog(AddressBookConfig * abc);
 #ifdef HAVE_SQLITE
 static GtkWidget *create_gpe_dialog(AddressBookConfig * abc);
 #endif
+#ifdef HAVE_RUBRICA
+static GtkWidget *create_rubrica_dialog(AddressBookConfig * abc);
+#endif
 
 static void help_button_cb(AddressBookConfig * abc);
 static gboolean handle_close(AddressBookConfig * abc);
@@ -104,7 +110,7 @@ balsa_address_book_config_new(LibBalsaAddressBook * address_book,
     abc = g_object_get_data(G_OBJECT(address_book), "balsa-abc");
     if (abc) {
         /* Only one dialog per address book. */
-        gdk_window_raise(abc->window->window);
+        gtk_window_present(GTK_WINDOW(abc->window));
         return;
     }
 
@@ -155,7 +161,8 @@ edit_book_response(GtkWidget * dialog, gint response,
     }
 
     gtk_widget_destroy(dialog);
-    g_object_set_data(G_OBJECT(abc->address_book), "balsa-abc", NULL);
+    if (abc->address_book)
+        g_object_set_data(G_OBJECT(abc->address_book), "balsa-abc", NULL);
     g_free(abc);
 }
 
@@ -206,10 +213,10 @@ add_radio_buttons(GtkWidget * table, gint row, AddressBookConfig * abc)
 }
 
 static GtkWidget *
-create_local_dialog(AddressBookConfig * abc)
+create_local_dialog(AddressBookConfig * abc, const gchar * type)
 {
     GtkWidget *dialog;
-    const gchar *title;
+    gchar *title;
     const gchar *action;
     const gchar *name;
     GtkWidget *table;
@@ -219,11 +226,11 @@ create_local_dialog(AddressBookConfig * abc)
 
     ab = abc->address_book;
     if (ab) {
-        title = _("Modify Address Book");
+        title = g_strdup_printf(_("Modify %s Address Book"), type);
         action = GTK_STOCK_APPLY;
         name = ab->name;
     } else {
-        title = _("Add Address Book");
+        title = g_strdup_printf(_("Add %s Address Book"), type);
         action = GTK_STOCK_ADD;
         name = NULL;
     }
@@ -235,6 +242,10 @@ create_local_dialog(AddressBookConfig * abc)
                                     action, GTK_RESPONSE_APPLY,
                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                     NULL);
+    g_free(title);
+#if HAVE_MACOSX_DESKTOP
+    libbalsa_macosx_menu_for_parent(dialog, abc->parent);
+#endif
     size_group = libbalsa_create_size_group(dialog);
 
     table = libbalsa_create_table(5, 2);
@@ -269,13 +280,13 @@ create_local_dialog(AddressBookConfig * abc)
 static GtkWidget *
 create_vcard_dialog(AddressBookConfig * abc)
 {
-    return create_local_dialog(abc);
+    return create_local_dialog(abc, "VCARD");
 }
 
 static GtkWidget *
 create_ldif_dialog(AddressBookConfig * abc)
 {
-    return create_local_dialog(abc);
+    return create_local_dialog(abc, "LDIF");
 }
 
 static GtkWidget *
@@ -294,6 +305,10 @@ create_dialog_from_type(AddressBookConfig * abc)
 #ifdef HAVE_SQLITE
     } else if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_GPE) {
         return create_gpe_dialog(abc);
+#endif
+#ifdef HAVE_RUBRICA
+    } else if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_RUBRICA) {
+        return create_rubrica_dialog(abc);
 #endif
     } else {
         g_assert_not_reached();
@@ -303,21 +318,21 @@ create_dialog_from_type(AddressBookConfig * abc)
 }
 
 static GtkWidget *
-create_generic_dialog(AddressBookConfig * abc)
+create_generic_dialog(AddressBookConfig * abc, const gchar * type)
 {
     GtkWidget *dialog;
-    const gchar *title;
+    gchar *title;
     const gchar *action;
     const gchar *name;
     LibBalsaAddressBook *ab;
 
     ab = abc->address_book;
     if (ab) {
-        title = _("Modify Address Book");
+        title = g_strdup_printf(_("Modify %s Address Book"), type);
         action = GTK_STOCK_APPLY;
         name = ab->name;
     } else {
-        title = _("Add Address Book");
+        title = g_strdup_printf(_("Add %s Address Book"), type);
         action = GTK_STOCK_ADD;
         name = NULL;
     }
@@ -329,9 +344,14 @@ create_generic_dialog(AddressBookConfig * abc)
                                     action, GTK_RESPONSE_APPLY,
                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                     NULL);
+    g_free(title);
+#if HAVE_MACOSX_DESKTOP
+    libbalsa_macosx_menu_for_parent(dialog, abc->parent);
+#endif
     gtk_container_set_border_width(GTK_CONTAINER(dialog), 5);
-    gtk_container_set_border_width(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),
-                                   12);
+    gtk_container_set_border_width(GTK_CONTAINER
+                                   (gtk_dialog_get_content_area
+                                    (GTK_DIALOG(dialog))), 12);
     g_signal_connect(G_OBJECT(dialog), "response",
                      G_CALLBACK(edit_book_response), abc);
 
@@ -361,16 +381,10 @@ create_externq_dialog(AddressBookConfig * abc)
     gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
     gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
 		     GTK_FILL, GTK_FILL, 0, 0);
-#if GTK_CHECK_VERSION(2, 6, 0)
     abc->ab_specific.externq.load =
         gtk_file_chooser_button_new
         (_("Select load program for address book"),
          GTK_FILE_CHOOSER_ACTION_OPEN);
-#else /* GTK_CHECK_VERSION(2, 6, 0) */
-    abc->ab_specific.externq.load =
-	gnome_file_entry_new("ExternAddressBookLoadPath",
-			     _("Select load program for address book"));
-#endif /* GTK_CHECK_VERSION(2, 6, 0) */
     gtk_table_attach(GTK_TABLE(table), abc->ab_specific.externq.load, 1, 2,
 		     1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), 
@@ -380,16 +394,10 @@ create_externq_dialog(AddressBookConfig * abc)
     gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
     gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
 		     GTK_FILL, GTK_FILL, 0, 0);
-#if GTK_CHECK_VERSION(2, 6, 0)
     abc->ab_specific.externq.save =
         gtk_file_chooser_button_new
         (_("Select save program for address book"),
          GTK_FILE_CHOOSER_ACTION_OPEN);
-#else /* GTK_CHECK_VERSION(2, 6, 0) */
-    abc->ab_specific.externq.save =
-	gnome_file_entry_new("ExternAddressBookSavePath",
-			     _("Select save program for address book"));
-#endif /* GTK_CHECK_VERSION(2, 6, 0) */
     gtk_table_attach(GTK_TABLE(table), abc->ab_specific.externq.save, 1, 2,
 		     2, 3, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), 
@@ -398,27 +406,18 @@ create_externq_dialog(AddressBookConfig * abc)
     add_radio_buttons(table, 3, abc);
 
     if (ab) {
-#if GTK_CHECK_VERSION(2, 6, 0)
         gtk_file_chooser_set_filename(GTK_FILE_CHOOSER
                                       (abc->ab_specific.externq.load),
                                       ab->load);
         gtk_file_chooser_set_filename(GTK_FILE_CHOOSER
                                       (abc->ab_specific.externq.save),
                                       ab->save);
-#else /* GTK_CHECK_VERSION(2, 6, 0) */
-	GtkWidget *entry;
-	entry = GTK_WIDGET(gnome_file_entry_gtk_entry
-			   (GNOME_FILE_ENTRY(abc->ab_specific.externq.load)));
-	gtk_entry_set_text(GTK_ENTRY(entry), ab->load);
-
-	entry = GTK_WIDGET(gnome_file_entry_gtk_entry
-			   (GNOME_FILE_ENTRY(abc->ab_specific.externq.save)));
-	gtk_entry_set_text(GTK_ENTRY(entry), ab->save);
-#endif /* GTK_CHECK_VERSION(2, 6, 0) */
     }
 
-    dialog = create_generic_dialog(abc);
-    gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), table);
+    dialog = create_generic_dialog(abc, "Extern");
+    gtk_container_add(GTK_CONTAINER
+                      (gtk_dialog_get_content_area(GTK_DIALOG(dialog))),
+                      table);
     return dialog;
 }
 
@@ -482,8 +481,10 @@ create_ldap_dialog(AddressBookConfig * abc)
     g_free(name);
     g_free(host);
     
-    dialog = create_generic_dialog(abc);
-    gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), table);
+    dialog = create_generic_dialog(abc, "LDAP");
+    gtk_container_add(GTK_CONTAINER
+                      (gtk_dialog_get_content_area(GTK_DIALOG(dialog))),
+                      table);
     return dialog;
 }
 #endif
@@ -511,18 +512,29 @@ create_gpe_dialog(AddressBookConfig * abc)
 
     add_radio_buttons(table, 1, abc);
 
-    dialog = create_generic_dialog(abc);
-    gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), table);
+    dialog = create_generic_dialog(abc, "GPE");
+    gtk_container_add(GTK_CONTAINER
+                      (gtk_dialog_get_content_area(GTK_DIALOG(dialog))),
+                      table);
     return dialog;
 }
 #endif
 
+#if HAVE_RUBRICA
+static GtkWidget *
+create_rubrica_dialog(AddressBookConfig * abc)
+{
+    return create_local_dialog(abc, "Rubrica");
+}
+#endif
+
 static void
 help_button_cb(AddressBookConfig * abc)
 {
     GError *err = NULL;
 
-    gnome_help_display("balsa", "preferences-1", &err);
+    gtk_show_uri(NULL, "ghelp:balsa?preferences-address-books",
+                 gtk_get_current_event_time(), &err);
 
     if (err) {
         libbalsa_information(LIBBALSA_INFORMATION_WARNING,
@@ -553,15 +565,6 @@ chooser_bad_path(GtkFileChooser * chooser, GtkWindow * window, gint type)
     return bad_path(gtk_file_chooser_get_filename(chooser), window, type);
 }
 
-#if !GTK_CHECK_VERSION(2, 6, 0)
-static gboolean
-entry_bad_path(GnomeFileEntry * entry, GtkWindow * window, gint type)
-{
-    return bad_path(gnome_file_entry_get_full_path(entry, TRUE), window,
-                    type);
-}
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
-
 static gboolean
 handle_close(AddressBookConfig * abc)
 {
@@ -576,7 +579,6 @@ handle_close(AddressBookConfig * abc)
                      ADDRESS_BOOK_CONFIG_PATH_FILE))
             return FALSE;
     } else if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_EXTERN) {
-#if GTK_CHECK_VERSION(2, 6, 0)
         if (chooser_bad_path(GTK_FILE_CHOOSER(abc->ab_specific.externq.load),
                      GTK_WINDOW(abc->window),
                      ADDRESS_BOOK_CONFIG_PATH_LOAD))
@@ -585,16 +587,6 @@ handle_close(AddressBookConfig * abc)
                      GTK_WINDOW(abc->window),
                      ADDRESS_BOOK_CONFIG_PATH_SAVE))
             return FALSE;
-#else /* GTK_CHECK_VERSION(2, 6, 0) */
-        if (entry_bad_path(GNOME_FILE_ENTRY(abc->ab_specific.externq.load),
-                     GTK_WINDOW(abc->window),
-                     ADDRESS_BOOK_CONFIG_PATH_LOAD))
-            return FALSE;
-        if (entry_bad_path(GNOME_FILE_ENTRY(abc->ab_specific.externq.save),
-                     GTK_WINDOW(abc->window),
-                     ADDRESS_BOOK_CONFIG_PATH_SAVE))
-            return FALSE;
-#endif /* GTK_CHECK_VERSION(2, 6, 0) */
     }
 
     if (abc->address_book == NULL)
@@ -612,9 +604,6 @@ handle_close(AddressBookConfig * abc)
 static gboolean
 bad_path(gchar * path, GtkWindow * window, gint type)
 {
-#if !GTK_CHECK_VERSION(2, 6, 0)
-    gchar *message, *question;
-#endif /* GTK_CHECK_VERSION(2, 6, 0) */
     GtkWidget *ask;
     gint clicked_button;
 
@@ -622,38 +611,15 @@ bad_path(gchar * path, GtkWindow * window, gint type)
         g_free(path);
         return FALSE;
     }
-#if GTK_CHECK_VERSION(2, 6, 0)
     ask = gtk_message_dialog_new(window,
 				 GTK_DIALOG_MODAL|
 				 GTK_DIALOG_DESTROY_WITH_PARENT,
                                  GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
                                  _("No path found.  "
 				   "Do you want to give one?"));
-#else /* GTK_CHECK_VERSION(2, 6, 0) */
-    switch (type) {
-        case ADDRESS_BOOK_CONFIG_PATH_FILE:
-            message =
-                _("The address book file path \"%s\" is not correct. %s");
-            break;
-        case ADDRESS_BOOK_CONFIG_PATH_LOAD:
-            message = _("The load program path \"%s\" is not correct. %s");
-            break;
-        case ADDRESS_BOOK_CONFIG_PATH_SAVE:
-            message = _("The save program path \"%s\" is not correct. %s");
-            break;
-        default:
-            message = _("The path \"%s\" is not correct. %s");
-            break;
-    }
-    question = _("Do you want to correct the path?");
-    ask = gtk_message_dialog_new(window,
-				 GTK_DIALOG_MODAL|
-				 GTK_DIALOG_DESTROY_WITH_PARENT,
-                                 GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
-                                 message, path, question);
-    g_free(path);
-#endif /* GTK_CHECK_VERSION(2, 6, 0) */
-
+#if HAVE_MACOSX_DESKTOP
+    libbalsa_macosx_menu_for_parent(ask, window);
+#endif
     gtk_dialog_set_default_response(GTK_DIALOG(ask), GTK_RESPONSE_YES);
     clicked_button = gtk_dialog_run(GTK_DIALOG(ask));
     gtk_widget_destroy(ask);
@@ -673,18 +639,15 @@ create_book(AddressBookConfig * abc)
             address_book = libbalsa_address_book_vcard_new(name, path);
         g_free(path);
     } else if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_EXTERN) {
-#if GTK_CHECK_VERSION(2, 6, 0)
 #define GET_FILENAME(chooser) \
   gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser))
-#else /* GTK_CHECK_VERSION(2, 6, 0) */
-#define GET_FILENAME(entry) \
-  gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(entry), FALSE)
-#endif /* GTK_CHECK_VERSION(2, 6, 0) */
         gchar *load = GET_FILENAME(abc->ab_specific.externq.load);
         gchar *save = GET_FILENAME(abc->ab_specific.externq.save);
         if (load != NULL && save != NULL)
             address_book =
                 libbalsa_address_book_externq_new(name, load, save);
+        g_free(load);
+        g_free(save);
     } else if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_LDIF) {
         gchar *path =
             gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(abc->window));
@@ -715,6 +678,14 @@ create_book(AddressBookConfig * abc)
     } else if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_GPE) {
         address_book =
             libbalsa_address_book_gpe_new(name);
+#endif
+#ifdef HAVE_RUBRICA
+    } else if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_RUBRICA) {
+        gchar *path =
+            gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(abc->window));
+        if (path != NULL)
+            address_book = libbalsa_address_book_rubrica_new(name, path);
+        g_free(path);
 #endif
     } else
         g_assert_not_reached();
@@ -740,7 +711,11 @@ modify_book(AddressBookConfig * abc)
         g_strdup(gtk_entry_get_text(GTK_ENTRY(abc->name_entry)));
 
     if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_VCARD
-        || abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_LDIF) {
+        || abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_LDIF
+#ifdef HAVE_RUBRICA
+        || abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_RUBRICA
+#endif /* HAVE_RUBRICA */
+        ) {
         LibBalsaAddressBookText *ab_text =
             LIBBALSA_ADDRESS_BOOK_TEXT(address_book);
         gchar *path =
@@ -752,23 +727,12 @@ modify_book(AddressBookConfig * abc)
         }
     } else if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_EXTERN) {
         LibBalsaAddressBookExtern *externq;
-#if GTK_CHECK_VERSION(2, 6, 0)
         gchar *load =
             gtk_file_chooser_get_filename(GTK_FILE_CHOOSER
                                            (abc->ab_specific.externq.load));
         gchar *save =
             gtk_file_chooser_get_filename(GTK_FILE_CHOOSER
                                            (abc->ab_specific.externq.save));
-#else /* GTK_CHECK_VERSION(2, 6, 0) */
-        gchar *load =
-            gnome_file_entry_get_full_path(GNOME_FILE_ENTRY
-                                           (abc->ab_specific.externq.load),
-                                           FALSE);
-        gchar *save =
-            gnome_file_entry_get_full_path(GNOME_FILE_ENTRY
-                                           (abc->ab_specific.externq.save),
-                                           FALSE);
-#endif /* GTK_CHECK_VERSION(2, 6, 0) */
 
         externq = LIBBALSA_ADDRESS_BOOK_EXTERN(address_book);
         if (load) {
@@ -877,6 +841,18 @@ add_gpe_cb(GtkWidget * widget, AddressBookConfig * abc)
 }
 #endif /* HAVE_SQLITE */
 
+#ifdef HAVE_RUBRICA
+static void
+add_rubrica_cb(GtkWidget * widget, AddressBookConfig * abc)
+{
+    g_object_weak_unref(G_OBJECT(gtk_widget_get_parent(widget)),
+                        (GWeakNotify) g_free, abc);
+    abc->type = LIBBALSA_TYPE_ADDRESS_BOOK_RUBRICA;
+    abc->window = create_rubrica_dialog(abc);
+    gtk_widget_show_all(abc->window);
+}
+#endif /* HAVE_SQLITE */
+
 GtkWidget *
 balsa_address_book_add_menu(BalsaAddressBookCallback callback,
                             GtkWindow * parent)
@@ -922,5 +898,12 @@ balsa_address_book_add_menu(BalsaAddressBookCallback callback,
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 #endif /* HAVE_SQLITE */
 
+#ifdef HAVE_RUBRICA
+    menuitem = gtk_menu_item_new_with_label(_("Rubrica2 Address Book"));
+    g_signal_connect(G_OBJECT(menuitem), "activate",
+                     G_CALLBACK(add_rubrica_cb), abc);
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+#endif /* HAVE_RUBRICA */
+
     return menu;
 }
