@@ -19,12 +19,14 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "balsa-app.h"
+
 #include <string.h>
 #include <stdlib.h>
 #ifdef BALSA_USE_THREADS
-/* _XOPEN_SOURCE is needed for rwlocks */
-#define _XOPEN_SOURCE 500
 #include <pthread.h>
 #endif
 
@@ -37,9 +39,12 @@
 #include "misc.h"
 #include "server.h"
 #include "smtp-server.h"
-#include "balsa-app.h"
 #include "save-restore.h"
 
+#if HAVE_MACOSX_DESKTOP
+#  include "macosx-helpers.h"
+#endif
+
 #include <glib/gi18n.h>	/* Must come after balsa-app.h. */
 
 /* Global application structure */
@@ -67,6 +72,7 @@ static gchar *
 ask_password_real(LibBalsaServer * server, LibBalsaMailbox * mbox)
 {
     GtkWidget *dialog, *entry, *rememb;
+    GtkWidget *content_area;
     gchar *prompt, *passwd = NULL;
 #if defined(HAVE_GNOME_KEYRING)
     static const gchar *remember_password_message =
@@ -93,21 +99,25 @@ ask_password_real(LibBalsaServer * server, LibBalsaMailbox * mbox)
                                          GTK_STOCK_OK, GTK_RESPONSE_OK,
                                          GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                          NULL); 
-    gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dialog)->vbox), HIG_PADDING);
-    gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),
+#if HAVE_MACOSX_DESKTOP
+    libbalsa_macosx_menu_for_parent(dialog, GTK_WINDOW(balsa_app.main_window));
+#endif
+    content_area = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
+    gtk_box_set_spacing(GTK_BOX(content_area), HIG_PADDING);
+    gtk_container_add(GTK_CONTAINER(content_area),
                       gtk_label_new_with_mnemonic(prompt));
     g_free(prompt);
-    gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),
+    gtk_container_add(GTK_CONTAINER(content_area),
                       entry = gtk_entry_new());
     gtk_entry_set_width_chars(GTK_ENTRY(entry), 20);
     gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
 
     rememb =  gtk_check_button_new_with_mnemonic(_(remember_password_message));
-    gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), rememb);
+    gtk_container_add(GTK_CONTAINER(content_area), rememb);
     if(server->remember_passwd)
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rememb), TRUE);
 
-    gtk_widget_show_all(GTK_WIDGET(GTK_DIALOG(dialog)->vbox));
+    gtk_widget_show_all(content_area);
     gtk_entry_set_activates_default(GTK_ENTRY(entry), TRUE);
     gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
     gtk_widget_grab_focus (entry);
@@ -327,7 +337,7 @@ balsa_app_init(void)
     balsa_app.expand_tree = FALSE;
     balsa_app.show_mblist = TRUE;
     balsa_app.show_notebook_tabs = FALSE;
-    balsa_app.alternative_layout = FALSE;
+    balsa_app.layout_type = LAYOUT_DEFAULT;
     balsa_app.view_message_on_open = TRUE;
     balsa_app.mw_action_after_move = NEXT_UNREAD;
 
@@ -351,10 +361,13 @@ balsa_app_init(void)
     balsa_app.mblist_totalmsg_width = TOTALMSGCOUNT_DEFAULT_WIDTH;
 
     /* Allocate the best colormap we can get */
-    gtk_widget_set_default_colormap(gdk_rgb_get_colormap());
     balsa_app.visual = gdk_visual_get_best();
-    balsa_app.colormap = gdk_colormap_new(balsa_app.visual, TRUE);
-
+    if (!(balsa_app.colormap = gdk_colormap_new(balsa_app.visual, TRUE))) {
+	balsa_app.visual = gdk_visual_get_system();
+	balsa_app.colormap = gdk_colormap_get_system();
+    }
+    g_assert(balsa_app.colormap);
+    
     /* arp */
     balsa_app.quote_str = NULL;
 
@@ -377,20 +390,14 @@ balsa_app_init(void)
     balsa_app.date_string = g_strdup(DEFAULT_DATE_FORMAT);
 
     /* printing */
-#ifdef HAVE_GTK_PRINT
     balsa_app.print_settings = gtk_print_settings_new();
     balsa_app.page_setup = gtk_page_setup_new();
-#else
-    balsa_app.paper_size = g_strdup(DEFAULT_PAPER_SIZE);
-#endif
 
     balsa_app.print_header_font = g_strdup(DEFAULT_PRINT_HEADER_FONT);
     balsa_app.print_footer_font = g_strdup(DEFAULT_PRINT_FOOTER_FONT);
     balsa_app.print_body_font   = g_strdup(DEFAULT_PRINT_BODY_FONT);
     balsa_app.print_highlight_cited = FALSE;
-#ifdef HAVE_GTK_PRINT
     balsa_app.print_highlight_phrases = FALSE;
-#endif
 
     /* address book */
     balsa_app.address_book_list = NULL;
@@ -421,14 +428,7 @@ balsa_app_init(void)
 
     balsa_app.notify_new_mail_sound = 1;
     balsa_app.notify_new_mail_dialog = 0;
-#if GTK_CHECK_VERSION(2, 10, 0)
     balsa_app.notify_new_mail_icon = 1;
-#endif                          /* GTK_CHECK_VERSION(2, 10, 0) */
-
-#if !GTK_CHECK_VERSION(2, 11, 0)
-    /* Tooltips */
-    balsa_app.tooltips = gtk_tooltips_new();
-#endif                          /* !GTK_CHECK_VERSION(2, 11, 0) */
 
     /* Local and IMAP */
     balsa_app.local_scan_depth = 1;
@@ -445,6 +445,9 @@ balsa_app_init(void)
     /* Message filing */
     balsa_app.folder_mru=NULL;
     balsa_app.fcc_mru=NULL;
+
+    g_object_set (gtk_settings_get_default (),
+		  "gtk-fallback-icon-theme", "gnome", NULL);
 }
 
 void
@@ -471,7 +474,7 @@ balsa_app_destroy(void)
 static gint
 check_new_messages_auto_cb(gpointer data)
 {
-    check_new_messages_real(data, TYPE_BACKGROUND);
+    check_new_messages_real(balsa_app.main_window, TYPE_BACKGROUND);
 
     if (balsa_app.debug)
         fprintf(stderr, "Auto-checked for new messages...\n");
@@ -484,21 +487,13 @@ check_new_messages_auto_cb(gpointer data)
 void
 update_timer(gboolean update, guint minutes)
 {
-    guint32 timeout;
-    timeout = minutes * 60 * 1000;
+    if (balsa_app.check_mail_timer_id)
+        g_source_remove(balsa_app.check_mail_timer_id);
 
-    if (update) {
-	if (balsa_app.check_mail_timer_id)
-	    g_source_remove(balsa_app.check_mail_timer_id);
-
-        balsa_app.check_mail_timer_id =
-            g_timeout_add(timeout,
-                          (GSourceFunc) check_new_messages_auto_cb, NULL);
-    } else {
-	if (balsa_app.check_mail_timer_id)
-	    g_source_remove(balsa_app.check_mail_timer_id);
-	balsa_app.check_mail_timer_id = 0;
-    }
+    balsa_app.check_mail_timer_id = update ?
+        g_timeout_add_seconds(minutes * 60,
+                              (GSourceFunc) check_new_messages_auto_cb,
+                              NULL) : 0;
 }
 
 
@@ -510,13 +505,43 @@ update_timer(gboolean update, guint minutes)
  */
 
 static void
-open_mailboxes_idle_func(const gchar * url, LibBalsaMailboxView * view,
-                         GString * str)
+append_url_if_open(const gchar * url, LibBalsaMailboxView * view,
+                   GPtrArray * array)
 {
-    if (view->open) {
-        if (str->len)
-            g_string_append_c(str, ';');
-        g_string_append(str, url);
+    if (view->open)
+        g_ptr_array_add(array, g_strdup(url));
+}
+
+static void
+open_mailbox_by_url(const gchar * url)
+{
+    LibBalsaMailbox *mailbox;
+
+    if (!(url && *url))
+        return;
+
+    mailbox = balsa_find_mailbox_by_url(url);
+    if (balsa_app.debug)
+        fprintf(stderr, "open_mailboxes_idle_cb: opening %s => %p..\n",
+                url, mailbox);
+    if (mailbox)
+        /* The first mailbox we open will be shown; later mailboxes will
+         * have notebook pages, but will not be shown. */
+        balsa_mblist_open_mailbox_hidden(mailbox);
+    else {
+        /* Do not try to open it next time. */
+        LibBalsaMailboxView *view =
+            g_hash_table_lookup(libbalsa_mailbox_view_table, url);
+        /* The mailbox may have been requested to be open because its
+         * stored view might say so or the user requested it from the
+         * command line - in which case, view may or may not be present.
+         * We will be careful here. */
+        if (view) {
+            view->open = FALSE;
+            view->in_sync = FALSE;
+        }
+        balsa_information(LIBBALSA_INFORMATION_WARNING,
+                          _("Couldn't open mailbox \"%s\""), url);
     }
 }
 
@@ -524,53 +549,35 @@ gboolean
 open_mailboxes_idle_cb(gchar ** urls)
 {
     gchar **tmp;
-    LibBalsaMailbox *mbox;
 
     gdk_threads_enter();
 
     if (!urls) {
-        GString *str;
+        GPtrArray *array;
 
         if (!libbalsa_mailbox_view_table) {
             gdk_threads_leave();
             return FALSE;
         }
 
-        str = g_string_new(NULL);
+        array = g_ptr_array_new();
         g_hash_table_foreach(libbalsa_mailbox_view_table,
-                             (GHFunc) open_mailboxes_idle_func, str);
-        urls = g_strsplit(str->str, ";", 0);
-        g_string_free(str, TRUE);
+                             (GHFunc) append_url_if_open, array);
+        g_ptr_array_add(array, NULL);
+        urls = (gchar **) g_ptr_array_free(array, FALSE);
     }
 
-    for (tmp = urls; *tmp; ++tmp) {
-        mbox = balsa_find_mailbox_by_url(*tmp);
-	if (balsa_app.debug)
-	    fprintf(stderr, "open_mailboxes_idle_cb: opening %s => %p..\n",
-		    *tmp, mbox);
-	if (mbox)
-	    balsa_mblist_open_mailbox(mbox);
-        else {
-	    /* Do not try to open it next time. */
-	    LibBalsaMailboxView *view =
-		g_hash_table_lookup(libbalsa_mailbox_view_table, *tmp);
-            /* The mailbox may have bee requested to be open because
-               its stored view might say so or the user requested it
-               from the command line - in which case, view may or may
-               not be present. We will be careful here. */
-            if(view) {
-                view->open = FALSE;
-                view->in_sync = FALSE;
-            }
-            balsa_information(LIBBALSA_INFORMATION_WARNING,
-                              _("Couldn't open mailbox \"%s\""), *tmp);
-	}
+    if (urls) {
+        if (*urls) {
+            open_mailbox_by_url(balsa_app.current_mailbox_url);
+
+            for (tmp = urls; *tmp; ++tmp)
+                open_mailbox_by_url(*tmp);
+        }
+
+        g_strfreev(urls);
     }
-    g_strfreev(urls);
 
-    if (gtk_notebook_get_current_page(GTK_NOTEBOOK(balsa_app.notebook)) >=
-        0)
-        gtk_notebook_set_current_page(GTK_NOTEBOOK(balsa_app.notebook), 0);
     gdk_threads_leave();
 
     return FALSE;
@@ -725,8 +732,9 @@ balsa_find_url(const gchar * url)
 
     bf.data = url;
     bf.mbnode = NULL;
-    gtk_tree_model_foreach(GTK_TREE_MODEL(balsa_app.mblist_tree_store),
-			   (GtkTreeModelForeachFunc) find_url, &bf);
+    if (balsa_app.mblist_tree_store)
+        gtk_tree_model_foreach(GTK_TREE_MODEL(balsa_app.mblist_tree_store),
+                               (GtkTreeModelForeachFunc) find_url, &bf);
     if (is_sub_thread)
 	gdk_threads_leave();
 
