@@ -1,6 +1,6 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2002 Stuart Parmenter and others,
+ * Copyright (C) 1997-2010 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,14 +22,16 @@
 #ifndef __BALSA_APP_H__
 #define __BALSA_APP_H__
 
-#include <gnome.h>
+#ifndef BALSA_VERSION
+# error "Include config.h before this file."
+#endif
+
 #include "libbalsa.h"
 #include "identity.h"
 #include "balsa-index.h"
 #include "balsa-mblist.h"
 #include "information-dialog.h"
 #include "main-window.h"
-#include "toolbar-factory.h"
 
 /* misc.h for LibBalsaCodeset */
 #include "misc.h"
@@ -85,8 +87,8 @@
 #define MESSAGEBOX_WIDTH 450
 #define MESSAGEBOX_HEIGHT 150
 
-#define DEFAULT_MESSAGE_FONT "Monospace 10"
-#define DEFAULT_SUBJECT_FONT "Monospace Bold 10"
+#define DEFAULT_MESSAGE_FONT "Monospace"
+#define DEFAULT_SUBJECT_FONT "Monospace Bold"
 #define DEFAULT_DATE_FORMAT "%x %X"
 #define DEFAULT_PAPER_SIZE "A4"
 #define DEFAULT_PRINT_HEADER_FONT "Monospace Regular 10"
@@ -208,6 +210,7 @@ extern struct BalsaApplication {
     gint notify_new_mail_sound;
     
     gint notify_new_mail_dialog;
+    gint notify_new_mail_icon;
 
     /* automatically close mailboxes after XX minutes */
     gboolean close_mailbox_auto;
@@ -267,6 +270,7 @@ extern struct BalsaApplication {
     GdkColormap *colormap;
 
     /* Colour of quoted text. */
+    gboolean mark_quoted;
     gchar *quote_regex;
     GdkColor quoted_color[MAX_QUOTED_COLOR];
 
@@ -287,7 +291,7 @@ extern struct BalsaApplication {
     gboolean expand_tree;
     gboolean show_mblist;
     gboolean show_notebook_tabs;
-    gboolean alternative_layout;
+    enum { LAYOUT_DEFAULT, LAYOUT_WIDE_MSG, LAYOUT_WIDE_SCREEN } layout_type;
     gboolean view_message_on_open;
     gboolean pgdownmod;
     gint pgdown_percent;
@@ -296,11 +300,22 @@ extern struct BalsaApplication {
     gboolean enable_view_filter;   /* enable quick view filter */
 #endif
 
+    /* Show toolbars, status bar, and subject-or-sender search bar */
+    gboolean show_main_toolbar;
+    gboolean show_message_toolbar;
+    gboolean show_compose_toolbar;
+    gboolean show_statusbar;
+    gboolean show_sos_bar;
+
     gboolean empty_trash_on_exit;
     gboolean previewpane;
-    gboolean source_escape_specials;
     gboolean debug;
 
+    /* Source viewer */
+    gboolean source_escape_specials;
+    gint source_width;
+    gint source_height;
+
     /* what to do with message window after moving the message */
     MwActionAfterMove mw_action_after_move;
 
@@ -327,6 +342,7 @@ extern struct BalsaApplication {
 
     /* list of currently open mailboxes */
     GList *open_mailbox_list;  /* data is a pointer to the mailbox */
+    gchar *current_mailbox_url;/* remember for next session */
     
     /* font used to display messages */
     gchar *message_font;
@@ -336,31 +352,17 @@ extern struct BalsaApplication {
     gchar *date_string;
 
     /* printing */
-#ifdef HAVE_GTK_PRINT
     GtkPageSetup *page_setup;
     GtkPrintSettings *print_settings;
     gdouble margin_left;
     gdouble margin_top;
     gdouble margin_right;
     gdouble margin_bottom;
-#else
-    gchar* paper_size; /* A4 or Letter */
-    gchar* margin_left;
-    gchar* margin_top;
-    gchar* margin_right;
-    gchar* margin_bottom;
-    gchar* print_unit;
-    gchar* print_layout;
-    gchar* paper_orientation;
-    gchar* page_orientation;
-#endif
     gchar* print_header_font;  /* font for printing headers */
     gchar* print_body_font;    /* font for printing text parts */
     gchar* print_footer_font;  /* font for printing footers */
     gboolean print_highlight_cited;
-#ifdef HAVE_GTK_PRINT
     gboolean print_highlight_phrases;
-#endif
 
     /* compose */
     gchar *compose_headers;
@@ -393,11 +395,6 @@ extern struct BalsaApplication {
     BalsaInformationShow debug_message;
     BalsaInformationShow fatal_message;
 
-#if !GTK_CHECK_VERSION(2, 11, 0)
-    /* Tooltips */
-    GtkTooltips *tooltips;
-#endif                          /* !GTK_CHECK_VERSION(2, 11, 0) */
-
     /* how to act if a MDN request is received */
     BalsaMDNReply mdn_reply_clean;
     BalsaMDNReply mdn_reply_notclean;
@@ -424,7 +421,6 @@ extern struct BalsaApplication {
 
     /* use as default email client for GNOME */
     int default_client;
-
 } balsa_app;
 
 #define BALSA_IS_MAILBOX_SPECIAL(a) ((a)==balsa_app.inbox || (a)==balsa_app.trash || (a)==balsa_app.outbox||(a)==balsa_app.draftbox || (a)==balsa_app.sentbox)
@@ -449,4 +445,8 @@ BalsaIndex* balsa_find_index_by_mailbox(LibBalsaMailbox* mailbox);
 
 void  balsa_remove_children_mailbox_nodes(BalsaMailboxNode * mbnode);
 
+#if USE_GREGEX
+GRegex *balsa_quote_regex_new(void);
+#endif                          /* USE_GREGEX */
+
 #endif				/* __BALSA_APP_H__ */
