@@ -20,14 +20,15 @@
  */
 
 #include <string.h>
-#include <gnome.h>
-#include <gdk/gdkx.h>
-
-#include "config.h"
 
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
 #include "balsa-icons.h"
+
 #include "libbalsa.h"
 #include "mailbox.h"
+#include "address-view.h"
 
 #define ELEMENTS(x) (sizeof (x) / sizeof (x[0]))
 
@@ -64,9 +65,7 @@ balsa_icon_create(const gchar ** data, GdkPixmap ** pmap, GdkBitmap ** bmap)
                                          bmap, 0, (gchar **) data);
 }
 
-#if GTK_CHECK_VERSION(2, 8, 0)
 static GHashTable *balsa_icon_table;
-#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
 
 static void
 load_balsa_pixmap(GtkIconTheme *icon_theme, GtkIconFactory *factory, 
@@ -80,6 +79,13 @@ load_balsa_pixmap(GtkIconTheme *icon_theme, GtkIconFactory *factory,
     static pixmap_fallback_t fallback_id[] = {
 	{ "user-trash", GTK_STOCK_DELETE },
 	{ "user-trash-full", GTK_STOCK_DELETE },
+	{ "mail-attachment", "stock_attach" },
+	{ "emblem-important", "stock_mail-flag-for-followup"},
+        { "stock_mail-reply", "stock_mail-replied"}, 
+        { "stock_mail-reply-to-all", "stock_mail-replied"}, 
+        { "stock_mail-forward", "stock_mail-replied"}, 
+        { "gnome-fs-directory-accept", GTK_STOCK_OPEN},
+        { "gnome-fs-directory", GTK_STOCK_DIRECTORY},
 	{ NULL, NULL } };
 
     BICONS_LOG("loading icon %s (stock id %s)", bpixmap->name,
@@ -91,19 +97,19 @@ load_balsa_pixmap(GtkIconTheme *icon_theme, GtkIconFactory *factory,
 	pixmap_fallback_t *fb = fallback_id;
 	while (fb->def_id && strcmp(fb->def_id, bpixmap->stock_id))
 	    fb++;
-	if (fb->def_id)
+	if (fb->def_id) {
 	    use_id = fb->fb_id;
-	else {
+            BICONS_LOG("\t(%s not found, fall back to %s)",
+                       bpixmap->stock_id, use_id);
+        } else {
 	    BICONS_ERR("icon %s unknown, no fallback", bpixmap->stock_id);
-	    return;
+	    use_id = GTK_STOCK_MISSING_IMAGE;
 	}
     } else
 	use_id = bpixmap->stock_id;
 
-#if GTK_CHECK_VERSION(2, 8, 0)
     g_hash_table_insert(balsa_icon_table, g_strdup(bpixmap->name), 
                         g_strdup(use_id));
-#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
 
     if (!gtk_icon_size_lookup(bpixmap->sizes[0], &width, &height)) {
 	BICONS_ERR("failed: could not look up default icon size %d",
@@ -133,8 +139,7 @@ load_balsa_pixmap(GtkIconTheme *icon_theme, GtkIconFactory *factory,
 	    if (!pixbuf) {
 		BICONS_ERR("additional size %d failed: %s", width,
 			   error->message);
-		g_error_free(error);
-		error = NULL;
+		g_clear_error(&error);
 	    } else {
 		GtkIconSource *icon_source = gtk_icon_source_new();
 		
@@ -144,12 +149,14 @@ load_balsa_pixmap(GtkIconTheme *icon_theme, GtkIconFactory *factory,
 		gtk_icon_source_set_size(icon_source, bpixmap->sizes[n]);
 		gtk_icon_source_set_size_wildcarded(icon_source, FALSE);
 		gtk_icon_set_add_source(icon_set, icon_source);
+                gtk_icon_source_free(icon_source);
 	    }
 	} else
 	    BICONS_ERR("bad size %d", bpixmap->sizes[n]);
     }
 
     gtk_icon_factory_add(factory, bpixmap->name, icon_set);
+    gtk_icon_set_unref(icon_set);
 }
 
 void
@@ -163,6 +170,8 @@ register_balsa_pixmaps(void)
 	  { GTK_ICON_SIZE_LARGE_TOOLBAR, GTK_ICON_SIZE_MENU } },
 	{ BALSA_PIXMAP_REPLY_GROUP,     "stock_mail-reply-to-all", 
 	  { GTK_ICON_SIZE_LARGE_TOOLBAR, GTK_ICON_SIZE_MENU } },
+	{ BALSA_PIXMAP_REQUEST_MDN,     "stock_mail-reply", 
+	  { GTK_ICON_SIZE_LARGE_TOOLBAR, GTK_ICON_SIZE_MENU } },
 	{ BALSA_PIXMAP_SEND,            "stock_mail-send",
 	  { GTK_ICON_SIZE_LARGE_TOOLBAR, GTK_ICON_SIZE_MENU } },
 	{ BALSA_PIXMAP_RECEIVE,         "stock_mail-receive",
@@ -185,7 +194,7 @@ register_balsa_pixmaps(void)
 	  { GTK_ICON_SIZE_LARGE_TOOLBAR, GTK_ICON_SIZE_MENU } },
 	{ BALSA_PIXMAP_MARK_ALL,	"balsa-mark-all",
 	  { GTK_ICON_SIZE_LARGE_TOOLBAR, GTK_ICON_SIZE_MENU } },
-	{ BALSA_PIXMAP_ATTACHMENT,	"stock_attach",
+	{ BALSA_PIXMAP_ATTACHMENT,	"mail-attachment",
 	  { GTK_ICON_SIZE_LARGE_TOOLBAR, GTK_ICON_SIZE_MENU } },
 	{ BALSA_PIXMAP_NEXT,		"balsa-next",
 	  { GTK_ICON_SIZE_LARGE_TOOLBAR, GTK_ICON_SIZE_MENU } },
@@ -270,20 +279,22 @@ register_balsa_pixmaps(void)
 	  { GTK_ICON_SIZE_MENU, GTK_ICON_SIZE_INVALID } },
         { BALSA_PIXMAP_INFO_NEW,        "stock_mail-unread",
 	  { GTK_ICON_SIZE_MENU, GTK_ICON_SIZE_INVALID } },
-	{ BALSA_PIXMAP_INFO_FLAGGED,    "stock_mail-flag-for-followup",
+	{ BALSA_PIXMAP_INFO_FLAGGED,    "emblem-important",
 	  { GTK_ICON_SIZE_MENU, GTK_ICON_SIZE_INVALID } },
 
+        /* drop-down icon for the address-view (16x16) */
+	{ BALSA_PIXMAP_DROP_DOWN,       "balsa-drop-down",
+	  { GTK_ICON_SIZE_MENU, GTK_ICON_SIZE_INVALID } },
 	};
 
     unsigned i;
     GtkIconFactory *factory = gtk_icon_factory_new();
     GtkIconTheme *icon_theme = gtk_icon_theme_get_default();
 
-#if GTK_CHECK_VERSION(2, 8, 0)
     balsa_icon_table =
         g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
-#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
     gtk_icon_factory_add_default(factory);
+    gtk_icon_theme_append_search_path(icon_theme, BALSA_STD_PREFIX "/share/icons");
     gtk_icon_theme_append_search_path(icon_theme, BALSA_DATA_PREFIX);
 
     for (i = 0; i < ELEMENTS(balsa_icons); i++)
@@ -311,6 +322,10 @@ register_balsa_pixbufs(GtkWidget * widget)
 	libbalsa_mailbox_set_sign_icon, BALSA_PIXMAP_SIGN}, {
 	libbalsa_mailbox_set_encr_icon, BALSA_PIXMAP_ENCR},
 #endif
+        {
+        libbalsa_address_view_set_book_icon,  BALSA_PIXMAP_BOOK_RED}, {
+        libbalsa_address_view_set_close_icon, GTK_STOCK_CLOSE}, {
+        libbalsa_address_view_set_drop_down_icon, BALSA_PIXMAP_DROP_DOWN},
     };
     guint i;
 
@@ -321,7 +336,6 @@ register_balsa_pixbufs(GtkWidget * widget)
 						 NULL));
 }
 
-#if GTK_CHECK_VERSION(2, 8, 0)
 const gchar *
 balsa_icon_id(const gchar * name)
 {
@@ -329,4 +343,3 @@ balsa_icon_id(const gchar * name)
 
     return retval ? retval : name;
 }
-#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
