@@ -19,22 +19,26 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "balsa-mblist.h"
 
-#include <gnome.h>
 /* #include <gtk/gtkfeatures.h> */
 #include <string.h>
-#include <gdk/gdkfont.h>
 
 #include "balsa-app.h"
 #include "balsa-icons.h"
 #include "balsa-index.h"
-#include "balsa-mblist.h"
 #include "libbalsa.h"
 #include <glib/gi18n.h>
 #include "mailbox-node.h"
 #include "main-window.h"
 
+#if HAVE_MACOSX_DESKTOP
+#  include "macosx-helpers.h"
+#endif
+
 /* Column numbers (used for sort_column_id): */
 typedef enum {
     BMBL_TREE_COLUMN_NAME = 1,
@@ -139,10 +143,10 @@ static void bmbl_expand_to_row(BalsaMBList * mblist, GtkTreePath * path);
 
 /* class methods */
 
-GtkType
+GType
 balsa_mblist_get_type(void)
 {
-    static GtkType mblist_type = 0;
+    static GType mblist_type = 0;
 
     if (!mblist_type) {
 	static const GTypeInfo mblist_info = {
@@ -225,11 +229,7 @@ bmbl_set_property_node_style(GSList * list)
             GtkTreeModel *model;
             GtkTreeIter iter;
 
-#if GTK_CHECK_VERSION(2, 8, 0)
             model = gtk_tree_row_reference_get_model(reference);
-#else                           /* GTK_CHECK_VERSION(2, 8, 0) */
-            model = GTK_TREE_MODEL(balsa_app.mblist_tree_store);
-#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
             if (gtk_tree_model_get_iter(model, &iter, path))
                 bmbl_node_style(model, &iter);
             gtk_tree_path_free(path);
@@ -359,11 +359,7 @@ bmbl_init(BalsaMBList * mblist)
     renderer = gtk_cell_renderer_pixbuf_new();
     gtk_tree_view_column_pack_start(column, renderer, FALSE);
     gtk_tree_view_column_set_attributes(column, renderer,
-#if GTK_CHECK_VERSION(2, 8, 0)
                                         "icon-name", ICON_COLUMN,
-#else                           /* GTK_CHECK_VERSION(2, 8, 0) */
-                                        "pixbuf", ICON_COLUMN,
-#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
                                         NULL);
     renderer = gtk_cell_renderer_text_new();
     gtk_tree_view_column_pack_start(column, renderer, FALSE);
@@ -487,11 +483,7 @@ balsa_mblist_get_store(void)
         balsa_app.mblist_tree_store =
             gtk_tree_store_new(N_COLUMNS,
                                G_TYPE_OBJECT,     /* MBNODE_COLUMN */
-#if GTK_CHECK_VERSION(2, 8, 0)
                                G_TYPE_STRING,     /* ICON_COLUMN   */
-#else                           /* GTK_CHECK_VERSION(2, 8, 0) */
-                               GDK_TYPE_PIXBUF,   /* ICON_COLUMN   */
-#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
                                G_TYPE_STRING,     /* NAME_COLUMN   */
                                PANGO_TYPE_WEIGHT, /* WEIGHT_COLUMN */
                                PANGO_TYPE_STYLE,  /* STYLE_COLUMN */
@@ -556,30 +548,11 @@ bmbl_tree_expand(GtkTreeView * tree_view, GtkTreeIter * iter,
     gtk_tree_model_get(model, iter, MBNODE_COLUMN, &mbnode, -1);
     balsa_mailbox_node_scan_children(mbnode);
 
-    if (!mbnode->mailbox) {
-#if GTK_CHECK_VERSION(2, 8, 0)
+    if (!mbnode->mailbox)
         gtk_tree_store_set(GTK_TREE_STORE(model), iter,
                            ICON_COLUMN,
                            balsa_icon_id(BALSA_PIXMAP_MBOX_DIR_OPEN),
                            -1);
-#else                           /* GTK_CHECK_VERSION(2, 8, 0) */
-        GdkPixbuf *pixbuf =
-            gtk_widget_render_icon(GTK_WIDGET(balsa_app.main_window),
-                                   BALSA_PIXMAP_MBOX_DIR_OPEN,
-                                   GTK_ICON_SIZE_MENU, NULL);
-        gtk_tree_store_set(GTK_TREE_STORE(model), iter,
-                           ICON_COLUMN, pixbuf,
-                           -1);
-        if(pixbuf) g_object_unref(pixbuf);
-        else { 
-            static int p=1;
-            if(p){
-                printf("icon %s not found.\n", BALSA_PIXMAP_MBOX_DIR_OPEN);
-                p = 0;
-            }
-        }
-#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
-    }
     g_object_unref(mbnode);
 
     if (gtk_tree_model_iter_children(model, &child_iter, iter)) {
@@ -647,30 +620,11 @@ bmbl_tree_collapse(GtkTreeView * tree_view, GtkTreeIter * iter,
 
     gtk_tree_model_get(model, iter, MBNODE_COLUMN, &mbnode, -1);
 
-    if (!mbnode->mailbox) {
-#if GTK_CHECK_VERSION(2, 8, 0)
+    if (!mbnode->mailbox)
         gtk_tree_store_set(GTK_TREE_STORE(model), iter,
                            ICON_COLUMN, 
                            balsa_icon_id(BALSA_PIXMAP_MBOX_DIR_CLOSED),
                            -1);
-#else                           /* GTK_CHECK_VERSION(2, 8, 0) */
-        GdkPixbuf *pixbuf =
-            gtk_widget_render_icon(GTK_WIDGET(balsa_app.main_window),
-                                   BALSA_PIXMAP_MBOX_DIR_CLOSED,
-                                   GTK_ICON_SIZE_MENU, NULL);
-        gtk_tree_store_set(GTK_TREE_STORE(model), iter,
-                           ICON_COLUMN, pixbuf,
-                           -1);
-        if(pixbuf) g_object_unref(pixbuf);
-        else { 
-            static int p=1;
-            if(p){
-                printf("icon %s not found.\n", BALSA_PIXMAP_MBOX_DIR_CLOSED);
-                p = 0;
-            }
-        }
-#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
-    }
     g_object_unref(mbnode);
 
     bmbl_tree_collapse_helper(model, iter);
@@ -803,12 +757,7 @@ bmbl_do_popup(GtkTreeView * tree_view, GtkTreePath * path,
 
     menu = balsa_mailbox_node_get_context_menu(mbnode);
     g_object_ref(menu);
-#if GLIB_CHECK_VERSION(2, 10, 0)
     g_object_ref_sink(menu);
-#else                           /* GLIB_CHECK_VERSION(2, 10, 0) */
-    g_object_ref(menu);
-    gtk_object_sink(GTK_OBJECT(menu));
-#endif                          /* GLIB_CHECK_VERSION(2, 10, 0) */
     gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
 		   event_button, event_time);
     g_object_unref(menu);
@@ -873,11 +822,12 @@ bmbl_drag_cb(GtkWidget * widget, GdkDragContext * context,
     BalsaIndex *orig_index;
     GArray *selected;
 
-    if (!selection_data || !selection_data->data)
+    if (!selection_data || !gtk_selection_data_get_data(selection_data))
 	/* Drag'n'drop is weird... */
 	return;
 
-    orig_index = *(BalsaIndex **) selection_data->data;
+    orig_index =
+        *(BalsaIndex **) gtk_selection_data_get_data(selection_data);
     selected = balsa_index_selected_msgnos_new(orig_index);
     if (selected->len == 0) {
 	/* it is actually possible to drag from GtkTreeView when no rows
@@ -956,7 +906,9 @@ bmbl_select_mailbox(GtkTreeSelection * selection, gpointer data)
     if (event->type != GDK_BUTTON_PRESS
             /* keyboard navigation */
         || event->button.button != 1
-            /* soft select */ ) {
+            /* soft select */
+        || event->button.window != gtk_tree_view_get_bin_window(tree_view)
+            /* click on a different widget */ ) {
         gdk_event_free(event);
         return;
     }
@@ -1152,6 +1104,10 @@ balsa_mblist_find_all_unread_mboxes(LibBalsaMailbox * mailbox)
     info.mailbox = mailbox;
     info.list = NULL;
 
+    if (!balsa_app.mblist_tree_store) /* We have no mailboxes, maybe
+					 we are about to quit? */
+	return NULL;
+
     gtk_tree_model_foreach(GTK_TREE_MODEL(balsa_app.mblist_tree_store),
                            bmbl_find_all_unread_mboxes_func, &info);
 
@@ -1164,8 +1120,8 @@ balsa_mblist_find_all_unread_mboxes(LibBalsaMailbox * mailbox)
  * mailbox page, or if a new page needs to be created and the mailbox
  * parsed.
  */
-void
-balsa_mblist_open_mailbox(LibBalsaMailbox * mailbox)
+static void
+bmbl_open_mailbox(LibBalsaMailbox * mailbox, gboolean set_current)
 {
     int i;
     GtkWidget *index;
@@ -1186,20 +1142,34 @@ balsa_mblist_open_mailbox(LibBalsaMailbox * mailbox)
     
     i = balsa_find_notebook_page_num(mailbox);
     if (i != -1) {
-	gtk_notebook_set_current_page(GTK_NOTEBOOK(balsa_app.notebook), i);
-        index = balsa_window_find_current_index(balsa_app.main_window);
-	time(&BALSA_INDEX(index)->mailbox_node->last_use);
-        balsa_index_set_column_widths(BALSA_INDEX(index));
+        if (set_current) {
+            gtk_notebook_set_current_page(GTK_NOTEBOOK(balsa_app.notebook),
+                                          i);
+            index = balsa_window_find_current_index(balsa_app.main_window);
+            time(&BALSA_INDEX(index)->mailbox_node->last_use);
+            balsa_index_set_column_widths(BALSA_INDEX(index));
+        }
     } else { /* page with mailbox not found, open it */
-	balsa_window_open_mbnode(balsa_app.main_window, mbnode);
+        balsa_window_open_mbnode(balsa_app.main_window, mbnode,
+                                 set_current);
 
 	if (balsa_app.mblist->display_info)
 	    balsa_mblist_update_mailbox(balsa_app.mblist_tree_store,
                                         mailbox);
     }
     g_object_unref(mbnode);
-    
-    balsa_mblist_set_status_bar(mailbox);
+}
+
+void
+balsa_mblist_open_mailbox(LibBalsaMailbox * mailbox)
+{
+    bmbl_open_mailbox(mailbox, TRUE);
+}
+
+void
+balsa_mblist_open_mailbox_hidden(LibBalsaMailbox * mailbox)
+{
+    bmbl_open_mailbox(mailbox, FALSE);
 }
 
 void
@@ -1330,9 +1300,6 @@ bmbl_store_redraw_mbnode(GtkTreeIter * iter, BalsaMailboxNode * mbnode)
     const gchar *icon;
     const gchar *name;
     gchar *tmp = NULL;
-#if !GTK_CHECK_VERSION(2, 8, 0)
-    GdkPixbuf *pixbuf;
-#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
     gboolean expose = FALSE;
 
     g_return_val_if_fail(mbnode, FALSE);
@@ -1407,7 +1374,6 @@ bmbl_store_redraw_mbnode(GtkTreeIter * iter, BalsaMailboxNode * mbnode)
         name = tmp = g_path_get_basename(mbnode->name);
     }
 
-#if GTK_CHECK_VERSION(2, 8, 0)
     gtk_tree_store_set(balsa_app.mblist_tree_store, iter,
                        MBNODE_COLUMN, mbnode,
                        ICON_COLUMN, balsa_icon_id(icon),
@@ -1417,25 +1383,6 @@ bmbl_store_redraw_mbnode(GtkTreeIter * iter, BalsaMailboxNode * mbnode)
                        UNREAD_COLUMN, "",
                        TOTAL_COLUMN,  "",
                        -1);
-#else                           /* GTK_CHECK_VERSION(2, 8, 0) */
-    /* render icon may fail if the installation was incomplete */
-    pixbuf = gtk_widget_render_icon(GTK_WIDGET(balsa_app.main_window),
-                                    icon, GTK_ICON_SIZE_MENU, NULL);
-    gtk_tree_store_set(balsa_app.mblist_tree_store, iter,
-                       MBNODE_COLUMN, mbnode,
-                       ICON_COLUMN, pixbuf,
-                       NAME_COLUMN,   name,
-                       WEIGHT_COLUMN, PANGO_WEIGHT_NORMAL,
-                       STYLE_COLUMN, PANGO_STYLE_NORMAL,
-                       UNREAD_COLUMN, "",
-                       TOTAL_COLUMN,  "",
-                       -1);
-    if(pixbuf) g_object_unref(pixbuf);
-    else { 
-        static int p=1;
-        if(p){ printf("icon %s not found.\n", icon); p = 0; }
-    }
-#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
     g_free(tmp);
 
     if (mbnode->mailbox) {
@@ -1479,7 +1426,7 @@ bmbl_update_mailbox(GtkTreeStore * store, LibBalsaMailbox * mailbox)
     if (!bindex || mailbox != BALSA_INDEX(bindex)->mailbox_node->mailbox)
         return;
 
-    balsa_mblist_set_status_bar(mailbox);
+    balsa_window_set_statusbar(balsa_app.main_window, mailbox);
 }
 
 void
@@ -1526,9 +1473,6 @@ bmbl_node_style(GtkTreeModel * model, GtkTreeIter * iter)
         const gchar *name;
         gchar *tmp = NULL;
         PangoWeight weight;
-#if !GTK_CHECK_VERSION(2, 8, 0)
-        GdkPixbuf *pixbuf;
-#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
 
         /* Set the style appropriate for unread_messages; we do this
          * even if the state hasn't changed, because we might be
@@ -1556,26 +1500,11 @@ bmbl_node_style(GtkTreeModel * model, GtkTreeIter * iter)
             mbnode->style &= ~MBNODE_STYLE_NEW_MAIL;
         }
 
-#if GTK_CHECK_VERSION(2, 8, 0)
         gtk_tree_store_set(GTK_TREE_STORE(model), iter,
                            ICON_COLUMN, balsa_icon_id(icon),
                            NAME_COLUMN, name,
                            WEIGHT_COLUMN, weight,
                            -1);
-#else                           /* GTK_CHECK_VERSION(2, 8, 0) */
-        pixbuf = gtk_widget_render_icon(GTK_WIDGET(balsa_app.main_window),
-                                        icon, GTK_ICON_SIZE_MENU, NULL);
-        gtk_tree_store_set(GTK_TREE_STORE(model), iter,
-                           ICON_COLUMN, pixbuf,
-                           NAME_COLUMN, name,
-                           WEIGHT_COLUMN, weight,
-                           -1);
-        if(pixbuf) g_object_unref(pixbuf);
-        else { 
-            static int p=1;
-            if(p){ printf("icon %s not found.\n", icon); p = 0; }
-        }
-#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
         g_free(tmp);
 
     }
@@ -1916,6 +1845,9 @@ bmbl_mru_show_tree(GtkWidget * widget, gpointer data)
         gtk_tree_view_get_selection(GTK_TREE_VIEW(mblist));
     GtkRequisition req;
 
+#if HAVE_MACOSX_DESKTOP
+    libbalsa_macosx_menu_for_parent(dialog, mru->window);
+#endif
     scroll = gtk_scrolled_window_new(NULL, NULL);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
                                    GTK_POLICY_AUTOMATIC,
@@ -1938,7 +1870,9 @@ bmbl_mru_show_tree(GtkWidget * widget, gpointer data)
     gtk_widget_set_size_request(GTK_WIDGET(mblist), req.width, req.height);
 
     gtk_container_add(GTK_CONTAINER(scroll), mblist);
-    gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), scroll);
+    gtk_container_add(GTK_CONTAINER
+                      (gtk_dialog_get_content_area(GTK_DIALOG(dialog))),
+                      scroll);
     gtk_widget_show_all(scroll);
 
     gtk_dialog_run(GTK_DIALOG(dialog));
@@ -2084,21 +2018,12 @@ bmbl_mru_combo_box_setup(GtkComboBox * combo_box)
     BalsaMBListMRUOption *mro =
         g_object_get_data(G_OBJECT(combo_box), "mro");
     GList *list;
-#if GTK_CHECK_VERSION(2, 6, 0)
     GtkListStore *store;
     GtkTreeIter iter;
-#else                           /* GTK_CHECK_VERSION(2, 6, 0) */
-    gint i;
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
 
     gtk_combo_box_set_active(combo_box, -1);
-#if GTK_CHECK_VERSION(2, 6, 0)
     store = GTK_LIST_STORE(gtk_combo_box_get_model(combo_box));
     gtk_list_store_clear(store);
-#else                           /* GTK_CHECK_VERSION(2, 6, 0) */
-    for (i = g_slist_length(mro->real_urls) + 1; --i >= 0;)
-        gtk_combo_box_remove_text(combo_box, i);
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
     g_slist_foreach(mro->real_urls, (GFunc) g_free, NULL);
     g_slist_free(mro->real_urls);
     mro->real_urls = NULL;
@@ -2108,20 +2033,14 @@ bmbl_mru_combo_box_setup(GtkComboBox * combo_box)
         LibBalsaMailbox *mailbox;
 
         if ((mailbox = balsa_find_mailbox_by_url(url)) || !*url) {
-#if GTK_CHECK_VERSION(2, 6, 0)
 	    gtk_list_store_append(store, &iter);
 	    gtk_list_store_set(store, &iter,
                                0, mailbox ? mailbox->name : "",
                                1, FALSE, -1);
-#else                           /* GTK_CHECK_VERSION(2, 6, 0) */
-            gtk_combo_box_append_text(GTK_COMBO_BOX(combo_box),
-                                      mailbox ? mailbox->name : "");
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
             mro->real_urls = g_slist_append(mro->real_urls, g_strdup(url));
         }
     }
 
-#if GTK_CHECK_VERSION(2, 6, 0)
     /* Separator: */
     gtk_list_store_append(store, &iter);
     gtk_list_store_set(store, &iter, 1, TRUE, -1);
@@ -2129,9 +2048,6 @@ bmbl_mru_combo_box_setup(GtkComboBox * combo_box)
     gtk_list_store_set(store, &iter,
                        0, _("Other..."),
 		       1, FALSE, -1);
-#else                           /* GTK_CHECK_VERSION(2, 6, 0) */
-    gtk_combo_box_append_text(GTK_COMBO_BOX(combo_box), _("Other..."));
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
     gtk_combo_box_set_active(combo_box, 0);
 }
 
@@ -2186,7 +2102,6 @@ bmbl_mru_combo_box_destroy_cb(BalsaMBListMRUOption * mro)
  * Adds a last entry that pops up the whole mailbox tree. When an item
  * is clicked, the url_list is updated.
  */
-#if GTK_CHECK_VERSION(2, 6, 0)
 static gboolean
 bmbl_mru_separator_func(GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
 {
@@ -2196,21 +2111,17 @@ bmbl_mru_separator_func(GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
 
     return is_sep;
 }
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
 
 GtkWidget *
 balsa_mblist_mru_option_menu(GtkWindow * window, GList ** url_list)
 {
     GtkWidget *combo_box;
     BalsaMBListMRUOption *mro;
-#if GTK_CHECK_VERSION(2, 6, 0)
     GtkListStore *store;
     GtkCellRenderer *renderer;
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
 
     g_return_val_if_fail(url_list != NULL, NULL);
 
-#if GTK_CHECK_VERSION(2, 6, 0)
     store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_BOOLEAN);
     combo_box = gtk_combo_box_new_with_model(GTK_TREE_MODEL(store));
     renderer = gtk_cell_renderer_text_new();
@@ -2220,9 +2131,6 @@ balsa_mblist_mru_option_menu(GtkWindow * window, GList ** url_list)
     gtk_combo_box_set_row_separator_func(GTK_COMBO_BOX(combo_box),
                                          bmbl_mru_separator_func, NULL,
                                          NULL);
-#else                           /* GTK_CHECK_VERSION(2, 6, 0) */
-    combo_box = gtk_combo_box_new_text();
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
     mro = g_new(BalsaMBListMRUOption, 1);
 
     mro->window = window;
@@ -2281,54 +2189,6 @@ balsa_mblist_mru_option_menu_get(GtkWidget * combo_box)
     return g_slist_nth_data(mro->real_urls, active);
 }
 
-void
-balsa_mblist_set_status_bar(LibBalsaMailbox * mailbox)
-{
-    gint total_messages = libbalsa_mailbox_total_messages(mailbox);
-    gint unread_messages = mailbox->unread_messages;
-    gint hidden_messages;
-    GString *desc = g_string_new(NULL);
-    GtkStatusbar *statusbar;
-    guint context_id;
-
-    hidden_messages =
-        mailbox->msg_tree ? total_messages -
-        (g_node_n_nodes(mailbox->msg_tree, G_TRAVERSE_ALL) - 1) : 0;
-
-    /* xgettext: this is the first part of the message
-     * "Shown mailbox: %s with %d messages, %d new, %d hidden". */
-    g_string_append_printf(desc, _("Shown mailbox: %s "), mailbox->name);
-    if (total_messages > 0) {
-        /* xgettext: this is the second part of the message
-         * "Shown mailbox: %s with %d messages, %d new, %d hidden". */
-        g_string_append_printf(desc,
-                               ngettext("with %d message",
-                                        "with %d messages",
-                                        total_messages), total_messages);
-        if (unread_messages > 0)
-            /* xgettext: this is the third part of the message
-             * "Shown mailbox: %s with %d messages, %d new, %d hidden". */
-            g_string_append_printf(desc,
-                                   ngettext(", %d new", ", %d new",
-                                            unread_messages),
-                                   unread_messages);
-        if (hidden_messages > 0)
-            /* xgettext: this is the fourth part of the message
-             * "Shown mailbox: %s with %d messages, %d new, %d hidden". */
-            g_string_append_printf(desc,
-                                   ngettext(", %d hidden", ", %d hidden",
-                                            hidden_messages),
-                                   hidden_messages);
-    }
-
-    statusbar = GTK_STATUSBAR(balsa_app.main_window->statusbar);
-    context_id = gtk_statusbar_get_context_id(statusbar, "BalsaMBList message");
-    gtk_statusbar_pop(statusbar, context_id);
-    gtk_statusbar_push(statusbar, context_id, desc->str);
-
-    g_string_free(desc, TRUE);
-}
-
 static void
 bmbl_expand_to_row(BalsaMBList * mblist, GtkTreePath * path)
 {
@@ -2345,6 +2205,24 @@ bmbl_expand_to_row(BalsaMBList * mblist, GtkTreePath * path)
 /* Make a new row for mbnode in balsa_app.mblist_tree_store; the row
  * will be a child to the row for root, if we find it, and a top-level
  * row otherwise. */
+static gboolean
+bmbl_sort_idle(gpointer data)
+{
+    GtkTreeSortable *sortable = data;
+
+    gdk_threads_enter();
+
+    gtk_tree_sortable_set_sort_column_id(sortable,
+                                         balsa_app.mblist->sort_column_id,
+                                         GTK_SORT_ASCENDING);
+    balsa_app.mblist->sort_idle_id = 0;
+    g_object_unref(sortable);
+
+    gdk_threads_leave();
+
+    return FALSE;
+}
+
 void 
 balsa_mblist_mailbox_node_append(BalsaMailboxNode * root,
 				 BalsaMailboxNode * mbnode)
@@ -2357,9 +2235,21 @@ balsa_mblist_mailbox_node_append(BalsaMailboxNode * root,
     gdk_threads_enter();
 
     model = GTK_TREE_MODEL(balsa_app.mblist_tree_store);
+
+    if (!balsa_app.mblist->sort_idle_id) {
+        GtkTreeSortable *sortable = GTK_TREE_SORTABLE(model);
+        gtk_tree_sortable_get_sort_column_id
+            (sortable, &balsa_app.mblist->sort_column_id, NULL);
+        gtk_tree_sortable_set_sort_column_id
+            (sortable, GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID,
+                       GTK_SORT_ASCENDING);
+        balsa_app.mblist->sort_idle_id =
+            g_idle_add(bmbl_sort_idle, g_object_ref(sortable));
+    }
+
     if (root && balsa_find_iter_by_data(&parent, root))
 	parent_iter = &parent;
-    gtk_tree_store_append(balsa_app.mblist_tree_store, &iter, parent_iter);
+    gtk_tree_store_prepend(balsa_app.mblist_tree_store, &iter, parent_iter);
     bmbl_store_redraw_mbnode(&iter, mbnode);
 
     if (parent_iter) {
