@@ -19,17 +19,22 @@
  * 02111-1307, USA.
  */
 
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "balsa-mime-widget-message.h"
+
 #include <string.h>
-#include "config.h"
+#include <gtk/gtk.h>
+
 #include "balsa-app.h"
 #include "balsa-icons.h"
 #include "send.h"
 #include "rfc3156.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 #include "balsa-mime-widget.h"
 #include "balsa-mime-widget-callbacks.h"
-#include "balsa-mime-widget-message.h"
-
+#include "sendmsg-window.h"
 
 typedef enum _rfc_extbody_t {
     RFC2046_EXTBODY_FTP,
@@ -90,7 +95,6 @@ balsa_mime_widget_new_message(BalsaMessage * bm,
     if (!g_ascii_strcasecmp("message/external-body", content_type)) {
 	gchar *access_type;
 	rfc_extbody_id *extbody_type = rfc_extbodys;
-	BalsaMimeWidget *mw = NULL;
 
 	access_type =
 	    libbalsa_message_body_get_parameter(mime_body, "access-type");
@@ -127,13 +131,10 @@ balsa_mime_widget_new_message(BalsaMessage * bm,
 				       BMW_MESSAGE_PADDING);
 	gtk_container_add(GTK_CONTAINER(mw->widget), mw->container);
 
-	emb_hdrs = bm_header_widget_new(bm, NULL);
+        mw->header_widget = emb_hdrs = bm_header_widget_new(bm, NULL);
 	gtk_box_pack_start(GTK_BOX(mw->container), emb_hdrs, FALSE, FALSE, 0);
 	
-	g_object_set_data(G_OBJECT(mw), "header-widget", emb_hdrs);
-	balsa_mime_widget_message_set_headers(bm, mw, mime_body->embhdrs,
-					      mime_body->parts,
-					      mime_body->embhdrs->subject);
+	balsa_mime_widget_message_set_headers(bm, mw, mime_body);
     }
 
     /* return the created widget (may be NULL) */
@@ -152,19 +153,14 @@ bmw_message_extbody_url(LibBalsaMessageBody * mime_body,
     BalsaMimeWidget *mw;
 
     if (url_type == RFC2046_EXTBODY_LOCALFILE) {
-	gchar *local_name;
+	url = libbalsa_message_body_get_parameter(mime_body, "name");
 
-	local_name =
-	    libbalsa_message_body_get_parameter(mime_body, "name");
-
-	if (!local_name)
+	if (!url)
 	    return NULL;
 
-	url = g_strdup_printf("file:%s", local_name);
 	msg = g_string_new(_("Content Type: external-body\n"));
 	g_string_append_printf(msg, _("Access type: local-file\n"));
-	g_string_append_printf(msg, _("File name: %s"), local_name);
-	g_free(local_name);
+	g_string_append_printf(msg, _("File name: %s"), url);
     } else if (url_type == RFC2017_EXTBODY_URL) {
 	gchar *local_name;
 
@@ -294,7 +290,7 @@ extbody_call_url(GtkWidget * button, gpointer data)
     GError *err = NULL;
 
     g_return_if_fail(url);
-    gnome_url_show(url, &err);
+    gtk_show_uri(NULL, url, gtk_get_current_event_time(), &err);
     if (err) {
 	balsa_information(LIBBALSA_INFORMATION_WARNING,
 			  _("Error showing %s: %s\n"), url, err->message);
@@ -309,19 +305,22 @@ extbody_send_mail(GtkWidget * button, LibBalsaMessageBody * mime_body)
     LibBalsaMessageBody *body;
     gchar *data;
     GError *err = NULL;
+    LibBalsaMsgCreateResult result;
 
     /* create a message */
     message = libbalsa_message_new();
-    data = internet_address_to_string(balsa_app.current_ident->ia, FALSE);
-    message->headers->from = internet_address_parse_string(data);
-    g_free(data);
+    message->headers->from = internet_address_list_new();
+    internet_address_list_add(message->headers->from,
+                              balsa_app.current_ident->ia);
 
     data = libbalsa_message_body_get_parameter(mime_body, "subject");
-    if (data)
-	LIBBALSA_MESSAGE_SET_SUBJECT(message, data);
+    if (data) {
+	libbalsa_message_set_subject(message, data);
+        g_free(data);
+    }
 
     data = libbalsa_message_body_get_parameter(mime_body, "server");
-    message->headers->to_list = internet_address_parse_string(data);
+    message->headers->to_list = internet_address_list_parse_string(data);
     g_free(data);
 
     /* the original body my have some data to be returned as commands... */
@@ -350,15 +349,20 @@ extbody_send_mail(GtkWidget * button, LibBalsaMessageBody * mime_body)
 	body->charset = g_strdup("US-ASCII");
     libbalsa_message_append_part(message, body);
 #if ENABLE_ESMTP
-    libbalsa_message_send(message, balsa_app.outbox, NULL,
-                          balsa_find_sentbox_by_url,
-                          balsa_app.current_ident->smtp_server,
-                          FALSE, balsa_app.debug);
+    result = libbalsa_message_send(message, balsa_app.outbox, NULL,
+				   balsa_find_sentbox_by_url,
+				   balsa_app.current_ident->smtp_server,
+				   FALSE, balsa_app.debug, &err);
 #else
-    libbalsa_message_send(message, balsa_app.outbox, NULL,
-                          balsa_find_sentbox_by_url,
-                          FALSE, balsa_app.debug);
+    result = libbalsa_message_send(message, balsa_app.outbox, NULL,
+				   balsa_find_sentbox_by_url,
+				   FALSE, balsa_app.debug, &err);
 #endif
+    if (result != LIBBALSA_MESSAGE_CREATE_OK)
+	libbalsa_information(LIBBALSA_INFORMATION_ERROR,
+			     _("Sending the external body request failed: %s"),
+			     err ? err->message : "?");
+    g_error_free(err);
     g_object_unref(G_OBJECT(message));
 }
 
@@ -376,9 +380,8 @@ balsa_mime_widget_new_message_tl(BalsaMessage * bm, GtkWidget * tl_buttons)
     mw->widget = gtk_vbox_new(FALSE, BMW_MESSAGE_PADDING);
     gtk_container_set_border_width(GTK_CONTAINER(mw->widget), BMW_MESSAGE_PADDING);
 
-    headers = bm_header_widget_new(bm, tl_buttons);
+    mw->header_widget = headers = bm_header_widget_new(bm, tl_buttons);
     gtk_box_pack_start(GTK_BOX(mw->widget), headers, FALSE, FALSE, 0);
-    g_object_set_data(G_OBJECT(mw), "header-widget", headers);
 
     mw->container = gtk_vbox_new(FALSE, BMW_MESSAGE_PADDING);
     gtk_box_pack_start(GTK_BOX(mw->widget), mw->container, TRUE, TRUE,
@@ -398,21 +401,21 @@ balsa_mime_widget_new_message_tl(BalsaMessage * bm, GtkWidget * tl_buttons)
 static void
 bm_header_widget_realized(GtkWidget * widget, BalsaMessage * bm)
 {
-    GtkWidget *tl_buttons = 
-	GTK_WIDGET(g_object_get_data(G_OBJECT(widget), "tl-buttons"));
+    GtkWidget *tl_buttons =
+        GTK_WIDGET(g_object_get_data(G_OBJECT(widget), "tl-buttons"));
+    GtkStyle *style = gtk_widget_get_style(GTK_WIDGET(bm));
 
     gtk_widget_modify_bg(widget, GTK_STATE_NORMAL,
-			 &GTK_WIDGET(bm)->style->dark[GTK_STATE_NORMAL]);
+                         &style->dark[GTK_STATE_NORMAL]);
     gtk_widget_modify_base(bm_header_widget_get_text_view(widget),
-			   GTK_STATE_NORMAL,
-			   &GTK_WIDGET(bm)->style->mid[GTK_STATE_NORMAL]);
+                           GTK_STATE_NORMAL,
+                           &style->mid[GTK_STATE_NORMAL]);
     if (tl_buttons) {
-	/* use a fresh style here to deal with pixmap themes correctly */
-	GtkStyle *new_style = gtk_style_new();
-	new_style->bg[GTK_STATE_NORMAL] =
-	    GTK_WIDGET(bm)->style->mid[GTK_STATE_NORMAL];
-	gtk_widget_set_style(tl_buttons, new_style);
-	g_object_unref(new_style);
+        /* use a fresh style here to deal with pixmap themes correctly */
+        GtkStyle *new_style = gtk_style_new();
+        new_style->bg[GTK_STATE_NORMAL] = style->mid[GTK_STATE_NORMAL];
+        gtk_widget_set_style(tl_buttons, new_style);
+        g_object_unref(new_style);
     }
 }
 
@@ -430,12 +433,40 @@ bm_header_widget_set_style(GtkWidget * widget, GtkStyle * previous_style,
 }
 
 static void
-bm_modify_font_from_string(GtkWidget * widget, const char *font)
+bm_header_ctx_menu_reply(GtkWidget * menu_item,
+                         LibBalsaMessageBody *part)
 {
-    PangoFontDescription *desc =
-	pango_font_description_from_string(balsa_app.message_font);
-    gtk_widget_modify_font(widget, desc);
-    pango_font_description_free(desc);
+    sendmsg_window_reply_embedded(part, SEND_REPLY);
+}
+
+static void
+bm_header_extend_popup(GtkTextView *textview, GtkMenu *menu, gpointer arg)
+{
+    GtkWidget *menu_item, *submenu;
+    GtkWidget *separator = gtk_separator_menu_item_new();
+
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), separator);
+    gtk_widget_show(separator);
+    menu_item = gtk_menu_item_new_with_label(_("Reply..."));
+    g_signal_connect(G_OBJECT(menu_item), "activate",
+                     G_CALLBACK(bm_header_ctx_menu_reply),
+                     arg);
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
+    gtk_widget_show(menu_item);
+
+
+    menu_item = gtk_menu_item_new_with_mnemonic(_("_Copy to folder..."));
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
+    gtk_widget_show(menu_item);
+
+    submenu =
+        balsa_mblist_mru_menu(GTK_WINDOW
+                              (gtk_widget_get_toplevel(GTK_WIDGET(textview))),
+                              &balsa_app.folder_mru,
+                              G_CALLBACK(balsa_message_copy_part), arg);
+    gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item),
+                              submenu);
+    gtk_widget_show_all(submenu);
 }
 
 static GtkWidget *
@@ -443,6 +474,8 @@ bm_header_widget_new(BalsaMessage * bm, GtkWidget * buttons)
 {
     GtkWidget *widget;
     GtkWidget *text_view;
+    GtkTextView *view;
+    GtkTextBuffer *buffer;
     GtkWidget *hbox;
 
     widget = gtk_frame_new(NULL);
@@ -462,13 +495,19 @@ bm_header_widget_new(BalsaMessage * bm, GtkWidget * buttons)
     g_signal_connect(G_OBJECT(text_view), "focus_out_event",
 		     G_CALLBACK(balsa_mime_widget_unlimit_focus),
 		     (gpointer) bm);
-    bm_modify_font_from_string(text_view, balsa_app.message_font);
-    gtk_text_view_set_editable(GTK_TEXT_VIEW(text_view), FALSE);
-    gtk_text_view_set_left_margin(GTK_TEXT_VIEW(text_view),
-				  BMW_HEADER_MARGIN_LEFT);
-    gtk_text_view_set_right_margin(GTK_TEXT_VIEW(text_view),
-				   BMW_HEADER_MARGIN_RIGHT);
-    gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text_view), GTK_WRAP_WORD);
+
+    view = GTK_TEXT_VIEW(text_view);
+    gtk_text_view_set_editable(view, FALSE);
+    gtk_text_view_set_left_margin(view, BMW_HEADER_MARGIN_LEFT);
+    gtk_text_view_set_right_margin(view, BMW_HEADER_MARGIN_RIGHT);
+    gtk_text_view_set_wrap_mode(view, GTK_WRAP_WORD_CHAR);
+
+
+    buffer = gtk_text_view_get_buffer(view);
+    gtk_text_buffer_create_tag(buffer, "subject-font", NULL);
+    gtk_text_buffer_create_tag(buffer, "message-font", NULL);
+    gtk_text_buffer_create_tag(buffer, "hanging-indent", NULL);
+    gtk_text_buffer_create_tag(buffer, "url", NULL);
 
     g_signal_connect(text_view, "key_press_event",
 		     G_CALLBACK(balsa_mime_widget_key_press_event), bm);
@@ -483,70 +522,95 @@ bm_header_widget_new(BalsaMessage * bm, GtkWidget * buttons)
     return widget;
 }
 
+static void
+bmwm_set_tabs(BalsaMessage * bm, GtkTextView * view,
+              GtkTextBuffer * buffer, gint tab_position)
+{
+    PangoTabArray *tabs;
+    GtkTextTagTable *table;
+    GtkTextTag *hanging_indent;
 
-/* Indents in pixels: */
-#define BALSA_ONE_CHAR     7
-#define BALSA_INDENT_CHARS 15
-#define BALSA_TAB1         (BALSA_ONE_CHAR * BALSA_INDENT_CHARS)
-#define BALSA_TAB2         (BALSA_TAB1 + BALSA_ONE_CHAR)
+    if (tab_position <= bm->tab_position)
+        return;
+
+    bm->tab_position = tab_position;
+    tabs = pango_tab_array_new_with_positions(1, TRUE, PANGO_TAB_LEFT,
+                                              tab_position);
+    gtk_text_view_set_tabs(view, tabs);
+    pango_tab_array_free(tabs);
+
+    table = gtk_text_buffer_get_tag_table(buffer);
+    hanging_indent = gtk_text_tag_table_lookup(table, "hanging-indent");
+    g_object_set(hanging_indent, "indent", -tab_position, NULL);
+}
+
+/* Indent in pixels: */
+#define BALSA_MESSAGE_HEADER_SEP     6
 
 static void
 add_header_gchar(BalsaMessage * bm, GtkTextView * view,
 		 const gchar * header, const gchar * label,
 		 const gchar * value)
 {
-    PangoTabArray *tab;
+    static const gchar * all_tag_const = N_("... [truncated]");
     GtkTextBuffer *buffer;
-    GtkTextTag *font_tag;
     GtkTextIter insert;
+    const gchar *font;
+    GdkRectangle location;
 
-    if (!(bm->show_all_headers || bm->shown_headers == HEADERS_ALL ||
+    if (!(bm->shown_headers == HEADERS_ALL ||
 	  libbalsa_find_word(header, balsa_app.selected_headers)))
 	return;
 
-    tab = pango_tab_array_new_with_positions(2, TRUE,
-					     PANGO_TAB_LEFT, BALSA_TAB1,
-					     PANGO_TAB_LEFT, BALSA_TAB2);
-    gtk_text_view_set_tabs(view, tab);
-    pango_tab_array_free(tab);
-
     /* always display the label in the predefined font */
     buffer = gtk_text_view_get_buffer(view);
-    font_tag = NULL;
-    if (strcmp(header, "subject") == 0)
-	font_tag =
-	    gtk_text_buffer_create_tag(buffer, NULL,
-				       "font", balsa_app.subject_font,
-				       NULL);
 
     gtk_text_buffer_get_iter_at_mark(buffer, &insert,
 				     gtk_text_buffer_get_insert(buffer));
-    if (gtk_text_buffer_get_char_count(buffer))
+    if (gtk_text_buffer_get_char_count(buffer) > 0)
 	gtk_text_buffer_insert(buffer, &insert, "\n", 1);
-    gtk_text_buffer_insert_with_tags(buffer, &insert,
-				     label, -1, font_tag, NULL);
+    font =
+        strcmp(header, "subject") == 0 ? "subject-font" : "message-font";
+    gtk_text_buffer_insert_with_tags_by_name(buffer, &insert, label, -1,
+                                             "hanging-indent", font,
+                                             NULL);
+
+    gtk_text_view_get_iter_location(view, &insert, &location);
+    bmwm_set_tabs(bm, view, buffer, location.x + BALSA_MESSAGE_HEADER_SEP);
 
     if (value && *value != '\0') {
-	GtkTextTagTable *table;
-	GtkTextTag *indent_tag;
-	gchar *wrapped_value;
-
-	table = gtk_text_buffer_get_tag_table(buffer);
-	indent_tag = gtk_text_tag_table_lookup(table, "indent");
-	if (!indent_tag)
-	    indent_tag =
-		gtk_text_buffer_create_tag(buffer, "indent",
-					   "indent", BALSA_TAB1, NULL);
+        gchar *sanitized;
+        const gchar *all_tag = _(all_tag_const);
+        gboolean truncated = FALSE;
 
 	gtk_text_buffer_insert(buffer, &insert, "\t", 1);
-	wrapped_value = g_strdup(value);
-	libbalsa_wrap_string(wrapped_value,
-			     balsa_app.wraplength - BALSA_INDENT_CHARS);
-	libbalsa_utf8_sanitize(&wrapped_value,
-			       balsa_app.convert_unknown_8bit, NULL);
-	gtk_text_buffer_insert_with_tags(buffer, &insert, wrapped_value,
-					 -1, indent_tag, font_tag, NULL);
-	g_free(wrapped_value);
+
+        sanitized = g_strdup(value);
+        libbalsa_utf8_sanitize(&sanitized,
+                               balsa_app.convert_unknown_8bit, NULL);
+
+        if(bm->shown_headers != HEADERS_ALL) {
+            static const gssize MAXLEN = 160;
+            ssize_t all_tag_len = g_utf8_strlen(all_tag, -1);
+            /* Look far enough into value to be sure that we can tell if
+             * the length is more than MAXLEN+all_tag_len: */
+            glong header_length = g_utf8_strlen(value, 4*(MAXLEN+all_tag_len+1));
+            if(header_length > MAXLEN+all_tag_len) {
+                gchar *p = g_utf8_offset_to_pointer(sanitized, MAXLEN);
+                *p = '\0';
+		truncated = TRUE;
+            }
+        }
+
+        gtk_text_buffer_insert_with_tags_by_name(buffer, &insert,
+                                                 sanitized, -1,
+                                                 font, NULL);
+        g_free(sanitized);
+
+	if (truncated)
+            gtk_text_buffer_insert_with_tags_by_name(buffer, &insert,
+                                                     all_tag, -1,
+                                                     "url", NULL);
     }
 }
 
@@ -557,10 +621,10 @@ add_header_address_list(BalsaMessage * bm, GtkTextView * view,
 {
     gchar *value;
 
-    if (list == NULL)
+    if (list == NULL || internet_address_list_length(list) == 0)
 	return;
 
-    if (!(bm->show_all_headers || bm->shown_headers == HEADERS_ALL ||
+    if (!(bm->shown_headers == HEADERS_ALL ||
 	  libbalsa_find_word(header, balsa_app.selected_headers)))
 	return;
 
@@ -573,8 +637,52 @@ add_header_address_list(BalsaMessage * bm, GtkTextView * view,
 
 void
 balsa_mime_widget_message_set_headers(BalsaMessage * bm, BalsaMimeWidget *mw,
-				      LibBalsaMessageHeaders * headers,
-				      LibBalsaMessageBody * sig_body, const gchar * subject)
+				      LibBalsaMessageBody * part)
+{
+    GtkWidget *widget;
+    GtkTextView *view;
+
+    balsa_mime_widget_message_set_headers_d(bm, mw, part->embhdrs, part->parts,
+                                            part->embhdrs->subject);
+    if (!(widget = mw->header_widget))
+	return;
+    view = bm_header_widget_get_text_view(widget);
+    if( !g_object_get_data(G_OBJECT(view), "popup-extended") ) {
+        g_signal_connect(G_OBJECT(view), "populate-popup",
+                         G_CALLBACK(bm_header_extend_popup),
+                         part);
+        g_object_set_data(G_OBJECT(view), "popup-extended",
+                          GINT_TO_POINTER(1));
+    }
+}
+
+/*
+ * Set up the message according to the user preferences.
+ * Do it each time we display a message, in case the prefs have been
+ * changed.
+ */
+static void
+bmwm_buffer_set_prefs(GtkTextBuffer * buffer)
+{
+    GtkTextTagTable *table = gtk_text_buffer_get_tag_table(buffer);
+    GtkTextTag *tag;
+
+    tag = gtk_text_tag_table_lookup(table, "subject-font");
+    g_object_set(tag, "font", balsa_app.subject_font, NULL);
+
+    tag = gtk_text_tag_table_lookup(table, "message-font");
+    g_object_set(tag, "font", balsa_app.message_font, NULL);
+
+    tag = gtk_text_tag_table_lookup(table, "url");
+    g_object_set(tag, "foreground-gdk", &balsa_app.url_color, NULL);
+}
+
+void
+balsa_mime_widget_message_set_headers_d(BalsaMessage * bm,
+                                        BalsaMimeWidget *mw,
+                                        LibBalsaMessageHeaders *headers,
+                                        LibBalsaMessageBody *part,
+                                        const gchar *subject)
 {
     GtkTextView *view;
     GtkTextBuffer *buffer;
@@ -582,22 +690,24 @@ balsa_mime_widget_message_set_headers(BalsaMessage * bm, BalsaMimeWidget *mw,
     gchar *date;
     GtkWidget * widget;
 
-    if (!(widget =
-	  GTK_WIDGET(g_object_get_data(G_OBJECT(mw), "header-widget"))))
+    if (!(widget = mw->header_widget))
 	return;
 
     view = bm_header_widget_get_text_view(widget);
     buffer = gtk_text_view_get_buffer(view);
+    bmwm_buffer_set_prefs(buffer);
 
     gtk_text_buffer_set_text(buffer, "", 0);
     g_return_if_fail(headers);
 
-    if (!bm->show_all_headers && bm->shown_headers == HEADERS_NONE) {
+    if (bm->shown_headers == HEADERS_NONE) {
 	gtk_widget_hide(widget);
 	return;
     } else
 	gtk_widget_show_all(widget);
 
+    bm->tab_position = 0;
+
     add_header_gchar(bm, view, "subject", _("Subject:"), subject);
 
     date = libbalsa_message_headers_date_to_utf8(headers,
@@ -646,21 +756,20 @@ balsa_mime_widget_message_set_headers(BalsaMessage * bm, BalsaMimeWidget *mw,
     }
 
 #ifdef HAVE_GPGME
-    if (sig_body) {
-	if (sig_body->parts
-	    && sig_body->parts->next
-	    && sig_body->parts->next->sig_info
-	    && sig_body->parts->next->sig_info->status !=
+    if (part) {
+	if (part->parts
+	    && part->parts->next
+	    && part->parts->next->sig_info
+	    && part->parts->next->sig_info->status !=
 	    GPG_ERR_NOT_SIGNED)
 	    /* top-level part is RFC 3156 or RFC 2633 signed */
-	    add_header_sigstate(view, sig_body->parts->next->sig_info);
-	else if (sig_body->sig_info
-		 && sig_body->sig_info->status != GPG_ERR_NOT_SIGNED)
+	    add_header_sigstate(view, part->parts->next->sig_info);
+	else if (part->sig_info
+		 && part->sig_info->status != GPG_ERR_NOT_SIGNED)
 	    /* top-level is OpenPGP (RFC 2440) signed */
-	    add_header_sigstate(view, sig_body->sig_info);
+	    add_header_sigstate(view, part->sig_info);
     }
 #endif
-
     gtk_widget_queue_resize(GTK_WIDGET(view));
 }
 
