@@ -1,6 +1,6 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2001 Stuart Parmenter and others,
+ * Copyright (C) 1997-2010 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,24 +19,32 @@
  * 02111-1307, USA.
  */
 
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "balsa-mime-widget-text.h"
+
 #include <string.h>
-#include "config.h"
+#include <stdlib.h>
 #include "balsa-app.h"
 #include "html.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 #include "quote-color.h"
 #include "sendmsg-window.h"
 #include "store-address.h"
 #include "balsa-mime-widget.h"
 #include "balsa-mime-widget-callbacks.h"
-#include "balsa-mime-widget-text.h"
 #include "balsa-cite-bar.h"
 
 #if HAVE_GTKSOURCEVIEW
 #include <gtksourceview/gtksourceview.h>
 #include <gtksourceview/gtksourcebuffer.h>
 #include <gtksourceview/gtksourcelanguage.h>
-#include <gtksourceview/gtksourcelanguagesmanager.h>
+#if (HAVE_GTKSOURCEVIEW == 1)
+#  include <gtksourceview/gtksourcelanguagesmanager.h>
+#else
+#  include <gtksourceview/gtksourcelanguagemanager.h>
+#endif
 #endif
 
 
@@ -47,9 +55,10 @@ static gboolean fix_text_widget(GtkWidget *widget, gpointer data);
 static void text_view_populate_popup(GtkTextView *textview, GtkMenu *menu,
 				     LibBalsaMessageBody * mime_body);
 
-#ifdef HAVE_GTKHTML
-static BalsaMimeWidget * bm_widget_new_html(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
-					    gchar * ptr, size_t len);
+#ifdef HAVE_HTML_WIDGET
+static BalsaMimeWidget *bm_widget_new_html(BalsaMessage * bm,
+                                           LibBalsaMessageBody *
+                                           mime_body);
 #endif
 static BalsaMimeWidget * bm_widget_new_vcard(BalsaMessage * bm,
                                              LibBalsaMessageBody * mime_body,
@@ -95,12 +104,13 @@ static gboolean store_button_coords(GtkWidget * widget, GdkEventButton * event,
 static gboolean check_over_url(GtkWidget * widget, GdkEventMotion * event, GList * url_list);
 static void pointer_over_url(GtkWidget * widget, message_url_t * url, gboolean set);
 static void prepare_url_offsets(GtkTextBuffer * buffer, GList * url_list);
-static void url_found_cb(GtkTextBuffer * buffer, GtkTextIter * iter, const gchar * buf, gpointer data);
+static void url_found_cb(GtkTextBuffer * buffer, GtkTextIter * iter,
+                         const gchar * buf, guint len, gpointer data);
 static gboolean check_call_url(GtkWidget * widget, GdkEventButton * event, GList * url_list);
 static message_url_t * find_url(GtkWidget * widget, gint x, gint y, GList * url_list);
-static void handle_url(const message_url_t* url);
+static void handle_url(const gchar* url);
 static void free_url_list(GList * url_list);
-static void balsa_gtk_html_on_url(GtkWidget *html, const gchar *url);
+static void bm_widget_on_url(const gchar *url);
 static void phrase_highlight(GtkTextBuffer * buffer, const gchar * id,
 			     gunichar tag_char, const gchar * property,
 			     gint value);
@@ -114,16 +124,23 @@ static gboolean draw_cite_bars(GtkWidget * widget, GdkEventExpose *event, GList
 #define BALSA_MIME_WIDGET_NEW_TEXT_NOTIFIED \
     "balsa-mime-widget-text-new-notified"
 
+#define BALSA_LEFT_MARGIN   2
+#define BALSA_RIGHT_MARGIN 15
+
 BalsaMimeWidget *
 balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 			   const gchar * content_type, gpointer data)
 {
     LibBalsaHTMLType html_type;
     gchar *ptr = NULL;
-    size_t alloced;
+    ssize_t alloced;
     BalsaMimeWidget *mw;
     GtkTextBuffer *buffer;
+#if USE_GREGEX
+    GRegex *rex;
+#else                           /* USE_GREGEX */
     regex_t rex;
+#endif                          /* USE_GREGEX */
     GList *url_list = NULL;
     const gchar *target_cs;
     GError *err = NULL;
@@ -133,9 +150,26 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
     g_return_val_if_fail(mime_body != NULL, NULL);
     g_return_val_if_fail(content_type != NULL, NULL);
 
+    /* handle HTML if possible */
+    html_type = libbalsa_html_type(content_type);
+    if (html_type) {
+        BalsaMimeWidget *html_widget = NULL;
+
+#ifdef HAVE_HTML_WIDGET
+	/* Force vertical scrollbar while we render the html, otherwise
+	 * the widget will make itself too wide to accept one, forcing
+	 * otherwise unnecessary horizontal scrolling. */
+        gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(bm->scroll),
+                                       GTK_POLICY_AUTOMATIC,
+                                       GTK_POLICY_ALWAYS);
+        html_widget = bm_widget_new_html(bm, mime_body);
+#endif
+        return html_widget;
+    }
+
     is_text_plain = !g_ascii_strcasecmp(content_type, "text/plain");
     alloced = libbalsa_message_body_get_content(mime_body, &ptr, &err);
-    if (!ptr) {
+    if (alloced < 0) {
         balsa_information(LIBBALSA_INFORMATION_ERROR,
                           _("Could not save a text part: %s"),
                           err ? err->message : "Unknown error");
@@ -143,24 +177,15 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
         return NULL;
     }
 
-    /* handle HTML if possible */
-    html_type = libbalsa_html_type(content_type);
-    if (html_type) {
-#ifdef HAVE_GTKHTML
-	alloced = libbalsa_html_filter(html_type, &ptr, alloced);
-	/* Force vertical scrollbar while we render the html, otherwise
-	 * the widget will make itself too wide to accept one, forcing
-	 * otherwise unnecessary horizontal scrolling. */
-        gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(bm->scroll),
-                                       GTK_POLICY_AUTOMATIC,
-                                       GTK_POLICY_ALWAYS);
-        return bm_widget_new_html(bm, mime_body, ptr, alloced);
-#else
-        return NULL;
-#endif
-    }
-    if(g_ascii_strcasecmp(content_type, "text/x-vcard") == 0) {
-        return bm_widget_new_vcard(bm, mime_body, ptr, alloced);
+    if(g_ascii_strcasecmp(content_type, "text/x-vcard") == 0 ||
+       g_ascii_strcasecmp(content_type, "text/directory") == 0) {
+        mw = bm_widget_new_vcard(bm, mime_body, ptr, alloced);
+        if (mw) {
+            g_free(ptr);
+            return mw;
+        }
+        /* else it was not a vCard with at least one address; we'll just
+         * show it as if it were text/plain. */
     }
 
     /* prepare a text part */
@@ -206,12 +231,10 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 	       )
 	libbalsa_wrap_string(ptr, balsa_app.browse_wrap_length);
 
-    mw = g_object_new(BALSA_TYPE_MIME_WIDGET, NULL);
-    mw->widget = create_text_widget(content_type);
     gtk_text_view_set_editable(GTK_TEXT_VIEW(mw->widget), FALSE);
-    gtk_text_view_set_left_margin(GTK_TEXT_VIEW(mw->widget), 2);
-    gtk_text_view_set_right_margin(GTK_TEXT_VIEW(mw->widget), 15);
-    gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(mw->widget), GTK_WRAP_WORD);
+    gtk_text_view_set_left_margin(GTK_TEXT_VIEW(mw->widget),  BALSA_LEFT_MARGIN);
+    gtk_text_view_set_right_margin(GTK_TEXT_VIEW(mw->widget), BALSA_RIGHT_MARGIN);
+    gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(mw->widget), GTK_WRAP_WORD_CHAR);
 
     /* set the message font */
     bm_modify_font_from_string(mw->widget, balsa_app.message_font);
@@ -224,25 +247,39 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 		     (gpointer)mime_body);
 
     buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(mw->widget));
-    gtk_text_buffer_create_tag(buffer, "soft", NULL, NULL);
     allocate_quote_colors(GTK_WIDGET(bm), balsa_app.quoted_color,
 			  0, MAX_QUOTED_COLOR - 1);
-    if (regcomp(&rex, balsa_app.quote_regex, REG_EXTENDED) != 0) {
-	g_warning
-	    ("part_info_init_mimetext: quote regex compilation failed.");
+#if USE_GREGEX
+    if (!(rex = balsa_quote_regex_new()))
 	gtk_text_buffer_insert_at_cursor(buffer, ptr, -1);
-    } else {
-	gchar *line_start;
+#else                           /* USE_GREGEX */
+    if (!balsa_app.mark_quoted
+        || regcomp(&rex, balsa_app.quote_regex, REG_EXTENDED)) {
+	if (balsa_app.mark_quoted)
+            g_warning("%s: quote regex compilation failed.", __func__);
+	gtk_text_buffer_insert_at_cursor(buffer, ptr, -1);
+    }
+#endif                          /* USE_GREGEX */
+    else {
+	const gchar *line_start;
 	LibBalsaUrlInsertInfo url_info;
 	GList * cite_bars_list;
 	guint cite_level;
 	guint cite_start;
 	gint margin;
-	PangoFontDescription *font;
+        gdouble char_width;
+        PangoContext *context = gtk_widget_get_pango_context(mw->widget);
+        PangoFontDescription *desc =
+            pango_context_get_font_description(context);
 
-	font = pango_font_description_from_string(balsa_app.message_font);
-	margin = pango_font_description_get_size(font) / PANGO_SCALE;
-	pango_font_description_free(font);
+        /* width of monospace characters is 3/5 of the size */
+        char_width = 0.6 * pango_font_description_get_size(desc);
+        if (!pango_font_description_get_size_is_absolute(desc))
+            char_width = char_width / PANGO_SCALE;
+
+        /* convert char_width from points to pixels */
+        margin = (char_width / 72.0) *
+            gdk_screen_get_resolution(gdk_screen_get_default());
 
 	gtk_text_buffer_create_tag(buffer, "url",
 				   "foreground-gdk",
@@ -254,30 +291,32 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 	url_info.callback = url_found_cb;
 	url_info.callback_data = &url_list;
 	url_info.buffer_is_flowed = libbalsa_message_body_is_flowed(mime_body);
-	url_info.ml_url = NULL;
 	url_info.ml_url_buffer = NULL;
 
 	line_start = ptr;
 	cite_bars_list = NULL;
 	cite_level = 0;
 	cite_start = 0;
-	while (line_start) {
-	    gchar *newline, *this_line;
+	while (*line_start) {
+	    const gchar *line_end;
 	    GtkTextTag *tag = NULL;
 	    int len;
 
-	    newline = strchr(line_start, '\n');
-	    if (newline)
-		this_line = g_strndup(line_start, newline - line_start);
-	    else
-		this_line = g_strdup(line_start);
+	    if (!(line_end = strchr(line_start, '\n')))
+                line_end = line_start + strlen(line_start);
 
 	    if (is_text_plain) {
 		guint quote_level;
 		guint cite_idx;
 
 		/* get the cite level only for text/plain parts */
-		libbalsa_match_regex(this_line, &rex, &quote_level, &cite_idx);
+#if USE_GREGEX
+		libbalsa_match_regex(line_start, rex, &quote_level,
+                                     &cite_idx);
+#else                           /* USE_GREGEX */
+		libbalsa_match_regex(line_start, &rex, &quote_level,
+                                     &cite_idx);
+#endif                          /* USE_GREGEX */
 	    
 		/* check if the citation level changed */
 		if (cite_level != quote_level) {
@@ -294,31 +333,26 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 		    cite_level = quote_level;
 		}
 
-		/* strip off the citation prefix */
+		/* skip the citation prefix */
 		if (quote_level) {
-		    gchar *new;
-		
-		    new = this_line + cite_idx;
-		    if (g_unichar_isspace(g_utf8_get_char(new)))
-			new = g_utf8_next_char(new);
-		    new = g_strdup(new);
-		    g_free(this_line);
-		    this_line = new;
+		    line_start += cite_idx;
+                    if (line_start < line_end
+                        && g_unichar_isspace(g_utf8_get_char(line_start)))
+                        line_start = g_utf8_next_char(line_start);
 		}
 		tag = quote_tag(buffer, quote_level, margin);
 	    }
 
-	    len = strlen(this_line);
-	    if (len > 0 && this_line[len - 1] == '\r')
-		this_line[len - 1] = '\0';
+            len = line_end - line_start;
+	    if (len > 0 && line_start[len - 1] == '\r')
+                --len;
 	    /* tag is NULL if the line isn't quoted, but it causes
 	     * no harm */
-	    if (!libbalsa_insert_with_url(buffer, this_line, line_start,
+	    if (!libbalsa_insert_with_url(buffer, line_start, len,
 					  tag, &url_info))
 		gtk_text_buffer_insert_at_cursor(buffer, "\n", 1);
 	    
-	    g_free(this_line);
-	    line_start = newline ? newline + 1 : NULL;
+	    line_start = *line_end ? line_end + 1 : line_end;
 	}
 
 	/* add any pending cited part */
@@ -339,7 +373,11 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 	    g_signal_connect_after(G_OBJECT(mw->widget), "expose-event",
 				   G_CALLBACK(draw_cite_bars), cite_bars_list);
 	}
+#if USE_GREGEX
+	g_regex_unref(rex);
+#else                           /* USE_GREGEX */
 	regfree(&rex);
+#endif                          /* USE_GREGEX */
     }
 
     prepare_url_offsets(buffer, url_list);
@@ -380,7 +418,7 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 static GtkWidget *
 create_text_widget(const char * content_type)
 {
-#if HAVE_GTKSOURCEVIEW
+#if (HAVE_GTKSOURCEVIEW == 1)
     static GtkSourceLanguagesManager * lm = NULL;
     GtkSourceLanguage * lang;
     GtkSourceBuffer * buffer;
@@ -405,7 +443,53 @@ create_text_widget(const char * content_type)
     widget = gtk_source_view_new_with_buffer(buffer);
     g_object_unref(buffer);
     return widget;
-#else
+#elif (HAVE_GTKSOURCEVIEW == 2)
+    static GtkSourceLanguageManager * lm = NULL;
+    static const gchar * const * lm_ids = NULL;
+    GtkWidget * widget = NULL;
+    gint n;
+
+    /* we use or own highlighting for text/plain */
+    if (!g_ascii_strcasecmp(content_type, "text/plain"))
+	return gtk_text_view_new();
+    
+    /* try to initialise the source language manager and return a "simple"
+     * text view if this fails */
+    if (!lm)
+	lm = gtk_source_language_manager_get_default();
+    if (lm && !lm_ids)
+	lm_ids = gtk_source_language_manager_get_language_ids(lm);
+    if (!lm_ids)
+	return gtk_text_view_new();
+    
+    /* search for a language supporting our mime type */
+    for (n = 0; !widget && lm_ids[n]; n++) {
+	GtkSourceLanguage * src_lang =
+	    gtk_source_language_manager_get_language(lm, lm_ids[n]);
+	gchar ** mime_types;
+
+	if (src_lang &&
+	    (mime_types = gtk_source_language_get_mime_types(src_lang))) {
+	    gint k;
+
+	    for (k = 0;
+		 mime_types[k] && g_ascii_strcasecmp(mime_types[k], content_type);
+		 k++);
+	    if (mime_types[k]) {
+		GtkSourceBuffer * buffer =
+		    gtk_source_buffer_new_with_language(src_lang);
+		
+		gtk_source_buffer_set_highlight_syntax(buffer, TRUE);
+		widget = gtk_source_view_new_with_buffer(buffer);
+		g_object_unref(buffer);
+	    }
+	    g_strfreev(mime_types);
+	}
+    }
+    
+    /* fall back to the simple text view if the mime type is not supported */
+    return widget ? widget : gtk_text_view_new();
+#else /* no GtkSourceview */
     return gtk_text_view_new();
 #endif
 }
@@ -448,7 +532,9 @@ quote_tag(GtkTextBuffer * buffer, gint level, gint margin)
             tag =
                 gtk_text_buffer_create_tag(buffer, name, "foreground-gdk",
                                            &balsa_app.quoted_color[q_level],
-					   "left-margin", margin * level,
+					   "left-margin",
+                                           BALSA_LEFT_MARGIN
+                                           + margin * level,
                                            NULL);
             /* Set a low priority, so we can set both quote color and
              * URL color, and URL color will take precedence. */
@@ -486,9 +572,15 @@ fix_text_widget(GtkWidget *widget, gpointer data)
 static void
 gtk_widget_destroy_insensitive(GtkWidget * widget)
 {
+#if GTK_CHECK_VERSION(2, 18, 0)
+    if (!gtk_widget_get_sensitive(widget) ||
+	GTK_IS_SEPARATOR_MENU_ITEM(widget))
+	gtk_widget_destroy(widget);
+#else                           /* GTK_CHECK_VERSION(2, 18, 0) */
     if (!GTK_WIDGET_SENSITIVE(widget) ||
 	GTK_IS_SEPARATOR_MENU_ITEM(widget))
 	gtk_widget_destroy(widget);
+#endif                          /* GTK_CHECK_VERSION(2, 18, 0) */
 }
 
 static void
@@ -536,7 +628,7 @@ url_copy_cb(GtkWidget * menu_item, message_url_t * uri)
 static void
 url_open_cb(GtkWidget * menu_item, message_url_t * uri)
 {
-    handle_url(uri);
+    handle_url(uri->url);
 }
 
 static void
@@ -544,7 +636,7 @@ url_send_cb(GtkWidget * menu_item, message_url_t * uri)
 {
     BalsaSendmsg * newmsg;
 
-    newmsg = sendmsg_window_compose(GTK_WIDGET(balsa_app.main_window));
+    newmsg = sendmsg_window_compose();
     sendmsg_window_set_field(newmsg, "body", uri->url);
 }
 
@@ -608,8 +700,8 @@ text_view_populate_popup(GtkTextView *textview, GtkMenu *menu,
                           (GtkCallback)gtk_widget_destroy_insensitive, NULL);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu),
 			  gtk_separator_menu_item_new ());
-    libbalsa_fill_vfs_menu_by_content_type(menu, "text/plain",
-					   G_CALLBACK (balsa_mime_widget_ctx_menu_vfs_cb),
+    libbalsa_vfs_fill_menu_by_content_type(menu, "text/plain",
+					   G_CALLBACK (balsa_mime_widget_ctx_menu_cb),
 					   (gpointer)mime_body);
 
     menu_item = gtk_menu_item_new_with_label (_("Save..."));
@@ -726,10 +818,10 @@ pointer_over_url(GtkWidget * widget, message_url_t * url, gboolean set)
     
     if (set) {
         gtk_text_buffer_apply_tag(buffer, tag, &start, &end);
-        balsa_gtk_html_on_url(NULL, url->url);
+        bm_widget_on_url(url->url);
     } else {
         gtk_text_buffer_remove_tag(buffer, tag, &start, &end);
-        balsa_gtk_html_on_url(NULL, NULL);
+        bm_widget_on_url(NULL);
     }
 }
 
@@ -760,7 +852,7 @@ prepare_url_offsets(GtkTextBuffer * buffer, GList * url_list)
 
 static void
 url_found_cb(GtkTextBuffer * buffer, GtkTextIter * iter,
-             const gchar * buf, gpointer data)
+             const gchar * buf, guint len, gpointer data)
 {
     GList **url_list = data;
     message_url_t *url_found;
@@ -768,7 +860,7 @@ url_found_cb(GtkTextBuffer * buffer, GtkTextIter * iter,
     url_found = g_new(message_url_t, 1);
     url_found->end_mark =
         gtk_text_buffer_create_mark(buffer, NULL, iter, TRUE);
-    url_found->url = g_strdup(buf);       /* gets freed later... */
+    url_found->url = g_strndup(buf, len);       /* gets freed later... */
     *url_list = g_list_append(*url_list, url_found);
 }
 
@@ -791,7 +883,7 @@ check_call_url(GtkWidget * widget, GdkEventButton * event,
         && (event->state & STORED_MASK_BITS) == stored_mask) {
         url = find_url(widget, x, y, url_list);
         if (url)
-            handle_url(url);
+            handle_url(url->url);
     }
     return FALSE;
 }
@@ -822,36 +914,48 @@ find_url(GtkWidget * widget, gint x, gint y, GList * url_list)
 }
 
 static gboolean
-status_bar_refresh(gpointer data)
+statusbar_pop(gpointer data)
 {
+    GtkStatusbar *statusbar;
+    guint context_id;
+
     gdk_threads_enter();
-    gnome_appbar_refresh(balsa_app.appbar);
+
+    statusbar = GTK_STATUSBAR(balsa_app.main_window->statusbar);
+    context_id = gtk_statusbar_get_context_id(statusbar, "BalsaMimeWidget message");
+    gtk_statusbar_pop(statusbar, context_id);
+
     gdk_threads_leave();
     return FALSE;
 }
-#define SCHEDULE_BAR_REFRESH()  g_timeout_add(5000, status_bar_refresh, NULL);
+
+#define SCHEDULE_BAR_REFRESH() \
+    g_timeout_add_seconds(5, statusbar_pop, NULL);
 
 static void
-handle_url(const message_url_t* url)
+handle_url(const gchar * url)
 {
-    if (!g_ascii_strncasecmp(url->url, "mailto:", 7)) {
-        BalsaSendmsg *snd = 
-            sendmsg_window_compose(GTK_WIDGET(balsa_app.main_window));
-        sendmsg_window_process_url(url->url + 7,
-                                   sendmsg_window_set_field, snd);      
+    if (!g_ascii_strncasecmp(url, "mailto:", 7)) {
+        BalsaSendmsg *snd = sendmsg_window_compose();
+        sendmsg_window_process_url(url + 7, sendmsg_window_set_field, snd);
     } else {
-        gchar *notice = g_strdup_printf(_("Calling URL %s..."),
-                                        url->url);
+        GtkStatusbar *statusbar;
+        guint context_id;
+        gchar *notice = g_strdup_printf(_("Calling URL %s..."), url);
         GError *err = NULL;
 
-        gnome_appbar_set_status(balsa_app.appbar, notice);
+        statusbar = GTK_STATUSBAR(balsa_app.main_window->statusbar);
+        context_id =
+            gtk_statusbar_get_context_id(statusbar,
+                                         "BalsaMimeWidget message");
+        gtk_statusbar_push(statusbar, context_id, notice);
         SCHEDULE_BAR_REFRESH();
         g_free(notice);
-        gnome_url_show(url->url, &err);
+        gtk_show_uri(NULL, url, gtk_get_current_event_time(), &err);
         if (err) {
             balsa_information(LIBBALSA_INFORMATION_WARNING,
-                    _("Error showing %s: %s\n"), url->url,
-                    err->message);
+                              _("Error showing %s: %s\n"),
+                              url, err->message);
             g_error_free(err);
         }
     }
@@ -990,8 +1094,10 @@ draw_cite_bar_real(cite_bar_t * bar, cite_bar_draw_mode_t * draw_mode)
 	bar->bar = balsa_cite_bar_new(height, bar->depth, draw_mode->dimension);
 	gtk_widget_modify_fg(bar->bar, GTK_STATE_NORMAL,
 			     &balsa_app.quoted_color[(bar->depth - 1) % MAX_QUOTED_COLOR]);
-	gtk_widget_modify_bg(bar->bar, GTK_STATE_NORMAL,
-			     &GTK_WIDGET(draw_mode->view)->style->base[GTK_STATE_NORMAL]);
+        gtk_widget_modify_bg(bar->bar, GTK_STATE_NORMAL,
+                             &gtk_widget_get_style(GTK_WIDGET
+                                                   (draw_mode->view))->
+                             base[GTK_STATE_NORMAL]);
 	gtk_widget_show(bar->bar);
 	gtk_text_view_add_child_in_window(draw_mode->view, bar->bar,
 					  GTK_TEXT_WINDOW_TEXT, 0, y_pos);
@@ -1023,16 +1129,22 @@ draw_cite_bars(GtkWidget * widget, GdkEventExpose *event, GList * cite_bars)
 
 /* --- HTML related functions -- */
 static void
-balsa_gtk_html_on_url(GtkWidget *html, const gchar *url)
+bm_widget_on_url(const gchar *url)
 {
+    GtkStatusbar *statusbar;
+    guint context_id;
+
+    statusbar = GTK_STATUSBAR(balsa_app.main_window->statusbar);
+    context_id = gtk_statusbar_get_context_id(statusbar, "BalsaMimeWidget URL");
+
     if( url ) {
-        gnome_appbar_set_status(balsa_app.appbar, url);
+        gtk_statusbar_push(statusbar, context_id, url);
         SCHEDULE_BAR_REFRESH();
     } else 
-        gnome_appbar_refresh(balsa_app.appbar);
+        gtk_statusbar_pop(statusbar, context_id);
 }
 
-#ifdef HAVE_GTKHTML
+#ifdef HAVE_HTML_WIDGET
 static void
 bm_zoom_in(BalsaMessage * bm)
 {
@@ -1051,51 +1163,70 @@ bm_zoom_reset(BalsaMessage * bm)
     balsa_message_zoom(bm, 0);
 }
 
-static gboolean
-balsa_gtk_html_popup(GtkWidget * html, BalsaMessage * bm)
+static void
+bmwt_populate_popup_menu(BalsaMessage * bm,
+                         GtkWidget    * html,
+                         GtkMenu      * menu)
 {
-    GtkWidget *menu, *menuitem;
+    GtkWidget *menuitem;
     gpointer mime_body = g_object_get_data(G_OBJECT(html), "mime-body");
 
-    menu = gtk_menu_new();
-
     menuitem = gtk_image_menu_item_new_from_stock(GTK_STOCK_ZOOM_IN, NULL);
     g_signal_connect_swapped(G_OBJECT(menuitem), "activate",
                              G_CALLBACK(bm_zoom_in), bm);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
-    menuitem = gtk_image_menu_item_new_from_stock(GTK_STOCK_ZOOM_OUT, NULL);
+    menuitem =
+        gtk_image_menu_item_new_from_stock(GTK_STOCK_ZOOM_OUT, NULL);
     g_signal_connect_swapped(G_OBJECT(menuitem), "activate",
                              G_CALLBACK(bm_zoom_out), bm);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
-    menuitem = gtk_image_menu_item_new_from_stock(GTK_STOCK_ZOOM_100, NULL);
+    menuitem =
+        gtk_image_menu_item_new_from_stock(GTK_STOCK_ZOOM_100, NULL);
     g_signal_connect_swapped(G_OBJECT(menuitem), "activate",
                              G_CALLBACK(bm_zoom_reset), bm);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
-    menuitem = gtk_separator_menu_item_new ();
+    menuitem = gtk_separator_menu_item_new();
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
-    libbalsa_fill_vfs_menu_by_content_type(GTK_MENU(menu), "text/html",
-					   G_CALLBACK (balsa_mime_widget_ctx_menu_vfs_cb),
-					   mime_body);
+    libbalsa_vfs_fill_menu_by_content_type(GTK_MENU(menu), "text/html",
+                                           G_CALLBACK
+                                           (balsa_mime_widget_ctx_menu_cb),
+                                           mime_body);
 
-    menuitem = gtk_menu_item_new_with_label (_("Save..."));
-    g_signal_connect (G_OBJECT (menuitem), "activate",
-                      G_CALLBACK (balsa_mime_widget_ctx_menu_save), mime_body);
-    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+    menuitem = gtk_menu_item_new_with_label(_("Save..."));
+    g_signal_connect(G_OBJECT(menuitem), "activate",
+                     G_CALLBACK(balsa_mime_widget_ctx_menu_save),
+                     mime_body);
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+
+    menuitem = gtk_separator_menu_item_new();
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+
+    menuitem =
+        gtk_image_menu_item_new_from_stock(GTK_STOCK_PRINT, NULL);
+    g_signal_connect_swapped(G_OBJECT(menuitem), "activate",
+                             G_CALLBACK(libbalsa_html_print), html);
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+    gtk_widget_set_sensitive(menuitem, libbalsa_html_can_print(html));
+}
+
+static gboolean
+balsa_gtk_html_popup(GtkWidget * html, BalsaMessage * bm)
+{
+    GtkWidget *menu;
+
+    menu = gtk_menu_new();
+    bmwt_populate_popup_menu(bm, html, GTK_MENU(menu));
 
     gtk_widget_show_all(menu);
-#if GLIB_CHECK_VERSION(2, 10, 0)
     g_object_ref_sink(menu);
-#else                           /* GLIB_CHECK_VERSION(2, 10, 0) */
-    g_object_ref(menu);
-    gtk_object_sink(GTK_OBJECT(menu));
-#endif                          /* GLIB_CHECK_VERSION(2, 10, 0) */
     gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
                    0, gtk_get_current_event_time());
     g_object_unref(menu);
+
     return TRUE;
 }
 
@@ -1107,59 +1238,48 @@ balsa_gtk_html_button_press_cb(GtkWidget * html, GdkEventButton * event,
             ? balsa_gtk_html_popup(html, bm) : FALSE);
 }
 
-/* balsa_gtk_html_size_request:
-   report the requested size of the HTML widget.
-*/
 static void
-balsa_gtk_html_size_request(GtkWidget * widget,
-                            GtkRequisition * requisition, gpointer data)
+bmwt_populate_popup_cb(GtkWidget * widget, GtkMenu * menu, gpointer data)
 {
-    g_return_if_fail(widget != NULL);
-    g_return_if_fail(requisition != NULL);
+    BalsaMessage *bm =
+        g_object_get_data(G_OBJECT(widget), "balsa-message");
+    GtkWidget *html = data;
 
-    requisition->width  = GTK_LAYOUT(widget)->hadjustment->upper;
-    requisition->height = GTK_LAYOUT(widget)->vadjustment->upper;
-}
-
-static void
-balsa_gtk_html_link_clicked(GObject *obj, const gchar *url)
-{
-    GError *err = NULL;
-
-    gnome_url_show(url, &err);
-    if (err) {
-        balsa_information(LIBBALSA_INFORMATION_WARNING,
-                _("Error showing %s: %s\n"), url, err->message);
-        g_error_free(err);
-    }
+    /* Remove WebKitWebView's items--they're irrelevant and confusing */
+    gtk_container_foreach(GTK_CONTAINER(menu),
+                          (GtkCallback) gtk_widget_destroy, NULL);
+    bmwt_populate_popup_menu(bm, html, menu);
+    gtk_widget_show_all(GTK_WIDGET(menu));
 }
 
 BalsaMimeWidget *
-bm_widget_new_html(BalsaMessage * bm, LibBalsaMessageBody * mime_body, gchar * ptr, size_t len)
+bm_widget_new_html(BalsaMessage * bm, LibBalsaMessageBody * mime_body)
 {
     BalsaMimeWidget *mw = g_object_new(BALSA_TYPE_MIME_WIDGET, NULL);
+    GtkWidget *widget;
 
     mw->widget =
-        libbalsa_html_new(ptr, len,
-			  libbalsa_message_body_charset(mime_body),
-			  bm->message,
-                          G_CALLBACK(balsa_gtk_html_link_clicked));
+        libbalsa_html_new(mime_body,
+                          (LibBalsaHtmlCallback) bm_widget_on_url,
+                          (LibBalsaHtmlCallback) handle_url);
     g_object_set_data(G_OBJECT(mw->widget), "mime-body", mime_body);
 
-    g_signal_connect(G_OBJECT(mw->widget), "size-request",
-                     G_CALLBACK(balsa_gtk_html_size_request), bm);
-    g_signal_connect(G_OBJECT(mw->widget), "on-url",
-                     G_CALLBACK(balsa_gtk_html_on_url), bm);
-    g_signal_connect(G_OBJECT(mw->widget), "button-press-event",
-                     G_CALLBACK(balsa_gtk_html_button_press_cb), bm);
     g_signal_connect(G_OBJECT(mw->widget), "key_press_event",
                      G_CALLBACK(balsa_mime_widget_key_press_event), bm);
-    g_signal_connect(G_OBJECT(mw->widget), "popup-menu",
-                     G_CALLBACK(balsa_gtk_html_popup), bm);
+    if ((widget = libbalsa_html_popup_menu_widget(mw->widget))) {
+        g_object_set_data(G_OBJECT(widget), "balsa-message", bm);
+        g_signal_connect(widget, "populate-popup",
+                         G_CALLBACK(bmwt_populate_popup_cb), mw->widget);
+    } else {
+        g_signal_connect(mw->widget, "button-press-event",
+                         G_CALLBACK(balsa_gtk_html_button_press_cb), bm);
+        g_signal_connect(mw->widget, "popup-menu",
+                         G_CALLBACK(balsa_gtk_html_popup), bm);
+    }
 
     return mw;
 }
-#endif /* defined HAVE_GTKHTML */
+#endif /* defined HAVE_HTML_WIDGET */
 
 #define TABLE_ATTACH(t,str,label) \
     if(str) { GtkWidget *lbl = gtk_label_new(label);              \
@@ -1172,7 +1292,7 @@ bm_widget_new_html(BalsaMessage * bm, LibBalsaMessageBody * mime_body, gchar * p
         row++;                                                    \
     }
 
-BalsaMimeWidget *
+static BalsaMimeWidget *
 bm_widget_new_vcard(BalsaMessage *bm, LibBalsaMessageBody *mime_body,
                     gchar *ptr, size_t len)
 {
@@ -1183,6 +1303,9 @@ bm_widget_new_vcard(BalsaMessage *bm, LibBalsaMessageBody *mime_body,
     GtkWidget *w;
     int row = 1;
 
+    if (!addr)
+        return NULL;
+
     mw->widget = gtk_table_new(10, 2, FALSE);
     table = (GtkTable*)mw->widget;
         
@@ -1208,6 +1331,5 @@ bm_widget_new_vcard(BalsaMessage *bm, LibBalsaMessageBody *mime_body,
         
     g_object_set_data(G_OBJECT(mw->widget), "mime-body", mime_body);
     gtk_widget_show_all(mw->widget);
-    g_free(ptr);
     return mw;
 }
