@@ -23,15 +23,15 @@
  * Callbacks for the filter edit dialog
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
 
-#define _XOPEN_SOURCE /* glibc2 needs this */
 /* yellow dog has crappy libc and needs pthread.h to be included here */
 #ifdef BALSA_USE_THREADS
 #  include <pthread.h>
 #endif
 #include <time.h>
-#include <gnome.h>
 
 #include <string.h>
 #include "filter.h"
@@ -41,17 +41,24 @@
 #include "balsa-app.h"
 #include "save-restore.h"
 #include "mailbox-filter.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 #include "libbalsa-conf.h"
+#include "missing.h"
+
+#if HAVE_MACOSX_DESKTOP
+#  include "macosx-helpers.h"
+#endif
 
 /* Defined in filter-edit-dialog.c*/
 extern option_list fe_search_type[4];
 extern GList * fe_user_headers_list;
 
+#if REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED
 static void fe_add_pressed(GtkWidget * widget, gpointer throwaway);
 static void fe_remove_pressed(GtkWidget * widget, gpointer throwaway);
 static void fe_regexs_selection_changed(GtkTreeSelection * selection,
                                         gpointer user_data);
+#endif                  /* REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED */
 static void fe_free_associated_filters(void);
 static void fe_free_associated_conditions(void);
 static GtkWidget *fe_date_sample(void);
@@ -99,7 +106,9 @@ GtkWidget *fe_type_date_low_entry,*fe_type_date_high_entry;
 /* widgets for the type notebook regex page */
 GtkTreeView *fe_type_regex_list;
 GtkWidget *fe_type_regex_label;
+#if REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED
 GtkWidget *fe_type_regex_entry;
+#endif                  /* REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED */
 
 /* widgets for the type notebook condition flag page */
 GtkWidget *fe_type_flag_label;
@@ -116,9 +125,19 @@ extern GtkWidget *fe_popup_entry;
 /* action field */
 extern GtkWidget *fe_action_option_menu;
 
+/* action list */
+extern option_list fe_actions[];
+
 /* Mailboxes option menu */
 extern GtkWidget * fe_mailboxes;
 
+/* Colorize widgets */
+extern GtkWidget * fe_color_buttons;
+extern GtkWidget * fe_foreground_set;
+extern GtkWidget * fe_foreground;
+extern GtkWidget * fe_background_set;
+extern GtkWidget * fe_background;
+
 /* Different buttons that need to be greyed or ungreyed */
 extern GtkWidget * fe_new_button;
 extern GtkWidget * fe_delete_button;
@@ -242,6 +261,7 @@ unique_filter_name(const gchar * name)
 
 /**************** Conditions *************************/
 
+#if REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED
 /* Callback function to fill the regex entry with the selected regex */
 
 static void
@@ -262,6 +282,7 @@ fe_regexs_selection_changed(GtkTreeSelection *selection,
     else gtk_entry_set_text(GTK_ENTRY(fe_type_regex_entry),"");
     gtk_widget_set_sensitive(fe_regex_remove_button, selected);
 }
+#endif                  /* REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED */
 
 /* Helper. */
 static gint
@@ -293,11 +314,16 @@ fe_typesmenu_cb(GtkWidget * widget, GtkWidget * field_frame)
 
     switch (type) {
     case CONDITION_STRING:
+#if REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED
     case CONDITION_REGEX:
+#endif                  /* REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED */
         gtk_widget_show(field_frame);
         break;
     case CONDITION_DATE:
     case CONDITION_FLAG:
+#if !REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED
+    case CONDITION_REGEX:
+#endif                  /* REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED */
         gtk_widget_hide(field_frame);
     default:
         break;
@@ -480,9 +506,11 @@ condition_validate(LibBalsaCondition* new_cnd)
     const gchar *c_str;
     gint row, col;
     struct tm date;
+#if REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED
     GtkTreeModel *model =
         gtk_tree_view_get_model(fe_type_regex_list);
     GtkTreeIter iter;
+#endif                  /* REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED */
 
 
     /* Sanity checks, prevent "empty" condition */
@@ -505,8 +533,8 @@ condition_validate(LibBalsaCondition* new_cnd)
 	    CONDITION_SETMATCH(new_cnd,CONDITION_MATCH_US_HEAD);
             str =
                 gtk_editable_get_chars(GTK_EDITABLE
-                                       (GTK_BIN(fe_user_header)->child), 0,
-                                       -1);
+                                       (gtk_bin_get_child
+                                        (GTK_BIN(fe_user_header))), 0, -1);
 	    if (!str[0]) {
                 balsa_information(LIBBALSA_INFORMATION_ERROR,
                                   _("You must specify the name of the "
@@ -535,12 +563,14 @@ condition_validate(LibBalsaCondition* new_cnd)
         }
         break;
     case CONDITION_REGEX:
+#if REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED
         if (!gtk_tree_model_get_iter_first(model, &iter)) {
             balsa_information(LIBBALSA_INFORMATION_ERROR,
                               _("You must provide at least one "
                                 "regular expression"));
             return FALSE;
         }
+#endif                  /* REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED */
         break;
     case CONDITION_DATE:
         c_str = gtk_entry_get_text(GTK_ENTRY(fe_type_date_low_entry));
@@ -586,8 +616,8 @@ condition_validate(LibBalsaCondition* new_cnd)
     if (CONDITION_CHKMATCH(new_cnd,CONDITION_MATCH_US_HEAD))
         new_cnd->match.string.user_header =
             gtk_editable_get_chars(GTK_EDITABLE
-                                   (GTK_BIN(fe_user_header)->child), 0,
-                                   -1);
+                                   (gtk_bin_get_child
+                                    (GTK_BIN(fe_user_header))), 0, -1);
     /* Set the type specific fields of the condition */
     switch (new_cnd->type) {
     case CONDITION_STRING:
@@ -629,12 +659,16 @@ condition_validate(LibBalsaCondition* new_cnd)
 static void
 clear_condition_widgets()
 {
+#if REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED
     GtkTreeModel *model =
         gtk_tree_view_get_model(fe_type_regex_list);
+#endif                  /* REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED */
 
     gtk_entry_set_text(GTK_ENTRY(fe_type_simple_entry),"");
+#if REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED
     gtk_entry_set_text(GTK_ENTRY(fe_type_regex_entry),"");          
     gtk_list_store_clear(GTK_LIST_STORE(model));
+#endif                  /* REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED */
     gtk_entry_set_text(GTK_ENTRY(fe_type_date_low_entry),"");
     gtk_entry_set_text(GTK_ENTRY(fe_type_date_high_entry),"");
 }
@@ -682,8 +716,10 @@ set_button_sensitivities(gboolean sensitive)
 static void
 fill_condition_widgets(LibBalsaCondition* cnd)
 {
+#if REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED
     GtkTreeModel *model =
         gtk_tree_view_get_model(fe_type_regex_list);
+#endif                  /* REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED */
     gchar str[20];
     struct tm date;
     gint row,col;
@@ -695,10 +731,12 @@ fill_condition_widgets(LibBalsaCondition* cnd)
     if (cnd->type!=CONDITION_STRING)
         gtk_entry_set_text(GTK_ENTRY(fe_type_simple_entry),"");
 
+#if REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED
     if (cnd->type!=CONDITION_REGEX)
         gtk_entry_set_text(GTK_ENTRY(fe_type_regex_entry),"");      
 
     gtk_list_store_clear(GTK_LIST_STORE(model));
+#endif                  /* REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED */
 
     gtk_notebook_set_current_page(GTK_NOTEBOOK(fe_type_notebook),
                                   cnd->type - 1);
@@ -727,7 +765,9 @@ fill_condition_widgets(LibBalsaCondition* cnd)
     }
     else {
 	gtk_widget_set_sensitive(fe_user_header,FALSE);
-	gtk_entry_set_text(GTK_ENTRY(GTK_BIN(fe_user_header)->child),"");
+        gtk_entry_set_text(GTK_ENTRY
+                           (gtk_bin_get_child(GTK_BIN(fe_user_header))),
+                           "");
     }	
     /* Next update type specific fields */
     switch (cnd->type) {
@@ -792,7 +832,7 @@ condition_dialog_response(GtkWidget * dialog, gint response,
                           gpointer throwaway)
 {
     LibBalsaCondition *new_cnd;
-    GError *err;
+    GError *err = NULL;
 
     switch (response) {
     case GTK_RESPONSE_OK:       /* OK button */
@@ -848,8 +888,8 @@ condition_dialog_response(GtkWidget * dialog, gint response,
         gtk_widget_hide(dialog);
         break;
     case GTK_RESPONSE_HELP:     /* Help button */
-	err = NULL;
-	gnome_help_display("balsa", "win-condition", &err);
+        gtk_show_uri(NULL, "ghelp:balsa?win-condition",
+                     gtk_get_current_event_time(), &err);
 	if (err) {
 	    balsa_information_parented(GTK_WINDOW(dialog),
 		    LIBBALSA_INFORMATION_WARNING,
@@ -938,7 +978,11 @@ static void
 build_type_notebook()
 {
     GtkWidget *page,*frame;
+#if REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED
     GtkWidget *scroll;
+#else                   /* REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED */
+    const gchar *msg;
+#endif                  /* REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED */
     GtkWidget *box;
     GtkWidget *button, *table;
     gint row,col;
@@ -955,7 +999,7 @@ build_type_notebook()
 
     box = gtk_vbox_new(FALSE, 5);
     table = gtk_table_new(7, 2, FALSE);
-    gtk_box_pack_start_defaults(GTK_BOX(box), table);
+    gtk_box_pack_start(GTK_BOX(box), table, TRUE, TRUE, 0);
 
     fe_type_simple_label = 
 	gtk_label_new_with_mnemonic(_("One of the specified f_ields contains"));
@@ -986,11 +1030,12 @@ build_type_notebook()
     /* The regex page of the type notebook */
     box = gtk_vbox_new(FALSE, 5);
 
-    page = gtk_table_new(5, 6, FALSE);
-    gtk_box_pack_start_defaults(GTK_BOX(box), page);
-
     gtk_notebook_append_page(GTK_NOTEBOOK(fe_type_notebook), box, NULL);
 
+#if REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED
+    page = gtk_table_new(5, 6, FALSE);
+    gtk_box_pack_start(GTK_BOX(box), page, TRUE, TRUE, 0);
+
     fe_type_regex_label = 
         gtk_label_new_with_mnemonic(_("_One of the regular expressions matches"));
     gtk_table_attach(GTK_TABLE(page),
@@ -1037,6 +1082,10 @@ build_type_notebook()
                      fe_type_regex_entry,
                      0, 5, 5, 6,
                      GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
+#else                   /* REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED */
+    msg = _("Filtering using regular expressions is not yet implemented.");
+    gtk_box_pack_start(GTK_BOX(box), gtk_label_new(msg), TRUE, TRUE, 0);
+#endif                  /* REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED */
 
     /* The date page of the notebook */
 
@@ -1125,6 +1174,7 @@ void build_condition_dialog(GtkWidget * condition_dialog)
 {
     GtkWidget *label,* box;
     GtkWidget *field_frame = get_field_frame();
+    GtkBox *content_box;
 
     /* builds the toggle buttons to specify fields concerned by the
      * conditions of the filter */
@@ -1136,14 +1186,14 @@ void build_condition_dialog(GtkWidget * condition_dialog)
                              G_CALLBACK(fe_typesmenu_cb), field_frame);
     gtk_box_pack_start(GTK_BOX(box), fe_search_option_menu, FALSE, FALSE, 5);
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), fe_search_option_menu);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(condition_dialog)->vbox),
-                       box, FALSE, FALSE, 2);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(condition_dialog)->vbox),
-                       field_frame, FALSE, FALSE, 2);
+
+    content_box =
+        GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(condition_dialog)));
+    gtk_box_pack_start(content_box, box, FALSE, FALSE, 2);
+    gtk_box_pack_start(content_box, field_frame, FALSE, FALSE, 2);
 
     build_type_notebook();
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(condition_dialog)->vbox),
-                       fe_type_notebook, FALSE, FALSE, 2);
+    gtk_box_pack_start(content_box, fe_type_notebook, FALSE, FALSE, 2);
 }
 
 /*
@@ -1194,6 +1244,9 @@ fe_edit_condition(GtkWidget * throwaway,gpointer is_new_cnd)
                                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                         GTK_STOCK_HELP, GTK_RESPONSE_HELP,
                                         NULL);
+#if HAVE_MACOSX_DESKTOP
+	libbalsa_macosx_menu_for_parent(condition_dialog, GTK_WINDOW(fe_window));
+#endif
         g_object_add_weak_pointer(G_OBJECT(condition_dialog),
                                   (gpointer) &condition_dialog);
 
@@ -1403,7 +1456,7 @@ fe_dialog_response(GtkWidget * dialog, gint response, gpointer data)
         gtk_tree_view_get_model(fe_filters_list);
     GtkTreeIter iter;
     gboolean valid;
-    GError *err;
+    GError *err = NULL;
     
     switch (response) {
     case GTK_RESPONSE_OK:       /* OK button */
@@ -1442,8 +1495,8 @@ fe_dialog_response(GtkWidget * dialog, gint response, gpointer data)
         break;
 
     case GTK_RESPONSE_HELP:     /* Help button */
-	err = NULL;
-	gnome_help_display("balsa", "win-filters", &err);
+        gtk_show_uri(NULL, "ghelp:balsa?win-filters",
+                     gtk_get_current_event_time(), &err);
 	if (err) {
 	    balsa_information_parented(GTK_WINDOW(dialog),
 		    LIBBALSA_INFORMATION_WARNING,
@@ -1469,8 +1522,15 @@ fe_action_selected(GtkWidget * widget, gpointer data)
 {
     FilterActionType type =
         (FilterActionType) fe_combo_box_get_value(widget);
-    gtk_widget_set_sensitive(GTK_WIDGET(fe_mailboxes),
-                             type != FILTER_TRASH);
+    if (type == FILTER_COLOR) {
+        gtk_widget_hide(fe_mailboxes);
+        gtk_widget_show_all(fe_color_buttons);
+    } else {
+        gtk_widget_hide(fe_color_buttons);
+        gtk_widget_show(fe_mailboxes);
+        gtk_widget_set_sensitive(GTK_WIDGET(fe_mailboxes),
+                                 type != FILTER_TRASH);
+    }
     set_button_sensitivities(TRUE);
 }                       /* end fe_action_selected() */
 
@@ -1506,6 +1566,7 @@ fe_action_changed(GtkWidget * widget, gpointer data)
     set_button_sensitivities(TRUE);
 }
 
+#if REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED
 /*
  * fe_add_pressed()
  *
@@ -1559,6 +1620,7 @@ fe_remove_pressed(GtkWidget * widget, gpointer throwaway)
     gtk_tree_path_free(path);
     condition_has_changed=TRUE;
 }                       /* end fe_remove_pressed() */
+#endif                  /* REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED */
 
 /************************************************************/
 /******** Functions handling filters ************************/
@@ -1723,9 +1785,6 @@ fe_delete_pressed(GtkWidget * widget, gpointer data)
         gtk_entry_set_text(GTK_ENTRY(fe_name_entry),"");
         gtk_entry_set_text(GTK_ENTRY(fe_popup_entry),"");
         /*gtk_option_menu_set_history(GTK_OPTION_MENU(fe_mailboxes), 0); */
-#if !GTK_CHECK_VERSION(2, 6, 0)
-        gtk_entry_set_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(fe_sound_entry))),"");
-#endif /* GTK_CHECK_VERSION(2, 6, 0) */
         gtk_list_store_clear(GTK_LIST_STORE
                              (gtk_tree_view_get_model
                               (fe_conditions_list)));
@@ -1838,7 +1897,36 @@ fe_apply_pressed(GtkWidget * widget, gpointer data)
 
     fil->action=action;
 
-    if (fil->action!=FILTER_TRASH)
+    if (fil->action == FILTER_COLOR) {
+        GdkColor color;
+        GString *string = g_string_new(NULL);
+        GtkToggleButton *toggle_button;
+        gchar *color_string;
+
+        toggle_button = (GTK_TOGGLE_BUTTON(fe_foreground_set));
+        if (gtk_toggle_button_get_active(toggle_button)) {
+            gtk_color_button_get_color(GTK_COLOR_BUTTON(fe_foreground),
+                                       &color);
+            color_string = gdk_color_to_string(&color);
+            g_string_append_printf(string, "foreground:%s", color_string);
+            g_free(color_string);
+            gtk_toggle_button_set_active(toggle_button, FALSE);
+        }
+
+        toggle_button = (GTK_TOGGLE_BUTTON(fe_background_set));
+        if (gtk_toggle_button_get_active(toggle_button)) {
+            gtk_color_button_get_color(GTK_COLOR_BUTTON(fe_background),
+                                       &color);
+            color_string = gdk_color_to_string(&color);
+            if (string->len > 0)
+                g_string_append_c(string, ';');
+            g_string_append_printf(string, "background:%s", color_string);
+            g_free(color_string);
+            gtk_toggle_button_set_active(toggle_button, FALSE);
+        }
+
+        fil->action_string = g_string_free(string, FALSE);
+    } else if (fil->action!=FILTER_TRASH)
         fil->action_string=g_strdup(balsa_mblist_mru_option_menu_get(fe_mailboxes));
     if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fe_popup_button))) {
         static gchar defstring[] = N_("Filter has matched");
@@ -1855,13 +1943,8 @@ fe_apply_pressed(GtkWidget * widget, gpointer data)
     if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fe_sound_button))) {
         gchar *tmpstr;
         
-#if GTK_CHECK_VERSION(2, 6, 0)
         tmpstr = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER
                                                (fe_sound_entry));
-#else /* GTK_CHECK_VERSION(2, 6, 0) */
-        tmpstr =
-            gtk_editable_get_chars(GTK_EDITABLE(fe_sound_entry), 0, -1);
-#endif /* GTK_CHECK_VERSION(2, 6, 0) */
         if ((!tmpstr) || (tmpstr[0] == '\0')) {
             g_free(tmpstr);
             libbalsa_filter_free(fil, GINT_TO_POINTER(TRUE));
@@ -1944,7 +2027,7 @@ fe_filters_list_selection_changed(GtkTreeSelection * selection,
     GtkTreeModel *model;
     GtkTreeIter iter;
     LibBalsaFilter* fil;
-    int pos;
+    int pos, i;
 
     if (!gtk_tree_selection_get_selected(selection, &model, &iter)) {
         if (gtk_tree_model_iter_n_children(model, NULL) == 0)
@@ -1963,21 +2046,40 @@ fe_filters_list_selection_changed(GtkTreeSelection * selection,
                        ? fil->popup_text : "");
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fe_sound_button),
                                  fil->sound!=NULL);
-#if GTK_CHECK_VERSION(2, 6, 0)
     if (fil->sound)
         gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(fe_sound_entry),
                                       fil->sound);
-#else /* GTK_CHECK_VERSION(2, 6, 0) */
-    gtk_entry_set_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(fe_sound_entry))),
-                       fil->sound!=NULL ? fil->sound : "");
-#endif /* GTK_CHECK_VERSION(2, 6, 0) */
     
-    gtk_combo_box_set_active(GTK_COMBO_BOX(fe_action_option_menu),
-                             fil->action - 1);
+    for (i = 0; i < FILTER_N_TYPES - 1; i++)
+        if (((FilterActionType) fe_actions[i].value) == fil->action)
+            gtk_combo_box_set_active(GTK_COMBO_BOX(fe_action_option_menu),
+                                     i);
     pos = (fil->condition
            && fil->condition->type == CONDITION_AND) ? 1 : 0;
     gtk_combo_box_set_active(GTK_COMBO_BOX(fe_op_codes_option_menu), pos);
-    if (fil->action!=FILTER_TRASH && fil->action_string)
+    if (fil->action == FILTER_COLOR) {
+        gchar **parts, **p;
+        GdkColor color;
+
+        parts = g_strsplit(fil->action_string, ";", 2);
+        for (p = parts; *p; p++) {
+            if (g_str_has_prefix(*p, "foreground:")) {
+                gdk_color_parse((*p) + 11, &color);
+                gtk_color_button_set_color(GTK_COLOR_BUTTON(fe_foreground),
+                                           &color);
+                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
+                                             (fe_foreground_set), TRUE);
+            }
+            if (g_str_has_prefix(*p, "background:")) {
+                gdk_color_parse((*p) + 11, &color);
+                gtk_color_button_set_color(GTK_COLOR_BUTTON(fe_background),
+                                           &color);
+                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
+                                             (fe_background_set), TRUE);
+            }
+        }
+        g_strfreev(parts);
+    } else if (fil->action!=FILTER_TRASH && fil->action_string)
         balsa_mblist_mru_option_menu_set(fe_mailboxes,
                                          fil->action_string);
     /* We free the conditions */
@@ -2033,7 +2135,6 @@ fe_date_sample(void)
     return label;
 }                               /* end fe_date_sample */
 
-#if GTK_CHECK_VERSION(2, 6, 0)
 /* Callback for the sound file-chooser-button's dialog. */
 void
 fe_sound_response(GtkDialog * dialog, gint response)
@@ -2041,4 +2142,20 @@ fe_sound_response(GtkDialog * dialog, gint response)
     if (response == GTK_RESPONSE_ACCEPT)
         set_button_sensitivities(TRUE);
 }
-#endif /* GTK_CHECK_VERSION(2, 6, 0) */
+
+/* Callback for color check-buttons' "toggled" signal */
+void
+fe_color_check_toggled(GtkToggleButton * check_button, gpointer data)
+{
+    GtkWidget *color_button = GTK_WIDGET(data);
+    gtk_widget_set_sensitive(color_button,
+                             gtk_toggle_button_get_active(check_button));
+    set_button_sensitivities(TRUE);
+}
+
+/* Callback for color buttons' "color-set" signal */
+void
+fe_color_set(GtkColorButton * color_button, gpointer data)
+{
+    set_button_sensitivities(TRUE);
+}
