@@ -19,7 +19,9 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
 
 #include "balsa-app.h"
 #include "filter-edit.h"
@@ -75,6 +77,13 @@ GtkWidget *fe_action_option_menu;
 /* Mailboxes option menu */
 GtkWidget * fe_mailboxes;
 
+/* Select colors */
+GtkWidget * fe_color_buttons;
+GtkWidget * fe_foreground_set;
+GtkWidget * fe_foreground;
+GtkWidget * fe_background_set;
+GtkWidget * fe_background;
+
 GtkWidget* fe_right_page;
 
 /* Different buttons that need to be greyed or ungreyed */
@@ -94,6 +103,7 @@ option_list fe_search_type[] = {
 option_list fe_actions[] = {
     {N_("Copy to folder:"), FILTER_COPY},
     {N_("Move to folder:"), FILTER_MOVE},
+    {N_("Colorize"), FILTER_COLOR},
     {N_("Print on printer:"), FILTER_PRINT},
     {N_("Run program:"), FILTER_RUN},
     {N_("Send to Trash"), FILTER_TRASH}
@@ -333,14 +343,45 @@ build_match_page()
  *
  * Builds the "Action" page of the main notebook
  */
+
+static GtkWidget *
+fe_make_color_buttons(void)
+{
+    GtkWidget *table_widget = gtk_table_new(2, 2, FALSE);
+    GtkTable  *table = GTK_TABLE(table_widget);
+    GdkColor color;
+
+    fe_foreground_set = gtk_check_button_new_with_mnemonic(_("Foreground"));
+    gtk_table_attach_defaults(table, fe_foreground_set, 0, 1, 0, 1);
+    gdk_color_parse("black", &color);
+    fe_foreground = gtk_color_button_new_with_color(&color);
+    gtk_widget_set_sensitive(fe_foreground, FALSE);
+    gtk_table_attach_defaults(table, fe_foreground, 1, 2, 0, 1);
+    g_signal_connect(fe_foreground_set, "toggled",
+                     G_CALLBACK(fe_color_check_toggled), fe_foreground);
+    g_signal_connect(fe_foreground, "color-set",
+                     G_CALLBACK(fe_color_set), NULL);
+
+    fe_background_set = gtk_check_button_new_with_mnemonic(_("Background"));
+    gtk_table_attach_defaults(table, fe_background_set, 0, 1, 1, 2);
+    gdk_color_parse("white", &color);
+    fe_background = gtk_color_button_new_with_color(&color);
+    gtk_widget_set_sensitive(fe_background, FALSE);
+    gtk_table_attach_defaults(table, fe_background, 1, 2, 1, 2);
+    g_signal_connect(fe_background_set, "toggled",
+                     G_CALLBACK(fe_color_check_toggled), fe_background);
+    g_signal_connect(fe_background, "color-set",
+                     G_CALLBACK(fe_color_set), NULL);
+
+    return table_widget;
+}
+
 static GtkWidget *
 build_action_page(GtkWindow * window)
 {
     GtkWidget *page, *frame, *table;
     GtkWidget *box;
-#if GTK_CHECK_VERSION(2, 6, 0)
     GtkWidget *dialog;
-#endif /* GTK_CHECK_VERSION(2, 6, 0) */
 
     page = gtk_vbox_new(TRUE, 5);
 
@@ -361,7 +402,6 @@ build_action_page(GtkWindow * window)
 		     0, 1, 0, 1,
 		     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 5, 5);
 
-#if GTK_CHECK_VERSION(2, 6, 0)
     dialog =
         gtk_file_chooser_dialog_new(_("Use Sound..."), NULL,
                                     GTK_FILE_CHOOSER_ACTION_OPEN,
@@ -374,10 +414,6 @@ build_action_page(GtkWindow * window)
                                             GTK_RESPONSE_ACCEPT,
                                             GTK_RESPONSE_CANCEL, -1);
     fe_sound_entry = gtk_file_chooser_button_new_with_dialog(dialog);
-#else /* GTK_CHECK_VERSION(2, 6, 0) */
-    fe_sound_entry =
-	gnome_file_entry_new("filter_sounds", _("Use Sound..."));
-#endif /* GTK_CHECK_VERSION(2, 6, 0) */
     gtk_table_attach(GTK_TABLE(table), fe_sound_entry, 1, 2, 0, 1,
 		     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 5, 5);
     /* fe_sound_entry is initially sensitive, so to be consistent 
@@ -385,15 +421,8 @@ build_action_page(GtkWindow * window)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fe_sound_button), TRUE);
     g_signal_connect(G_OBJECT(fe_sound_button), "toggled",
                      G_CALLBACK(fe_button_toggled), fe_sound_entry);
-#if GTK_CHECK_VERSION(2, 6, 0)
     g_signal_connect(G_OBJECT(dialog), "response",
                      G_CALLBACK(fe_sound_response), NULL);
-#else /* GTK_CHECK_VERSION(2, 6, 0) */
-    g_signal_connect(G_OBJECT
-                     (gnome_file_entry_gtk_entry
-                      (GNOME_FILE_ENTRY(fe_sound_entry))), "changed",
-                     G_CALLBACK(fe_action_changed), NULL);
-#endif /* GTK_CHECK_VERSION(2, 6, 0) */
 
     fe_popup_button = gtk_check_button_new_with_label(_("Popup text:"));
     gtk_table_attach(GTK_TABLE(table),
@@ -442,6 +471,8 @@ build_action_page(GtkWindow * window)
                      G_CALLBACK(fe_action_changed), NULL);
     gtk_box_pack_start(GTK_BOX(box), fe_mailboxes, TRUE, FALSE, 1);
 
+    fe_color_buttons = fe_make_color_buttons();
+    gtk_box_pack_start(GTK_BOX(box), fe_color_buttons, TRUE, FALSE, 1);
     return page;
 }				/* end build_action_page() */
 
@@ -538,7 +569,7 @@ filters_edit_dialog(void)
 	return;
     }
     if (fe_already_open) {
-	gdk_window_raise(fe_window->window);
+	gtk_window_present(GTK_WINDOW(fe_window));
 	return;
     }
     
@@ -567,8 +598,9 @@ filters_edit_dialog(void)
 
     /* main hbox */
     hbox = gtk_hbox_new(FALSE, FILTER_EDIT_PADDING);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(fe_window)->vbox),
-		       hbox, TRUE, TRUE, FILTER_EDIT_PADDING);
+    gtk_box_pack_start(GTK_BOX
+                       (gtk_dialog_get_content_area(GTK_DIALOG(fe_window))),
+                       hbox, TRUE, TRUE, FILTER_EDIT_PADDING);
     gtk_box_pack_start(GTK_BOX(hbox), piece, FALSE, FALSE,
                        FILTER_EDIT_PADDING);
 
