@@ -19,18 +19,24 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
-#include <gnome.h>
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "folder-conf.h"
+
 #include <string.h>
 #include "balsa-app.h"
 #include "balsa-icons.h"
-#include "folder-conf.h"
 #include "mailbox-conf.h"
 #include "mailbox-node.h"
 #include "save-restore.h"
 #include "pref-manager.h"
 #include "imap-server.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
+
+#if HAVE_MACOSX_DESKTOP
+#  include "macosx-helpers.h"
+#endif
 
 typedef struct _CommonDialogData CommonDialogData;
 typedef struct _FolderDialogData FolderDialogData;
@@ -55,7 +61,8 @@ struct _FolderDialogData {
     GtkWidget *folder_name, *port, *username, *anonymous, *remember,
         *password, *subscribed, *list_inbox, *prefix;
     GtkWidget *use_ssl, *tls_mode;
-    GtkWidget *connection_limit, *enable_persistent, *has_bugs, *use_status;
+    GtkWidget *connection_limit, *enable_persistent,
+        *use_idle, *has_bugs, *use_status;
 };
 
 /* FIXME: identity_name will leak on cancelled folder edition */
@@ -82,8 +89,6 @@ folder_conf_destroy_cdd(CommonDialogData * cdd)
         g_free(cdd);
 }
 
-static const gchar folder_config_section[] = "folder-config";
-
 static void
 folder_conf_response(GtkDialog * dialog, int response,
                      CommonDialogData * cdd)
@@ -98,11 +103,12 @@ folder_conf_response(GtkDialog * dialog, int response,
 	g_object_ref(cdd->mbnode);
     switch (response) {
     case GTK_RESPONSE_HELP:
-        gnome_help_display("balsa", folder_config_section, &err);
+        gtk_show_uri(NULL, "ghelp:balsa?folder-config",
+                     gtk_get_current_event_time(), &err);
         if (err) {
             balsa_information(LIBBALSA_INFORMATION_WARNING,
-		    _("Error displaying %s: %s\n"), folder_config_section,
-                    err->message);
+                              _("Error displaying config help: %s\n"),
+                              err->message);
             g_error_free(err);
         }
 	if (cdd->mbnode)
@@ -194,6 +200,9 @@ folder_conf_clicked_ok(FolderDialogData * fcw)
         (LIBBALSA_IMAP_SERVER(s),
          gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fcw->enable_persistent)));
 #endif
+    libbalsa_imap_server_set_use_idle
+        (LIBBALSA_IMAP_SERVER(s), 
+         gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fcw->use_idle)));
     libbalsa_imap_server_set_bug
         (LIBBALSA_IMAP_SERVER(s), ISBUG_FETCH,
          gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fcw->has_bugs)));
@@ -262,12 +271,20 @@ folder_conf_imap_node(BalsaMailboxNode *mn)
     gchar *default_server;
     int r = 0;
 
+#if defined(HAVE_GNOME_KEYRING)
+    static const gchar *remember_password_message =
+        N_("_Remember password in keyring");
+#else
+    static const gchar *remember_password_message =
+        N_("_Remember password");
+#endif
+
     /* Allow only one dialog per mailbox node, and one with mn == NULL
      * for creating a new folder. */
     fcw = mn ? g_object_get_data(G_OBJECT(mn), BALSA_FOLDER_CONF_IMAP_KEY)
              : fcw_new;
     if (fcw) {
-        gdk_window_raise(GTK_WIDGET(fcw->dialog)->window);
+        gtk_window_present(GTK_WINDOW(fcw->dialog));
         return;
     }
 
@@ -284,6 +301,9 @@ folder_conf_imap_node(BalsaMailboxNode *mn)
                     mn ? _("_Update") : _("C_reate"), GTK_RESPONSE_OK,
                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                     GTK_STOCK_HELP, GTK_RESPONSE_HELP, NULL));
+#if HAVE_MACOSX_DESKTOP
+    libbalsa_macosx_menu_for_parent(GTK_WIDGET(fcw->dialog), GTK_WINDOW(balsa_app.main_window));
+#endif
     g_object_add_weak_pointer(G_OBJECT(fcw->dialog),
                               (gpointer) &fcw->dialog);
     gtk_window_set_wmclass(GTK_WINDOW(fcw->dialog), 
@@ -299,7 +319,7 @@ folder_conf_imap_node(BalsaMailboxNode *mn)
     }
 
     notebook = gtk_notebook_new();
-    gtk_box_pack_start(GTK_BOX(fcw->dialog->vbox),
+    gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(fcw->dialog)),
                        notebook, TRUE, TRUE, 0);
     table = libbalsa_create_table(9, 2);
     gtk_container_set_border_width(GTK_CONTAINER(table), 12);
@@ -322,6 +342,13 @@ folder_conf_imap_node(BalsaMailboxNode *mn)
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fcw->enable_persistent),
                                      TRUE);
 #endif
+    fcw->use_idle = 
+        balsa_server_conf_add_checkbox(&fcw->bsc,
+                                       _("Use IDLE command"));
+    if(s &&
+       libbalsa_imap_server_get_use_idle(LIBBALSA_IMAP_SERVER(s)))
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fcw->use_idle),
+                                     TRUE);
     fcw->has_bugs = 
         balsa_server_conf_add_checkbox(&fcw->bsc,
                                        _("Enable _bug workarounds"));
@@ -340,7 +367,7 @@ folder_conf_imap_node(BalsaMailboxNode *mn)
                              gtk_label_new_with_mnemonic(_("_Advanced")));
 
     /* INPUT FIELD CREATION */
-    label = libbalsa_create_label(_("Descriptive _Name:"), table, 0);
+    label = libbalsa_create_label(_("Descriptive _name:"), table, 0);
     fcw->folder_name = libbalsa_create_entry(table,
                                    G_CALLBACK(validate_folder),
                                    fcw, r++, mn ? mn->name : NULL, 
@@ -370,7 +397,7 @@ folder_conf_imap_node(BalsaMailboxNode *mn)
                                 table, r++, s ? s->try_anonymous : FALSE);
     g_signal_connect(G_OBJECT(fcw->anonymous), "toggled",
                      G_CALLBACK(anonymous_cb), fcw);
-    fcw->remember = libbalsa_create_check(_("_Remember password"), 
+    fcw->remember = libbalsa_create_check(_(remember_password_message), 
                                 table, r++, s ? s->remember_passwd : TRUE);
     g_signal_connect(G_OBJECT(fcw->remember), "toggled",
                      G_CALLBACK(remember_cb), fcw);
@@ -532,10 +559,14 @@ browse_button_cb(GtkWidget * widget, SubfolderDialogData * sdd)
                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                     GTK_STOCK_OK, GTK_RESPONSE_OK,
                                     NULL);
+#if HAVE_MACOSX_DESKTOP
+    libbalsa_macosx_menu_for_parent(dialog, GTK_WINDOW(sdd->dialog));
+#endif
     
     scroll = gtk_scrolled_window_new(NULL, NULL);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), scroll,
-                       TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX
+                       (gtk_dialog_get_content_area(GTK_DIALOG(dialog))),
+                       scroll, TRUE, TRUE, 0);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
                                    GTK_POLICY_AUTOMATIC,
                                    GTK_POLICY_ALWAYS);
@@ -616,8 +647,12 @@ folder, parent);
                                                              _("Cancel"),
                                                              GTK_RESPONSE_CANCEL,
                                                              NULL);
-                gtk_container_add(GTK_CONTAINER(GTK_DIALOG(ask)->vbox),
-                                  gtk_label_new(msg));
+#if HAVE_MACOSX_DESKTOP
+		libbalsa_macosx_menu_for_parent(ask, GTK_WINDOW(sdd->dialog));
+#endif
+                gtk_container_add(GTK_CONTAINER
+                                  (gtk_dialog_get_content_area
+                                   (GTK_DIALOG(ask))), gtk_label_new(msg));
                 g_free(msg);
                 button = gtk_dialog_run(GTK_DIALOG(ask));
                 gtk_widget_destroy(ask);
@@ -715,7 +750,7 @@ folder_conf_imap_sub_node(BalsaMailboxNode * mn)
     sdd = mn ? g_object_get_data(G_OBJECT(mn), BALSA_FOLDER_CONF_IMAP_KEY)
              : sdd_new;
     if (sdd) {
-        gdk_window_raise(GTK_WIDGET(sdd->dialog)->window);
+        gtk_window_present(GTK_WINDOW(sdd->dialog));
         return;
     }
 
@@ -748,6 +783,9 @@ folder_conf_imap_sub_node(BalsaMailboxNode * mn)
                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, 
                     GTK_STOCK_HELP,   GTK_RESPONSE_HELP,
                     NULL));
+#if HAVE_MACOSX_DESKTOP
+    libbalsa_macosx_menu_for_parent(GTK_WIDGET(sdd->dialog), GTK_WINDOW(balsa_app.main_window));
+#endif
     g_object_add_weak_pointer(G_OBJECT(sdd->dialog),
                               (gpointer) &sdd->dialog);
     /* `Enter' key => Create: */
@@ -767,7 +805,7 @@ folder_conf_imap_sub_node(BalsaMailboxNode * mn)
 
     frame = gtk_frame_new(mn ? _("Rename or move subfolder") :
 			       _("Create subfolder"));
-    gtk_box_pack_start(GTK_BOX(sdd->dialog->vbox),
+    gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(sdd->dialog)),
                        frame, TRUE, TRUE, 0);
     table = libbalsa_create_table(3, 3);
     gtk_container_set_border_width(GTK_CONTAINER(table), 12);
@@ -837,6 +875,9 @@ folder_conf_delete(BalsaMailboxNode* mbnode)
                                    "You may use \"New IMAP Folder\" "
                                    "later to add this folder again.\n"),
                                  mbnode->name);
+#if HAVE_MACOSX_DESKTOP
+    libbalsa_macosx_menu_for_parent(ask, GTK_WINDOW(balsa_app.main_window));
+#endif
     gtk_window_set_title(GTK_WINDOW(ask), _("Confirm"));
 
     response = gtk_dialog_run(GTK_DIALOG(ask));
