@@ -39,21 +39,29 @@
    The mailbox_name field is displayed only for special mailboxes
    and POP3 mailboxes.
 */
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "mailbox-conf.h"
 
-#include <gnome.h>
+#include <gtk/gtk.h>
 #include <string.h>
 
 #include "balsa-app.h"
 #include "balsa-mblist.h"
-#include "mailbox-conf.h"
 #include "mailbox-node.h"
 #include "pref-manager.h"
 #include "save-restore.h"
 
 #include "libbalsa.h"
 #include "imap-server.h"
-#include "i18n.h"
+#include "mailbox-filter.h"
+#include "libbalsa-conf.h"
+#include <glib/gi18n.h>
+
+#if HAVE_MACOSX_DESKTOP
+#  include "macosx-helpers.h"
+#endif
 
 struct _BalsaMailboxConfView {
     GtkWindow *window;
@@ -73,17 +81,11 @@ struct _MailboxConfWindow {
     void (*ok_handler)(MailboxConfWindow*);
     const gchar *ok_button_name;
     GtkWidget *mailbox_name;
-    GtkType mailbox_type;
+    GType mailbox_type;
     BalsaMailboxConfView *view_info;
     gboolean ok_sensitive;
 
     union {
-#if !GTK_CHECK_VERSION(2, 6, 0)
-	/* for local mailboxes */
-	struct local { 
-	    GnomeFileEntry *path;
-	} local;
-#endif /* GTK_CHECK_VERSION(2, 6, 0) */
 	/* for imap mailboxes & directories */
 	struct {
 	    GtkWidget *port;
@@ -110,9 +112,6 @@ struct _MailboxConfWindow {
     } mb_data;
 };
 
-/* callback */
-static void check_for_blank_fields(GtkWidget *widget, MailboxConfWindow *mcw);
-
 static void mailbox_conf_update(MailboxConfWindow *conf_window);
 static void mailbox_conf_add(MailboxConfWindow *conf_window);
 
@@ -124,11 +123,12 @@ static void fill_in_imap_data(MailboxConfWindow *mcw, gchar ** name,
 static void update_imap_mailbox(MailboxConfWindow *mcw);
 
 static void update_pop_mailbox(MailboxConfWindow *mcw);
-static BalsaMailboxConfView
-    *mailbox_conf_view_new_with_size_group(LibBalsaMailbox * mailbox,
-                                           GtkWindow * window,
-                                           GtkWidget * table, gint row,
-                                           GtkSizeGroup * size_group);
+static BalsaMailboxConfView *
+    mailbox_conf_view_new_full(LibBalsaMailbox * mailbox,
+                               GtkWindow * window,
+                               GtkWidget * table, gint row,
+                               GtkSizeGroup * size_group,
+                               MailboxConfWindow * mcw);
 
 /* pages */
 static GtkWidget *create_dialog(MailboxConfWindow *mcw);
@@ -371,6 +371,7 @@ mailbox_conf_delete(BalsaMailboxNode * mbnode)
     gint button;
     GtkWidget *ask;
     LibBalsaMailbox* mailbox = mbnode->mailbox;
+    gchar *url, *group;
 
     if(BALSA_IS_MAILBOX_SPECIAL(mailbox)) {
 	balsa_information(
@@ -440,6 +441,9 @@ mailbox_conf_delete(BalsaMailboxNode * mbnode)
                                NULL);
     }
     
+#if HAVE_MACOSX_DESKTOP
+    libbalsa_macosx_menu_for_parent(ask, GTK_WINDOW(balsa_app.main_window));
+#endif
     button = gtk_dialog_run(GTK_DIALOG(ask));
     gtk_widget_destroy(ask);
 
@@ -449,6 +453,9 @@ mailbox_conf_delete(BalsaMailboxNode * mbnode)
     if ( button < 0)
 	return;
 
+    /* Save the mailbox URL */
+    url = g_strdup(mailbox->url ? mailbox->url : mailbox->name);
+
     /* Delete it from the config file and internal nodes */
     config_mailbox_delete(mailbox);
 
@@ -492,6 +499,18 @@ mailbox_conf_delete(BalsaMailboxNode * mbnode)
     } else
 	balsa_mblist_mailbox_node_remove(mbnode);
     update_mail_servers();
+
+    /* Clean up filters */
+    group = mailbox_filters_section_lookup(url);
+    if (group) {
+        libbalsa_conf_remove_group(group);
+        g_free(group);
+    }
+
+    /* Remove view */
+    config_view_remove(url);
+
+    g_free(url);
 }
 
 #define MCW_RESPONSE 1
@@ -514,7 +533,7 @@ conf_response_cb(GtkDialog* dialog, gint response, MailboxConfWindow * mcw)
 }
 
 static GtkWidget *
-run_mailbox_conf(BalsaMailboxNode* mbnode, GtkType mailbox_type, 
+run_mailbox_conf(BalsaMailboxNode* mbnode, GType mailbox_type, 
 		 gboolean update)
 {
     MailboxConfWindow* mcw;
@@ -557,12 +576,12 @@ run_mailbox_conf(BalsaMailboxNode* mbnode, GtkType mailbox_type,
  * If the used clicks save add the new mailbox to the tree.
  */
 void
-mailbox_conf_new(GtkType mailbox_type)
+mailbox_conf_new(GType mailbox_type)
 {
     static GtkWidget *dialog;
 
     if (dialog) {
-        gdk_window_raise(dialog->window);
+        gtk_window_present(GTK_WINDOW(dialog));
         return;
     }
 
@@ -583,7 +602,7 @@ mailbox_conf_edit(BalsaMailboxNode * mbnode)
     dialog = g_object_get_data(G_OBJECT(mbnode->mailbox),
                                BALSA_MAILBOX_CONF_DIALOG);
     if (dialog) {
-        gdk_window_raise(dialog->window);
+        gtk_window_present(GTK_WINDOW(dialog));
         return;
     }
 
@@ -614,7 +633,10 @@ mailbox_conf_set_values(MailboxConfWindow *mcw)
             GtkFileChooser *chooser = GTK_FILE_CHOOSER(mcw->window);
 	    LibBalsaMailboxLocal *local = LIBBALSA_MAILBOX_LOCAL(mailbox);
             const gchar *path = libbalsa_mailbox_local_get_path(local);
+            gchar *basename = g_path_get_basename(path);
             gtk_file_chooser_set_filename(chooser, path);
+            gtk_file_chooser_set_current_name(chooser, basename);
+            g_free(basename);
         }
     } else if (LIBBALSA_IS_MAILBOX_POP3(mailbox)) {
 	LibBalsaMailboxPop3 *pop3;
@@ -703,7 +725,7 @@ mailbox_conf_set_values(MailboxConfWindow *mcw)
  * Checks for blank fields in the dialog.
  * Sets the sensitivity of the Update/Add button accordingly.
  * This function should be attached to a change event signal 
- * on any widget which can effect the validity of the input.
+ * on any widget which can affect the validity of the input.
  */
 static void
 check_for_blank_fields(GtkWidget *widget, MailboxConfWindow *mcw)
@@ -963,23 +985,24 @@ mailbox_conf_add(MailboxConfWindow * mcw)
     if ( LIBBALSA_IS_MAILBOX_LOCAL(mcw->mailbox) ) {
 	LibBalsaMailboxLocal *ml  = LIBBALSA_MAILBOX_LOCAL(mcw->mailbox);
 	gchar *path;
-	int res;
 
         path =
             gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(mcw->window));
-	res = libbalsa_mailbox_local_set_path(ml, path, TRUE);
 
-	if( res != 0) {
+        if (libbalsa_mailbox_local_set_path(ml, path, TRUE) != 0) {
+            g_free(path);
 	    g_object_unref(G_OBJECT(mcw->mailbox));
 	    mcw->mailbox = NULL;
 	    return;
 	}
 
-	save_to_config = balsa_app.local_mail_directory == NULL
-	    || strncmp(balsa_app.local_mail_directory, path,
-		       strlen(balsa_app.local_mail_directory)) != 0;
+	save_to_config =
+            !libbalsa_path_is_below_dir(path,
+                                        balsa_app.local_mail_directory);
         printf("Save to config: %d\n", save_to_config);
 	mcw->mailbox->name = g_path_get_basename(path);
+        g_free(path);
+
 	balsa_mailbox_local_append(mcw->mailbox);
     }
     mbnode = balsa_mailbox_node_new_from_mailbox(mcw->mailbox);
@@ -1056,6 +1079,35 @@ balsa_get_entry(GtkWidget * widget, GtkWidget ** entry)
                               (GtkCallback) balsa_get_entry, entry);
 }
 
+/*
+ * Callback for the file chooser's "selection-changed" signal and its
+ * entry's "changed" signal
+ *
+ * If the path has really changed, call check_for_blank_fields to set
+ * the sensitivity of the buttons appropriately.  If it hasn't, this is
+ * probably just the file chooser being initialized in an idle callback,
+ * so we don't change button sensitivity.
+ */
+static void
+local_mailbox_dialog_cb(GtkWidget * widget, MailboxConfWindow * mcw)
+{
+    gchar *filename =
+        gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(mcw->window));
+
+    if (filename) {
+        gboolean changed = TRUE;
+        if (mcw->mailbox) {
+            LibBalsaMailboxLocal *local =
+                LIBBALSA_MAILBOX_LOCAL(mcw->mailbox);
+            const gchar *path = libbalsa_mailbox_local_get_path(local);
+            changed = strcmp(filename, path);
+        }
+        g_free(filename);
+        if (changed)
+            check_for_blank_fields(widget, mcw);
+    }
+}
+
 static GtkWidget *
 create_local_mailbox_dialog(MailboxConfWindow *mcw)
 {
@@ -1074,7 +1126,7 @@ create_local_mailbox_dialog(MailboxConfWindow *mcw)
         label = libbalsa_create_label(_("_Mailbox Name:"), table, ++row);
         mcw->mailbox_name = 
             libbalsa_create_entry(table,
-                         GTK_SIGNAL_FUNC(check_for_blank_fields),
+                         G_CALLBACK(check_for_blank_fields),
                          mcw, row, NULL, label);
     } else mcw->mailbox_name = NULL;
 
@@ -1088,6 +1140,9 @@ create_local_mailbox_dialog(MailboxConfWindow *mcw)
                                     mcw->ok_button_name, MCW_RESPONSE,
                                     GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                                     NULL);
+#if HAVE_MACOSX_DESKTOP
+    libbalsa_macosx_menu_for_parent(dialog, GTK_WINDOW(balsa_app.main_window));
+#endif
 
     size_group = libbalsa_create_size_group(dialog);
     if (label)
@@ -1097,16 +1152,15 @@ create_local_mailbox_dialog(MailboxConfWindow *mcw)
     gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog),
 	                                balsa_app.local_mail_directory);
     g_signal_connect(G_OBJECT(dialog), "selection-changed",
-                     G_CALLBACK(check_for_blank_fields), mcw);
+                     G_CALLBACK(local_mailbox_dialog_cb), mcw);
     balsa_get_entry(dialog, &entry);
     if (entry)
 	g_signal_connect(G_OBJECT(entry), "changed",
-                         G_CALLBACK(check_for_blank_fields), mcw);
+                         G_CALLBACK(local_mailbox_dialog_cb), mcw);
 
     mcw->view_info =
-        mailbox_conf_view_new_with_size_group(mcw->mailbox,
-                                              GTK_WINDOW(dialog),
-                                              table, ++row, size_group);
+        mailbox_conf_view_new_full(mcw->mailbox, GTK_WINDOW(dialog), table,
+                                   ++row, size_group, mcw);
 
     return dialog;
 }
@@ -1121,6 +1175,9 @@ create_generic_dialog(MailboxConfWindow * mcw)
                                     mcw->ok_button_name, MCW_RESPONSE,
                                     GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                                     NULL);
+#if HAVE_MACOSX_DESKTOP
+    libbalsa_macosx_menu_for_parent(dialog, GTK_WINDOW(balsa_app.main_window));
+#endif
     return dialog;
 }
 
@@ -1137,14 +1194,14 @@ create_pop_mailbox_dialog(MailboxConfWindow *mcw)
                              gtk_label_new_with_mnemonic(_("_Basic")));
 
     /* mailbox name */
-    label = libbalsa_create_label(_("Mailbox _Name:"), table, 0);
+    label = libbalsa_create_label(_("Mailbox _name:"), table, 0);
     mcw->mailbox_name = libbalsa_create_entry(table,
-				     GTK_SIGNAL_FUNC(check_for_blank_fields),
+				     G_CALLBACK(check_for_blank_fields),
 				     mcw, 0, NULL, label);
     /* pop server */
     label = libbalsa_create_label(_("_Server:"), table, 1);
     mcw->mb_data.pop3.bsc.server = libbalsa_create_entry(table,
-				     GTK_SIGNAL_FUNC(check_for_blank_fields),
+				     G_CALLBACK(check_for_blank_fields),
 				     mcw, 1, "localhost", label);
     mcw->mb_data.pop3.bsc.default_ports = POP3_DEFAULT_PORTS;
 
@@ -1152,7 +1209,7 @@ create_pop_mailbox_dialog(MailboxConfWindow *mcw)
     /* username  */
     label= libbalsa_create_label(_("Use_r name:"), table, 2);
     mcw->mb_data.pop3.username = libbalsa_create_entry(table,
-				     GTK_SIGNAL_FUNC(check_for_blank_fields),
+				     G_CALLBACK(check_for_blank_fields),
 				     mcw, 2, g_get_user_name(), label);
 
     /* password field */
@@ -1181,7 +1238,7 @@ create_pop_mailbox_dialog(MailboxConfWindow *mcw)
     label = libbalsa_create_label(_("Fi_lter Command:"), table, 7);
     mcw->mb_data.pop3.filter_cmd =
 	libbalsa_create_entry(table,
-		     GTK_SIGNAL_FUNC(check_for_blank_fields),
+		     G_CALLBACK(check_for_blank_fields),
 		     mcw, 7, "procmail -f -", label);
 
     advanced =
@@ -1199,7 +1256,9 @@ create_pop_mailbox_dialog(MailboxConfWindow *mcw)
     gtk_widget_grab_focus(mcw->mailbox_name);
 
     dialog = create_generic_dialog(mcw);
-    gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), notebook);
+    gtk_container_add(GTK_CONTAINER
+                      (gtk_dialog_get_content_area(GTK_DIALOG(dialog))),
+                      notebook);
     return dialog;
 }
 
@@ -1232,6 +1291,14 @@ create_imap_mailbox_dialog(MailboxConfWindow *mcw)
     GtkWidget *entry;
     gint row = -1;
 
+#if defined(HAVE_GNOME_KEYRING)
+    static const gchar *remember_password_message =
+        N_("_Remember password in keyring");
+#else
+    static const gchar *remember_password_message =
+        N_("_Remember password");
+#endif
+
     notebook = gtk_notebook_new();
     table = libbalsa_create_table(8, 2);
     gtk_container_set_border_width(GTK_CONTAINER(table), 12);
@@ -1239,16 +1306,16 @@ create_imap_mailbox_dialog(MailboxConfWindow *mcw)
                              gtk_label_new_with_mnemonic(_("_Basic")));
 
     /* mailbox name */
-    label = libbalsa_create_label(_("Mailbox _Name:"), table, ++row);
+    label = libbalsa_create_label(_("Mailbox _name:"), table, ++row);
     mcw->mailbox_name =  libbalsa_create_entry(table,
-                                      GTK_SIGNAL_FUNC(check_for_blank_fields),
+                                      G_CALLBACK(check_for_blank_fields),
                                       mcw, row, NULL, label);
 
     /* imap server */
     label = libbalsa_create_label(_("_Server:"), table, ++row);
     mcw->mb_data.imap.bsc.server = 
 	libbalsa_create_entry(table,
-		     GTK_SIGNAL_FUNC(check_for_blank_fields),
+		     G_CALLBACK(check_for_blank_fields),
 		     mcw, row, "localhost", label);
     mcw->mb_data.imap.bsc.default_ports = IMAP_DEFAULT_PORTS;
 
@@ -1256,7 +1323,7 @@ create_imap_mailbox_dialog(MailboxConfWindow *mcw)
     label = libbalsa_create_label(_("_Username:"), table, ++row);
     mcw->mb_data.imap.username = 
 	libbalsa_create_entry(table,
-		     GTK_SIGNAL_FUNC(check_for_blank_fields),
+		     G_CALLBACK(check_for_blank_fields),
 		     mcw, row, g_get_user_name(), label);
 
     /* toggle for anonymous password */
@@ -1267,7 +1334,7 @@ create_imap_mailbox_dialog(MailboxConfWindow *mcw)
                      G_CALLBACK(anon_toggle_cb), mcw);
     /* toggle for remember password */
     mcw->mb_data.imap.remember = 
-	libbalsa_create_check(_("_Remember Password"), table, ++row,
+	libbalsa_create_check(_(remember_password_message), table, ++row,
 		     FALSE);
     g_signal_connect(G_OBJECT(mcw->mb_data.imap.remember), "toggled",
                      G_CALLBACK(remember_toggle_cb), mcw);
@@ -1278,7 +1345,7 @@ create_imap_mailbox_dialog(MailboxConfWindow *mcw)
 	libbalsa_create_entry(table, NULL, NULL, row, NULL, label);
     gtk_entry_set_visibility(GTK_ENTRY(mcw->mb_data.imap.password), FALSE);
 
-    label = libbalsa_create_label(_("F_older Path:"), table, ++row);
+    label = libbalsa_create_label(_("F_older path:"), table, ++row);
 
     mcw->mb_data.imap.folderpath = entry = gtk_entry_new();
     gtk_entry_set_text(GTK_ENTRY(mcw->mb_data.imap.folderpath), "INBOX");
@@ -1313,11 +1380,13 @@ create_imap_mailbox_dialog(MailboxConfWindow *mcw)
                           mcw->mailbox_name : mcw->mb_data.imap.bsc.server);
 
     dialog = create_generic_dialog(mcw);
-    gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), notebook);
+    gtk_container_add(GTK_CONTAINER
+                      (gtk_dialog_get_content_area(GTK_DIALOG(dialog))),
+                      notebook);
 
     mcw->view_info =
-        mailbox_conf_view_new(mcw->mailbox, GTK_WINDOW(dialog), table,
-                              ++row);
+        mailbox_conf_view_new_full(mcw->mailbox, GTK_WINDOW(dialog), table,
+                                   ++row, NULL, mcw);
 
     return dialog;
 }
@@ -1347,10 +1416,11 @@ mailbox_conf_view_free(BalsaMailboxConfView *view_info)
  * row:         the row of the table in which to start.
  */
 static BalsaMailboxConfView *
-mailbox_conf_view_new_with_size_group(LibBalsaMailbox * mailbox,
-                                      GtkWindow * window,
-                                      GtkWidget * table, gint row,
-                                      GtkSizeGroup * size_group)
+mailbox_conf_view_new_full(LibBalsaMailbox * mailbox,
+                           GtkWindow * window,
+                           GtkWidget * table, gint row,
+                           GtkSizeGroup * size_group,
+                           MailboxConfWindow * mcw)
 {
     GtkWidget *label;
     BalsaMailboxConfView *view_info;
@@ -1369,6 +1439,9 @@ mailbox_conf_view_new_with_size_group(LibBalsaMailbox * mailbox,
         gtk_size_group_add_widget(size_group, label);
 
     view_info->identity_combo_box = widget = gtk_combo_box_new_text();
+    if (mcw)
+        g_signal_connect(view_info->identity_combo_box, "changed",
+                         G_CALLBACK(check_for_blank_fields), mcw);
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), widget);
     identity_name = libbalsa_mailbox_get_identity_name(mailbox);
     for (list = balsa_app.identities, active = 0; list;
@@ -1409,6 +1482,9 @@ mailbox_conf_view_new_with_size_group(LibBalsaMailbox * mailbox,
         gtk_combo_box_set_active(GTK_COMBO_BOX(view_info->chk_crypt),
                                  libbalsa_mailbox_get_crypto_mode
                                  (mailbox));
+        if (mcw)
+            g_signal_connect(view_info->chk_crypt, "changed",
+                             G_CALLBACK(check_for_blank_fields), mcw);
 	gtk_table_attach(GTK_TABLE(table), view_info->chk_crypt,
 		         1, 2, row, row + 1,
 			 GTK_EXPAND | GTK_FILL, 0, 0, 0);
@@ -1421,6 +1497,9 @@ mailbox_conf_view_new_with_size_group(LibBalsaMailbox * mailbox,
                               table, ++row,
                               libbalsa_mailbox_get_show(mailbox) ==
                               LB_MAILBOX_SHOW_TO);
+    if (mcw)
+        g_signal_connect(view_info->show_to, "toggled",
+                         G_CALLBACK(check_for_blank_fields), mcw);
     
     /* Subscribe check button */
     view_info->subscribe =
@@ -1428,6 +1507,9 @@ mailbox_conf_view_new_with_size_group(LibBalsaMailbox * mailbox,
                               ++row,
                               libbalsa_mailbox_get_subscribe(mailbox) !=
                               LB_MAILBOX_SUBSCRIBE_NO);
+    if (mcw)
+        g_signal_connect(view_info->subscribe, "toggled",
+                         G_CALLBACK(check_for_blank_fields), mcw);
 
     return view_info;
 }
@@ -1436,8 +1518,8 @@ BalsaMailboxConfView *
 mailbox_conf_view_new(LibBalsaMailbox * mailbox,
                       GtkWindow * window, GtkWidget * table, gint row)
 {
-    return mailbox_conf_view_new_with_size_group(mailbox, window, table,
-                                                 row, NULL);
+    return mailbox_conf_view_new_full(mailbox, window, table, row,
+                                      NULL, NULL);
 }
 
 #ifdef HAVE_GPGME
