@@ -19,17 +19,25 @@
  * 02111-1307, USA.
  */
 
-#define _XOPEN_SOURCE 500
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
 
+#if HAVE_UNIQUE
+#include <gtk/gtk.h>
+#include <unique/unique.h>
+#endif                          /* HAVE_UNIQUE */
+
+#if HAVE_GNOME
 #include <gnome.h>
-#include <libgnomeui/gnome-window-icon.h>
+#endif
 
 #ifdef GTKHTML_HAVE_GCONF
 # include <gconf/gconf.h>
 #endif
 
 #include <signal.h>
+#include <stdlib.h>
 
 #ifdef BALSA_USE_THREADS
 #include <pthread.h>
@@ -57,12 +65,14 @@
 #include "information.h"
 #include "imap-server.h"
 
-#include "libinit_balsa/init_balsa.h"
+#include "libinit_balsa/assistant_init.h"
 
+#if !HAVE_UNIQUE && HAVE_GNOME
 #include "Balsa.h"
 #include "balsa-bonobo.h"
 #include <bonobo-activation/bonobo-activation.h>
 #include <bonobo/bonobo-exception.h>
+#endif                          /* HAVE_UNIQUE */
 
 #ifdef HAVE_GPGME
 #include <string.h>
@@ -88,15 +98,17 @@ static void threads_init(void);
 static void threads_destroy(void);
 #endif				/* BALSA_USE_THREADS */
 
-static void balsa_init(int argc, char **argv);
+static gboolean balsa_init(int argc, char **argv);
 static void config_init(gboolean check_only);
 static void mailboxes_init(gboolean check_only);
 static void balsa_cleanup(void);
+#if HAVE_GNOME
 static gint balsa_kill_session(GnomeClient * client, gpointer client_data);
 static gint balsa_save_session(GnomeClient * client, gint phase,
 			       GnomeSaveStyle save_style, gint is_shutdown,
 			       GnomeInteractStyle interact_style,
 			       gint is_fast, gpointer client_data);
+#endif
 gboolean initial_open_unread_mailboxes(void); 
 /* yes void is there cause gcc is tha suck */
 gboolean initial_open_inbox(void);
@@ -132,10 +144,207 @@ accel_map_save(void)
     g_free(accel_map_filename);
 }
 
+static gboolean
+balsa_main_check_new_messages(gpointer data)
+{
+    check_new_messages_real(data, TYPE_CALLBACK);
+    return FALSE;
+}
+
+void balsa_get_stats(long *unread, long *unsent);
+
+#if HAVE_UNIQUE
+enum {
+    COMMAND_0,                  /* unused: 0 is an invalid command */
+
+    COMMAND_CHECK_MAIL,
+    COMMAND_GET_STATS,
+    COMMAND_OPEN_UNREAD,
+    COMMAND_OPEN_INBOX,
+    COMMAND_OPEN_MAILBOX,
+    COMMAND_COMPOSE
+};
+
+static UniqueResponse
+mw_message_received_cb(UniqueApp         *app,
+                       UniqueCommand      command,
+                       UniqueMessageData *message,
+                       guint              message_time,
+                       gpointer           user_data)
+{
+    GtkWindow *window = GTK_WINDOW(user_data);
+    UniqueResponse res = UNIQUE_RESPONSE_OK;
+    glong unread, unsent;
+    GError *err = NULL;
+    gchar *text;
+    gchar *filename;
+    gchar **uris, **p;
+    BalsaSendmsg *snd;
+
+    switch (command) {
+    case UNIQUE_ACTIVATE:
+        /* move the main window to the screen that sent us the command */
+        gtk_window_set_screen(window,
+                              unique_message_data_get_screen(message));
+        gtk_window_present(window);
+        break;
+    case COMMAND_CHECK_MAIL:
+        balsa_main_check_new_messages(balsa_app.main_window);
+        break;
+    case COMMAND_GET_STATS:
+        balsa_get_stats(&unread, &unsent);
+        text =
+            g_strdup_printf("Unread: %ld Unsent: %ld\n", unread, unsent);
+#if UNIQUE_CHECK_VERSION(1, 0, 2)
+        filename = unique_message_data_get_filename(message);
+#else                           /* UNIQUE_CHECK_VERSION(1, 0, 2) */
+        filename = unique_message_data_get_text(message);
+#endif                          /* UNIQUE_CHECK_VERSION(1, 0, 2) */
+        if (!g_file_set_contents(filename, text, -1, &err)) {
+            balsa_information_parented(window,
+                                       LIBBALSA_INFORMATION_WARNING,
+                                       _("Could not write to %s: %s"),
+                                       filename, err->message);
+            g_error_free(err);
+        }
+        g_free(filename);
+        g_free(text);
+        break;
+    case COMMAND_OPEN_UNREAD:
+        initial_open_unread_mailboxes();
+        break;
+    case COMMAND_OPEN_INBOX:
+        initial_open_inbox();
+        break;
+    case COMMAND_OPEN_MAILBOX:
+        text = unique_message_data_get_text(message);
+        uris = g_strsplit(text, ";", 20);
+        g_free(text);
+        g_idle_add((GSourceFunc) open_mailboxes_idle_cb, uris);
+        break;
+    case COMMAND_COMPOSE:
+        gdk_threads_enter();
+        snd = sendmsg_window_compose();
+        gdk_threads_leave();
+
+        uris = unique_message_data_get_uris(message);
+        text = uris[0];
+        if (text) {
+            gchar *decoded = libbalsa_urldecode(text);
+            if (g_ascii_strncasecmp(decoded, "mailto:", 7) == 0)
+                sendmsg_window_process_url(decoded + 7,
+                                           sendmsg_window_set_field, snd);
+            else
+                sendmsg_window_set_field(snd, "to", decoded);
+            g_free(decoded);
+            for (p = uris + 1; *p; p++)
+                add_attachment(snd, *p, FALSE, NULL);
+        }
+        g_strfreev(uris);
+        snd->quit_on_close = FALSE;
+        break;
+    default:
+        break;
+    }
+
+    return res;
+}
+
+static void
+balsa_handle_automation_options(UniqueApp * app)
+{
+    printf("Another Balsa found. Talking to it...\n");
+
+    if (!(opt_compose_email || opt_attach_list))
+        /* Move the main window to the request's screen */
+        unique_app_send_message(app, UNIQUE_ACTIVATE, NULL);
+
+    if (cmd_check_mail_on_startup)
+        unique_app_send_message(app, COMMAND_CHECK_MAIL, NULL);
+
+    if (cmd_get_stats) {
+        gint fd;
+        gchar *name_used;
+        GError *err = NULL;
+
+        fd = g_file_open_tmp("balsa-get-stats-XXXXXX", &name_used, &err);
+        if (fd < 0) {
+            g_warning("Could not create temporary file: %s", err->message);
+            g_error_free(err);
+        } else {
+            UniqueMessageData *message;
+            UniqueResponse response;
+
+            close(fd);
+
+            message = unique_message_data_new();
+#if UNIQUE_CHECK_VERSION(1, 0, 2)
+            unique_message_data_set_filename(message, name_used);
+#else                           /* UNIQUE_CHECK_VERSION(1, 0, 2) */
+            unique_message_data_set_text(message, name_used, -1);
+#endif                          /* UNIQUE_CHECK_VERSION(1, 0, 2) */
+            response =
+                unique_app_send_message(app, COMMAND_GET_STATS, message);
+            unique_message_data_free(message);
+
+            if (response == UNIQUE_RESPONSE_OK) {
+                gchar *text;
+
+                if (!g_file_get_contents(name_used, &text, NULL, &err)) {
+                    g_warning("Could not read %s: %s",
+                              name_used, err->message);
+                    g_error_free(err);
+                } else {
+                    g_print("%s", text);
+                    g_free(text);
+                }
+            }
+            if (unlink(name_used) < 0)
+                g_warning("Could not unlink temporary file %s: %s",
+                          name_used, g_strerror(errno));
+            g_free(name_used);
+        }
+    }
+
+    if (cmd_open_unread_mailbox)
+        unique_app_send_message(app, COMMAND_OPEN_UNREAD, NULL);
+
+    if (cmd_open_inbox)
+        unique_app_send_message(app, COMMAND_OPEN_INBOX, NULL);
+
+    if (cmd_line_open_mailboxes) {
+        UniqueMessageData *message = unique_message_data_new();
+        unique_message_data_set_text(message, cmd_line_open_mailboxes, -1);
+        unique_app_send_message(app, COMMAND_OPEN_MAILBOX, message);
+        unique_message_data_free(message);
+    }
+
+    if (opt_compose_email || opt_attach_list) {
+        UniqueMessageData *message = unique_message_data_new();
+        GSList *l;
+        gchar **uris = g_new(gchar *, g_slist_length(opt_attach_list) + 2);
+        gint i;
+
+        uris[0] =
+            libbalsa_urlencode(opt_compose_email ?
+                               opt_compose_email : "mailto");
+
+        for (l = opt_attach_list, i = 1; l; l = l->next, i++)
+            uris[i] = g_strdup(l->data ? l->data : "");
+        uris[i] = NULL;
+
+        unique_message_data_set_uris(message, uris);
+        g_strfreev(uris);
+
+        unique_app_send_message(app, COMMAND_COMPOSE, message);
+        unique_message_data_free(message);
+    }
+}
+#elif HAVE_GNOME
 static void
 balsa_handle_automation_options() {
    CORBA_Object factory;
-   CORBA_Environment ev;
+   CORBA_Environment ev, *ev_p = &ev;
    BonoboObject *balsacomposer;
    BonoboObject *balsaapp;
  
@@ -146,7 +355,7 @@ balsa_handle_automation_options() {
 	Bonobo_ACTIVATION_FLAG_EXISTING_ONLY,
 	NULL, &ev);
    
-   if ( !(BONOBO_EX (&ev) || factory == CORBA_OBJECT_NIL) ) {
+   if ( !(BONOBO_EX (ev_p) || factory == CORBA_OBJECT_NIL) ) {
        /* there already is a server. good */
        CORBA_Object app;
        printf("Another Balsa found. Talking to it...\n");
@@ -216,14 +425,15 @@ balsa_handle_automation_options() {
    }
    
 }
+#endif                          /* HAVE_UNIQUE */
 
 /* balsa_init:
    FIXME - check for memory leaks.
 */
-static void
+static gboolean
 balsa_init(int argc, char **argv)
 {
-#ifndef GNOME_PARAM_GOPTION_CONTEXT
+#if (HAVE_GNOME && !defined(GNOME_PARAM_GOPTION_CONTEXT))
     static char *attachment = NULL;
     int opt;
     poptContext context;
@@ -284,7 +494,7 @@ balsa_init(int argc, char **argv)
 	{"compose", 'm', 0, G_OPTION_ARG_STRING, &(opt_compose_email),
 	 N_("Compose a new email to EMAIL@ADDRESS"), "EMAIL@ADDRESS"},
 	{"attach", 'a', 0, G_OPTION_ARG_FILENAME_ARRAY, &(attach_vect),
-	 N_("Attach file at PATH"), "PATH"},
+	 N_("Attach file at URI"), "URI"},
 	{"open-mailbox", 'o', 0, G_OPTION_ARG_STRING,
          &(cmd_line_open_mailboxes),
 	 N_("Opens MAILBOXNAME"), N_("MAILBOXNAME")},
@@ -307,6 +517,7 @@ balsa_init(int argc, char **argv)
           "Special option that collects any remaining arguments for us" },
         { NULL }
     };
+#if HAVE_GNOME
     GOptionContext *option_context = g_option_context_new("balsa");
     GnomeProgram *my_app;
     g_option_context_add_main_entries(option_context, option_entries, NULL);
@@ -317,6 +528,19 @@ balsa_init(int argc, char **argv)
                                 GNOME_PARAM_APP_DATADIR,
                                 BALSA_STD_PREFIX "/share",
                                 GNOME_PARAM_NONE);
+    gtk_init_check(&argc, &argv);
+#else /* HAVE_GNOME */
+    GError *err = NULL;
+
+    if (!gtk_init_with_args(&argc, &argv, PACKAGE, option_entries, NULL,
+                            &err)) {
+        g_print("%s\n", err->message);
+        g_print(_("Run '%s --help' to see a full list"
+                  " of available command line options.\n"), argv[0]);
+        g_error_free(err);
+        return FALSE;
+    }
+#endif  /* HAVE_GNOME */
 
     if (remaining_args != NULL) {
         gint i, num_args;
@@ -340,8 +564,12 @@ balsa_init(int argc, char **argv)
         attach_vect = NULL;
     }
 #endif /* OPTION HANDLING */
-    balsa_handle_automation_options();  
-    
+
+#if !HAVE_UNIQUE && HAVE_GNOME
+    balsa_handle_automation_options();
+#endif /* HAVE_UNIQUE */
+
+    return TRUE;
 }
 
 /* check_special_mailboxes: 
@@ -389,7 +617,9 @@ config_init(gboolean check_only)
 {
     while(!config_load() && !check_only) {
 	balsa_init_begin();
+#if HAVE_GNOME
         config_defclient_save();
+#endif
     }
 }
 
@@ -400,7 +630,9 @@ mailboxes_init(gboolean check_only)
     if (!balsa_app.inbox && !check_only) {
 	g_warning("*** error loading mailboxes\n");
 	balsa_init_begin();
+#if HAVE_GNOME
         config_defclient_save();
+#endif
 	return;
     }
 }
@@ -465,14 +697,16 @@ threads_destroy(void)
 gboolean
 initial_open_unread_mailboxes()
 {
-    GList *i, *gl;
+    GList *l, *gl;
     gdk_threads_enter();
     gl = balsa_mblist_find_all_unread_mboxes(NULL);
 
     if (gl) {
-        for (i = g_list_first(gl); i; i = g_list_next(i)) {
-            printf("opening %s..\n", (LIBBALSA_MAILBOX(i->data))->name);
-            balsa_mblist_open_mailbox(LIBBALSA_MAILBOX(i->data));
+        for (l = gl; l; l = l->next) {
+            LibBalsaMailbox *mailbox = LIBBALSA_MAILBOX(l->data);
+
+            printf("opening %s..\n", mailbox->name);
+            balsa_mblist_open_mailbox(mailbox);
         }
         g_list_free(gl);
     }
@@ -487,15 +721,14 @@ initial_open_inbox()
     if (!balsa_app.inbox)
 	return FALSE;
 
-    printf("opening %s..\n", (LIBBALSA_MAILBOX(balsa_app.inbox))->name);
+    printf("opening %s..\n", balsa_app.inbox->name);
     gdk_threads_enter();
-    balsa_mblist_open_mailbox(LIBBALSA_MAILBOX(balsa_app.inbox));
+    balsa_mblist_open_mailbox_hidden(balsa_app.inbox);
     gdk_threads_leave();
     
     return FALSE;
 }
 
-void balsa_get_stats(long *unread, long *unsent);
 void
 balsa_get_stats(long *unread, long *unsent)
 {
@@ -721,11 +954,13 @@ int
 main(int argc, char *argv[])
 {
     GtkWidget *window;
+#if HAVE_GNOME
     GnomeClient *client;
-    gchar *default_icon;
-#ifdef GTKHTML_HAVE_GCONF
-    GError *gconf_error;
 #endif
+    gchar *default_icon;
+#if HAVE_UNIQUE
+    UniqueApp *app;
+#endif                          /* HAVE_UNIQUE */
 
 #ifdef ENABLE_NLS
     /* Initialize the i18n stuff */
@@ -745,6 +980,39 @@ main(int argc, char *argv[])
     LIBXML_TEST_VERSION
 #endif
 
+    /* FIXME: do we need to allow a non-GUI mode? */
+    if (!balsa_init(argc, argv))
+        return 0;
+
+#if HAVE_UNIQUE
+    /* as soon as we create the UniqueApp instance we either have the name
+     * we requested ("org.mydomain.MyApplication", in the example) or we
+     * don't because there already is an application using the same name
+     */
+    app = unique_app_new_with_commands("org.desktop.Balsa", NULL,
+                                       "check-mail",   COMMAND_CHECK_MAIL,
+                                       "get-stats",    COMMAND_GET_STATS,
+                                       "open-unread",  COMMAND_OPEN_UNREAD,
+                                       "open-inbox",   COMMAND_OPEN_INBOX,
+                                       "open-mailbox", COMMAND_OPEN_MAILBOX,
+                                       "compose",      COMMAND_COMPOSE,
+                                       NULL);
+
+    /* if there already is an instance running, this will return TRUE; there
+     * is no race condition because the check is already performed at
+     * construction time
+     */
+    if (unique_app_is_running(app)) {
+        balsa_handle_automation_options(app);
+        g_object_unref(app);
+        return 0;
+    }
+
+    /* this is the first instance, so we can proceed with the usual
+     * application construction sequence
+     */
+#endif                          /* HAVE_UNIQUE */
+
 #ifdef HAVE_GPGME
     /* initialise the gpgme library */
     g_message("init gpgme version %s", gpgme_check_version(NULL));
@@ -769,14 +1037,8 @@ main(int argc, char *argv[])
     }
 #endif
 
-    /* FIXME: do we need to allow a non-GUI mode? */
-    gtk_init_check(&argc, &argv);
-    balsa_init(argc, argv);
-
 #ifdef GTKHTML_HAVE_GCONF
-    if (!gconf_init(argc, argv, &gconf_error))
-	g_error_free(gconf_error);
-    gconf_error = NULL;
+    gconf_init(argc, argv, NULL);
 #endif
 
     balsa_app_init();
@@ -806,14 +1068,31 @@ main(int argc, char *argv[])
     }
 
     signal( SIGPIPE, SIG_IGN );
+#if HAVE_GNOME
     gnome_triggers_do("", "program", "balsa", "startup", NULL);
+#endif
 
     window = balsa_window_new();
     balsa_app.main_window = BALSA_WINDOW(window);
     g_object_add_weak_pointer(G_OBJECT(window),
 			      (gpointer) &balsa_app.main_window);
-    g_signal_connect(G_OBJECT(window), "destroy",
-                     G_CALLBACK(balsa_cleanup), NULL);
+
+#if HAVE_UNIQUE
+    /* the UniqueApp instance must "watch" all the top-level windows the
+     * application creates, so that it can terminate the startup
+     * notification sequence for us
+     */
+    unique_app_watch_window(app, GTK_WINDOW(window));
+
+    /* using this signal we get notifications from the newly launched instances
+     * and we can reply to them; the default signal handler will just return
+     * UNIQUE_RESPONSE_OK and terminate the startup notification sequence on each
+     * watched window, so you can connect to the message-received signal only if
+     * you want to handle the commands and responses
+     */
+    g_signal_connect(app, "message-received",
+                     G_CALLBACK(mw_message_received_cb), window);
+#endif                          /* HAVE_UNIQUE */
 
     /* load mailboxes */
     config_load_sections();
@@ -827,11 +1106,13 @@ main(int argc, char *argv[])
     }
 
     /* session management */
+#if HAVE_GNOME
     client = gnome_master_client();
     g_signal_connect(G_OBJECT(client), "save_yourself",
 		     G_CALLBACK(balsa_save_session), argv[0]);
     g_signal_connect(G_OBJECT(client), "die",
 		     G_CALLBACK(balsa_kill_session), NULL);
+#endif
 
 #ifdef HAVE_GPGME
     balsa_app.has_openpgp = 
@@ -859,25 +1140,30 @@ main(int argc, char *argv[])
         for(lst = opt_attach_list; lst; lst = g_slist_next(lst))
             add_attachment(snd, lst->data, FALSE, NULL);
 	snd->quit_on_close = FALSE;
-    };
+    }
     gtk_widget_show(window);
 
-    if (cmd_check_mail_on_startup || balsa_app.check_mail_upon_startup)
-	check_new_messages_cb(NULL, NULL);
-
-
     g_idle_add((GSourceFunc) scan_mailboxes_idle_cb, NULL);
-    g_timeout_add(1801*1000, (GSourceFunc) periodic_expunge_cb, NULL);
+    g_timeout_add_seconds(1801, (GSourceFunc) periodic_expunge_cb, NULL);
+
+    if (cmd_check_mail_on_startup || balsa_app.check_mail_upon_startup)
+        g_idle_add((GSourceFunc) balsa_main_check_new_messages,
+                   balsa_app.main_window);
 
     accel_map_load();
     gdk_threads_enter();
     gtk_main();
     gdk_threads_leave();
+
+    balsa_cleanup();
     accel_map_save();
 
 #ifdef BALSA_USE_THREADS
     threads_destroy();
 #endif
+#if HAVE_UNIQUE
+    g_object_unref(app);
+#endif                          /* HAVE_UNIQUE */
     libbalsa_imap_server_close_all_connections();
     return 0;
 }
@@ -924,9 +1210,12 @@ balsa_cleanup(void)
     g_hash_table_destroy(libbalsa_mailbox_view_table);
     libbalsa_mailbox_view_table = NULL;
 
+#if HAVE_GNOME
     gnome_sound_shutdown();
+#endif
 }
 
+#if HAVE_GNOME
 static gint
 balsa_kill_session(GnomeClient * client, gpointer client_data)
 {
@@ -988,3 +1277,4 @@ balsa_save_session(GnomeClient * client, gint phase,
 
     return TRUE;
 }
+#endif /* HAVE_GNOME */
