@@ -19,9 +19,12 @@
  * 02111-1307, USA.
  */
 
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+
 #include <string.h>
 #include <gtk/gtk.h>
-#include "config.h"
 #include "balsa-app.h"
 #include "print.h"
 #include "misc.h"
@@ -309,8 +312,7 @@ begin_print(GtkPrintOperation * operation, GtkPrintContext * context,
 	pdata->setup.c_height -= P_TO_C(p_height) + C_HEADER_SEP;
 
 	/* remember in the context */
-	pdata->footer = footer_string->str;
-	g_string_free(footer_string, FALSE);
+	pdata->footer = g_string_free(footer_string, FALSE);
     }
     g_object_unref(G_OBJECT(layout));
 
@@ -377,7 +379,7 @@ get_default_user_units(void)
      */
     gchar *e = _("default:mm");
   
-#ifdef HAVE_LANGINFO
+#if (HAVE_LANGINFO && defined(_NL_MEASUREMENT_MEASUREMENT))
     gchar *imperial = NULL;
   
     imperial = nl_langinfo(_NL_MEASUREMENT_MEASUREMENT);
@@ -461,10 +463,14 @@ add_margin_spinbtn(const gchar * text, gdouble min, gdouble max, gdouble dflt,
 }
 
 static void
-check_margins(GtkAdjustment *adjustment, GtkAdjustment *other)
+check_margins(GtkAdjustment * adjustment, GtkAdjustment * other)
 {
-    if (adjustment->value + other->value > adjustment->upper)
-	gtk_adjustment_set_value(adjustment, adjustment->upper - other->value);
+    if (gtk_adjustment_get_value(adjustment) +
+        gtk_adjustment_get_value(other) >
+        gtk_adjustment_get_upper(adjustment))
+        gtk_adjustment_set_value(adjustment,
+                                 gtk_adjustment_get_upper(adjustment) -
+                                 gtk_adjustment_get_value(other));
 }
 
 static GtkWidget *
@@ -639,9 +645,11 @@ message_prefs_apply(GtkPrintOperation * operation, GtkWidget * widget,
 	g_strdup(gtk_font_button_get_font_name
 		 (GTK_FONT_BUTTON(print_prefs->footer_font)));
     balsa_app.print_highlight_cited =
-	GTK_TOGGLE_BUTTON(print_prefs->highlight_cited)->active;
+	gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
+                                     (print_prefs->highlight_cited));
     balsa_app.print_highlight_phrases =
-	GTK_TOGGLE_BUTTON(print_prefs->highlight_phrases)->active;
+	gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
+                                     (print_prefs->highlight_phrases));
 
     balsa_app.margin_top =
 	gtk_spin_button_get_value(GTK_SPIN_BUTTON(print_prefs->margin_top)) * 72.0;
