@@ -1,6 +1,6 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2003 Stuart Parmenter and others,
+ * Copyright (C) 1997-2010 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,16 +18,19 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
  * 02111-1307, USA.
  */
-#include "config.h"
 
-#define _XOPEN_SOURCE 500
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "save-restore.h"
 
+#include <stdlib.h>
 #include <string.h>
+#if HAVE_GNOME
 #include <gconf/gconf-client.h>
-#include <gnome.h>
+#endif
 #include <glib/gi18n.h>
 #include "balsa-app.h"
-#include "save-restore.h"
 #include "server.h"
 #include "quote-color.h"
 #include "toolbar-prefs.h"
@@ -195,11 +198,10 @@ config_mailbox_set_as_special(LibBalsaMailbox * mailbox, specialType which)
     if (*special) {
 	g_free((*special)->config_prefix); 
 	(*special)->config_prefix = NULL;
-        if(balsa_app.local_mail_directory == NULL
-           || !LIBBALSA_IS_MAILBOX_LOCAL(*special)
-           || strncmp(balsa_app.local_mail_directory, 
-                      libbalsa_mailbox_local_get_path(*special),
-                      strlen(balsa_app.local_mail_directory)) != 0) 
+        if (!LIBBALSA_IS_MAILBOX_LOCAL(*special)
+            || !libbalsa_path_is_below_dir(libbalsa_mailbox_local_get_path
+                                           (*special),
+                                           balsa_app.local_mail_directory))
             config_mailbox_add(*special, NULL);
 	g_object_remove_weak_pointer(G_OBJECT(*special), (gpointer) special);
 
@@ -390,6 +392,18 @@ pop3_progress_notify(LibBalsaMailbox* mailbox, int msg_type, int prog, int tot,
 
 /* Initialize the specified mailbox, creating the internal data
    structures which represent the mailbox. */
+static void
+sr_special_notify(gpointer data, GObject * mailbox)
+{
+    LibBalsaMailbox **special = data;
+
+    if (special == &balsa_app.trash && !balsa_app.mblist_tree_store
+        && balsa_app.empty_trash_on_exit)
+        empty_trash(balsa_app.main_window);
+
+    *special = NULL;
+}
+
 static gint
 config_mailbox_init(const gchar * prefix)
 {
@@ -442,7 +456,8 @@ config_mailbox_init(const gchar * prefix)
 	     * and outbox, and view->subscribe gets set correctly for
              * trashbox. */
 	    *special = mailbox;
-	    g_object_add_weak_pointer(G_OBJECT(mailbox), (gpointer) special);
+            g_object_weak_ref(G_OBJECT(mailbox),
+                              (GWeakNotify) sr_special_notify, special);
 	}
 
         balsa_mblist_mailbox_node_append(NULL, mbnode);
@@ -489,6 +504,8 @@ config_load_smtp_server(const gchar * key, const gchar * value, gpointer data)
 
     libbalsa_conf_push_group(key);
     smtp_server = libbalsa_smtp_server_new_from_config(value);
+    libbalsa_server_connect_signals(LIBBALSA_SERVER(smtp_server),
+				    G_CALLBACK(ask_password), NULL);
     libbalsa_conf_pop_group();
     libbalsa_smtp_server_add_to_list(smtp_server, smtp_servers);
 
@@ -496,7 +513,6 @@ config_load_smtp_server(const gchar * key, const gchar * value, gpointer data)
 }
 #endif                          /* ENABLE_ESMTP */
 
-#ifdef HAVE_GTK_PRINT
 static gboolean
 load_gtk_print_setting(const gchar * key, const gchar * value, gpointer data)
 {
@@ -563,7 +579,6 @@ restore_gtk_page_setup()
     
     return page_setup;
 }
-#endif
 
 static gint
 config_global_load(void)
@@ -698,6 +713,8 @@ config_global_load(void)
     }
 
     /* ... Quote colouring */
+    balsa_app.mark_quoted =
+        libbalsa_conf_get_bool_with_default("MarkQuoted=true", &def_used);
     g_free(balsa_app.quote_regex);
     balsa_app.quote_regex =
 	libbalsa_conf_get_string("QuoteRegex=" DEFAULT_QUOTE_REGEX);
@@ -737,14 +754,14 @@ config_global_load(void)
 	libbalsa_conf_get_string("SubjectFont=" DEFAULT_SUBJECT_FONT);
 
     /* ... wrap words */
-    balsa_app.browse_wrap = libbalsa_conf_get_bool("WordWrap=true");
+    balsa_app.browse_wrap = libbalsa_conf_get_bool("WordWrap=false");
     balsa_app.browse_wrap_length = libbalsa_conf_get_int("WordWrapLength=79");
     if (balsa_app.browse_wrap_length < 40)
 	balsa_app.browse_wrap_length = 40;
 
     /* ... handling of Multipart/Alternative */
     balsa_app.display_alt_plain = 
-	libbalsa_conf_get_bool("DisplayAlternativeAsPlain=false");
+	libbalsa_conf_get_bool("DisplayAlternativeAsPlain=true");
 
     /* ... handling of broken mails with 8-bit chars */
     balsa_app.convert_unknown_8bit = 
@@ -763,7 +780,11 @@ config_global_load(void)
     balsa_app.show_notebook_tabs = libbalsa_conf_get_bool("ShowTabs=false");
 
     /* ... alternative layout of main window */
-    balsa_app.alternative_layout = libbalsa_conf_get_bool("AlternativeLayout=false");
+    balsa_app.layout_type = libbalsa_conf_get_int("LayoutType=0");
+    if (balsa_app.layout_type != LAYOUT_DEFAULT && 
+        balsa_app.layout_type != LAYOUT_WIDE_MSG &&
+        balsa_app.layout_type != LAYOUT_WIDE_SCREEN)
+        balsa_app.layout_type = LAYOUT_DEFAULT;
     balsa_app.view_message_on_open = libbalsa_conf_get_bool("ViewMessageOnOpen=true");
     balsa_app.pgdownmod = libbalsa_conf_get_bool("PageDownMod=false");
     balsa_app.pgdown_percent = libbalsa_conf_get_int("PageDownPercent=50");
@@ -775,6 +796,17 @@ config_global_load(void)
     balsa_app.enable_view_filter = libbalsa_conf_get_bool("ViewFilter=false");
 #endif /* ENABLE_TOUCH_UI */
 
+    balsa_app.show_main_toolbar =
+        libbalsa_conf_get_bool("ShowMainWindowToolbar=true");
+    balsa_app.show_message_toolbar =
+        libbalsa_conf_get_bool("ShowMessageWindowToolbar=true");
+    balsa_app.show_compose_toolbar =
+        libbalsa_conf_get_bool("ShowComposeWindowToolbar=true");
+    balsa_app.show_statusbar =
+        libbalsa_conf_get_bool("ShowStatusbar=true");
+    balsa_app.show_sos_bar =
+        libbalsa_conf_get_bool("ShowSOSbar=true");
+
     /* ... Progress Window Dialog */
     balsa_app.pwindow_option = d_get_gint("ProgressWindow", WHILERETR);
 
@@ -805,13 +837,14 @@ config_global_load(void)
     libbalsa_conf_push_group("SourcePreview");
     balsa_app.source_escape_specials = 
         libbalsa_conf_get_bool("EscapeSpecials=true");
+    balsa_app.source_width  = libbalsa_conf_get_int("Width=500");
+    balsa_app.source_height = libbalsa_conf_get_int("Height=400");
     libbalsa_conf_pop_group();
 
     /* Printing options ... */
     libbalsa_conf_push_group("Printing");
 
     /* ... Printing */
-#ifdef HAVE_GTK_PRINT
     if (balsa_app.page_setup)
 	g_object_unref(G_OBJECT(balsa_app.page_setup));
     balsa_app.page_setup = restore_gtk_page_setup();
@@ -819,27 +852,6 @@ config_global_load(void)
     balsa_app.margin_top = libbalsa_conf_get_double("TopMargin");
     balsa_app.margin_right = libbalsa_conf_get_double("RightMargin");
     balsa_app.margin_bottom = libbalsa_conf_get_double("BottomMargin");
-#else
-    g_free(balsa_app.paper_size);
-    balsa_app.paper_size =
-	libbalsa_conf_get_string("PaperSize=" DEFAULT_PAPER_SIZE);
-    g_free(balsa_app.margin_left);
-    balsa_app.margin_left   = libbalsa_conf_get_string("LeftMargin");
-    g_free(balsa_app.margin_top);
-    balsa_app.margin_top    = libbalsa_conf_get_string("TopMargin");
-    g_free(balsa_app.margin_right);
-    balsa_app.margin_right  = libbalsa_conf_get_string("RightMargin");
-    g_free(balsa_app.margin_bottom);
-    balsa_app.margin_bottom = libbalsa_conf_get_string("BottomMargin");
-    g_free(balsa_app.print_unit);
-    balsa_app.print_unit    = libbalsa_conf_get_string("PrintUnit");
-    g_free(balsa_app.print_layout);
-    balsa_app.print_layout  = libbalsa_conf_get_string("PrintLayout");
-    g_free(balsa_app.paper_orientation);
-    balsa_app.paper_orientation  = libbalsa_conf_get_string("PaperOrientation");
-    g_free(balsa_app.page_orientation);
-    balsa_app.page_orientation   = libbalsa_conf_get_string("PageOrientation");
-#endif
 
     g_free(balsa_app.print_header_font);
     balsa_app.print_header_font =
@@ -855,19 +867,15 @@ config_global_load(void)
                                 DEFAULT_PRINT_BODY_FONT);
     balsa_app.print_highlight_cited =
         libbalsa_conf_get_bool("PrintHighlightCited=false");
-#ifdef HAVE_GTK_PRINT
     balsa_app.print_highlight_phrases =
         libbalsa_conf_get_bool("PrintHighlightPhrases=false");
-#endif
     libbalsa_conf_pop_group();
 
-#ifdef HAVE_GTK_PRINT
     /* GtkPrint printing */
     libbalsa_conf_push_group("GtkPrint");
     libbalsa_conf_foreach_keys("GtkPrint", load_gtk_print_setting,
 			       balsa_app.print_settings);
     libbalsa_conf_pop_group();
-#endif
 
     /* Spelling options ... */
     libbalsa_conf_push_group("Spelling");
@@ -909,6 +917,8 @@ config_global_load(void)
 	d_get_gint("NewMailNotificationDialog", 0);
     balsa_app.notify_new_mail_sound =
 	d_get_gint("NewMailNotificationSound", 1);
+    balsa_app.notify_new_mail_icon =
+	d_get_gint("NewMailNotificationIcon", 1);
     balsa_app.check_mail_upon_startup =
 	libbalsa_conf_get_bool("OnStartup=false");
     balsa_app.check_mail_auto = libbalsa_conf_get_bool("Auto=false");
@@ -1063,6 +1073,8 @@ config_global_load(void)
     
     balsa_app.remember_open_mboxes =
 	libbalsa_conf_get_bool("RememberOpenMailboxes=false");
+    balsa_app.current_mailbox_url =
+	libbalsa_conf_get_string("CurrentMailboxURL");
 
     balsa_app.empty_trash_on_exit =
 	libbalsa_conf_get_bool("EmptyTrash=false");
@@ -1129,7 +1141,6 @@ config_global_load(void)
     return TRUE;
 }				/* config_global_load */
 
-#ifdef HAVE_GTK_PRINT
 static void
 save_gtk_print_setting(const gchar *key, const gchar *value, gpointer user_data)
 {
@@ -1175,7 +1186,6 @@ save_gtk_page_setup(GtkPageSetup *page_setup)
     libbalsa_conf_set_int("Orientation",
 			  gtk_page_setup_get_orientation(page_setup));
 }
-#endif
 
 gint
 config_save(void)
@@ -1253,6 +1263,7 @@ config_save(void)
 			 libbalsa_mailbox_get_sort_field(NULL));
     libbalsa_conf_set_int("ThreadingType",
 			 libbalsa_mailbox_get_threading_type(NULL));
+    libbalsa_conf_set_bool("MarkQuoted", balsa_app.mark_quoted);
     libbalsa_conf_set_string("QuoteRegex", balsa_app.quote_regex);
     libbalsa_conf_set_string("MessageFont", balsa_app.message_font);
     libbalsa_conf_set_string("SubjectFont", balsa_app.subject_font);
@@ -1286,7 +1297,7 @@ config_save(void)
     libbalsa_conf_set_bool("ShowMailboxList", balsa_app.show_mblist);
     libbalsa_conf_set_bool("ShowTabs", balsa_app.show_notebook_tabs);
     libbalsa_conf_set_int("ProgressWindow", balsa_app.pwindow_option);
-    libbalsa_conf_set_bool("AlternativeLayout", balsa_app.alternative_layout);
+    libbalsa_conf_set_int("LayoutType",     balsa_app.layout_type);
     libbalsa_conf_set_bool("ViewMessageOnOpen", balsa_app.view_message_on_open);
     libbalsa_conf_set_bool("PageDownMod", balsa_app.pgdownmod);
     libbalsa_conf_set_int("PageDownPercent", balsa_app.pgdown_percent);
@@ -1294,6 +1305,18 @@ config_save(void)
     libbalsa_conf_set_bool("FileFormatCheck", balsa_app.do_file_format_check);
     libbalsa_conf_set_bool("ViewFilter",      balsa_app.enable_view_filter);
 #endif /* ENABLE_TOUCH_UI */
+
+    libbalsa_conf_set_bool("ShowMainWindowToolbar",
+                           balsa_app.show_main_toolbar);
+    libbalsa_conf_set_bool("ShowMessageWindowToolbar",
+                           balsa_app.show_message_toolbar);
+    libbalsa_conf_set_bool("ShowComposeWindowToolbar",
+                           balsa_app.show_compose_toolbar);
+    libbalsa_conf_set_bool("ShowStatusbar",
+                           balsa_app.show_statusbar);
+    libbalsa_conf_set_bool("ShowSOSbar",
+                           balsa_app.show_sos_bar);
+
     libbalsa_conf_set_bool("HideDeleted", libbalsa_mailbox_get_filter(NULL) & 1);
     libbalsa_conf_set_bool("ExpungeOnClose", balsa_app.expunge_on_close);
     libbalsa_conf_set_bool("AutoExpunge", balsa_app.expunge_auto);
@@ -1308,36 +1331,17 @@ config_save(void)
     libbalsa_conf_push_group("SourcePreview");
     libbalsa_conf_set_bool("EscapeSpecials",
                           balsa_app.source_escape_specials);
+    libbalsa_conf_set_int("Width",  balsa_app.source_width);
+    libbalsa_conf_set_int("Height", balsa_app.source_height);
     libbalsa_conf_pop_group();
 
     /* Printing options ... */
     libbalsa_conf_push_group("Printing");
-#ifdef HAVE_GTK_PRINT
     save_gtk_page_setup(balsa_app.page_setup);
     libbalsa_conf_set_double("LeftMargin", balsa_app.margin_left);
     libbalsa_conf_set_double("TopMargin", balsa_app.margin_top);
     libbalsa_conf_set_double("RightMargin", balsa_app.margin_right);
     libbalsa_conf_set_double("BottomMargin", balsa_app.margin_bottom);
-#else
-    libbalsa_conf_set_string("PaperSize",balsa_app.paper_size);
-    if(balsa_app.margin_left)
-	libbalsa_conf_set_string("LeftMargin",   balsa_app.margin_left);
-    if(balsa_app.margin_top)
-	libbalsa_conf_set_string("TopMargin",    balsa_app.margin_top);
-    if(balsa_app.margin_bottom)
-	libbalsa_conf_set_string("RightMargin",  balsa_app.margin_right);
-    if(balsa_app.margin_bottom)
-	libbalsa_conf_set_string("BottomMargin", balsa_app.margin_bottom);
-    if(balsa_app.print_unit)
-	libbalsa_conf_set_string("PrintUnit",    balsa_app.print_unit);
-    if(balsa_app.print_layout)
-	libbalsa_conf_set_string("PrintLayout", balsa_app.print_layout);
-    if(balsa_app.margin_bottom)
-	libbalsa_conf_set_string("PaperOrientation", 
-				balsa_app.paper_orientation);
-    if(balsa_app.margin_bottom)
-	libbalsa_conf_set_string("PageOrientation", balsa_app.page_orientation);
-#endif
 
     libbalsa_conf_set_string("PrintHeaderFont",
                             balsa_app.print_header_font);
@@ -1346,13 +1350,10 @@ config_save(void)
                             balsa_app.print_footer_font);
     libbalsa_conf_set_bool("PrintHighlightCited",
                           balsa_app.print_highlight_cited);
-#ifdef HAVE_GTK_PRINT
     libbalsa_conf_set_bool("PrintHighlightPhrases",
                           balsa_app.print_highlight_phrases);
-#endif
     libbalsa_conf_pop_group();
 
-#ifdef HAVE_GTK_PRINT
     /* GtkPrintSettings stuff */
     libbalsa_conf_remove_group("GtkPrint");
     libbalsa_conf_push_group("GtkPrint");
@@ -1360,7 +1361,6 @@ config_save(void)
 	gtk_print_settings_foreach(balsa_app.print_settings,
 				   save_gtk_print_setting, NULL);
     libbalsa_conf_pop_group();
-#endif
 
     /* Spelling options ... */
     libbalsa_conf_remove_group("Spelling");
@@ -1392,10 +1392,12 @@ config_save(void)
     /* Maibox checking options ... */
     libbalsa_conf_push_group("MailboxChecking");
 
-	libbalsa_conf_set_int("NewMailNotificationDialog",
-				balsa_app.notify_new_mail_dialog);
-	libbalsa_conf_set_int("NewMailNotificationSound",
-				balsa_app.notify_new_mail_sound);
+    libbalsa_conf_set_int("NewMailNotificationDialog",
+                          balsa_app.notify_new_mail_dialog);
+    libbalsa_conf_set_int("NewMailNotificationSound",
+                          balsa_app.notify_new_mail_sound);
+    libbalsa_conf_set_int("NewMailNotificationIcon",
+                          balsa_app.notify_new_mail_icon);
     libbalsa_conf_set_bool("OnStartup", balsa_app.check_mail_upon_startup);
     libbalsa_conf_set_bool("Auto", balsa_app.check_mail_auto);
     libbalsa_conf_set_int("AutoDelay", balsa_app.check_mail_timer);
@@ -1472,6 +1474,9 @@ config_save(void)
 
     libbalsa_conf_set_bool("RememberOpenMailboxes",
 			  balsa_app.remember_open_mboxes);
+    libbalsa_conf_set_string("CurrentMailboxURL",
+                             balsa_app.current_mailbox_url);
+
     libbalsa_conf_set_bool("EmptyTrash", balsa_app.empty_trash_on_exit);
 
     if (balsa_app.default_address_book)
@@ -1701,17 +1706,12 @@ config_identities_save(void)
 {
     LibBalsaIdentity* ident;
     GList* list;
-    gchar** conf_vec, *prefix;
+    gchar *prefix;
 
-    conf_vec = g_malloc(sizeof(gchar*) * g_list_length(balsa_app.identities));
-
-    g_assert(conf_vec != NULL);
-    
     libbalsa_conf_push_group("identity");
     libbalsa_conf_set_string("CurrentIdentity", 
                             balsa_app.current_ident->identity_name);
     libbalsa_conf_pop_group();
-    g_free(conf_vec);
 
     config_remove_groups(IDENTITY_SECTION_PREFIX);
 
@@ -1753,7 +1753,7 @@ config_view_load(const gchar * key, const gchar * value, gpointer data)
             libbalsa_conf_get_string_with_default("MailingListAddress",
                                                   &def);
         view->mailing_list_address =
-            def ? NULL : internet_address_parse_string(address);
+            def ? NULL : internet_address_list_parse_string(address);
         g_free(address);
 
         view->identity_name = libbalsa_conf_get_string("Identity");
@@ -1826,22 +1826,33 @@ config_views_load(void)
                                 GINT_TO_POINTER(FALSE));
 }
 
+/* Get viewByUrl prefix */
+static gchar *
+view_by_url_prefix(const gchar * url)
+{
+    gchar *url_enc;
+    gchar *prefix;
+
+    url_enc = libbalsa_urlencode(url);
+    prefix = g_strconcat(VIEW_BY_URL_SECTION_PREFIX, url_enc, NULL);
+    g_free(url_enc);
+
+    return prefix;
+}
+
 /* config_views_save:
    iterates over all mailbox views.
 */
 static void
 save_view(const gchar * url, LibBalsaMailboxView * view)
 {
-    gchar *url_enc;
     gchar *prefix;
 
     if (!view || (view->in_sync && view->used))
 	return;
     view->in_sync = TRUE;
 
-    url_enc = libbalsa_urlencode(url);
-    prefix = g_strconcat(VIEW_BY_URL_SECTION_PREFIX, url_enc, NULL);
-    g_free(url_enc);
+    prefix = view_by_url_prefix(url);
 
     /* Remove the view--it will be recreated if any member needs to be
      * saved. */
@@ -1907,6 +1918,15 @@ config_views_save(void)
 			 NULL);
 }
 
+void
+config_view_remove(const gchar * url)
+{
+    gchar *prefix = view_by_url_prefix(url);
+    libbalsa_conf_remove_group(prefix);
+    g_free(prefix);
+    g_hash_table_remove(libbalsa_mailbox_view_table, url);
+}
+
 static void
 save_color(gchar * key, GdkColor * color)
 {
@@ -1922,7 +1942,6 @@ static gboolean
 config_filter_load(const gchar * key, const gchar * value, gpointer data)
 {
     char *endptr;
-    guint *save = data;
     LibBalsaFilter *fil;
     long int dummy;
 
@@ -1935,20 +1954,6 @@ config_filter_load(const gchar * key, const gchar * value, gpointer data)
     libbalsa_conf_push_group(key);
 
     fil = libbalsa_filter_new_from_config();
-    if (!fil->condition) {
-        /* Try pre-2.1 style: */
-        FilterOpType op = libbalsa_conf_get_int("Operation");
-        ConditionMatchType cmt =
-            op == FILTER_OP_OR ? CONDITION_OR : CONDITION_AND;
-        fil->condition = libbalsa_condition_new_2_0(key, cmt);
-        if (fil->condition) {
-            if (fil->action > FILTER_TRASH)
-                /* Some 2.0.x versions had a new action code which
-                 * changed the value of FILTER_TRASH. */
-                fil->action = FILTER_TRASH;
-            ++*save;
-        }
-    }
     if (!fil->condition) {
         g_idle_add((GSourceFunc) config_warning_idle,
                    _("Filter with no condition was omitted"));
@@ -1966,14 +1971,9 @@ config_filter_load(const gchar * key, const gchar * value, gpointer data)
 static void
 config_filters_load(void)
 {
-    guint save = 0;
-
     filter_errno = FILTER_NOERR;
     libbalsa_conf_foreach_group(FILTER_SECTION_PREFIX,
-                                config_filter_load, &save);
-
-    if (save)
-        config_filters_save();
+                                config_filter_load, NULL);
 }
 
 #define FILTER_SECTION_MAX "9999"
@@ -2127,6 +2127,7 @@ check_for_old_sigs(GList * id_list_tmp)
     }
 }
 
+#if HAVE_GNOME
 void
 config_defclient_save(void)
 {
@@ -2176,3 +2177,4 @@ config_defclient_save(void)
         }
     }
 }
+#endif /* HAVE_GNOME */
