@@ -19,30 +19,33 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "spell-check.h"
 
 #define USE_ORIGINAL_MANAGER_FUNCS
 
 #include <ctype.h>
-#include <gnome.h>
 /* use old 0.11-style API in pspell */
 #define USE_ORIGINAL_MANAGER_FUNCS
 #include <pspell/pspell.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
-#include "i18n.h"
+#include <glib/gi18n.h>
 
-#ifdef HAVE_PCRE
-#  include <pcre.h>
-#  include <pcreposix.h>
-#else
-#  include <sys/types.h>
-#  include <regex.h>
-#endif
+#if !USE_GREGEX
+#  ifdef HAVE_PCRE
+#    include <pcreposix.h>
+#  else
+#    include <sys/types.h>
+#    include <regex.h>
+#  endif
+#endif                          /* USE_GREGEX */
 
 #include "balsa-app.h"
 #include "quote-color.h"
-#include "spell-check.h"
 #include "balsa-icons.h"
 
 #define SPELLMGR_CODESET "UTF-8"
@@ -158,10 +161,10 @@ static GtkDialogClass *parent_class = NULL;
 
 
 /* begin actual code */
-GtkType
+GType
 balsa_spell_check_get_type()
 {
-    static GtkType balsa_spell_check_type = 0;
+    static GType balsa_spell_check_type = 0;
 
     if (!balsa_spell_check_type) {
 	static const GTypeInfo balsa_spell_check_info = {
@@ -198,7 +201,7 @@ balsa_spell_check_class_init(BalsaSpellCheckClass * klass)
     widget_class = (GtkWidgetClass *) klass;
     container_class = (GtkContainerClass *) klass;
 
-    parent_class = gtk_type_class(GTK_TYPE_FRAME);
+    parent_class = g_type_class_peek_parent(klass);
 
     /* GObject signals */
     o_class->set_property = spch_set_property;
@@ -461,6 +464,7 @@ balsa_spell_check_init(BalsaSpellCheck * spell_check)
     GtkWidget *learn;
     GtkWidget *done;
     GtkWidget *cancel;
+    GtkBox *content_box, *action_box;
 
     const guint padding = 4;
 
@@ -479,15 +483,15 @@ balsa_spell_check_init(BalsaSpellCheck * spell_check)
     /* setup suggestion display */
     new_word_text = gtk_entry_new();
     spell_check->entry = GTK_ENTRY(new_word_text);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(spell_check)->vbox),
-                       new_word_text, FALSE, FALSE, 0);
+    content_box =
+        GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(spell_check)));
+    gtk_box_pack_start(content_box, new_word_text, FALSE, FALSE, 0);
 
     sw = gtk_scrolled_window_new(NULL, NULL);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
 				   GTK_POLICY_AUTOMATIC,
 				   GTK_POLICY_AUTOMATIC);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(spell_check)->vbox),
-                       sw, TRUE, TRUE, 0);
+    gtk_box_pack_start(content_box, sw, TRUE, TRUE, 0);
 
     /* setup suggestion list */
     store = gtk_list_store_new(1, G_TYPE_STRING);
@@ -506,99 +510,55 @@ balsa_spell_check_init(BalsaSpellCheck * spell_check)
     /* setup buttons to perform actions */
     vbox = gtk_vbox_new(FALSE, padding);
     change = balsa_stock_button_with_label(GTK_STOCK_REDO,
-					   "_Change");
-#if GTK_CHECK_VERSION(2, 11, 0)
+					   _("_Change"));
     gtk_widget_set_tooltip_text(change,
                                 _("Replace the current word "
                                   "with the selected suggestion"));
-#else                           /* GTK_CHECK_VERSION(2, 11, 0) */
-    gtk_tooltips_set_tip(balsa_app.tooltips, change,
-			 _("Replace the current word "
-                           "with the selected suggestion"),
-			 NULL);
-#endif                          /* GTK_CHECK_VERSION(2, 11, 0) */
     gtk_box_pack_start(GTK_BOX(vbox), change, FALSE, FALSE, 0);
 
     change_all = balsa_stock_button_with_label(GTK_STOCK_REFRESH,
-					       "Change _All");
-#if GTK_CHECK_VERSION(2, 11, 0)
+					       _("Change _All"));
     gtk_widget_set_tooltip_text(change_all,
                                 _("Replace all occurrences of the current word "
                                   "with the selected suggestion"));
-#else                           /* GTK_CHECK_VERSION(2, 11, 0) */
-    gtk_tooltips_set_tip(balsa_app.tooltips, change_all,
-			 _("Replace all occurrences of the current word "
-                           "with the selected suggestion"),
-			 NULL);
-#endif                          /* GTK_CHECK_VERSION(2, 11, 0) */
     gtk_box_pack_start(GTK_BOX(vbox), change_all, FALSE, FALSE, 0);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(spell_check)->action_area),
-                       vbox, FALSE, FALSE, 0);
+    action_box =
+        GTK_BOX(gtk_dialog_get_action_area(GTK_DIALOG(spell_check)));
+    gtk_box_pack_start(action_box, vbox, FALSE, FALSE, 0);
 
     vbox = gtk_vbox_new(FALSE, padding);
     ignore = balsa_stock_button_with_label(GTK_STOCK_GO_FORWARD,
-                                           "_Ignore");
+                                           _("_Ignore"));
 
-#if GTK_CHECK_VERSION(2, 11, 0)
     gtk_widget_set_tooltip_text(ignore,
                                 _("Skip the current word"));
-#else                           /* GTK_CHECK_VERSION(2, 11, 0) */
-    gtk_tooltips_set_tip(balsa_app.tooltips, ignore,
-			 _("Skip the current word"), NULL);
-#endif                          /* GTK_CHECK_VERSION(2, 11, 0) */
     gtk_box_pack_start(GTK_BOX(vbox), ignore, FALSE, FALSE, 0);
 
     ignore_all = balsa_stock_button_with_label(GTK_STOCK_GOTO_LAST,
-                                               "Ignore A_ll");
-#if GTK_CHECK_VERSION(2, 11, 0)
+                                               _("Ignore A_ll"));
     gtk_widget_set_tooltip_text(ignore_all,
                                 _("Skip all occurrences of the current word"));
-#else                           /* GTK_CHECK_VERSION(2, 11, 0) */
-    gtk_tooltips_set_tip(balsa_app.tooltips, ignore_all,
-			 _("Skip all occurrences of the current word"),
-			 NULL);
-#endif                          /* GTK_CHECK_VERSION(2, 11, 0) */
     gtk_box_pack_start(GTK_BOX(vbox), ignore_all, FALSE, FALSE, 0);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(spell_check)->action_area),
-                       vbox, FALSE, FALSE, 0);
+    gtk_box_pack_start(action_box, vbox, FALSE, FALSE, 0);
 
     vbox = gtk_vbox_new(FALSE, padding);
     learn = balsa_stock_button_with_label(BALSA_PIXMAP_BOOK_OPEN,
-                                          "_Learn");
-#if GTK_CHECK_VERSION(2, 11, 0)
+                                          _("_Learn"));
     gtk_widget_set_tooltip_text(learn,
                                 _("Add the current word to your personal dictionary"));
-#else                           /* GTK_CHECK_VERSION(2, 11, 0) */
-    gtk_tooltips_set_tip(balsa_app.tooltips, learn,
-			 _("Add the current word to your personal dictionary"),
-			 NULL);
-#endif                          /* GTK_CHECK_VERSION(2, 11, 0) */
     gtk_box_pack_start(GTK_BOX(vbox), learn, FALSE, FALSE, 0);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(spell_check)->action_area),
-                       vbox, FALSE, FALSE, 0);
+    gtk_box_pack_start(action_box, vbox, FALSE, FALSE, 0);
 
     vbox = gtk_vbox_new(FALSE, padding);
-    done = balsa_stock_button_with_label(GTK_STOCK_OK, "_Done");
-#if GTK_CHECK_VERSION(2, 11, 0)
+    done = balsa_stock_button_with_label(GTK_STOCK_OK, _("_Done"));
     gtk_widget_set_tooltip_text(done, _("Finish spell checking"));
-#else                           /* GTK_CHECK_VERSION(2, 11, 0) */
-    gtk_tooltips_set_tip(balsa_app.tooltips, done,
-			 _("Finish spell checking"), NULL);
-#endif                          /* GTK_CHECK_VERSION(2, 11, 0) */
     gtk_box_pack_start(GTK_BOX(vbox), done, FALSE, FALSE, 0);
 
     cancel = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
-#if GTK_CHECK_VERSION(2, 11, 0)
-    gtk_widget_set_tooltip_text(learn,
+    gtk_widget_set_tooltip_text(cancel,
                                 _("Revert all changes and finish spell checking"));
-#else                           /* GTK_CHECK_VERSION(2, 11, 0) */
-    gtk_tooltips_set_tip(balsa_app.tooltips, cancel,
-			 _("Revert all changes and finish spell checking"),
-			 NULL);
-#endif                          /* GTK_CHECK_VERSION(2, 11, 0) */
     gtk_box_pack_start(GTK_BOX(vbox), cancel, FALSE, FALSE, 0);
-    gtk_box_pack_end(GTK_BOX(GTK_DIALOG(spell_check)->action_area),
-                     vbox, FALSE, FALSE, 0);
+    gtk_box_pack_end(action_box, vbox, FALSE, FALSE, 0);
 
     /* connect signal handlers */
     g_signal_connect(G_OBJECT(selection), "changed",
@@ -803,7 +763,11 @@ done_cb(GtkButton * button, gpointer data)
  * PSpellManager to do the checking.
  * */
 
+#if USE_GREGEX
+static GRegex *quoted_rex;
+#else                           /* USE_GREGEX */
 static regex_t quoted_rex;
+#endif                          /* USE_GREGEX */
 static gboolean quoted_rex_compiled = FALSE;
 
 void
@@ -815,7 +779,6 @@ balsa_spell_check_start(BalsaSpellCheck * spell_check, GtkWindow *parent_wnd)
     GtkTextIter start, end, iter;
     GtkTextMark *insert;
 
-
     /* Config the spell check */
     spell_check->spell_config = new_pspell_config();
     pspell_config_replace(spell_check->spell_config,
@@ -845,6 +808,7 @@ balsa_spell_check_start(BalsaSpellCheck * spell_check, GtkWindow *parent_wnd)
 	/* quit without breaking things */
 	balsa_information_parented(parent_wnd,
                                    LIBBALSA_INFORMATION_ERROR,
+                                   "%s",
                                    pspell_error_message(spell_error));
 
 	/* Generate a response signal. */
@@ -886,15 +850,25 @@ balsa_spell_check_start(BalsaSpellCheck * spell_check, GtkWindow *parent_wnd)
      * balsa_app.quote_regex may change, so compile it new every
      * time!)
      */
+#if USE_GREGEX
+    if (quoted_rex_compiled)
+        g_regex_unref(quoted_rex);
+    quoted_rex = balsa_quote_regex_new();
+    if (!quoted_rex)
+        quoted_rex_compiled = FALSE;
+    else
+        quoted_rex_compiled = TRUE;
+#else                           /* USE_GREGEX */
     if (quoted_rex_compiled)
         regfree(&quoted_rex);
     if (regcomp(&quoted_rex, balsa_app.quote_regex, REG_EXTENDED)) {
         balsa_information(LIBBALSA_INFORMATION_ERROR,
-                          "BalsaSpellCheck: Quoted text "
-                          "regular expression compilation failed\n");
+                          _("BalsaSpellCheck: Quoted text "
+                            "regular expression compilation failed\n"));
         quoted_rex_compiled = FALSE;
     } else
         quoted_rex_compiled = TRUE;
+#endif                          /* USE_GREGEX */
 
     spell_check->end_iter = start;
 
@@ -1013,12 +987,13 @@ balsa_spell_check_learn(BalsaSpellCheck * spell_check,
     if (!result) {
 	if (pspell_manager_error_number(spell_check->spell_manager) != 0) {
 	    balsa_information(LIBBALSA_INFORMATION_ERROR,
-			      "BalsaSpellCheck: Learn operation failed;\n%s\n",
+			      _("BalsaSpellCheck: "
+                                "Learn operation failed;\n%s\n"),
 			      pspell_manager_error_message
 			      (spell_check->spell_manager));
 	} else {
 	    balsa_information(LIBBALSA_INFORMATION_ERROR,
-			      "BalsaSpellCheck: Learn operation failed.\n");
+			      _("BalsaSpellCheck: Learn operation failed.\n"));
 	}
     }
 
@@ -1128,7 +1103,11 @@ balsa_spell_check_destroy(GtkObject * object)
     }
 
     if (quoted_rex_compiled) {
+#if USE_GREGEX
+        g_regex_unref(quoted_rex);
+#else                           /* USE_GREGEX */
         regfree(&quoted_rex);
+#endif                          /* USE_GREGEX */
         quoted_rex_compiled = FALSE;
     }
 
@@ -1213,8 +1192,8 @@ setup_suggestions(BalsaSpellCheck * spell_check)
 
         if (balsa_app.debug)
             balsa_information(LIBBALSA_INFORMATION_DEBUG,
-                              "BalsaSpellCheck: Suggest %s (%s)\n", new_word,
-			      cword);
+                              _("BalsaSpellCheck: Suggest %s (%s)\n"),
+                              new_word, cword);
 	if(new_word)
 	    new_word=cword;
 
@@ -1315,7 +1294,7 @@ check_pspell_errors(PspellManager * manager)
 {
     if (pspell_manager_error_number(manager) != 0) {
 	balsa_information(LIBBALSA_INFORMATION_WARNING,
-			  "BalsaSpellCheck: Pspell Error: %s\n",
+			  _("BalsaSpellCheck: Pspell Error: %s\n"),
 			  pspell_manager_error_message(manager));
 	return TRUE;
     }
@@ -1365,8 +1344,13 @@ next_word(BalsaSpellCheck * spell_check)
                                         &line_end, FALSE);
             skip_sig = (!balsa_app.check_sig
                         && strcmp(line, "-- \n") == 0);
+#if USE_GREGEX
+            skip_quoted = (!balsa_app.check_quoted && quoted_rex_compiled
+                           && is_a_quote(line, quoted_rex));
+#else                           /* USE_GREGEX */
             skip_quoted = (!balsa_app.check_quoted && quoted_rex_compiled
                            && is_a_quote(line, &quoted_rex));
+#endif                          /* USE_GREGEX */
             g_free(line);
 
             if (skip_sig)
